@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2000-2012
+ *			Copyright (c) Telecom ParisTech 2000-2019
  *					All rights reserved
  *
  *			Authors: Jean Le Feuvre
@@ -26,23 +26,17 @@
  */
 
 
-#include <gpac/constants.h>
-#include <gpac/isomedia.h>
+#include "mp4box.h"
 
 #ifndef GPAC_DISABLE_SENG
 #include <gpac/scene_engine.h>
 #endif
 #ifndef GPAC_DISABLE_STREAMING
-#include <gpac/filestreamer.h>
 #include <gpac/rtp_streamer.h>
 #endif
 
 #include <gpac/mpegts.h>
 
-#ifndef GPAC_DISABLE_STREAMING
-#include <gpac/internal/ietf_dev.h>
-#endif
-
 #if defined(GPAC_DISABLE_ISOM) || defined(GPAC_DISABLE_ISOM_WRITE)
 
 #error "Cannot compile MP4Box if GPAC is not built with ISO File Format support"
@@ -51,164 +45,16 @@
 
 #if !defined(GPAC_DISABLE_STREAMING) && !defined(GPAC_DISABLE_SENG)
 
-void PrintStreamerUsage()
-{
-	fprintf(stderr, "File Streamer Options\n"
-	        "\n"
-	        "MP4Box can stream ISO files to RTP. The streamer currently doesn't support\n"
-	        "data carrouselling and will therefore not handle BIFS and OD streams properly.\n"
-	        "\n"
-            "-rtp         enables streamer\n"
-            "-run-for=T   runs for T seconds of the media then exits\n"
-	        "-noloop      disables looping when streaming\n"
-	        "-mpeg4       forces MPEG-4 ES Generic for all RTP streams\n"
-	        "-dst=IP      IP destination (uni/multi-cast). Default: 127.0.0.1\n"
-	        "-port=PORT   output port of the first stream. Default: 7000\n"
-	        "-mtu=MTU     path MTU for RTP packets. Default is 1450 bytes\n"
-	        "-ifce=IFCE   IP address of the physical interface to use. Default: NULL (ANY)\n"
-	        "-ttl=TTL     time to live for multicast packets. Default: 1\n"
-	        "-sdp=Name    file name of the generated SDP. Default: \"session.sdp\"\n"
-	        "\n"
-	       );
-}
-
-static void on_logs(void *cbk, GF_LOG_Level ll, GF_LOG_Tool lm, const char *fmt, va_list list)
-{
-	FILE *logs = (FILE*)cbk;
-	vfprintf(logs, fmt, list);
-	fflush(logs);
-}
-
-int stream_file_rtp(int argc, char **argv)
-{
-	GF_ISOMRTPStreamer *file_streamer;
-	char *sdp_file = "session.sdp";
-	char *ip_dest = "127.0.0.1";
-	char *ifce_addr = NULL;
-	char *inName = NULL;
-	char *logs=NULL;
-	FILE *logfile=NULL;
-	u16 port = 7000;
-	u32 ttl = 1;
-	Bool loop = GF_TRUE;
-    GF_MemTrackerType mem_track = GF_MemTrackerNone;
-	Bool force_mpeg4 = GF_FALSE;
-    u32 path_mtu = 1450;
-    Double run_for = -1.0;
-	u32 i;
-
-	for (i = 1; i < (u32) argc ; i++) {
-		char *arg = argv[i];
-
-		if (arg[0] != '-') {
-			if (inName) {
-				fprintf(stderr, "Error - 2 input names specified, please check usage\n");
-				return 1;
-			}
-			inName = arg;
-		}
-		else if (!stricmp(arg, "-noloop")) loop = GF_FALSE;
-		else if (!stricmp(arg, "-mpeg4")) force_mpeg4 = GF_TRUE;
-		else if (!strnicmp(arg, "-port=", 6)) port = atoi(arg+6);
-		else if (!strnicmp(arg, "-mtu=", 5)) path_mtu = atoi(arg+5);
-		else if (!strnicmp(arg, "-dst=", 5)) ip_dest = arg+5;
-		else if (!strnicmp(arg, "-ttl=", 5)) ttl = atoi(arg+5);
-		else if (!strnicmp(arg, "-ifce=", 6)) ifce_addr = arg+6;
-		else if (!strnicmp(arg, "-sdp=", 5)) sdp_file = arg+5;
-        else if (!stricmp(arg, "-mem-track")) mem_track = GF_MemTrackerSimple;
-        else if (!stricmp(arg, "-mem-track-stack")) mem_track = GF_MemTrackerBackTrace;
-		else if (!strnicmp(arg, "-logs=", 6)) logs = arg+6;
-		else if (!strnicmp(arg, "-lf=", 4)) logfile = gf_fopen(arg+4, "wt");
-        else if (!strnicmp(arg, "-run-for=", 9)) run_for = atof(arg+9);
-	}
-
-	gf_sys_init(mem_track);
-	if (logs)
-		gf_log_set_tools_levels(logs);
-	else
-		gf_log_set_tool_level(GF_LOG_RTP, GF_LOG_INFO); //set to debug to have packet list
-	if (logfile) {
-		gf_log_set_callback(logfile, on_logs);
-	}
-
-	if (!gf_isom_probe_file(inName)) {
-		fprintf(stderr, "File %s is not a valid ISO Media file and cannot be streamed\n", inName);
-		if (logfile) gf_fclose(logfile);
-		gf_sys_close();
-		return 1;
-	}
-
-	file_streamer = gf_isom_streamer_new(inName, ip_dest, port, loop, force_mpeg4, path_mtu, ttl, ifce_addr);
-	if (!file_streamer) {
-		fprintf(stderr, "Cannot create file streamer\n");
-	} else {
-        Bool run = GF_TRUE;
-		u32 check = 50;
-		fprintf(stderr, "Starting streaming %s to %s:%d\n", inName, ip_dest, port);
-		gf_isom_streamer_write_sdp(file_streamer, sdp_file);
-
-        if (run_for==0) run=GF_FALSE;
-        
-		while (run) {
-			gf_isom_streamer_send_next_packet(file_streamer, 0, 0);
-			check--;
-			if (!check) {
-				if (gf_prompt_has_input()) {
-					char c = (char) gf_prompt_get_char();
-					if (c=='q') break;
-				}
-				check = 50;
-			}
-            if ((run_for > 0) && (run_for < gf_isom_streamer_get_current_time(file_streamer)) )
-                break;
-		}
-		gf_isom_streamer_del(file_streamer);
-	}
-	if (logfile) gf_fclose(logfile);
-	gf_sys_close();
-	return 0;
-}
 
 
-void PrintLiveUsage()
-{
-	fprintf(stderr,
 
-	        "Live scene encoder options:\n"
-	        "-dst=IP    destination IP - default: NULL\n"
-	        "-port=PORT destination port - default: 7000\n"
-	        "-mtu=MTU   path MTU for RTP packets. Default is 1450 bytes\n"
-	        "-ifce=IFCE IP address of the physical interface to use. Default: NULL(ANY)\n"
-	        "-ttl=TTL   time to live for multicast packets. Default: 1\n"
-	        "-sdp=Name  ouput SDP file - default: session.sdp\n"
-	        "\n"
-	        "-dims      turns on DIMS mode for SVG input - default: off\n"
-	        "-no-rap    disabled RAP sending - this also disables carousel generation. Default: off\n"
-	        "-src=file  source of updates - default: null\n"
-	        "-rap=time  duration in ms of base carousel - default: 0 (off)\n"
-	        "            you can specify the RAP period of a single ESID (not in DIMS):\n"
-	        "                -rap=ESID=X:time\n"
-	        "\n"
-	        "Runtime options:\n"
-	        "q:         quits\n"
-	        "u:         inputs some commands to be sent\n"
-	        "U:         same as u but signals the updates as critical\n"
-	        "e:         inputs some commands to be sent without being aggregated\n"
-	        "E:         same as e but signals the updates as critical\n"
-	        "f:         forces RAP sending\n"
-	        "F:         forces RAP regeneration and sending\n"
-	        "p:         dumps current scene\n"
-	        "\n"
-	        "GPAC version: " GPAC_FULL_VERSION "\n"
-	        "");
-}
 typedef struct
 {
 	GF_RTPStreamer *rtp;
 	Bool manual_rtcp;
 	u16 ESID;
 
-	char *carousel_data;
+	u8 *carousel_data;
 	u32 carousel_size, carousel_alloc;
 	u32 last_carousel_time;
 	u64 carousel_ts, time_at_carousel_store;
@@ -265,7 +111,7 @@ RTPChannel *next_carousel(LiveSession *sess, u32 *timeout)
 }
 
 
-static void live_session_callback(void *calling_object, u16 ESID, char *data, u32 size, u64 ts)
+static void live_session_callback(void *calling_object, u16 ESID, u8 *data, u32 size, u64 ts)
 {
 	LiveSession *livesess = (LiveSession *) calling_object;
 	RTPChannel *rtpch;
@@ -310,7 +156,7 @@ static void live_session_callback(void *calling_object, u16 ESID, char *data, u3
 static void live_session_send_carousel(LiveSession *livesess, RTPChannel *ch)
 {
 	u32 now = gf_sys_clock();
-	u64 ts=0;
+	u64 ts;
 	if (ch) {
 		if (ch->carousel_size) {
 			ts = ch->carousel_ts + ch->timescale * ( (ch->adjust_carousel_time ? (u64)gf_sys_clock() : ch->time_at_carousel_store) - ch->init_time + ch->ts_delta)/1000;
@@ -346,7 +192,7 @@ static void live_session_send_carousel(LiveSession *livesess, RTPChannel *ch)
 	}
 }
 
-static void live_session_setup(LiveSession *livesess, char *ip, u16 port, u32 path_mtu, u32 ttl, char *ifce_addr, char *sdp_name)
+static Bool live_session_setup(LiveSession *livesess, char *ip, u16 port, u32 path_mtu, u32 ttl, char *ifce_addr, char *sdp_name)
 {
 	RTPChannel *rtpch;
 	u32 count = gf_seng_get_stream_count(livesess->seng);
@@ -355,10 +201,16 @@ static void live_session_setup(LiveSession *livesess, char *ip, u16 port, u32 pa
 	char *sdp = gf_rtp_streamer_format_sdp_header("GPACSceneStreamer", ip, NULL, iod64);
 	if (iod64) gf_free(iod64);
 
+#ifdef GPAC_ENABLE_COVERAGE
+	if (gf_sys_is_cov_mode()) {
+		GF_Descriptor *desc = gf_seng_get_iod(livesess->seng);
+		if (desc) gf_odf_desc_del(desc);
+	}
+#endif
 	for (i=0; i<count; i++) {
 		u16 ESID;
 		u32 st, oti, ts;
-		char *config = NULL;
+		const u8 *config = NULL;
 		u32 config_len;
 		gf_seng_get_stream_config(livesess->seng, i, &ESID, &config, &config_len, &st, &oti, &ts);
 
@@ -373,9 +225,9 @@ static void live_session_setup(LiveSession *livesess, char *ip, u16 port, u32 pa
 		switch (st) {
 		case GF_STREAM_OD:
 		case GF_STREAM_SCENE:
-			rtpch->rtp = gf_rtp_streamer_new_extended(st, oti, ts, ip, port, path_mtu, ttl, ifce_addr,
-			             GP_RTP_PCK_SYSTEMS_CAROUSEL, (char *) config, config_len,
-			             96, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4);
+			rtpch->rtp = gf_rtp_streamer_new(st, oti, ts, ip, port, path_mtu, ttl, ifce_addr,
+			             GP_RTP_PCK_SYSTEMS_CAROUSEL, config, config_len,
+			             96, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4, GF_FALSE);
 
 			if (rtpch->rtp) {
 				gf_rtp_streamer_disable_auto_rtcp(rtpch->rtp);
@@ -383,18 +235,20 @@ static void live_session_setup(LiveSession *livesess, char *ip, u16 port, u32 pa
 			}
 			break;
 		default:
-			rtpch->rtp = gf_rtp_streamer_new(st, oti, ts, ip, port, path_mtu, ttl, ifce_addr, GP_RTP_PCK_SIGNAL_RAP, (char *) config, config_len);
+			rtpch->rtp = gf_rtp_streamer_new(st, oti, ts, ip, port, path_mtu, ttl, ifce_addr, GP_RTP_PCK_SIGNAL_RAP, config, config_len, 96, 0, 0, GF_FALSE, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, GF_FALSE);
 			break;
 		}
 		rtpch->ESID = ESID;
 		rtpch->adjust_carousel_time = 1;
 		gf_list_add(livesess->streams, rtpch);
 
-		gf_rtp_streamer_append_sdp(rtpch->rtp, ESID, (char *) config, config_len, NULL, &sdp);
+		if (!rtpch->rtp)
+			return GF_FALSE;
+
+		gf_rtp_streamer_append_sdp(rtpch->rtp, ESID, config, config_len, NULL, &sdp);
 
 		/*fetch initial config of the broadcast*/
 		gf_seng_get_stream_carousel_info(livesess->seng, ESID, &rtpch->carousel_period, &rtpch->aggregate_on_stream);
-
 		port += 2;
 	}
 	if (sdp) {
@@ -403,6 +257,7 @@ static void live_session_setup(LiveSession *livesess, char *ip, u16 port, u32 pa
 		gf_fclose(out);
 		gf_free(sdp);
 	}
+	return GF_TRUE;
 }
 
 void live_session_shutdown(LiveSession *livesess)
@@ -475,7 +330,7 @@ int live_session(int argc, char **argv)
 	u32 i;
 	char *filename = NULL;
 	char *dst = NULL;
-	char *ifce_addr = NULL;
+	const char *ifce_addr = NULL;
 	char *sdp_name = "session.sdp";
 	u16 dst_port = 7000;
 	u32 load_type=0;
@@ -483,7 +338,7 @@ int live_session(int argc, char **argv)
 	u32 ttl = 1;
 	u32 path_mtu = 1450;
 	s32 next_time;
-	u64 last_src_modif, mod_time;
+	u64 last_src_modif, mod_time, runfor=0, start_time;
 	char *src_name = NULL;
 	Bool run, has_carousel, no_rap;
 	Bool udp = 0;
@@ -502,12 +357,15 @@ int live_session(int argc, char **argv)
 	aggregate_au = 1;
 	es_id = 0;
 	no_rap = 0;
-	gf_sys_init(GF_MemTrackerNone);
+	gf_sys_init(GF_MemTrackerNone, NULL);
 
 	memset(&livesess, 0, sizeof(LiveSession));
 
 	gf_log_set_tool_level(GF_LOG_ALL, GF_LOG_INFO);
 
+	gf_sys_set_args(argc, (const char **) argv);
+
+
 	for (i=1; i<(u32) argc; i++) {
 		char *arg = argv[i];
 		if (arg[0] != '-') filename = arg;
@@ -516,7 +374,6 @@ int live_session(int argc, char **argv)
 		else if (!strnicmp(arg, "-sdp=", 5)) sdp_name = arg+5;
 		else if (!strnicmp(arg, "-mtu=", 5)) path_mtu = atoi(arg+5);
 		else if (!strnicmp(arg, "-ttl=", 5)) ttl = atoi(arg+5);
-		else if (!strnicmp(arg, "-ifce=", 6)) ifce_addr = arg+6;
 		else if (!strnicmp(arg, "-no-rap", 7)) no_rap = 1;
 		else if (!strnicmp(arg, "-dims", 5)) load_type = GF_SM_LOAD_DIMS;
 		else if (!strnicmp(arg, "-src=", 5)) src_name = arg+5;
@@ -528,21 +385,35 @@ int live_session(int argc, char **argv)
 			sk_port = atoi(arg+5);
 			udp = 0;
 		}
+		else if (!stricmp(arg, "-run-for")) {
+			runfor = 1 + 1000 * atoi(argv[i+1]);
+			i++;
+		}
 	}
 	if (!filename) {
-		fprintf(stderr, "Missing filename\n");
+		M4_LOG(GF_LOG_ERROR, ("Missing filename\n"));
 		PrintLiveUsage();
 		return 1;
 	}
+	ifce_addr = gf_opts_get_key("core", "ifce");
 
 	if (dst_port && dst) livesess.streams = gf_list_new();
 
 	livesess.seng = gf_seng_init(&livesess, filename, load_type, NULL, (load_type == GF_SM_LOAD_DIMS) ? 1 : 0);
 	if (!livesess.seng) {
-		fprintf(stderr, "Cannot create scene engine\n");
+		M4_LOG(GF_LOG_ERROR, ("Cannot create scene engine\n"));
 		return 1;
 	}
-	if (livesess.streams) live_session_setup(&livesess, dst, dst_port, path_mtu, ttl, ifce_addr, sdp_name);
+	if (livesess.streams) {
+		Bool res = live_session_setup(&livesess, dst, dst_port, path_mtu, ttl, (char *) ifce_addr, sdp_name);
+		if (!res) {
+			live_session_shutdown(&livesess);
+			if (update_buffer) gf_free(update_buffer);
+			if (sk) gf_sk_del(sk);
+			gf_sys_close();
+			return e ? 1 : 0;
+		}
+	}
 
 	has_carousel = 0;
 	last_src_modif = src_name ? gf_file_modification_time(src_name) : 0;
@@ -563,14 +434,13 @@ int live_session(int argc, char **argv)
 	for (i=0; i<(u32) argc; i++) {
 		char *arg = argv[i];
 		if (!strnicmp(arg, "-rap=", 5)) {
-			u32 period, id, j;
-			RTPChannel *ch;
+			u32 id, j;
 			period = id = 0;
 			if (strchr(arg, ':')) {
 				sscanf(arg, "-rap=ESID=%u:%u", &id, &period);
 				e = gf_seng_enable_aggregation(livesess.seng, id, 1);
 				if (e) {
-					fprintf(stderr, "Cannot enable aggregation on stream %u: %s\n", id, gf_error_to_string(e));
+					M4_LOG(GF_LOG_ERROR, ("Cannot enable aggregation on stream %u: %s\n", id, gf_error_to_string(e)));
 					goto exit;
 				}
 			} else {
@@ -604,6 +474,21 @@ int live_session(int argc, char **argv)
 
 	live_session_send_carousel(&livesess, NULL);
 
+
+#ifdef GPAC_ENABLE_COVERAGE
+	if (gf_sys_is_cov_mode()) {
+		aggregate_on_stream = (u16) -1;
+		adjust_carousel_time = force_rap = discard_pending = signal_rap = signal_critical = 0;
+		aggregate_au = version_inc = 1;
+		period = -1;
+		ts_delta = 0;
+		es_id = 0;
+
+		set_broadcast_params(&livesess, es_id, period, ts_delta, aggregate_on_stream, adjust_carousel_time, force_rap, aggregate_au, discard_pending, signal_rap, signal_critical, version_inc);
+	}
+#endif
+
+	start_time = gf_sys_clock_high_res();
 	check = 10;
 	run = 1;
 	while (run) {
@@ -617,14 +502,12 @@ int live_session(int argc, char **argv)
 					run=0;
 					break;
 				case 'U':
-					livesess.critical = 1;
 				case 'u':
 				{
-					GF_Err e;
 					char szCom[8192];
 					fprintf(stderr, "Enter command to send:\n");
 					szCom[0] = 0;
-					if (1 > scanf("%[^\t\n]", szCom)) {
+					if (1 > scanf("%8191[^\t\n]", szCom)) {
 						fprintf(stderr, "No command entered properly, aborting.\n");
 						break;
 					}
@@ -634,19 +517,17 @@ int live_session(int argc, char **argv)
 					if (e) fprintf(stderr, "Processing command failed: %s\n", gf_error_to_string(e));
 					e = gf_seng_aggregate_context(livesess.seng, 0);
 					if (e) fprintf(stderr, "Aggregating context failed: %s\n", gf_error_to_string(e));
-					livesess.critical = 0;
+					livesess.critical = (c=='U') ? 1 : 0;
 					update_context = 1;
 				}
 				break;
 				case 'E':
-					livesess.critical = 1;
 				case 'e':
 				{
-					GF_Err e;
 					char szCom[8192];
 					fprintf(stderr, "Enter command to send:\n");
 					szCom[0] = 0;
-					if (1 > scanf("%[^\t\n]", szCom)) {
+					if (1 > scanf("%8191[^\t\n]", szCom)) {
 						printf("No command entered properly, aborting.\n");
 						break;
 					}
@@ -654,7 +535,7 @@ int live_session(int argc, char **argv)
 					while (getchar()!='\n') {}
 					e = gf_seng_encode_from_string(livesess.seng, 0, 1, szCom, live_session_callback);
 					if (e) fprintf(stderr, "Processing command failed: %s\n", gf_error_to_string(e));
-					livesess.critical = 0;
+					livesess.critical = (c=='E') ? 1 : 0;
 					e = gf_seng_aggregate_context(livesess.seng, 0);
 					if (e) fprintf(stderr, "Aggregating context failed: %s\n", gf_error_to_string(e));
 
@@ -665,8 +546,8 @@ int live_session(int argc, char **argv)
 				{
 					char rad[GF_MAX_PATH];
 					fprintf(stderr, "Enter output file name - \"std\" for stderr: ");
-					if (1 > scanf("%s", rad)) {
-						fprintf(stderr, "No ouput file name entered, aborting.\n");
+					if (1 > scanf("%1023s", rad)) {
+						fprintf(stderr, "No output file name entered, aborting.\n");
 						break;
 					}
 					e = gf_seng_save_context(livesess.seng, !strcmp(rad, "std") ? NULL : rad);
@@ -696,7 +577,7 @@ int live_session(int argc, char **argv)
 				if (!srcf) continue;
 
 				/*checks if we have a broadcast config*/
-				if (!fgets(flag_buf, 200, srcf))
+				if (!gf_fgets(flag_buf, 200, srcf))
 					flag_buf[0] = '\0';
 				gf_fclose(srcf);
 
@@ -712,7 +593,7 @@ int live_session(int argc, char **argv)
 				if (flag) {
 					flag += strlen("gpac_broadcast_config ");
 					/*move to next word*/
-					while (flag && (flag[0]==' ')) flag++;
+					while (flag[0]==' ') flag++;
 
 					while (1) {
 						char *sep = strchr(flag, ' ');
@@ -750,13 +631,13 @@ int live_session(int argc, char **argv)
 
 		/*process updates from socket source*/
 		if (sk) {
-			char buffer[2049];
+			u8 buffer[2049];
 			u32 bytes_read;
 			u32 update_length;
 			u32 bytes_received;
 
 
-			e = gf_sk_receive(sk, buffer, 2048, 0, &bytes_read);
+			e = gf_sk_receive(sk, buffer, 2048, &bytes_read);
 			if (e == GF_OK) {
 				u32 hdr_length = 0;
 				u8 cmd_type = buffer[0];
@@ -801,7 +682,7 @@ int live_session(int argc, char **argv)
 					bytes_received = bytes_read-hdr_length;
 				}
 				while (bytes_received<update_length) {
-					e = gf_sk_receive(sk, buffer, 2048, 0, &bytes_read);
+					e = gf_sk_receive(sk, buffer, 2048, &bytes_read);
 					switch (e) {
 					case GF_IP_NETWORK_EMPTY:
 						break;
@@ -818,7 +699,9 @@ int live_session(int argc, char **argv)
 
 				if (update_length) {
 					e = gf_seng_encode_from_string(livesess.seng, es_id, aggregate_au ? 0 : 1, update_buffer, live_session_callback);
-					if (e) fprintf(stderr, "Processing command failed: %s\n", gf_error_to_string(e));
+					if (e) {
+						M4_LOG(GF_LOG_ERROR, ("Processing command failed: %s\n", gf_error_to_string(e)));
+					}
 					e = gf_seng_aggregate_context(livesess.seng, 0);
 
 					update_context = 1;
@@ -839,6 +722,10 @@ int live_session(int argc, char **argv)
 			continue;
 		}
 
+		if (runfor && (gf_sys_clock_high_res() > start_time+runfor))
+			break;
+
+
 		if (!has_carousel) {
 			gf_sleep(10);
 			continue;
@@ -852,6 +739,16 @@ int live_session(int argc, char **argv)
 		live_session_send_carousel(&livesess, ch);
 	}
 
+#ifdef GPAC_ENABLE_COVERAGE
+	if (gf_sys_is_cov_mode()) {
+/*		gf_seng_save_context(livesess.seng, NULL);
+		gf_seng_aggregate_context
+		gf_seng_encode_from_string
+		gf_seng_encode_from_file
+*/
+	}
+#endif
+
 exit:
 	live_session_shutdown(&livesess);
 	if (update_buffer) gf_free(update_buffer);
@@ -864,98 +761,3 @@ exit:
 #endif /*!defined(GPAC_DISABLE_STREAMING) && !defined(GPAC_DISABLE_SENG)*/
 
 #endif /*defined(GPAC_DISABLE_ISOM) || defined(GPAC_DISABLE_ISOM_WRITE)*/
-
-#ifndef GPAC_DISABLE_MPEG2TS
-
-u32 grab_live_m2ts(const char *grab_m2ts, const char *grab_ifce, const char *outName)
-{
-	char data[0x80000];
-	u32 check = 50;
-	u64 nb_pck;
-	Bool first_run, is_rtp;
-	FILE *output;
-#ifndef GPAC_DISABLE_STREAMING
-	u16 seq_num;
-	GF_RTPReorder *ch = NULL;
-#endif
-	GF_Socket *sock;
-	GF_Err e = gf_m2ts_get_socket(grab_m2ts, grab_ifce, GF_M2TS_UDP_BUFFER_SIZE, &sock);
-
-	if (e) {
-		fprintf(stderr, "Cannot open %s: %s\n", grab_m2ts, gf_error_to_string(e));
-		return 1;
-	}
-	output = gf_fopen(outName, "wb");
-	if (!output) {
-		fprintf(stderr, "Cannot open %s: check path and rights\n", outName);
-		gf_sk_del(sock);
-		return 1;
-	}
-
-	fprintf(stderr, "Dumping %s stream to %s - press q to abort\n", grab_m2ts, outName);
-
-	first_run = 1;
-	is_rtp = 0;
-	while (1) {
-		u32 size = 0;
-
-		check--;
-		if (!check) {
-			if (gf_prompt_has_input()) {
-				char c = (char) gf_prompt_get_char();
-				if (c=='q') break;
-			}
-			check = 50;
-		}
-
-		/*m2ts chunks by chunks*/
-		e = gf_sk_receive(sock, data, 0x40000, 0, &size);
-		if (!size || e) {
-			gf_sleep(1);
-			continue;
-		}
-		if (first_run) {
-			first_run = 0;
-			/*FIXME: we assume only simple RTP packaging (no CSRC nor extensions)*/
-			if ((data[0] != 0x47) && ((data[1] & 0x7F) == 33) ) {
-				is_rtp = 1;
-#ifndef GPAC_DISABLE_STREAMING
-				ch = gf_rtp_reorderer_new(100, 500);
-#endif
-			}
-		}
-		/*process chunk*/
-		if (is_rtp) {
-#ifndef GPAC_DISABLE_STREAMING
-			char *pck;
-			seq_num = ((data[2] << 8) & 0xFF00) | (data[3] & 0xFF);
-			gf_rtp_reorderer_add(ch, (void *) data, size, seq_num);
-
-			pck = (char *) gf_rtp_reorderer_get(ch, &size);
-			if (pck) {
-				fwrite(pck+12, size-12, 1, output);
-				gf_free(pck);
-			}
-#else
-			fwrite(data+12, size-12, 1, output);
-#endif
-		} else {
-			fwrite(data, size, 1, output);
-		}
-	}
-	nb_pck = gf_ftell(output);
-	nb_pck /= 188;
-	fprintf(stderr, "Captured "LLU" TS packets\n", nb_pck );
-	gf_fclose(output);
-	gf_sk_del(sock);
-
-#ifndef GPAC_DISABLE_STREAMING
-	if (ch)
-		gf_rtp_reorderer_del(ch);
-#endif
-	return 0;
-}
-
-#endif /* GPAC_DISABLE_MPEG2TS */
-
-
