@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2000-2020
+ *			Copyright (c) Telecom ParisTech 2000-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / common tools sub-project
@@ -67,7 +67,7 @@ Macro transforming its input name into a string
 \brief 4CC Formatting
 \hideinitializer
 
-Macro formating a 4-character code (or 4CC) "abcd" as 0xAABBCCDD
+Macro formatting a 4-character code (or 4CC) "abcd" as 0xAABBCCDD
 */
 #ifndef GF_4CC
 #define GF_4CC(a,b,c,d) ((((u32)a)<<24)|(((u32)b)<<16)|(((u32)c)<<8)|((u32)d))
@@ -82,6 +82,12 @@ Macro formating a 4-character code (or 4CC) "abcd" as 0xAABBCCDD
 */
 const char *gf_4cc_to_str(u32 type);
 
+/*! converts a 4CC string to its 32 bits value
+\param val  four character string
+\return code value or 0 if error
+*/
+u32 gf_4cc_parse(const char *val);
+
 /*! @} */
 
 /*!
@@ -131,7 +137,7 @@ typedef enum
 	GF_SCRIPT_ERROR							= -8,
 	/*! Buffer is too small to contain decoded data. Decoders shall use this error whenever they need to resize their output memory buffers*/
 	GF_BUFFER_TOO_SMALL						= -9,
-	/*! Bitstream is not compliant to the specfication it refers to*/
+	/*! The bitstream is not compliant to the specfication it refers to*/
 	GF_NON_COMPLIANT_BITSTREAM				= -10,
 	/*! No filter could be found to handle the desired media type*/
 	GF_FILTER_NOT_FOUND						= -11,
@@ -143,6 +149,9 @@ typedef enum
 	GF_REMOTE_SERVICE_ERROR					= -14,
 	/*! The desired stream could not be found in the service*/
 	GF_STREAM_NOT_FOUND						= -15,
+    /*! The URL no longer exists*/
+    GF_URL_REMOVED                          = -16,
+
 	/*! The IsoMedia file is not a valid one*/
 	GF_ISOM_INVALID_FILE					= -20,
 	/*! The IsoMedia file is not complete. Either the file is being downloaded, or it has been truncated*/
@@ -218,9 +227,9 @@ const char *gf_error_to_string(GF_Err e);
 Macro allocating memory and zero-ing it
 */
 #define GF_SAFEALLOC(__ptr, __struct) {\
-		__ptr = (__struct *) gf_malloc(sizeof(__struct));\
+		(__ptr) = (__struct *) gf_malloc(sizeof(__struct));\
 		if (__ptr) {\
-			memset((void *) __ptr, 0, sizeof(__struct));\
+			memset((void *) (__ptr), 0, sizeof(__struct));\
 		}\
 	}
 
@@ -231,9 +240,9 @@ Macro allocating memory and zero-ing it
 Macro allocating memory for n structures and zero-ing it
 */
 #define GF_SAFE_ALLOC_N(__ptr, __n, __struct) {\
-		__ptr = (__struct *) gf_malloc( __n * sizeof(__struct));\
+		(__ptr) = (__struct *) gf_malloc( __n * sizeof(__struct));\
 		if (__ptr) {\
-			memset((void *) __ptr, 0, __n * sizeof(__struct));\
+			memset((void *) (__ptr), 0, __n * sizeof(__struct));\
 		}\
 	}
 
@@ -249,6 +258,121 @@ Dynamic concatenation of string with optional separator
  */
 GF_Err gf_dynstrcat(char **str, const char *to_append, const char *sep);
 
+
+/*!
+\brief fraction parsing
+
+Parse a 64 bit fraction from string
+\param str string to parse
+\param frac fraction to fill
+\return GF_TRUE if success, GF_FALSE otherwise ( fraction being set to {0,0} )
+ */
+Bool gf_parse_lfrac(const char *str, GF_Fraction64 *frac);
+
+/*!
+\brief fraction parsing
+
+Parse a 32 bit fraction from string
+\param str string to parse
+\param frac fraction to fill
+\return GF_TRUE if success, GF_FALSE otherwise ( fraction being set to {0,0} )
+ */
+Bool gf_parse_frac(const char *str, GF_Fraction *frac);
+
+/*!
+\brief search string without case
+
+Search a aubstring in a string witout checking for case
+\param text text to search
+\param subtext string to find
+\param subtext_len length of string to find
+\return GF_TRUE if success, GF_FALSE otherwise
+ */
+Bool gf_strnistr(const char *text, const char *subtext, u32 subtext_len);
+
+
+/*!
+\brief safe timestamp rescale
+
+Rescale a 64 bit timestamp value to new timescale, i.e. performs value * new_timescale / timescale
+\param value value to rescale. A value of -1 means no timestamp defined and is returned unmodified
+\param timescale timescale of value. Assumed to be less than 0xFFFFFFFF
+\param new_timescale new timescale? Assumed to be less than 0xFFFFFFFF
+\return new value
+ */
+u64 gf_timestamp_rescale(u64 value, u64 timescale, u64 new_timescale);
+
+/*!
+\brief safe signed timestamp rescale
+
+Rescale a 64 bit timestamp value to new timescale, i.e. performs value * new_timescale / timescale
+\param value value to rescale
+\param timescale timescale of value. Assumed to be less than 0xFFFFFFFF
+\param new_timescale new timescale. Assumed to be less than 0xFFFFFFFF
+\return new value
+ */
+s64 gf_timestamp_rescale_signed(s64 value, u64 timescale, u64 new_timescale);
+
+/*!
+\brief compare timestamps
+
+Compares two timestamps
+\param value1 value to rescale
+\param timescale1 timescale of value. Assumed to be less than 0xFFFFFFFF
+\param value2 value to rescale
+\param timescale2 timescale of value. Assumed to be less than 0xFFFFFFFF
+\return GF_TRUE if (value1 / timescale1) is stricly less than (value2 / timescale2)
+ */
+Bool gf_timestamp_less(u64 value1, u64 timescale1, u64 value2, u64 timescale2);
+
+/*!
+\brief compare timestamps
+
+Compares two timestamps
+\param value1 value to rescale
+\param timescale1 timescale of value. Assumed to be less than 0xFFFFFFFF
+\param value2 value to rescale
+\param timescale2 timescale of value. Assumed to be less than 0xFFFFFFFF
+\return GF_TRUE if (value1 / timescale1) is stricly less than or equal to (value2 / timescale2)
+ */
+Bool gf_timestamp_less_or_equal(u64 value1, u64 timescale1, u64 value2, u64 timescale2);
+
+/*!
+\brief compare timestamps
+
+Compares two timestamps
+\param value1 value to rescale
+\param timescale1 timescale of value. Assumed to be less than 0xFFFFFFFF
+\param value2 value to rescale
+\param timescale2 timescale of value. Assumed to be less than 0xFFFFFFFF
+\return GF_TRUE if (value1 / timescale1) is stricly greater than (value2 / timescale2)
+ */
+Bool gf_timestamp_greater(u64 value1, u64 timescale1, u64 value2, u64 timescale2);
+
+/*!
+\brief compare timestamps
+
+Compares two timestamps
+\param value1 value to rescale
+\param timescale1 timescale of value. Assumed to be less than 0xFFFFFFFF
+\param value2 value to rescale
+\param timescale2 timescale of value. Assumed to be less than 0xFFFFFFFF
+\return GF_TRUE if (value1 / timescale1) is stricly greater than or equal to (value2 / timescale2)
+ */
+Bool gf_timestamp_greater_or_equal(u64 value1, u64 timescale1, u64 value2, u64 timescale2);
+
+/*!
+\brief compare timestamps
+
+Compares two timestamps
+\param value1 value to rescale
+\param timescale1 timescale of value. Assumed to be less than 0xFFFFFFFF
+\param value2 value to rescale
+\param timescale2 timescale of value. Assumed to be less than 0xFFFFFFFF
+\return GF_TRUE if (value1 / timescale1) is equal to (value2 / timescale2)
+ */
+Bool gf_timestamp_equal(u64 value1, u64 timescale1, u64 value2, u64 timescale2);
+
 /*! @} */
 
 /*!
@@ -298,7 +422,7 @@ The profile allows using a different global config file than the default, and ma
 GF_Err gf_sys_init(GF_MemTrackerType mem_tracker_type, const char *profile);
 /*!
 \brief System closing
-Closes the system high-resolution clock and any CPU associated ressources.
+Closes the system high-resolution clock and any CPU associated resources.
 \note This can be called several times but the system will be closed when no more users are counted.
  */
 void gf_sys_close();
@@ -308,7 +432,7 @@ void gf_sys_close();
 
 Sets the user app arguments (used by GUI mode)
 \param argc Number of arguments
-\param argv Array of arguments
+\param argv Array of arguments - the first string is ignored (considered to be the executable name)
 \return error code if any, GF_OK otherwise
  */
 GF_Err gf_sys_set_args(s32 argc, const char **argv);
@@ -338,6 +462,14 @@ Gets the number of argument of the user application if any
  */
 const char *gf_sys_get_arg(u32 arg);
 
+/*!
+\brief Locate a global filter arg
+
+Looks for a filter option specified as global argument
+\param arg name of option to search, without "--" or "-+" specififers
+\return argument value string, empty string for booleans or NULL if not found
+ */
+const char *gf_sys_find_global_arg(const char *arg);
 
 /*!
 \brief Mark arg as used
@@ -507,13 +639,13 @@ typedef enum
 \brief Log exits at first error assignment
 
 When GF_LOG_ERROR happens, program leaves with instruction exit(1);
-\param strict strict behaviour when encoutering a serious error.
+\param strict strict behavior when encoutering a serious error.
 \return old value before the call.
  */
 Bool gf_log_set_strict_error(Bool strict);
 
 /*!
-\brief gets string-formated log tools
+\brief gets string-formatted log tools
 
 Gets the string-formatted log tools and levels. Returned string shall be freed by the caller.
 \return string-formatted log tools.
@@ -540,15 +672,11 @@ typedef enum
 	GF_LOG_HTTP,
 	/*! Log message from the RTP/RTCP stack (TS info) and packet structure & hinting (debug)*/
 	GF_LOG_RTP,
-	/*! Log message from authoring subsystem (file manip, import/export)*/
-	GF_LOG_AUTHOR,
-	/*! Log message from the sync layer of the terminal*/
-	GF_LOG_SYNC,
 	/*! Log message from a codec*/
 	GF_LOG_CODEC,
-	/*! Log message from any XML parser (context loading, etc)*/
+	/*! Log message from any textual (XML, ...) parser (context loading, etc)*/
 	GF_LOG_PARSER,
-	/*! Log message from the terminal/compositor, indicating media object state*/
+	/*! Generic log message from a filter (not from filter core library)*/
 	GF_LOG_MEDIA,
 	/*! Log message from the scene graph/scene manager (handling of nodes and attribute modif, DOM core)*/
 	GF_LOG_SCENE,
@@ -558,14 +686,14 @@ typedef enum
 	GF_LOG_INTERACT,
 	/*! Log message from compositor*/
 	GF_LOG_COMPOSE,
+	/*! Log message from the terminal/compositor, indicating media object state*/
+	GF_LOG_COMPTIME,
 	/*! Log for video object cache */
 	GF_LOG_CACHE,
 	/*! Log message from multimedia I/O devices (audio/video input/output, ...)*/
 	GF_LOG_MMIO,
 	/*! Log for runtime info (times, memory, CPU usage)*/
 	GF_LOG_RTI,
-	/*! Log for SMIL timing and animation*/
-	GF_LOG_SMIL,
 	/*! Log for memory tracker*/
 	GF_LOG_MEMORY,
 	/*! Log for audio compositor*/
@@ -578,12 +706,12 @@ typedef enum
 	GF_LOG_CONDITION,
 	/*! Log for all HTTP streaming */
 	GF_LOG_DASH,
-	/*! Log for all messages coming from filters */
+	/*! Log for all messages from filter core library (not from a filter) */
 	GF_LOG_FILTER,
 	/*! Log for filter scheduler only */
 	GF_LOG_SCHEDULER,
-	/*! Log for all ATSC3 message */
-	GF_LOG_ATSC,
+	/*! Log for all ROUTE message */
+	GF_LOG_ROUTE,
 	/*! Log for all messages coming from GF_Terminal or script alert()*/
 	GF_LOG_CONSOLE,
 	/*! Log for all messages coming the application, not used by libgpac or the modules*/
@@ -632,9 +760,9 @@ gf_log_cbk gf_log_set_callback(void *usr_cbk, gf_log_cbk cbk);
  \cond DUMMY_DOXY_SECTION
 */
 #ifndef GPAC_DISABLE_LOG
-/*note:
-		to turn log on, change to GPAC_ENABLE_LOG
-		to turn log off, change to GPAC_DISABLE_LOG
+/*\note
+	- to turn log on, change to GPAC_ENABLE_LOG
+	- to turn log off, change to GPAC_DISABLE_LOG
 	this is needed by configure+sed to modify this file directly
 */
 #define GPAC_ENABLE_LOG
@@ -846,6 +974,13 @@ Parses 128 bit from string
  */
 GF_Err gf_bin128_parse(const char *string, bin128 value);
 
+
+enum
+{
+    GF_BLOB_IN_TRANSFER = 1,
+    GF_BLOB_CORRUPTED = 1<<1,
+};
+
 /*!
  * Blob structure used to pass data pointer around
  */
@@ -855,16 +990,28 @@ typedef struct
 	u8 *data;
 	/*! size of blob */
 	u32 size;
+    /*! blob flags */
+    u32 flags;
+    /*! blob mutex for multi-thread access */
+    struct __tag_mutex *mx;
 } GF_Blob;
 
 /*!
- * Retrieves data associated with a blob url
+ * Retrieves data associated with a blob url. If success, \ref gf_blob_release must be called after this
 \param blob_url URL of blob object (ie gmem://%p)
 \param out_data if success, set to blob data pointer
 \param out_size if success, set to blob data size
+\param blob_flags if success, set to blob flags - may be NULL
 \return error code
  */
-GF_Err gf_blob_get_data(const char *blob_url, u8 **out_data, u32 *out_size);
+GF_Err gf_blob_get(const char *blob_url, u8 **out_data, u32 *out_size, u32 *blob_flags);
+
+/*!
+ * Releases blob data
+\param blob_url URL of blob object (ie gmem://%p)
+\return error code
+ */
+GF_Err gf_blob_release(const char *blob_url);
 
 /*!
 \addtogroup time_grp
@@ -919,7 +1066,7 @@ u64 gf_net_get_utc();
 \brief converts an ntp timestamp into UTC time in milliseconds
 
 Converts NTP 64-bit timestamp to UTC clock in milliseconds
-\u64 ntp NTP timestamp
+\param ntp NTP timestamp
 \return UTC time in milliseconds
  */
 u64 gf_net_ntp_to_utc(u64 ntp);
@@ -957,6 +1104,14 @@ Gets timezone adjustment in seconds, with localtime - timezone = UTC time
  */
 s32 gf_net_get_timezone();
 
+/*!
+\brief gets timezone daylight saving time status
+
+Gets timezone daylight saving time
+\return GF_TRUE if DST is active
+ */
+Bool gf_net_time_is_dst();
+
 /*!
 \brief gets time from UTC timestamp
 
@@ -1131,7 +1286,7 @@ Cleanup a directory within the full path, removing all the files and the directo
 \param DirPathName the dir path name.
 \return error if any
  */
-GF_Err gf_cleanup_dir(const char* DirPathName);
+GF_Err gf_dir_cleanup(const char* DirPathName);
 
 
 /**
@@ -1274,10 +1429,11 @@ Opens a file, potentially using file IO if the parent URL is a File IO wrapper
 \param file_name same as fopen
 \param parent_url URL of parent file. If not a file io wrapper (gfio://), the function is equivalent to gf_fopen
 \param mode same as fopen
+\param no_warn if GF_TRUE, do not throw log message if failure
 \return stream handle of the file object
 \note You only need to call this function if you're suspecting the file to be a large one (usually only media files), otherwise use regular stdio.
 \return stream habdle of the file or file IO object*/
-FILE *gf_fopen_ex(const char *file_name, const char *parent_url, const char *mode);
+FILE *gf_fopen_ex(const char *file_name, const char *parent_url, const char *mode, Bool no_warn);
 
 /*!
 \brief file closing
@@ -1323,7 +1479,6 @@ char* gf_file_basename(const char* filename);
 */
 char* gf_file_ext_start(const char* filename);
 
-
 /*!\brief FileEnum info object
 
 The FileEnumInfo object is used to get file attributes upon enumeration of a directory.
@@ -1403,7 +1558,7 @@ FILE *gf_file_temp(char ** const fileName);
 /*!
 \brief File Modification Time
 
-Gets the modification time of the given file. The exact meaning of this value is system dependent
+Gets the UTC modification time of the given file in microseconds
 \param filename file to check
 \return modification time of the file
  */
@@ -1433,11 +1588,12 @@ typedef struct __gf_file_io GF_FileIO;
 /*! open proc for memory file IO
 \param fileio_ref reference file_io. A file can be opened multiple times for the same reference, your code must handle this
 \param url target file name.
-\param mode opening mode of file, same as fopen mode. The following additionnal modes are defined:
+\param mode opening mode of file, same as fopen mode. The following additional modes are defined:
 	- "ref": indicates this FileIO object is used by some part of the code and must not be destroyed upon closing of the file. Associated URL is null
 	- "unref": indicates this FileIO object is not used by some part of the code and may be destroyed if no more references to this object are set. Associated URL is null
 	- "url": indicates to create a new FileIO object for the given URL without opening the output file. The resulting FileIO object must be garbage collected by the app in case its is never used by the callers
 	- "probe": checks if the file exists, but no need to open the file. The function should return NULL in this case. If file does not exist, set out_error to GF_URL_ERROR
+	- "close": indicates the fileIO object is being closed (fclose)
 \param out_error must be set to error code if any (never NULL)
 \return the opened GF_FileIO if success, or NULL otherwise
  */
@@ -1556,6 +1712,22 @@ Bool gf_fileio_get_stats(GF_FileIO *fileio, u64 *bytes_done, u64 *file_size, Boo
 */
 GF_FileIO *gf_fileio_open_url(GF_FileIO *fileio, const char *url, const char *mode, GF_Err *out_err);
 
+
+/*! Tags a FileIO object to be accessed from main thread only
+\param fileio target file IO object
+\return error if any
+*/
+GF_Err gf_fileio_tag_main_thread(GF_FileIO *fileio);
+
+/*! Check if a FileIO object is to be accessed from main thread only
+
+ \note Filters accessing FileIO objects by other means that a filter option must manually tag themselves as main thread, potentially rescheduling a configure call to next process. Not doing so can result in binding crashes in multi-threaded mode.
+
+\param url target url
+\return GF_TRUE if object is tagged for main thread, GF_FALSE otherwise
+*/
+Bool gf_fileio_is_main_thread(const char *url);
+
 /*! Gets GF_FileIO object from its URL
  The url uses the protocol scheme "gfio://"
 \param url the URL of  the File IO object
@@ -1564,7 +1736,7 @@ GF_FileIO *gf_fileio_open_url(GF_FileIO *fileio, const char *url, const char *mo
 GF_FileIO *gf_fileio_from_url(const char *url);
 
 /*! Constructs a new GF_FileIO object from a URL
- The url can be absolute or relative to the parent GF_FileIO. This is typcically needed by filters (dash input, dasher, NHML/NHNT writers...) generating or consuming additionnal files associated with the main file IO object but being written or read by other filters.
+ The url can be absolute or relative to the parent GF_FileIO. This is typcically needed by filters (dash input, dasher, NHML/NHNT writers...) generating or consuming additional files associated with the main file IO object but being written or read by other filters.
  The function will not open the associated resource, only create the file IO wrapper for later usage
  If you need to create a new fileIO to be opened immediately, use \ref gf_fopen_ex.
 
@@ -1769,6 +1941,13 @@ void gf_sha1_finish(GF_SHA1Context *ctx, u8 digest[GF_SHA1_DIGEST_SIZE] );
 */
 GF_Err gf_sha1_file(const char *filename, u8 digest[GF_SHA1_DIGEST_SIZE]);
 
+/*! gets SHA1 message digest of a opened file
+\param file  handle to open file
+\param digest buffer to store message digest
+\return error if any
+*/
+GF_Err gf_sha1_file_ptr(FILE *file, u8 digest[GF_SHA1_DIGEST_SIZE] );
+
 /*! gets SHA-1 of input buffer
 \param buf input buffer to hash
 \param buflen sizeo of input buffer in bytes
@@ -1909,8 +2088,8 @@ extern void gf_fm_request_set_callback(void *cbk_obj, fm_callback_func cbk_func)
 void gf_fm_request_call(u32 type, u32 param, int *value);
 #endif //GPAC_CONFIG_ANDROID
 
-/*to call whenever the OpenGL library is opened - this function is needed to bind openGL and remotery, and to load
-openGL extensions on windows
+/*to call whenever the OpenGL library is opened - this function is needed to bind OpenGL and remotery, and to load
+OpenGL extensions on windows
 not exported, and not included in src/compositor/gl_inc.h since it may be needed even when no OpenGL 
 calls are made by the caller*/
 void gf_opengl_init();
@@ -1937,6 +2116,7 @@ typedef struct _gl_texture_wrap
 	Bool is_yuv;
 	u32 bit_depth, uv_w, uv_h;
 	u32 scale_10bit;
+	u32 init_active_texture;
 
 	u32 gl_format;
 	u32 bytes_per_pix;
@@ -1947,13 +2127,18 @@ typedef struct _gl_texture_wrap
 	struct _gf_filter_frame_interface *frame_ifce;
 	Bool first_tx_load;
 
-	//PBO state - must be managed by caller, especially if using seperated push and texImg steps through gf_gl_txw_setup calls
+	//PBO state - must be managed by caller, especially if using separated push and texImg steps through gf_gl_txw_setup calls
 	GF_GLPBOState pbo_state;
 	Bool flip;
+	//YUV is full video range
+	Bool fullrange;
+	s32 mx_cicp;
+
+	u32 last_program;
 } GF_GLTextureWrapper;
 
-Bool gf_gl_txw_insert_fragment_shader(u32 pix_fmt, const char *tx_name, char **f_source);
-Bool gf_gl_txw_setup(GF_GLTextureWrapper *tx, u32 pix_fmt, u32 width, u32 height, u32 stride, u32 uv_stride, Bool linear_interp, struct _gf_filter_frame_interface *frame_ifce);
+Bool gf_gl_txw_insert_fragment_shader(u32 pix_fmt, const char *tx_name, char **f_source, Bool y_flip);
+Bool gf_gl_txw_setup(GF_GLTextureWrapper *tx, u32 pix_fmt, u32 width, u32 height, u32 stride, u32 uv_stride, Bool linear_interp, struct _gf_filter_frame_interface *frame_ifce, Bool full_range, s32 matrix_coef_or_neg);
 Bool gf_gl_txw_upload(GF_GLTextureWrapper *tx, const u8 *data, struct _gf_filter_frame_interface *frame_ifce);
 Bool gf_gl_txw_bind(GF_GLTextureWrapper *tx, const char *tx_name, u32 gl_program, u32 texture_unit);
 void gf_gl_txw_reset(GF_GLTextureWrapper *tx);
