@@ -86,11 +86,12 @@ static void flush_text_node_edit(GF_Compositor *compositor, Bool final_flush)
 	}
 	if (compositor->sel_buffer_len) {
 		char *txt;
-		size_t len;
+		u32 len;
 		const u16 *lptr;
 		txt = (char*)gf_malloc(sizeof(char)*2*compositor->sel_buffer_len);
 		lptr = compositor->sel_buffer;
 		len = gf_utf8_wcstombs(txt, 2*compositor->sel_buffer_len, &lptr);
+		if (len == GF_UTF8_FAIL) len = 0;
 		txt[len] = 0;
 		*compositor->edited_text = gf_strdup(txt);
 		gf_free(txt);
@@ -134,18 +135,19 @@ static void flush_text_node_edit(GF_Compositor *compositor, Bool final_flush)
 GF_Err gf_sc_paste_text(GF_Compositor *compositor, const char *text)
 {
 	u16 *conv_buf;
-	size_t len;
+	u32 len;
 	if (!compositor->sel_buffer || !compositor->edited_text) return GF_BAD_PARAM;
 	if (!text) return GF_OK;
-	len = strlen(text);
+	len = (u32) strlen(text);
 	if (!len) return GF_OK;
 
 	gf_sc_lock(compositor, GF_TRUE);
 
 	conv_buf = (u16*)gf_malloc(sizeof(u16)*(len+1));
 	len = gf_utf8_mbstowcs(conv_buf, len, &text);
+	if (len == GF_UTF8_FAIL) return GF_IO_ERR;
 
-	compositor->sel_buffer_alloc += (u32) len;
+	compositor->sel_buffer_alloc += len;
 	if (compositor->sel_buffer_len == compositor->sel_buffer_alloc)
 		compositor->sel_buffer_alloc++;
 
@@ -153,8 +155,8 @@ GF_Err gf_sc_paste_text(GF_Compositor *compositor, const char *text)
 	memmove(&compositor->sel_buffer[compositor->caret_pos+len], &compositor->sel_buffer[compositor->caret_pos], sizeof(u16)*(compositor->sel_buffer_len-compositor->caret_pos));
 	memcpy(&compositor->sel_buffer[compositor->caret_pos], conv_buf, sizeof(u16)*len);
 	gf_free(conv_buf);
-	compositor->sel_buffer_len += (u32) len;
-	compositor->caret_pos += (u32) len;
+	compositor->sel_buffer_len += len;
+	compositor->caret_pos += len;
 	compositor->sel_buffer[compositor->sel_buffer_len]=0;
 	flush_text_node_edit(compositor, GF_FALSE);
 	gf_sc_lock(compositor, GF_FALSE);
@@ -280,7 +282,7 @@ static Bool load_text_node(GF_Compositor *compositor, u32 cmd_type)
 							gf_node_list_insert_child(&children, t, pos);
 							res = &((GF_DOMText *)child->node)->textContent;
 						} else {
-							size_t len;
+							u32 len;
 							GF_DOMText *cur;
 
 							gf_node_list_insert_child(&children, t, pos+1);
@@ -298,6 +300,7 @@ static Bool load_text_node(GF_Compositor *compositor, u32 cmd_type)
 							cur->textContent = (char*)gf_malloc(sizeof(char)*(len+1));
 							srcp = compositor->sel_buffer;
 							len = gf_utf8_wcstombs(cur->textContent, len, &srcp);
+							if (len == GF_UTF8_FAIL) len = 0;
 							cur->textContent[len] = 0;
 							compositor->sel_buffer[compositor->caret_pos] = end;
 
@@ -306,6 +309,7 @@ static Bool load_text_node(GF_Compositor *compositor, u32 cmd_type)
 								ntext->textContent = (char*)gf_malloc(sizeof(char)*(len+1));
 								srcp = compositor->sel_buffer + compositor->caret_pos + 1;
 								len = gf_utf8_wcstombs(ntext->textContent, len, &srcp);
+								if (len == GF_UTF8_FAIL) len = 0;
 								ntext->textContent[len] = 0;
 							} else {
 								ntext->textContent = gf_strdup("");
@@ -369,15 +373,17 @@ static Bool load_text_node(GF_Compositor *compositor, u32 cmd_type)
 		compositor->sel_buffer = (u16*)gf_realloc(compositor->sel_buffer, sizeof(u16)*compositor->sel_buffer_alloc);
 
 		if (caret_pos>=0) {
-			size_t l = gf_utf8_mbstowcs(compositor->sel_buffer, compositor->sel_buffer_alloc, &src);
-			compositor->sel_buffer_len = (u32) l;
+			u32 l = gf_utf8_mbstowcs(compositor->sel_buffer, compositor->sel_buffer_alloc, &src);
+			if (l == GF_UTF8_FAIL) return GF_FALSE;
+			compositor->sel_buffer_len = l;
 			memmove(&compositor->sel_buffer[caret_pos+1], &compositor->sel_buffer[caret_pos], sizeof(u16)*(compositor->sel_buffer_len-caret_pos));
 			compositor->sel_buffer[caret_pos] = GF_CARET_CHAR;
 			compositor->caret_pos = caret_pos;
 
 		} else {
-			size_t l = gf_utf8_mbstowcs(compositor->sel_buffer, compositor->sel_buffer_alloc, &src);
-			compositor->sel_buffer_len = (u32) l;
+			u32 l = gf_utf8_mbstowcs(compositor->sel_buffer, compositor->sel_buffer_alloc, &src);
+			if (l == GF_UTF8_FAIL) return GF_FALSE;
+			compositor->sel_buffer_len = l;
 			compositor->sel_buffer[compositor->sel_buffer_len] = GF_CARET_CHAR;
 			compositor->caret_pos = compositor->sel_buffer_len;
 		}
@@ -900,7 +906,7 @@ Bool gf_sc_exec_event_vrml(GF_Compositor *compositor, GF_Event *ev)
 		if (keynav) gf_sc_change_key_navigator(compositor, keynav);
 
 		/*call the sensor LAST, as this may triger a destroy of the scene the sensor is in
-		this is only true for anchors, as other other sensors output events are queued as routes untill next pass*/
+		this is only true for anchors, as other other sensors output events are queued as routes until next pass*/
 		res += hs->OnUserEvent(hs, GF_TRUE, GF_FALSE, ev, compositor);
 		if (stype == TAG_MPEG4_Anchor) check_anchor = GF_TRUE;
 #ifndef GPAC_DISABLE_X3D
@@ -989,7 +995,7 @@ Bool gf_sc_exec_event_vrml(GF_Compositor *compositor, GF_Event *ev)
 	}
 	if (res) {
 		GF_SceneGraph *sg;
-		/*apply event cascade - this is needed for cases where several events are processed inbetween
+		/*apply event cascade - this is needed for cases where several events are processed between
 		2 simulation tick. If we don't flush the routes stack, the result will likely be wrong
 		*/
 		gf_sg_activate_routes(compositor->scene);
