@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2000-2018
+ *			Copyright (c) Telecom ParisTech 2000-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / Scene Compositor sub-project
@@ -183,7 +183,7 @@ GF_MediaObject *gf_mo_register(GF_Node *node, MFURL *url, Bool lock_timelines, B
 	while (scene->secondary_resource && scene->root_od->parentscene)
 		scene = scene->root_od->parentscene;
 
-	res = gf_scene_get_media_object_ex(scene, url, obj_type, lock_timelines, syncRef, force_new_res, node);
+	res = gf_scene_get_media_object_ex(scene, url, obj_type, lock_timelines, syncRef, force_new_res, node, NULL);
 	return res;
 }
 
@@ -212,7 +212,7 @@ Bool gf_mo_get_visual_info(GF_MediaObject *mo, u32 *width, u32 *height, u32 *str
 {
 	if ((mo->type != GF_MEDIA_OBJECT_VIDEO) && (mo->type!=GF_MEDIA_OBJECT_TEXT)) return GF_FALSE;
 
-	if (mo->config_changed) {
+	if (mo->config_changed || !mo->width || !mo->height) {
 		gf_mo_update_caps(mo);
 	}
 	if (width) *width = mo->width;
@@ -230,13 +230,6 @@ void gf_mo_get_nb_views(GF_MediaObject *mo, u32 *nb_views)
 	if (mo) *nb_views = mo->nb_views;
 }
 
-GF_EXPORT
-
-void gf_mo_get_nb_layers(GF_MediaObject *mo, u32 *nb_layers)
-{
-	if (mo) *nb_layers = mo->nb_layers;
-}
-
 GF_EXPORT
 Bool gf_mo_get_audio_info(GF_MediaObject *mo, u32 *sample_rate, u32 *bits_per_sample, u32 *num_channels, u64 *channel_config, Bool *forced_layout)
 {
@@ -257,7 +250,7 @@ Bool gf_mo_get_audio_info(GF_MediaObject *mo, u32 *sample_rate, u32 *bits_per_sa
 
 	if (mo->odm->ambi_ch_id) {
 		if (mo->num_channels>1) {
-			GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[ODM%d]: tagged as ambisonic channel %d but has %d channels, ignoring ambisonic tag\n",  mo->odm->ID, mo->odm->ambi_ch_id, mo->num_channels ));
+			GF_LOG(GF_LOG_WARNING, GF_LOG_COMPTIME, ("[ODM%d]: tagged as ambisonic channel %d but has %d channels, ignoring ambisonic tag\n",  mo->odm->ID, mo->odm->ambi_ch_id, mo->num_channels ));
 		} else {
 			if (num_channels) *num_channels = 1;
 			if (channel_config) *channel_config = (u64) ( 1 << (mo->odm->ambi_ch_id - 1) );
@@ -270,7 +263,7 @@ Bool gf_mo_get_audio_info(GF_MediaObject *mo, u32 *sample_rate, u32 *bits_per_sa
 }
 
 
-void gf_mo_update_caps(GF_MediaObject *mo)
+void gf_mo_update_caps_ex(GF_MediaObject *mo, Bool check_unchanged)
 {
 	Bool changed = GF_FALSE;
 	const GF_PropertyValue *v, *v2;
@@ -281,11 +274,15 @@ void gf_mo_update_caps(GF_MediaObject *mo)
 #define UPDATE_CAP(_code, _field) \
 		v = gf_filter_pid_get_property(mo->odm->pid, _code);\
 		if (v) {\
-			if (mo->_field && (mo->_field != v->value.uint)) changed=GF_TRUE;\
+			if (mo->_field != v->value.uint) changed=GF_TRUE;\
 			mo->_field = v->value.uint;\
+		} else if (mo->_field) {\
+			changed=GF_TRUE;\
+			mo->_field=0;\
 		}\
 
 	if (mo->odm->type==GF_STREAM_VISUAL) {
+		Bool check_mx = GF_TRUE;
 
 		UPDATE_CAP(GF_PROP_PID_WIDTH, width)
 		UPDATE_CAP(GF_PROP_PID_HEIGHT, height)
@@ -293,6 +290,35 @@ void gf_mo_update_caps(GF_MediaObject *mo)
 		UPDATE_CAP(GF_PROP_PID_PIXFMT, pixelformat)
 		UPDATE_CAP(GF_PROP_PID_BITRATE, bitrate)
 
+		UPDATE_CAP(GF_PROP_PID_ROTATE, rotate)
+		if (v) check_mx = GF_FALSE;
+
+		UPDATE_CAP(GF_PROP_PID_MIRROR, flip)
+		if (v) check_mx = GF_FALSE;
+
+		if (check_mx) {
+			v = gf_filter_pid_get_property(mo->odm->pid, GF_PROP_PID_ISOM_TRACK_MATRIX);
+			if (v) {
+				GF_Err gf_prop_matrix_decompose(const GF_PropertyValue *p, u32 *flip_mode, u32 *rot_mode);
+				u32 flip, rotate;
+
+				if (gf_prop_matrix_decompose(v, &flip, &rotate)==GF_OK) {
+					if (flip != mo->flip) {
+						mo->flip = flip;
+						changed = GF_TRUE;
+					}
+					if (rotate != mo->rotate) {
+						mo->rotate = rotate;
+						changed = GF_TRUE;
+					}
+				}
+			} else {
+				if (mo->flip || mo->rotate) changed = GF_TRUE;
+				mo->flip = 0;
+				mo->rotate = 0;
+			}
+		}
+
 		v = gf_filter_pid_get_property(mo->odm->pid, GF_PROP_PID_SAR);
 		if (v) {
 			u32 n_par = (v->value.frac.num) << 16 | (v->value.frac.den);
@@ -343,6 +369,20 @@ void gf_mo_update_caps(GF_MediaObject *mo)
 				mo->srd_full_h = v2->value.vec2i.y;
 			}
 		}
+
+		v = gf_filter_pid_get_property(mo->odm->pid, GF_PROP_PID_NUM_VIEWS);
+		mo->nb_views = v ? v->value.uint : 0;
+
+		mo->c_w = mo->c_h = mo->c_x = mo->c_y = 0;
+		v = gf_filter_pid_get_property(mo->odm->pid, GF_PROP_PID_CLAP_W);
+		if (v && v->value.frac.den) { mo->c_w = (Float) v->value.frac.num; mo->c_w /= v->value.frac.den; }
+		v = gf_filter_pid_get_property(mo->odm->pid, GF_PROP_PID_CLAP_H);
+		if (v && v->value.frac.den) { mo->c_h = (Float) v->value.frac.num; mo->c_h /= v->value.frac.den; }
+		v = gf_filter_pid_get_property(mo->odm->pid, GF_PROP_PID_CLAP_X);
+		if (v && v->value.frac.den) { mo->c_x = (Float) v->value.frac.num; mo->c_x /= v->value.frac.den; }
+		v = gf_filter_pid_get_property(mo->odm->pid, GF_PROP_PID_CLAP_Y);
+		if (v && v->value.frac.den) { mo->c_y = (Float) v->value.frac.num; mo->c_y /= v->value.frac.den; }
+
 	} else if (mo->odm->type==GF_STREAM_AUDIO) {
 		UPDATE_CAP(GF_PROP_PID_SAMPLE_RATE, sample_rate)
 		UPDATE_CAP(GF_PROP_PID_NUM_CHANNELS, num_channels)
@@ -366,7 +406,7 @@ void gf_mo_update_caps(GF_MediaObject *mo)
 	} else if (mo->odm->type==GF_STREAM_TEXT) {
 		//nothing to do
 	} else {
-		GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("Unknwon scene object type %d\n", mo->odm->type));
+		GF_LOG(GF_LOG_WARNING, GF_LOG_COMPTIME, ("Unknwon scene object type %d\n", mo->odm->type));
 	}
 
 	if (changed) {
@@ -375,26 +415,101 @@ void gf_mo_update_caps(GF_MediaObject *mo)
 		memset(&evt, 0, sizeof(GF_Event));
 		evt.type = GF_EVENT_QUALITY_SWITCHED;
 		gf_sc_send_event(scene->compositor, &evt);
+	} else if (check_unchanged) {
+		//reset config changed if nothing changed, this avoid resetting up texture or mixer input
+		mo->config_changed = GF_FALSE;
 	}
 }
+void gf_mo_update_caps(GF_MediaObject *mo)
+{
+	gf_mo_update_caps_ex(mo, GF_FALSE);
 
+}
 static u64 convert_ts_to_ms(GF_MediaObject *mo, u64 ts, u32 timescale, Bool *discard)
 {
-	if (mo->odm->delay) {
-		if (mo->odm->delay >= 0) {
-			ts += mo->odm->delay;
-		} else if (ts < (u64) -mo->odm->delay) {
+	if (mo->odm->timestamp_offset) {
+		if (mo->odm->timestamp_offset >= 0) {
+			ts += mo->odm->timestamp_offset;
+		} else if (ts < (u64) -mo->odm->timestamp_offset) {
 			*discard = GF_TRUE;
 			return 0;
 		} else {
-			ts -= -mo->odm->delay;
+			ts -= -mo->odm->timestamp_offset;
+		}
+	}
+
+	ts = gf_timestamp_rescale(ts, timescale, 1000);
+	
+	//if addon, translate back into main content timing
+	if (mo->odm->parentscene && mo->odm->parentscene->root_od->addon) {
+		if (!mo->odm->parentscene->root_od->addon->timeline_ready) {
+			ts = 0;
+		} else {
+			s64 res = gf_scene_adjust_timestamp_for_addon(mo->odm->parentscene->root_od->addon, ts);
+			if (res<0) res=0;
+			ts = (u64) res;
 		}
 	}
-	ts *= 1000;
-	ts /= timescale;
 	return ts;
 }
 
+
+static void check_temi(GF_MediaObject *mo)
+{
+	u32 idx=0;
+	if (!(mo->odm->flags & GF_ODM_HAS_TEMI)) return;
+
+	while (1) {
+		const GF_PropertyValue *p;
+		u32 p4cc = 0;
+		const char *pname = NULL;
+		p = gf_filter_pck_enum_properties(mo->pck, &idx, &p4cc, &pname);
+		if (!p) break;
+		if (!pname) continue;
+		if (p->type != GF_PROP_DATA) continue;
+		if (!strncmp(pname, "temi_l:", 7)) {
+			GF_AssociatedContentLocation temi_l;
+			u8 *data = p->value.data.ptr;
+			u32 len = (u32) strlen(data);
+			memset(&temi_l, 0, sizeof(GF_AssociatedContentLocation));
+			temi_l.timeline_id = atoi(pname+7);
+			temi_l.is_announce = (data[len+1] & 0x80) ? GF_TRUE : GF_FALSE;
+			temi_l.is_splicing = (data[len+1] & 0x40) ? GF_TRUE : GF_FALSE;
+			temi_l.reload_external = (data[len+1] & 0x20) ? GF_TRUE : GF_FALSE;
+			if (temi_l.is_announce) {
+				temi_l.activation_countdown.den = GF_4CC(data[len+2], data[len+3], data[len+4], data[len+5]);
+				temi_l.activation_countdown.num = GF_4CC(data[len+6], data[len+7], data[len+8], data[len+9]);
+			}
+			temi_l.external_URL = data;
+
+			gf_scene_register_associated_media(mo->odm->subscene ? mo->odm->subscene : mo->odm->parentscene, &temi_l);
+			continue;
+		}
+		if (!strncmp(pname, "temi_t:", 7)) {
+			GF_BitStream *bs;
+			GF_AssociatedContentTiming temi_t;
+			memset(&temi_t, 0, sizeof(GF_AssociatedContentTiming));
+			temi_t.timeline_id = atoi(pname+7);
+			bs = gf_bs_new(p->value.data.ptr, p->value.data.size, GF_BITSTREAM_READ);
+			temi_t.media_timescale = gf_bs_read_u32(bs);
+			temi_t.media_timestamp = gf_bs_read_u64(bs);
+			temi_t.media_pts = gf_bs_read_u64(bs);
+			temi_t.force_reload = gf_bs_read_int(bs, 1);
+			temi_t.is_paused = gf_bs_read_int(bs, 1);
+			temi_t.is_discontinuity = gf_bs_read_int(bs, 1);
+			temi_t.ntp = gf_bs_read_int(bs, 1);
+			gf_bs_read_int(bs, 4);
+			if (temi_t.ntp)
+				temi_t.ntp = gf_bs_read_u64(bs);
+
+			gf_bs_del(bs);
+
+			gf_scene_notify_associated_media_timeline(mo->odm->subscene ? mo->odm->subscene : mo->odm->parentscene, &temi_t);
+			continue;
+		}
+	}
+}
+
 GF_EXPORT
 u8 *gf_mo_fetch_data(GF_MediaObject *mo, GF_MOFetchMode resync, u32 upload_time_ms, Bool *eos, u32 *timestamp, u32 *size, s32 *ms_until_pres, s32 *ms_until_next, GF_FilterFrameInterface **outFrame, u32 *planar_size)
 {
@@ -421,7 +536,7 @@ u8 *gf_mo_fetch_data(GF_MediaObject *mo, GF_MOFetchMode resync, u32 upload_time_
 
 	/*if frame locked return it*/
 	if (mo->nb_fetch) {
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[ODM%d] ODM %d: CU already fetched, returning\n", mo->odm->ID));
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPTIME, ("[ODM%d] ODM %d: CU already fetched, returning\n", mo->odm->ID));
 		mo->nb_fetch ++;
 		if (planar_size) *planar_size = mo->framesize / mo->num_channels;
 		return mo->frame;
@@ -433,9 +548,16 @@ u8 *gf_mo_fetch_data(GF_MediaObject *mo, GF_MOFetchMode resync, u32 upload_time_
 	}
 
 	if ( gf_odm_check_buffering(mo->odm, NULL) ) {
-		//if buffering, first frame fetched and still buffering return
-		if (mo->first_frame_fetched && mo->odm->nb_buffering)
+		//special flag set for tiles only, return NULL until we are done buffering
+		if (mo->odm->flags & GF_ODM_TILED_SHARED_CLOCK) {
 			return NULL;
+		}
+		if (mo->type==GF_MEDIA_OBJECT_AUDIO)
+			return NULL;
+		//if buffering, first frame fetched and still buffering return last frame
+		if (mo->first_frame_fetched && mo->odm->nb_buffering) {
+			return mo->frame_ifce ? (u8 *) mo->frame_ifce : mo->frame;
+		}
 	}
 
 retry:
@@ -458,18 +580,18 @@ retry:
 		} else {
 			gf_filter_pck_ref(&mo->pck);
 			gf_filter_pid_drop_packet(mo->odm->pid);
+			check_temi(mo);
 		}
 		is_first = GF_TRUE;
 	}
 	assert(mo->pck);
 	mo->first_frame_fetched = GF_TRUE;
-	mo->is_eos = GF_FALSE;
-
+	*eos = mo->is_eos = GF_FALSE;
 
 	/*not running and no resync (ie audio)*/
 	if (!gf_clock_is_started(mo->odm->ck)) {
 		if (!resync) {
-			GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[ODM%d] ODM %d: CB not running, returning\n", mo->odm->ID));
+			GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPTIME, ("[ODM%d] ODM %d: CB not running, returning\n", mo->odm->ID));
 			return NULL;
 		} else if (mo->odm->ck->nb_buffering && mo->odm->type==GF_STREAM_AUDIO) {
 			return NULL;
@@ -505,11 +627,12 @@ retry:
 		} else {
 			if (gf_filter_pid_is_eos(mo->odm->pid)) {
 				if (!mo->is_eos) {
-					mo->is_eos = GF_TRUE;
-					*eos = mo->is_eos;
+					*eos = mo->is_eos = GF_TRUE;
 					mediasensor_update_timing(mo->odm, GF_TRUE);
 					gf_odm_on_eos(mo->odm, mo->odm->pid);
 					force_decode_mode=0;
+					if (!mo->pck)
+						goto retry;
 				}
 				break;
 			}
@@ -520,7 +643,7 @@ retry:
 		gf_filter_pid_try_pull(mo->odm->pid);
 	}
 	if (!retry_pull && (force_decode_mode==1)) {
-		GF_LOG(GF_LOG_INFO, GF_LOG_MEDIA, ("[ODM%d] At %d could not force a pull from pid - POTENTIAL blank frame after TS %u\n", mo->odm->ID, gf_clock_time(mo->odm->ck), mo->timestamp));
+		GF_LOG(GF_LOG_INFO, GF_LOG_COMPTIME, ("[ODM%d] At %d could not force a pull from pid - POTENTIAL blank frame after TS %u\n", mo->odm->ID, gf_clock_time(mo->odm->ck), mo->timestamp));
 	}
 
 	/*resync*/
@@ -543,11 +666,11 @@ retry:
 		if (ABS(diff_pck_old) > ABS(diff_pck_new)) {
 			//don't reset discontinuity flag for audio
 			if (resync>GF_MO_FETCH) {
-				GF_LOG(GF_LOG_INFO, GF_LOG_SYNC, ("[ODM%d] end of clock discontinuity: diff pck TS to old clock %d to new clock %d\n", mo->odm->ID, diff_pck_old, diff_pck_new));
+				GF_LOG(GF_LOG_INFO, GF_LOG_COMPTIME, ("[ODM%d] end of clock discontinuity: diff pck TS to old clock %d to new clock %d\n", mo->odm->ID, diff_pck_old, diff_pck_new));
 				mo->odm->prev_clock_at_discontinuity_plus_one = 0;
 			}
 		} else if (diff_old < diff_new) {
-			GF_LOG(GF_LOG_INFO, GF_LOG_SYNC, ("[ODM%d] in clock discontinuity: time since fetch old clock %d new clock %d\n", mo->odm->ID, diff_old, diff_new));
+			GF_LOG(GF_LOG_INFO, GF_LOG_COMPTIME, ("[ODM%d] in clock discontinuity: time since fetch old clock %d new clock %d\n", mo->odm->ID, diff_old, diff_new));
 
 			obj_time = old_timebase_time;
 		}
@@ -558,7 +681,7 @@ retry:
 	if ((mo->odm->ck->speed == FIX_ONE)
 		&& (mo->type==GF_MEDIA_OBJECT_VIDEO)
 		//if no buffer playout we are in low latency configuration, don"t skip resync
-		&& mo->odm->buffer_playout_us
+		&& mo->odm->buffer_playout_ms
 	) {
 		assert(mo->odm->parentscene);
 		if (! mo->odm->parentscene->compositor->drop) {
@@ -578,7 +701,7 @@ retry:
 			else if (next_ts + 300 >= obj_time) {
 				skip_resync = GF_TRUE;
 			} else if (next_ts) {
-				GF_LOG(GF_LOG_DEBUG, GF_LOG_SYNC, ("[ODM%d] At %u frame TS %u next frame TS %d too late in no-drop mode, enabling drop - resync mode %d\n", mo->odm->ID, obj_time, pck_ts, next_ts, resync));
+				GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPTIME, ("[ODM%d] At %u frame TS %u next frame TS %d too late in no-drop mode, enabling drop - resync mode %d\n", mo->odm->ID, obj_time, pck_ts, next_ts, resync));
 				mo->flags |= GF_MO_IN_RESYNC;
 			}
 		}
@@ -597,7 +720,7 @@ retry:
 			//drop current and go to next - we use the same loop as regular resync below
 			resync = GF_MO_FETCH_RESYNC;
 			move_to_next_only = GF_TRUE;
-			GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[ODM%d] Switching to CU CTS %u (next %d) now %u\n", mo->odm->ID, pck_ts, next_ts, obj_time));
+			GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPTIME, ("[ODM%d] Switching to CU CTS %u (next %d) now %u\n", mo->odm->ID, pck_ts, next_ts, obj_time));
 		}
 	}
 	if (resync!=GF_MO_FETCH) {
@@ -607,7 +730,7 @@ retry:
 				if (mo->odm->ck->speed > 0 ? pck_ts >= obj_time : pck_ts <= obj_time )
 					break;
 
-				GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[ODM%d] Try to drop frame TS %u next frame TS %u obj time %u\n", mo->odm->ID, pck_ts, next_ts, obj_time));
+				GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPTIME, ("[ODM%d] Try to drop frame TS %u next frame TS %u obj time %u\n", mo->odm->ID, pck_ts, next_ts, obj_time));
 
 				//nothing ready yet
 				if ( gf_filter_pid_first_packet_is_empty(mo->odm->pid) ) {
@@ -622,7 +745,7 @@ retry:
 
 				nb_dropped ++;
 				if (nb_dropped>=1) {
-					GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[ODM%d] At OTB %u dropped frame TS %u\n", mo->odm->ID, obj_time, pck_ts));
+					GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPTIME, ("[ODM%d] At OTB %u dropped frame TS %u\n", mo->odm->ID, obj_time, pck_ts));
 
 					mo->odm->nb_dropped++;
 				}
@@ -631,7 +754,9 @@ retry:
 			//delete our packet
 			gf_filter_pck_unref(mo->pck);
 			mo->pck = gf_filter_pid_get_packet(mo->odm->pid);
+			assert(mo->pck);
 			gf_filter_pck_ref( &mo->pck);
+			check_temi(mo);
 
 			pck_ts = convert_ts_to_ms(mo, gf_filter_pck_get_cts(mo->pck), timescale, &discard);
 			//drop next packet from pid
@@ -642,7 +767,7 @@ retry:
 				s32 diff_pck_new = (s32) pck_ts - (s32) obj_time_orig;
 
 				if (ABS(diff_pck_old) > ABS(diff_pck_new)) {
-					GF_LOG(GF_LOG_INFO, GF_LOG_SYNC, ("[ODM%d] end of clock discontinuity, moving from old time base %d to new %d\n", mo->odm->ID, obj_time, obj_time_orig));
+					GF_LOG(GF_LOG_INFO, GF_LOG_COMPTIME, ("[ODM%d] end of clock discontinuity, moving from old time base %d to new %d\n", mo->odm->ID, obj_time, obj_time_orig));
 					obj_time = obj_time_orig;
 					mo->odm->prev_clock_at_discontinuity_plus_one = 0;
 				}
@@ -661,15 +786,16 @@ retry:
 	mo->frame = (char *) gf_filter_pck_get_data(mo->pck, &mo->size);
 	mo->framesize = mo->size - mo->RenderedLength;
 
-	//planar mode, RenderedLength correspond to all channels, so move frame pointer
-	//to first sample non consumed = RenderedLength/nb_channels
-	if (mo->planar_audio) {
-		mo->frame += mo->RenderedLength / mo->num_channels;
-	} else {
-		mo->frame += mo->RenderedLength;
+	if (mo->type == GF_MEDIA_OBJECT_AUDIO) {
+		//planar mode, RenderedLength correspond to all channels, so move frame pointer
+		//to first sample non consumed = RenderedLength/nb_channels
+		if (mo->planar_audio) {
+			mo->frame += mo->RenderedLength / mo->num_channels;
+		} else {
+			mo->frame += mo->RenderedLength;
+		}
 	}
 	mo->frame_ifce = gf_filter_pck_get_frame_interface(mo->pck);
-//	mo->media_frame = CU->frame;
 
 	diff = (s32) ( (mo->speed >= 0) ? ( (s64) pck_ts - (s64) obj_time) : ( (s64) obj_time - (s64) pck_ts) );
 	mo->ms_until_pres = FIX2INT(diff * mo->speed);
@@ -720,7 +846,7 @@ retry:
 			mediasensor_update_timing(mo->odm, GF_FALSE);
 #endif
 
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[ODM%d (%s)] At OTB %u fetch frame TS %u size %d (previous TS %u) - %d unit in CB - UTC "LLU" ms - %d ms until CTS is due - %d ms until next frame\n", mo->odm->ID, mo->odm->scene_ns->url, gf_clock_time(mo->odm->ck), pck_ts, mo->framesize, mo->timestamp, gf_filter_pid_get_packet_count(mo->odm->pid), gf_net_get_utc(), mo->ms_until_pres, mo->ms_until_next ));
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPTIME, ("[ODM%d (%s)] At OTB %u fetch frame TS %u size %d (previous TS %u) - %d unit in CB - UTC "LLU" ms - %d ms until CTS is due - %d ms until next frame\n", mo->odm->ID, mo->odm->scene_ns->url, gf_clock_time(mo->odm->ck), pck_ts, mo->framesize, mo->timestamp, gf_filter_pid_get_packet_count(mo->odm->pid), gf_net_get_utc(), mo->ms_until_pres, mo->ms_until_next ));
 
 		v = gf_filter_pck_get_property(mo->pck, GF_PROP_PCK_SENDER_NTP);
 		if (v) {
@@ -739,7 +865,7 @@ retry:
 				mo->odm->last_drawn_frame_ntp_diff -= v->value.sint;
 			}
 			gf_filter_release_property(pe);
-			GF_LOG(GF_LOG_INFO, GF_LOG_MEDIA, ("[ODM%d (%s)] Frame TS %u NTP diff with sender %d ms\n", mo->odm->ID, mo->odm->scene_ns->url, pck_ts, mo->odm->last_drawn_frame_ntp_diff));
+			GF_LOG(GF_LOG_INFO, GF_LOG_COMPTIME, ("[ODM%d (%s)] Frame TS %u NTP diff with sender %d ms\n", mo->odm->ID, mo->odm->scene_ns->url, pck_ts, mo->odm->last_drawn_frame_ntp_diff));
 
 			if (mo->odm->parentscene->compositor->ntpsync
 				&& (mo->odm->last_drawn_frame_ntp_diff > (s32) mo->odm->parentscene->compositor->ntpsync)
@@ -759,7 +885,7 @@ retry:
 		//already rendered the last frame, consider we no longer have pending late frame on this stream
 		mo->ms_until_pres = 0;
 	} else {
-//		GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[ODM%d (%s)] At OTB %u same frame fetch TS %u\n", mo->odm->ID, mo->odm->net_service->url, obj_time, CU->TS ));
+//		GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPTIME, ("[ODM%d (%s)] At OTB %u same frame fetch TS %u\n", mo->odm->ID, mo->odm->net_service->url, obj_time, CU->TS ));
 
 		//if paused force a high value for next frame
 		if (!gf_clock_is_started(mo->odm->ck)) {
@@ -767,7 +893,7 @@ retry:
 		}
 	}
 
-	/*also adjust CU time based on consummed bytes in input, since some codecs output very large audio chunks*/
+	/*also adjust CU time based on consumed bytes in input, since some codecs output very large audio chunks*/
 	if (mo->bytes_per_sec) mo->timestamp += mo->RenderedLength * 1000 / mo->bytes_per_sec;
 
 	if (mo->odm->parentscene->compositor->bench_mode) {
@@ -785,12 +911,13 @@ retry:
 	if (ms_until_pres) *ms_until_pres = mo->ms_until_pres;
 	if (ms_until_next) *ms_until_next = mo->ms_until_next;
 	if (outFrame) *outFrame = mo->frame_ifce;
-	if (planar_size) *planar_size = mo->framesize / mo->num_channels;
+	//planar size is computed based on original size, not framesize (= size - renderedLength)
+	if (planar_size) *planar_size = mo->size / mo->num_channels;
 
 //	gf_odm_service_media_event(mo->odm, GF_EVENT_MEDIA_TIME_UPDATE);
 
 	if (mo->frame_ifce)
-		return (char *) mo->frame_ifce;
+		return (u8 *) mo->frame_ifce;
 
 	return mo->frame;
 }
@@ -837,7 +964,7 @@ void gf_mo_release_data(GF_MediaObject *mo, u32 nb_bytes, s32 drop_mode)
 		if (drop_mode) {
 			gf_filter_pck_unref(mo->pck);
 			mo->pck = NULL;
-			GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[ODM%d] At OTB %u released frame TS %u\n", mo->odm->ID,gf_clock_time(mo->odm->ck), mo->timestamp));
+			GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPTIME, ("[ODM%d] At OTB %u released frame TS %u\n", mo->odm->ID,gf_clock_time(mo->odm->ck), mo->timestamp));
 		} else {
 			/*we cannot drop since we don't know the speed of the playback (which can even be frame by frame)*/
 		}
@@ -874,7 +1001,7 @@ void gf_mo_play(GF_MediaObject *mo, Double clipBegin, Double clipEnd, Bool can_l
 				return;
 			}
 		}
-		if (mo->odm->flags & GF_ODM_NO_TIME_CTRL) {
+		if ( (mo->odm->flags & GF_ODM_NO_TIME_CTRL) || (clipBegin<0) ) {
 			mo->odm->media_start_time = 0;
 		} else {
 			mo->odm->media_start_time = (u64) (clipBegin*1000);
@@ -922,6 +1049,11 @@ void gf_mo_stop(GF_MediaObject **_mo)
 			return;
 		}
 
+		if (mo->pck) {
+			gf_filter_pck_unref(mo->pck);
+			mo->pck = NULL;
+		}
+
 		/*signal STOP request*/
 		if ((mo->OD_ID==GF_MEDIA_EXTERNAL_ID) || (mo->odm && mo->odm->ID && (mo->odm->ID==GF_MEDIA_EXTERNAL_ID))) {
 			gf_odm_disconnect(mo->odm, 2);
@@ -953,6 +1085,10 @@ void gf_mo_restart(GF_MediaObject *mo)
 		}
 	}
 	/*all other cases, call restart to take into account clock references*/
+	mo->first_frame_fetched = GF_FALSE;
+	mo->is_eos = GF_FALSE;
+	mo->ms_until_next = 0;
+	mo->ms_until_pres = 0;
 	mediacontrol_restart(mo->odm);
 }
 
@@ -987,7 +1123,7 @@ u32 gf_mo_get_od_id(MFURL *url)
 				j++;
 			}
 			j = sscanf(s_url, "%u", &tmpid);
-			/*be carefull, an url like "11-regression-test.mp4" will return 1 on sscanf :)*/
+			/*be careful, an url like "11-regression-test.mp4" will return 1 on sscanf :)*/
 			if (j==1) {
 				char szURL[20];
 				sprintf(szURL, "%u", tmpid);
@@ -1061,14 +1197,14 @@ Bool gf_mo_is_same_url(GF_MediaObject *obj, MFURL *an_url, Bool *keep_fragment,
 			}
 
 			scene = gf_scene_get_root_scene(obj->odm->parentscene ? obj->odm->parentscene : obj->odm->subscene);
-			while ( (sns = (GF_SceneNamespace*) gf_list_enum(scene->namespaces, &j) ) ) {
-				/*sub-service of an existing service - don't touch any fragment*/
-#ifdef FILTER_FIXME
-				if (gf_term_service_can_handle_url(sns, an_url->vals[i].url)) {
-					*keep_fragment = GF_TRUE;
-					return GF_FALSE;
+			if (scene->root_od->scene_ns && scene->root_od->scene_ns->url) {
+				while ( (sns = (GF_SceneNamespace*) gf_list_enum(scene->namespaces, &j) ) ) {
+					/*sub-service of an existing service - don't touch any fragment*/
+					if (gf_filter_is_supported_source(scene->compositor->filter, an_url->vals[i].url, scene->root_od->scene_ns->url)) {
+						*keep_fragment = GF_TRUE;
+						return GF_FALSE;
+					}
 				}
-#endif
 			}
 		}
 	}
@@ -1335,9 +1471,7 @@ void gf_mo_set_flag(GF_MediaObject *mo, GF_MOUserFlags flag, Bool set_on)
 GF_EXPORT
 u32 gf_mo_has_audio(GF_MediaObject *mo)
 {
-#ifdef FILTER_FIXME
 	char *sub_url;
-#endif
 	u32 i;
 	GF_SceneNamespace *ns;
 	GF_Scene *scene;
@@ -1347,22 +1481,19 @@ u32 gf_mo_has_audio(GF_MediaObject *mo)
 
 	ns = mo->odm->scene_ns;
 	scene = mo->odm->parentscene;
-#ifdef FILTER_FIXME
 	sub_url = strchr(ns->url, '#');
-#endif
+
 	for (i=0; i<gf_list_count(scene->resources); i++) {
 		GF_ObjectManager *odm = (GF_ObjectManager *)gf_list_get(scene->resources, i);
 		if (odm->scene_ns != ns) continue;
 		//object already associated
 		if (odm->mo) continue;
 
-#ifdef FILTER_FIXME
 		if (sub_url) {
 			char *ext = mo->URLs.count ? mo->URLs.vals[0].url : NULL;
 			if (ext) ext = strchr(ext, '#');
 			if (!ext || strcmp(sub_url, ext)) continue;
 		}
-#endif
 		/*we have one audio object not bound with the scene from the same service, let's use it*/
 		if (odm->type == GF_STREAM_AUDIO) return 1;
 	}
