@@ -25,11 +25,11 @@
 
 #include "nodes_stacks.h"
 #include "visual_manager.h"
-/*for default scene view*/
-#include <gpac/internal/terminal_dev.h>
+
 #include <gpac/options.h>
 
 
+GF_EXPORT
 GF_Err gf_sc_get_viewpoint(GF_Compositor *compositor, u32 viewpoint_idx, const char **outName, Bool *is_bound)
 {
 #ifndef GPAC_DISABLE_VRML
@@ -43,18 +43,25 @@ GF_Err gf_sc_get_viewpoint(GF_Compositor *compositor, u32 viewpoint_idx, const c
 	n = (GF_Node*)gf_list_get(compositor->visual->view_stack, viewpoint_idx-1);
 	switch (gf_node_get_tag(n)) {
 	case TAG_MPEG4_Viewport:
-		*outName = ((M_Viewport*)n)->description.buffer;
-		*is_bound = ((M_Viewport*)n)->isBound;
+		if (outName)
+			*outName = ((M_Viewport*)n)->description.buffer;
+		if (is_bound)
+			*is_bound = ((M_Viewport*)n)->isBound;
 		return GF_OK;
 	case TAG_MPEG4_Viewpoint:
 #ifndef GPAC_DISABLE_X3D
 	case TAG_X3D_Viewpoint:
 #endif
-		*outName = ((M_Viewpoint*)n)->description.buffer;
-		*is_bound = ((M_Viewpoint*)n)->isBound;
+		if (outName)
+			*outName = ((M_Viewpoint*)n)->description.buffer;
+		if (is_bound)
+			*is_bound = ((M_Viewpoint*)n)->isBound;
 		return GF_OK;
 	default:
-		*outName = NULL;
+		if (outName)
+			*outName = NULL;
+		if (is_bound)
+			*is_bound = GF_FALSE;
 		return GF_OK;
 	}
 #else
@@ -62,6 +69,7 @@ GF_Err gf_sc_get_viewpoint(GF_Compositor *compositor, u32 viewpoint_idx, const c
 #endif
 }
 
+GF_EXPORT
 GF_Err gf_sc_set_viewpoint(GF_Compositor *compositor, u32 viewpoint_idx, const char *viewpoint_name)
 {
 #ifndef GPAC_DISABLE_VRML
@@ -112,7 +120,7 @@ GF_Err gf_sc_set_viewpoint(GF_Compositor *compositor, u32 viewpoint_idx, const c
 #ifndef GPAC_DISABLE_VRML
 
 
-#define VPCHANGED(__rend) { GF_Event evt; evt.type = GF_EVENT_VIEWPOINTS; gf_term_send_event(__rend->term, &evt); }
+#define VPCHANGED(__comp) { GF_Event evt; evt.type = GF_EVENT_VIEWPOINTS; gf_sc_send_event(__comp, &evt); }
 
 
 static void DestroyViewStack(GF_Node *node)
@@ -381,9 +389,14 @@ static void TraverseViewpoint(GF_Node *node, void *rs, Bool is_destroy)
 	/*not bound or in 2D visual*/
 	if (!vp->isBound || !tr_state->navigations) return;
 
-	if (!gf_node_dirty_get(node)) return;
+	if (!gf_node_dirty_get(node)) {
+		if ((tr_state->vp_size.x == st->last_vp_size.x) &&  (tr_state->vp_size.y == st->last_vp_size.y)) return;
+	}
 	gf_node_dirty_clear(node, 0);
 
+	st->last_vp_size.x = tr_state->vp_size.x;
+	st->last_vp_size.y = tr_state->vp_size.y;
+
 	/*move to local system*/
 	gf_mx_copy(mx, st->world_view_mx);
 	gf_mx_add_translation(&mx, vp->position.x, vp->position.y, vp->position.z);
@@ -421,9 +434,11 @@ void compositor_init_viewpoint(GF_Compositor *compositor, GF_Node *node)
 
 static void navinfo_set_bind(GF_Node *node, GF_Route *route)
 {
-	ViewStack *st = (ViewStack *) gf_node_get_private(node);
-	Bindable_OnSetBind(node, st->reg_stacks, NULL);
-	gf_sc_invalidate( gf_sc_get_compositor(node), NULL);
+	if (node) {
+		ViewStack *st = (ViewStack *) gf_node_get_private(node);
+		Bindable_OnSetBind(node, st->reg_stacks, NULL);
+		gf_sc_invalidate( gf_sc_get_compositor(node), NULL);
+	}
 }
 
 static void TraverseNavigationInfo(GF_Node *node, void *rs, Bool is_destroy)
@@ -518,7 +533,8 @@ static void TraverseNavigationInfo(GF_Node *node, void *rs, Bool is_destroy)
 	if (ni->avatarSize.count>1) tr_state->camera->avatar_size.y = gf_mulfix(scale, ni->avatarSize.vals[1]);
 	if (ni->avatarSize.count>2) tr_state->camera->avatar_size.z = gf_mulfix(scale, ni->avatarSize.vals[2]);
 
-	if (0 && tr_state->pixel_metrics) {
+#if 0
+	if (tr_state->pixel_metrics) {
 		u32 s = MAX(tr_state->visual->width, tr_state->visual->height);
 		s /= 2;
 //		tr_state->camera->speed = ni->speed;
@@ -529,6 +545,8 @@ static void TraverseNavigationInfo(GF_Node *node, void *rs, Bool is_destroy)
 	}
 #endif
 
+#endif
+
 }
 
 void compositor_init_navigation_info(GF_Compositor *compositor, GF_Node *node)
@@ -544,6 +562,12 @@ void compositor_init_navigation_info(GF_Compositor *compositor, GF_Node *node)
 	gf_node_set_private(node, st);
 	gf_node_set_callback_function(node, TraverseNavigationInfo);
 	((M_NavigationInfo*)node)->on_set_bind = navinfo_set_bind;
+
+#ifdef GPAC_ENABLE_COVERAGE
+	if (gf_sys_is_cov_mode()) {
+		navinfo_set_bind(NULL, NULL);
+	}
+#endif
 }
 
 
