@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2017-2018
+ *			Copyright (c) Telecom ParisTech 2017-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / filters sub-project
@@ -48,10 +48,15 @@ GF_Err gf_filter_pck_merge_properties_filter(GF_FilterPacket *pck_src, GF_Filter
 	pck_src=pck_src->pck;
 	pck_dst=pck_dst->pck;
 
+	//keep internal flags
+	u32 iflags = pck_dst->info.flags & (GF_PCKF_PROPS_REFERENCE | GF_PCKF_FORCE_MAIN) ;
 	pck_dst->info = pck_src->info;
-	pck_dst->info.flags &= ~GF_PCKF_PROPS_REFERENCE;
+	//remove internal props flags of reference
+	pck_dst->info.flags &= ~ (GF_PCKF_PROPS_REFERENCE | GF_PCKF_FORCE_MAIN);
+	//restore internal props flags of packet
+	pck_dst->info.flags |= iflags;
 
-	if (!pck_src->props) {
+	if (!pck_src->props || pck_dst->is_dangling) {
 		return GF_OK;
 	}
 	if (!pck_dst->props) {
@@ -95,7 +100,7 @@ static Bool pck_queue_enum(void *udta, void *item)
 	return GF_FALSE;
 }
 
-static GF_FilterPacket *gf_filter_pck_new_alloc_internal(GF_FilterPid *pid, u32 data_size, u8 **data, Bool no_block_check)
+static GF_FilterPacket *gf_filter_pck_new_alloc_internal(GF_FilterPid *pid, u32 data_size, u8 **data)
 {
 	GF_FilterPacket *pck=NULL;
 	GF_FilterPacket *closest=NULL;
@@ -105,8 +110,6 @@ static GF_FilterPacket *gf_filter_pck_new_alloc_internal(GF_FilterPid *pid, u32
 		GF_LOG(GF_LOG_ERROR, GF_LOG_FILTER, ("Attempt to allocate a packet on an input PID in filter %s\n", pid->filter->name));
 		return NULL;
 	}
-	if (!no_block_check && gf_filter_pid_would_block(pid))
-		return NULL;
 
 	count = gf_fq_count(pid->filter->pcks_alloc_reservoir);
 	if (count) {
@@ -128,6 +131,11 @@ static GF_FilterPacket *gf_filter_pck_new_alloc_internal(GF_FilterPid *pid, u32
 		assert(closest);
 		closest->alloc_size = data_size;
 		closest->data = gf_realloc(closest->data, closest->alloc_size);
+		if (!closest->data) {
+			gf_free(closest);
+			GF_LOG(GF_LOG_ERROR, GF_LOG_FILTER, ("Failed to allocate new packet on PID %s of filter %s\n", pid->name, pid->filter->name));
+			return NULL;
+		}
 		pck = closest;
 #ifdef GPAC_MEMORY_TRACKING
 		pid->filter->session->nb_realloc_pck++;
@@ -136,9 +144,16 @@ static GF_FilterPacket *gf_filter_pck_new_alloc_internal(GF_FilterPid *pid, u32
 
 	if (!pck) {
 		GF_SAFEALLOC(pck, GF_FilterPacket);
-		if (!pck)
+		if (!pck) {
+			GF_LOG(GF_LOG_ERROR, GF_LOG_FILTER, ("Failed to allocate new packet on PID %s of filter %s\n", pid->name, pid->filter->name));
 			return NULL;
+		}
 		pck->data = gf_malloc(sizeof(char)*data_size);
+		if (!pck->data) {
+			gf_free(pck);
+			GF_LOG(GF_LOG_ERROR, GF_LOG_FILTER, ("Failed to allocate new packet on PID %s of filter %s\n", pid->name, pid->filter->name));
+			return NULL;
+		}
 		pck->alloc_size = data_size;
 #ifdef GPAC_MEMORY_TRACKING
 		pid->filter->session->nb_alloc_pck+=2;
@@ -171,17 +186,60 @@ static GF_FilterPacket *gf_filter_pck_new_alloc_internal(GF_FilterPid *pid, u32
 GF_EXPORT
 GF_FilterPacket *gf_filter_pck_new_alloc(GF_FilterPid *pid, u32 data_size, u8 **data)
 {
-	return gf_filter_pck_new_alloc_internal(pid, data_size, data, GF_TRUE);
+	return gf_filter_pck_new_alloc_internal(pid, data_size, data);
 }
 
-static GF_FilterPacket *clone_frame_interface(GF_FilterPid *pid, GF_FilterPacket *pck_source, u8 **data)
+static GF_FilterPacket *gf_filter_pck_new_dangling_packet(GF_FilterPacket *cached_pck, u32 data_length)
+{
+	GF_FilterPacket *dst;
+
+	if (cached_pck && cached_pck->reference) {
+		gf_filter_pck_discard(cached_pck);
+		cached_pck = NULL;
+	}
+	if (cached_pck) {
+		if (data_length > cached_pck->alloc_size) {
+			cached_pck->alloc_size = data_length;
+			cached_pck->data = gf_realloc(cached_pck->data, cached_pck->alloc_size);
+			if (!cached_pck->data) {
+				gf_free(cached_pck);
+				GF_LOG(GF_LOG_ERROR, GF_LOG_FILTER, ("Failed to re-allocate dangling packet data\n"));
+				return NULL;
+			}
+		}
+		cached_pck->data_length = data_length;
+		return cached_pck;
+	}
+
+	GF_SAFEALLOC(dst, GF_FilterPacket);
+	if (!dst) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_FILTER, ("Failed to allocate new dangling packet\n"));
+		return NULL;
+	}
+	dst->data = gf_malloc(sizeof(char) * data_length);
+	if (!dst->data) {
+		gf_free(dst);
+		GF_LOG(GF_LOG_ERROR, GF_LOG_FILTER, ("Failed to allocate new dangling packet\n"));
+		return NULL;
+	}
+	dst->alloc_size = dst->data_length = data_length;
+	dst->filter_owns_mem = 0;
+	dst->pck = dst;
+	dst->is_dangling = 1;
+	return dst;
+}
+
+static GF_FilterPacket *gf_filter_pck_clone_frame_interface(GF_FilterPid *pid, GF_FilterPacket *pck_source, u8 **data, Bool dangling_packet, GF_FilterPacket *cached_pck)
 {
 	u32 i, w, h, stride, stride_uv, pf, osize;
 	u32 nb_planes, uv_height;
 	GF_FilterPacket *dst, *ref;
+	u8 *pck_data;
 	GF_FilterPacketInstance *pcki = (GF_FilterPacketInstance *) pck_source;
 	const GF_PropertyValue *p;
 
+	if (data) *data = NULL;
+
 	ref = pcki->pck;
 
 	p = gf_filter_pid_get_property(ref->pid , GF_PROP_PID_WIDTH);
@@ -198,18 +256,27 @@ static GF_FilterPacket *clone_frame_interface(GF_FilterPid *pid, GF_FilterPacket
 	stride = stride_uv = 0;
 
 	if (gf_pixel_get_size_info(pf, w, h, &osize, &stride, &stride_uv, &nb_planes, &uv_height) == GF_FALSE) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_FILTER, ("Unknown pixel format, cannot grab underlying video data\n"));
 		return NULL;
 	}
-	p = gf_filter_pid_get_property(pid, GF_PROP_PID_STRIDE);
-	if (!p || (p->value.uint != stride)) {
-		gf_filter_pid_set_property(pid, GF_PROP_PID_STRIDE, &PROP_UINT(stride));
-		gf_filter_pid_set_property(pid, GF_PROP_PID_STRIDE_UV, &PROP_UINT(stride_uv));
+
+	if (!dangling_packet) {
+		p = gf_filter_pid_get_property(pid, GF_PROP_PID_STRIDE);
+		if (!p || (p->value.uint != stride)) {
+			gf_filter_pid_set_property(pid, GF_PROP_PID_STRIDE, &PROP_UINT(stride));
+			gf_filter_pid_set_property(pid, GF_PROP_PID_STRIDE_UV, &PROP_UINT(stride_uv));
+		}
+		dst = gf_filter_pck_new_alloc(pid, osize, &pck_data);
+		if (!dst) return NULL;
+	} else {
+		dst = gf_filter_pck_new_dangling_packet(cached_pck, osize);
+		if (!dst) return NULL;
+		pck_data = dst->data;
 	}
 
-	dst = gf_filter_pck_new_alloc(pid, osize, data);
 	if (!dst) return NULL;
-	
-	u8 *out_ptr = *data;
+	if (data) *data = pck_data;
+
 	for (i=0; i<nb_planes; i++) {
 		u32 j, write_h, dst_stride;
 		const u8 *in_ptr;
@@ -227,25 +294,32 @@ static GF_FilterPacket *clone_frame_interface(GF_FilterPid *pid, GF_FilterPacket
 			dst_stride = stride;
 		}
 		for (j=0; j<write_h; j++) {
-			memcpy(out_ptr, in_ptr, dst_stride);
+			memcpy(pck_data, in_ptr, dst_stride);
 			in_ptr += src_stride;
-			out_ptr += dst_stride;
+			pck_data += dst_stride;
 		}
 	}
 	gf_filter_pck_merge_properties(pck_source, dst);
 	return dst;
 }
 
-static GF_FilterPacket *gf_filter_pck_new_clone_internal(GF_FilterPid *pid, GF_FilterPacket *pck_source, u8 **data, Bool force_copy)
+static GF_FilterPacket *gf_filter_pck_new_clone_internal(GF_FilterPid *pid, GF_FilterPacket *pck_source, u8 **data, Bool force_copy, Bool dangling_packet, GF_FilterPacket *cached_pck)
 {
 	GF_FilterPacket *dst, *ref;
 	u32 max_ref = 0;
 	GF_FilterPacketInstance *pcki;
 	if (!pck_source) return NULL;
 
+	if (dangling_packet) {
+		if (PCK_IS_OUTPUT(pck_source)) {
+			GF_LOG(GF_LOG_ERROR, GF_LOG_FILTER, ("Cannot create dangling packet from non-source packet\n"));
+			return NULL;
+		}
+	}
+
 	pcki = (GF_FilterPacketInstance *) pck_source;
 	if (pcki->pck->frame_ifce)
-		return clone_frame_interface(pid, pck_source, data);
+		return gf_filter_pck_clone_frame_interface(pid, pck_source, data, dangling_packet, cached_pck);
 
 	if (force_copy) {
 		max_ref = 2;
@@ -263,39 +337,84 @@ static GF_FilterPacket *gf_filter_pck_new_clone_internal(GF_FilterPid *pid, GF_F
 	}
 	
 	if (max_ref>1) {
-		dst = gf_filter_pck_new_alloc_internal(pid, pcki->pck->data_length, data, GF_TRUE);
-		if (dst && data) {
-			memcpy(*data, pcki->pck->data, sizeof(char)*pcki->pck->data_length);
+		u8 *data_new;
+		if (dangling_packet) {
+			dst = gf_filter_pck_new_dangling_packet(cached_pck, pcki->pck->data_length);
+			if (!dst) return NULL;
+
+			data_new = dst->data;
+		} else {
+			dst = gf_filter_pck_new_alloc_internal(pid, pcki->pck->data_length, &data_new);
+		}
+		if (dst && data_new) {
+			memcpy(data_new, pcki->pck->data, sizeof(char)*pcki->pck->data_length);
+			if (data) *data = data_new;
 		}
 		if (dst) gf_filter_pck_merge_properties(pck_source, dst);
 		return dst;
 	}
-	dst = gf_filter_pck_new_ref(pid, NULL, 0, pck_source);
+
+	if (dangling_packet) {
+		if (cached_pck && cached_pck->reference) {
+			gf_filter_pck_discard(cached_pck);
+		}
+		GF_SAFEALLOC(dst, GF_FilterPacket);
+		if (!dst) {
+			GF_LOG(GF_LOG_ERROR, GF_LOG_FILTER, ("Failed to allocate new dangling packet (source filter %s)\n", pck_source->pck->src_filter->name));
+			return NULL;
+		}
+		pck_source = pck_source->pck;
+		dst->data = pck_source->data;
+		dst->data_length = pck_source->data_length;
+		dst->reference = pck_source;
+		dst->pck = dst;
+		dst->is_dangling = 1;
+
+		assert(pck_source->reference_count);
+		//cf gf_filter_pck_new_ref for these
+		safe_int_inc(&pck_source->reference_count);
+		safe_int_inc(&pck_source->pid->nb_shared_packets_out);
+		safe_int_inc(&pck_source->pid->filter->nb_shared_packets_out);
+		gf_filter_pck_merge_properties(pck_source, dst);
+		if (data) *data = dst->data;
+		return dst;
+	}
+
+	dst = gf_filter_pck_new_ref(pid, 0, 0, pck_source);
 	if (dst) {
 		gf_filter_pck_merge_properties(pck_source, dst);
-		*data = dst->data;
+		if (data) *data = dst->data;
 	}
 	return dst;
 }
 
+GF_EXPORT
+GF_FilterPacket *gf_filter_pck_dangling_copy(GF_FilterPacket *pck_source, GF_FilterPacket *cached_pck)
+{
+	return gf_filter_pck_new_clone_internal(NULL, pck_source, NULL, GF_FALSE, GF_TRUE, cached_pck);
+}
 
 GF_EXPORT
 GF_FilterPacket *gf_filter_pck_new_clone(GF_FilterPid *pid, GF_FilterPacket *pck_source, u8 **data)
 {
-	return gf_filter_pck_new_clone_internal(pid, pck_source, data, GF_FALSE);
+	return gf_filter_pck_new_clone_internal(pid, pck_source, data, GF_FALSE, GF_FALSE, NULL);
 }
 GF_EXPORT
 GF_FilterPacket *gf_filter_pck_new_copy(GF_FilterPid *pid, GF_FilterPacket *pck_source, u8 **data)
 {
-	return gf_filter_pck_new_clone_internal(pid, pck_source, data, GF_TRUE);
+	return gf_filter_pck_new_clone_internal(pid, pck_source, data, GF_TRUE, GF_FALSE, NULL);
 }
 
 
 GF_EXPORT
 GF_FilterPacket *gf_filter_pck_new_alloc_destructor(GF_FilterPid *pid, u32 data_size, u8 **data, gf_fsess_packet_destructor destruct)
 {
-	GF_FilterPacket *pck = gf_filter_pck_new_alloc_internal(pid, data_size, data, GF_TRUE);
-	if (pck) pck->destructor = destruct;
+	GF_FilterPacket *pck = gf_filter_pck_new_alloc_internal(pid, data_size, data);
+	if (pck && destruct) {
+		pck->destructor = destruct;
+		if (pid->filter->freg->flags & GF_FS_REG_MAIN_THREAD)
+			pck->info.flags |= GF_PCKF_FORCE_MAIN;
+	}
 	return pck;
 }
 
@@ -327,6 +446,9 @@ GF_FilterPacket *gf_filter_pck_new_shared_internal(GF_FilterPid *pid, const u8 *
 	}
 	gf_filter_pck_reset_props(pck, pid);
 
+	if (destruct && (pid->filter->freg->flags & GF_FS_REG_MAIN_THREAD))
+		pck->info.flags |= GF_PCKF_FORCE_MAIN;
+
 	assert(pck->pid);
 	return pck;
 }
@@ -339,21 +461,39 @@ GF_FilterPacket *gf_filter_pck_new_shared(GF_FilterPid *pid, const u8 *data, u32
 }
 
 GF_EXPORT
-GF_FilterPacket *gf_filter_pck_new_ref(GF_FilterPid *pid, const u8 *data, u32 data_size, GF_FilterPacket *reference)
+GF_FilterPacket *gf_filter_pck_new_ref(GF_FilterPid *pid, u32 data_offset, u32 data_size, GF_FilterPacket *reference)
 {
 	GF_FilterPacket *pck;
 	if (!reference) return NULL;
 	reference=reference->pck;
-	pck = gf_filter_pck_new_shared(pid, data, data_size, NULL);
+
+	if (reference->data) {
+		if (data_offset > reference->data_length)
+			return NULL;
+
+		if (!data_size)
+			data_size = reference->data_length - data_offset;
+
+		if (data_offset + data_size > reference->data_length)
+			return NULL;
+	}
+
+	pck = gf_filter_pck_new_shared(pid, reference->data, data_size, NULL);
 	if (!pck) return NULL;
 	pck->reference = reference;
+	//apply offset
+	if (reference->data)
+		pck->data += data_offset;
 	assert(reference->reference_count);
 	safe_int_inc(&reference->reference_count);
-	if (!data && !data_size) {
+	if (!data_offset && (!data_size || (data_size==reference->data_length))) {
 		pck->data = reference->data;
 		pck->data_length = reference->data_length;
 		pck->frame_ifce = reference->frame_ifce;
 	}
+	if (reference->info.flags & GF_PCKF_FORCE_MAIN)
+		pck->info.flags |= GF_PCKF_FORCE_MAIN;
+
 	safe_int_inc(&reference->pid->nb_shared_packets_out);
 	safe_int_inc(&reference->pid->filter->nb_shared_packets_out);
 	return pck;
@@ -380,6 +520,8 @@ GF_FilterPacket *gf_filter_pck_new_frame_interface(GF_FilterPid *pid, GF_FilterF
 	pck->destructor = destruct;
 	pck->frame_ifce = frame_ifce;
 	pck->filter_owns_mem = 2;
+	if (frame_ifce && frame_ifce->get_gl_texture)
+		pck->info.flags |= GF_PCKF_FORCE_MAIN;
 	return pck;
 }
 
@@ -401,6 +543,8 @@ GF_Err gf_filter_pck_forward(GF_FilterPacket *reference, GF_FilterPid *pid)
 	pck->data = reference->data;
 	pck->data_length = reference->data_length;
 	pck->frame_ifce = reference->frame_ifce;
+	if (reference->info.flags & GF_PCKF_FORCE_MAIN)
+		pck->info.flags |= GF_PCKF_FORCE_MAIN;
 
 	return gf_filter_pck_send(pck);
 }
@@ -434,7 +578,7 @@ void gf_filter_packet_destroy(GF_FilterPacket *pck)
 		is_filter_destroyed = pck->src_filter->finalized;
 	}
 
-	if (!is_filter_destroyed) {
+	if (!is_filter_destroyed && !pck->is_dangling) {
 		assert(pck->pid);
 		if (pck->pid->filter) {
 			if (pck->info.cts != GF_FILTER_NO_TS) {
@@ -444,8 +588,10 @@ void gf_filter_packet_destroy(GF_FilterPacket *pck)
 			}
 		}
 	}
+	//never set for dangling packets
 	if (pck->destructor) pck->destructor(pid->filter, pid, pck);
 
+	//never set for dangling packets (theyr are never sent)
 	if (pck->pid_props) {
 		GF_PropertyMap *props = pck->pid_props;
 		pck->pid_props = NULL;
@@ -481,6 +627,7 @@ void gf_filter_packet_destroy(GF_FilterPacket *pck)
 			gf_props_del(props);
 		}
 	}
+	//never set for dangling packets, they are either standalone mem or packet references
 	if (pck->filter_owns_mem && !(pck->info.flags & GF_PCK_CMD_MASK) ) {
 		assert(pck->pid);
 		assert(pck->pid->nb_shared_packets_out);
@@ -506,6 +653,8 @@ void gf_filter_packet_destroy(GF_FilterPacket *pck)
 			gf_filter_packet_destroy(pck->reference);
 		}
 		pck->reference = NULL;
+		if (pck->is_dangling)
+			pck->data = NULL;
 	}
 	/*this is a property reference packet, its destruction may happen at ANY time*/
 	if (is_ref_props_packet) {
@@ -517,7 +666,11 @@ void gf_filter_packet_destroy(GF_FilterPacket *pck)
 	} else if (is_filter_destroyed) {
 		if (!pck->filter_owns_mem && pck->data) gf_free(pck->data);
 		gf_free(pck);
-	} else if (pck->filter_owns_mem ) {
+	} else if (pck->is_dangling) {
+		if (pck->data) gf_free(pck->data);
+		gf_free(pck);
+	}
+	else if (pck->filter_owns_mem ) {
 		if (pid->filter && pid->filter->pcks_shared_reservoir) {
 			gf_fq_add(pid->filter->pcks_shared_reservoir, pck);
 		} else {
@@ -533,7 +686,7 @@ void gf_filter_packet_destroy(GF_FilterPacket *pck)
 	}
 }
 
-static Bool gf_filter_aggregate_packets(GF_FilterPidInst *dst)
+Bool gf_filter_aggregate_packets(GF_FilterPidInst *dst)
 {
 	u32 size=0, pos=0;
 	u64 byte_offset = 0;
@@ -548,13 +701,15 @@ static Bool gf_filter_aggregate_packets(GF_FilterPidInst *dst)
 	count=gf_list_count(dst->pck_reassembly);
 	//no packet to reaggregate
 	if (!count) return GF_FALSE;
+
+	dst->nb_reagg_pck++;
+
 	//single packet, update PID buffer and dispatch to packet queue
 	if (count==1) {
 		GF_FilterPacketInstance *pcki = gf_list_pop_back(dst->pck_reassembly);
 		safe_int_inc(&dst->filter->pending_packets);
 		if (pcki->pck->info.duration && pcki->pck->pid_props->timescale) {
-			s64 duration = ((u64)pcki->pck->info.duration) * 1000000;
-			duration /= pcki->pck->pid_props->timescale;
+			u64 duration = gf_timestamp_rescale(pcki->pck->info.duration, pcki->pck->pid_props->timescale, 1000000);
 			safe_int64_add(&dst->buffer_duration, duration);
 		}
 		pcki->pck->info.flags |= GF_PCKF_BLOCK_START | GF_PCKF_BLOCK_END;
@@ -637,8 +792,7 @@ static Bool gf_filter_aggregate_packets(GF_FilterPidInst *dst)
 			safe_int_inc(&dst->filter->pending_packets);
 
 			if (info.duration && pck->pid_props && pck->pid_props->timescale) {
-				s64 duration = ((u64) info.duration) * 1000000;
-				duration /= pck->pid_props->timescale;
+				u64 duration = gf_timestamp_rescale(info.duration, pck->pid_props->timescale, 1000000);
 				safe_int64_add(&dst->buffer_duration, duration);
 			}
 			//not continous set of bytes reaggregated
@@ -685,6 +839,7 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 	s64 duration=0;
 	u32 timescale=0;
 	GF_FilterClockType cktype;
+	Bool is_cmd_pck;
 #ifdef GPAC_MEMORY_TRACKING
 	u32 nb_allocs=0, nb_reallocs=0, prev_nb_allocs=0, prev_nb_reallocs=0;
 #endif
@@ -716,8 +871,10 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 		return GF_OK;
 	}
 
+	is_cmd_pck = (pck->info.flags & GF_PCK_CMD_MASK);
+
 	//special case for source filters (no input pids), mark as playing once we have a packet sent
-	if (!pid->filter->num_input_pids && !pid->initial_play_done && !pid->is_playing) {
+	if (!is_cmd_pck && !pid->filter->num_input_pids && !pid->initial_play_done && !pid->is_playing) {
 		pid->initial_play_done = GF_TRUE;
 		pid->is_playing = GF_TRUE;
 		pid->filter->nb_pids_playing++;
@@ -730,18 +887,23 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 
 	gf_rmt_begin(pck_send, GF_RMT_AGGREGATE);
 
+	cktype = ( pck->info.flags & GF_PCK_CKTYPE_MASK) >> GF_PCK_CKTYPE_POS;
+
 	//send from filter, update flags
 	if (from_filter) {
-		Bool is_cmd = (pck->info.flags & GF_PCK_CKTYPE_MASK) ? GF_TRUE : GF_FALSE;
-		if (! is_cmd ) {
+		Bool is_cmd = (pck->info.flags & GF_PCK_CMD_MASK) ? GF_TRUE : GF_FALSE;
+		//not a clock, flush any pending clock
+		if (!  (pck->info.flags & GF_PCK_CKTYPE_MASK) ) {
 			gf_filter_forward_clock(pck->pid->filter);
 		}
 		if ( (pck->info.flags & GF_PCK_CMD_MASK) == GF_PCK_CMD_PID_EOS) {
 			if (!pid->has_seen_eos) {
 				pid->has_seen_eos = GF_TRUE;
-				GF_LOG(GF_LOG_INFO, GF_LOG_FILTER, ("Filter %s PID %s EOS detected\n", pck->pid->filter->name, pck->pid->name));
+				GF_LOG(GF_LOG_DEBUG, GF_LOG_FILTER, ("Filter %s PID %s EOS detected\n", pck->pid->filter->name, pck->pid->name));
 			}
-		} else if (pid->has_seen_eos && !is_cmd) {
+		}
+		//reset eos only if not a command and not a clock signaling
+		else if (pid->has_seen_eos && !is_cmd && !cktype) {
 			pid->has_seen_eos = GF_FALSE;
 		}
 
@@ -754,13 +916,17 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 		//a new property map was created -  flag the packet; don't do this if first packet dispatched on pid
 		pck->info.flags &= ~GF_PCKF_PROPS_CHANGED;
 
-		if (!pid->request_property_map && !(pck->info.flags & GF_PCK_CMD_MASK) && (pid->nb_pck_sent || pid->props_changed_since_connect) ) {
+		if (!pid->request_property_map && !is_cmd_pck && (pid->nb_pck_sent || pid->props_changed_since_connect) ) {
 			GF_LOG(GF_LOG_DEBUG, GF_LOG_FILTER, ("Filter %s PID %s properties modified, marking packet\n", pck->pid->filter->name, pck->pid->name));
 
+			if (pid->filter->user_pid_props)
+				gf_filter_pid_set_args(pid->filter, pid);
+
 			pck->info.flags |= GF_PCKF_PROPS_CHANGED;
 		}
 		//any new pid_set_property after this packet will trigger a new property map
-		if (! (pck->info.flags & GF_PCK_CMD_MASK)) {
+		//note we don't reset if packet is a clock info so that next non-clock packet still has the discontinuity marker
+		if (! is_cmd_pck && !cktype) {
 			pid->request_property_map = GF_TRUE;
 			pid->props_changed_since_connect = GF_FALSE;
 		}
@@ -800,7 +966,7 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 	pck->src_filter = NULL;
 
 	assert(pck->pid);
-	if (! (pck->info.flags & GF_PCK_CMD_MASK) ) {
+	if (! is_cmd_pck ) {
 		pid->nb_pck_sent++;
 		if (pck->data_length) {
 			pid->filter->nb_pck_sent++;
@@ -808,14 +974,12 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 		} else if (pck->frame_ifce) {
 			pid->filter->nb_hw_pck_sent++;
 		}
-		if (pck->info.cts!=GF_FILTER_NO_TS) {
+		if (timescale && (pck->info.cts!=GF_FILTER_NO_TS)) {
 			pid->last_ts_sent.num = pck->info.cts;
-			pid->last_ts_sent.den = pck->pid_props->timescale;
+			pid->last_ts_sent.den = timescale;
 		}
 	}
 
-	cktype = ( pck->info.flags & GF_PCK_CKTYPE_MASK) >> GF_PCK_CKTYPE_POS;
-
 	if (cktype == GF_FILTER_CLOCK_PCR_DISC) {
 		pid->duration_init = GF_FALSE;
 		pid->min_pck_cts = pid->max_pck_cts = 0;
@@ -901,6 +1065,7 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 			if (!pid->min_pck_duration) pid->min_pck_duration = (u32) duration;
 			else if ((u32) duration < pid->min_pck_duration) pid->min_pck_duration = (u32) duration;
 		}
+
 		if (!pck->info.duration && pid->min_pck_duration)
 			pck->info.duration = (u32) duration;
 
@@ -908,6 +1073,7 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 		if ((s32) pck->info.duration < 0) {
 			pck->info.duration = 0;
 		}
+		pid->last_pck_dur = pck->info.duration;
 
 #ifndef GPAC_DISABLE_LOG
 		if (gf_log_tool_level_on(GF_LOG_FILTER, GF_LOG_DEBUG)) {
@@ -944,13 +1110,16 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 
 	assert(pck->pid);
 	count = pck->pid->num_destinations;
+	//check if processing this packet must be done on main thread (OpenGL interface or source filter asked fo this) 
+	Bool force_main_thread = (pck->info.flags & GF_PCKF_FORCE_MAIN) ? GF_TRUE : GF_FALSE;
+
 	for (i=0; i<count; i++) {
 		Bool post_task=GF_FALSE;
 		GF_FilterPacketInstance *inst;
 		GF_FilterPidInst *dst = gf_list_get(pck->pid->destinations, i);
-		if (!dst->filter || dst->filter->finalized || dst->filter->removed || !dst->filter->freg->process) continue;
+		if (!dst->filter || dst->filter->finalized || (dst->filter->removed==1) || !dst->filter->freg->process) continue;
 
-		if (dst->discard_inputs) {
+		if (dst->discard_inputs==1) {
 			//in discard input mode, we drop all input packets but trigger reconfigure as they happen
 			if ((pck->info.flags & GF_PCKF_PROPS_CHANGED) && (dst->props != pck->pid_props)) {
 				//unassign old property list and set the new one
@@ -972,11 +1141,14 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 				//in which a previously blacklisted filter (failing (re)configure for previous state) could
 				//now work, eg moving from formatA to formatB then back to formatA
 				gf_list_reset(dst->filter->blacklisted);
+				dst->discard_inputs = 2;
 				//and post a reconfigure task
-				gf_fs_post_task(dst->filter->session, gf_filter_pid_reconfigure_task, dst->filter, dst->pid, "pidinst_reconfigure", NULL);
+				gf_fs_post_task(dst->filter->session, gf_filter_pid_reconfigure_task_discard, dst->filter, (GF_FilterPid *)dst, "pidinst_reconfigure", NULL);
+				//keep packets, they will be trashed if we are still in discard when executing gf_filter_pid_reconfigure_task_discard
+			} else {
+				nb_discard++;
+				continue;
 			}
-			nb_discard++;
-			continue;
 		}
 
 		inst = gf_fq_pop(pck->pid->filter->pcks_inst_reservoir);
@@ -988,9 +1160,11 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 		inst->pid = dst;
 		inst->pid_props_change_done = 0;
 		inst->pid_info_change_done = 0;
-		//if packet is an openGL interface, force scheduling on main thread for the destination
-		if (pck->frame_ifce&&pck->frame_ifce->get_gl_texture)
-			dst->filter->main_thread_forced = GF_TRUE;
+
+		//if packet is forcing main thread processing increase destination filter main_thread
+		if (force_main_thread) {
+			safe_int_inc(&dst->filter->nb_main_thread_forced);
+		}
 
 		if ((inst->pck->info.flags & GF_PCK_CMD_MASK) == GF_PCK_CMD_PID_EOS)  {
 			safe_int_inc(&inst->pid->nb_eos_signaled);
@@ -1034,14 +1208,14 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 				//single block packet, direct dispatch in packet queue (aggregation done before)
 				else {
 					assert(dst->last_block_ended);
+					if (!is_cmd_pck)
+						dst->nb_reagg_pck++;
 
 					if (pck->info.duration && timescale) {
-						duration = ((u64)pck->info.duration) * 1000000;
-						duration /= timescale;
+						duration = gf_timestamp_rescale(pck->info.duration, timescale, 1000000);
 						safe_int64_add(&dst->buffer_duration, duration);
 					}
 					inst->pck->info.flags |= GF_PCKF_BLOCK_START;
-
 					safe_int_inc(&dst->filter->pending_packets);
 					gf_fq_add(dst->packets, inst);
 				}
@@ -1059,7 +1233,12 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 				if (inst->pck->filter_owns_mem) {
 					u8 *data;
 					u32 alloc_size;
-					inst->pck = gf_filter_pck_new_alloc_internal(pck->pid, pck->data_length, &data, GF_TRUE);
+					inst->pck = gf_filter_pck_new_alloc_internal(pck->pid, pck->data_length, &data);
+					if (!inst->pck) {
+						GF_LOG(GF_LOG_ERROR, GF_LOG_FILTER, ("Filter %s: failed to allocate new packet\n", pid->filter->name));
+						continue;
+					}
+
 					alloc_size = inst->pck->alloc_size;
 					memcpy(inst->pck, pck, sizeof(GF_FilterPacket));
 					inst->pck->pck = inst->pck;
@@ -1114,11 +1293,9 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 			}
 
 			if (duration && timescale) {
-				duration *= 1000000;
-				duration /= timescale;
+				duration = gf_timestamp_rescale(duration, timescale, 1000000);
 				safe_int64_add(&dst->buffer_duration, duration);
 			}
-
 			safe_int_inc(&dst->filter->pending_packets);
 //
 			gf_fq_add(dst->packets, inst);
@@ -1170,6 +1347,11 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 GF_EXPORT
 GF_Err gf_filter_pck_send(GF_FilterPacket *pck)
 {
+	//dangling packet
+	if (pck->is_dangling) {
+		gf_filter_pck_discard(pck);
+		return GF_OK;
+	}
 	assert(pck->pid);
 	return gf_filter_pck_send_internal(pck, GF_TRUE);
 }
@@ -1182,9 +1364,23 @@ GF_Err gf_filter_pck_ref(GF_FilterPacket **pck)
 
 	(*pck) = (*pck)->pck;
 	safe_int_inc(& (*pck)->reference_count);
+	//keep track of number of ref packets for this pid at filter level
+	safe_int_inc(& (*pck)->pid->filter->nb_ref_packets);
 	return GF_OK;
 }
 
+GF_EXPORT
+GF_FilterPacket *gf_filter_pck_ref_ex(GF_FilterPacket *pck)
+{
+	GF_FilterPacket *ref_pck;
+	if (! pck ) return NULL;
+	ref_pck = pck->pck;
+	safe_int_inc(& ref_pck->reference_count);
+	//keep track of number of ref packets for this pid at filter level
+	safe_int_inc(& ref_pck->pid->filter->nb_ref_packets);
+	return ref_pck;
+}
+
 GF_EXPORT
 GF_Err gf_filter_pck_ref_props(GF_FilterPacket **pck)
 {
@@ -1232,6 +1428,11 @@ void gf_filter_pck_unref(GF_FilterPacket *pck)
 	pck=pck->pck;
 
 	assert(pck->reference_count);
+	//decrease number of ref packets for this pid at filter level if not a props reference
+	if (! (pck->info.flags & GF_PCKF_PROPS_REFERENCE)) {
+		assert(pck->pid->filter->nb_ref_packets);
+		safe_int_dec(&pck->pid->filter->nb_ref_packets);
+	}
 	if (safe_int_dec(&pck->reference_count) == 0) {
 		gf_filter_packet_destroy(pck);
 	}
@@ -1289,6 +1490,15 @@ GF_Err gf_filter_pck_set_property_dyn(GF_FilterPacket *pck, char *name, const GF
 	return gf_filter_pck_set_property_full(pck, 0, NULL, name, value);
 }
 
+GF_EXPORT
+Bool gf_filter_pck_has_properties(GF_FilterPacket *pck)
+{
+	//get true packet pointer
+	pck = pck->pck;
+	if (!pck->props) return GF_FALSE;
+	return GF_TRUE;
+}
+
 GF_EXPORT
 const GF_PropertyValue *gf_filter_pck_get_property(GF_FilterPacket *pck, u32 prop_4cc)
 {
