@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2017-2018
+ *			Copyright (c) Telecom ParisTech 2017-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / filters sub-project
@@ -58,6 +58,10 @@ struct __gf_prop_entry
 	u32 alloc_size;
 };
 
+//we use the same value internally but with reverse meaning
+#define GF_FS_FLAG_IMPLICIT_MODE	GF_FS_FLAG_NO_IMPLICIT
+
+
 #ifndef GF_PROPS_HASHTABLE_SIZE
 #define GF_PROPS_HASHTABLE_SIZE 0
 #endif
@@ -72,7 +76,7 @@ typedef struct
 	GF_List *properties;
 #endif
 	volatile u32 reference_count;
-	//number of references hold by packet references - since these may be destroyed at the end of the refering filter
+	//number of references hold by packet references - since these may be destroyed at the end of the referring filter
 	//the pid might be dead. This is only used for pid props maps
 	volatile u32 pckrefs_reference_count;
 	GF_FilterSession *session;
@@ -139,6 +143,32 @@ typedef struct __gf_filter_pck_inst
 } GF_FilterPacketInstance;
 
 
+/*URI relocators are used for containers like zip or ISO FF with file items. The relocator
+is in charge of translating the URI, potentially extracting the associated resource and sending
+back the new (local or not) URI. Only the interface is defined, URI translators are free to derive from them
+
+relocate a URI - if NULL is returned, this relocator is not concerned with the URI
+otherwise returns the translated URI
+*/
+
+#define GF_FS_URI_RELOCATOR	\
+	Bool (*relocate_uri)(void *__self, const char *parent_uri, const char *uri, char *out_relocated_uri, char *out_localized_uri);		\
+
+typedef struct __gf_uri_relocator GF_URIRelocator;
+
+struct __gf_uri_relocator
+{
+	GF_FS_URI_RELOCATOR
+};
+
+typedef struct
+{
+	GF_FS_URI_RELOCATOR
+	GF_FilterSession *sess;
+	char *szAbsRelocatedPath;
+} GF_FSLocales;
+
+
 //packet flags
 enum
 {
@@ -165,7 +195,9 @@ enum
 	//2 bits for crypt type
 	GF_PCK_CMD_POS = 13,
 	GF_PCK_CMD_MASK = 0x3 << GF_PCK_CMD_POS,
-	//RESERVED bits [9,12]
+	GF_PCKF_FORCE_MAIN = 1<<12,
+	//RESERVED bits [8,11]
+
 	//2 bits for is_leading
 	GF_PCK_ISLEADING_POS = 6,
 	GF_PCK_ISLEADING_MASK = 0x3 << GF_PCK_ISLEADING_POS,
@@ -221,9 +253,6 @@ struct __gf_filter_pck
 
 	//for allocated memory packets
 	u32 alloc_size;
-	//for shared memory packets: 0: cloned mem, 1: read/write mem from source filter, 2: read-only mem from filter
-	//note that packets with frame_ifce are always considered as read-only memory
-	u32 filter_owns_mem;
 	gf_fsess_packet_destructor destructor;
 	//for packet reference  packets (sharing data from other packets)
 	struct __gf_filter_pck *reference;
@@ -234,6 +263,12 @@ struct __gf_filter_pck
 	GF_PropertyMap *props;
 	//pid properties applying to this packet
 	GF_PropertyMap *pid_props;
+
+	//for shared memory packets: 0: cloned mem, 1: read/write mem from source filter, 2: read-only mem from filter
+	//note that packets with frame_ifce are always considered as read-only memory
+	u8 filter_owns_mem;
+	u8 is_dangling;
+
 };
 
 /*!
@@ -250,6 +285,7 @@ struct __gf_fs_task
 	Bool requeue_request;
 	Bool can_swap;
 	Bool blocking;
+	Bool force_main;
 
 	u64 schedule_next_time;
 
@@ -262,7 +298,10 @@ struct __gf_fs_task
 };
 
 void gf_fs_post_task(GF_FilterSession *fsess, gf_fs_task_callback fun, GF_Filter *filter, GF_FilterPid *pid, const char *log_name, void *udta);
-void gf_fs_post_task_ex(GF_FilterSession *fsess, gf_fs_task_callback task_fun, GF_Filter *filter, GF_FilterPid *pid, const char *log_name, void *udta, Bool requires_main_thread, Bool force_direct_call);
+/* extended version of gf_fs_post_task
+force_direct_call shall only be true for gf_filter_process_task
+*/
+void gf_fs_post_task_ex(GF_FilterSession *fsess, gf_fs_task_callback task_fun, GF_Filter *filter, GF_FilterPid *pid, const char *log_name, void *udta, Bool is_configure, Bool force_main_thread, Bool force_direct_call);
 
 void gf_filter_pid_send_event_downstream(GF_FSTask *task);
 
@@ -287,14 +326,21 @@ typedef struct __gf_fs_thread
 
 } GF_SessionThread;
 
+typedef enum {
+	GF_ARGTYPE_LOCAL = 0, //:arg syntax
+	GF_ARGTYPE_GLOBAL, //--arg syntax
+	GF_ARGTYPE_META, //old -+arg syntax
+	GF_ARGTYPE_META_REPORTING
+} GF_FSArgItemType;
+
 typedef struct
 {
 	char *argname;
-	u32 type;
+	GF_FSArgItemType type;
 	Bool found;
 } GF_FSArgItem;
 
-void gf_fs_push_arg(GF_FilterSession *session, const char *szArg, Bool was_found, u32 type);
+void gf_fs_push_arg(GF_FilterSession *session, const char *szArg, Bool was_found, GF_FSArgItemType type, GF_Filter *meta_filter);
 
 enum
 {
@@ -303,6 +349,9 @@ enum
 	GF_FS_NOBLOCK
 };
 
+//#define GF_FS_ENABLE_LOCALES
+
+
 struct __gf_filter_session
 {
 	u32 flags;
@@ -310,7 +359,11 @@ struct __gf_filter_session
 	Bool direct_mode;
 	volatile u32 tasks_in_process;
 	Bool requires_solved_graph;
-	Bool no_main_thread;
+	//non blicking session mode:
+	//0: session is blocking
+	//1: session is non-blocking and first call to gf_fs_run (extra threads not started)
+	//2: session is non-blocking and not first call to gf_fs_run (extra threads started)
+	u32 non_blocking;
 
 	GF_List *registry;
 	GF_List *filters;
@@ -428,6 +481,15 @@ struct __gf_filter_session
 	GF_List *jstasks;
 	struct __jsfs_task *new_f_task, *del_f_task, *on_evt_task;
 #endif
+
+	gf_fs_on_filter_creation on_filter_create_destroy;
+	void *rt_udta;
+	Bool force_main_thread_tasks;
+
+#ifdef GF_FS_ENABLE_LOCALES
+	GF_List *uri_relocators;
+	GF_FSLocales locales;
+#endif
 };
 
 #ifdef GPAC_HAS_QJS
@@ -460,6 +522,16 @@ typedef enum
 	GF_FILTER_ARG_EXPLICIT_SINK,
 } GF_FilterArgType;
 
+typedef enum
+{
+	//filter cannot be cloned
+	GF_FILTER_NO_CLONE=0,
+	//filter can be cloned
+	GF_FILTER_CLONE,
+	//filter may be cloned in implicit link session mode only
+	GF_FILTER_CLONE_PROBE,
+} GF_FilterCloneType;
+
 struct __gf_filter
 {
 	const GF_FilterRegister *freg;
@@ -475,13 +547,17 @@ struct __gf_filter
 	//but dynamic_source_ids still contains the unresolved pattern (eg ServiceID=*)
 	char *dynamic_source_ids;
 
+	char *restricted_source_id;
+	
 	//parent media session
 	GF_FilterSession *session;
 
 	//indicates the max number of additional input PIDs - muxers and scalable filters typically set this to (u32) -1
 	u32 max_extra_pids;
 
-	void (*on_setup_error)(GF_Filter *f, void *on_setup_error_udta, GF_Err e);
+	u32 subsession_id, subsource_id;
+
+	Bool (*on_setup_error)(GF_Filter *f, void *on_setup_error_udta, GF_Err e);
 	void *on_setup_error_udta;
 	GF_Filter *on_setup_error_filter;
 
@@ -493,7 +569,9 @@ struct __gf_filter
 	char *src_args;
 	//allocated argument string of the destingation filter for filters dynamically loaded, if any
 	char *dst_args;
-
+	//filter tag
+	char *tag;
+	
 	//tasks pending for this filter. The first task in this list is also present in the filter session
 	//task list in order to avoid locking the main task list with a mutex
 	GF_FilterQueue *tasks;
@@ -513,7 +591,9 @@ struct __gf_filter
 	Bool in_process_callback;
 	Bool no_probe;
 	Bool no_inputs;
-	
+	Bool is_blocking_source;
+	Bool force_demux;
+
 	s32 nb_pids_playing;
 
 	//list of pids connected to this filter
@@ -546,11 +626,13 @@ struct __gf_filter
 	volatile u32 in_pid_connection_pending;
 	volatile u32 out_pid_connection_pending;
 	volatile u32 pending_packets;
+	volatile u32 nb_ref_packets;
 
 	volatile u32 stream_reset_pending;
 	volatile u32 num_events_queued;
 	volatile u32 detach_pid_tasks_pending;
 	volatile u32 nb_shared_packets_out;
+	volatile u32 abort_pending;
 	GF_List *postponed_packets;
 
 	//list of blacklisted filtered registries
@@ -568,6 +650,8 @@ struct __gf_filter
 	GF_List *destination_filters;
 	//list of potential destination for which we need to perform link resolution
 	GF_List *destination_links;
+	//temp list of input PIDs during graph resolution, added during pid_init task and removed at pid_configure task
+	GF_List *temp_input_pids;
 
 	//statistics
 	//number of tasks executed by this filter
@@ -607,11 +691,12 @@ struct __gf_filter
 	Bool finalized;
 	//filter is scheduled for removal: any filter connected to this filter will
 	//not be checked for graph resolution - destroy has not yet been posted
-	Bool removed;
+	//if value is 2, packets are still dispatched to this PID instance (pid reconfig)
+	u32 removed;
 	//setup has been notified
 	Bool setup_notified;
-	//filter loaded to solve a filter chain
-	Bool dynamic_filter;
+	//filter loaded to solve a filter chain - special value 2 is for dummy reframer when forcing demux
+	u32 dynamic_filter;
 	//filter block EOS queries
 	Bool block_eos;
 	//set when one input pid of the filter has been marked for removal through gf_filter_remove_src
@@ -621,7 +706,9 @@ struct __gf_filter
 	//2 means temporary sticky, used when reconfiguring filter chain
 	u32 sticky;
 	//explicitly loaded filters are usually not cloned, except if this flag is set
-	Bool clonable;
+	GF_FilterCloneType clonable;
+	//set to true during pid link resolution for filters accepting a single pid
+	Bool in_link_resolution;
 	//one of the output PID needs reconfiguration
 	volatile u32 nb_caps_renegociate;
 
@@ -666,6 +753,8 @@ struct __gf_filter
 	//for encoder filters, set to the corresponding stream type - used to discard filters during the resolution
 	u32 encoder_stream_type;
 
+	Bool act_as_sink;
+	Bool require_source_id;
 #ifndef GPAC_DISABLE_REMOTERY
 	rmtU32 rmt_hash;
 #endif
@@ -686,7 +775,7 @@ struct __gf_filter
 
 	GF_Err in_connect_err;
 
-	Bool main_thread_forced;
+	volatile u32 nb_main_thread_forced;
 	Bool no_dst_arg_inherit;
 	GF_List *source_filters;
 
@@ -701,14 +790,24 @@ struct __gf_filter
 
 	Bool event_target;
 
+	u64 last_schedule_task_time;
+
+	//set to NULL, or to the only source filter for this filter
+	//this is a helper for the graph resolver to avoid browing all input pids when checking
+	//for cycles or fetching last defined ID.
+	//typically helps for tiling case with hundreds of tiles
+	GF_Filter *single_source;
+
 #ifdef GPAC_HAS_QJS
 	char *iname;
 	JSValue jsval;
 #endif
+	//for external bindings
+	void *rt_udta;
 };
 
 GF_Filter *gf_filter_new(GF_FilterSession *fsess, const GF_FilterRegister *freg, const char *args, const char *dst_args, GF_FilterArgType arg_type, GF_Err *err, GF_Filter *multi_sink_target, Bool dynamic_filter);
-GF_Filter *gf_filter_clone(GF_Filter *filter);
+GF_Filter *gf_filter_clone(GF_Filter *filter, GF_Filter *source_filter);
 void gf_filter_del(GF_Filter *filter);
 
 Bool gf_filter_swap_source_register(GF_Filter *filter);
@@ -758,12 +857,12 @@ struct __gf_filter_pid_inst
 	Bool last_block_ended;
 	Bool first_block_started;
 	//set during play/stop/reset phases
-	Bool discard_packets;
+	volatile u32 discard_packets;
 
 	Bool force_reconfig;
 
 	//set by filter
-	Bool discard_inputs;
+	u32 discard_inputs;
 
 	//amount of media data in us in the packet queue - concurrent inc/dec
 	volatile s64 buffer_duration;
@@ -771,17 +870,20 @@ struct __gf_filter_pid_inst
 	volatile s32 detach_pending;
 
 	void *udta;
+	u32 udta_flags;
 
 	//statistics per pid instance
 	u64 last_pck_fetch_time;
 	u64 stats_start_ts, stats_start_us;
 	u32 cur_bit_size, avg_bit_rate, max_bit_rate, avg_process_rate, max_process_rate;
-	u32 nb_processed, nb_sap_processed;
+	u32 nb_processed, nb_sap_processed, nb_reagg_pck;
 	//all times in us
 	u64 total_process_time, total_sap_process_time;
 	u64 max_process_time, max_sap_process_time;
 	u64 first_frame_time;
 	Bool is_end_of_stream;
+	Bool is_playing, is_paused;
+	
 	volatile u32 nb_eos_signaled;
 
 	Bool is_encoder_input;
@@ -805,6 +907,8 @@ struct __gf_filter_pid_inst
 
 	GF_Fraction64 last_ts_drop;
 
+	u64 last_buf_query_clock;
+	u64 last_buf_query_dur;
 };
 
 struct __gf_filter_pid
@@ -856,8 +960,9 @@ struct __gf_filter_pid
 
 	Bool duration_init;
 	u64 last_pck_dts, last_pck_cts, min_pck_cts, max_pck_cts;
-	u32 min_pck_duration, nb_unreliable_dts;
+	u32 min_pck_duration, nb_unreliable_dts, last_pck_dur;
 	Bool recompute_dts;
+	Bool ignore_blocking;
 
 	u32 nb_pck_sent;
 	//1000x speed value
@@ -868,9 +973,11 @@ struct __gf_filter_pid
 	Bool initial_play_done;
 	Bool is_playing;
 	void *udta;
+	u32 udta_flags;
 
 	GF_PropertyMap *caps_negociate;
-	GF_FilterPidInst *caps_negociate_pidi;
+	Bool caps_negociate_direct;
+	GF_List *caps_negociate_pidi_list;
 	GF_List *adapters_blacklist;
 	GF_Filter *caps_dst_filter;
 
@@ -896,6 +1003,7 @@ void gf_filter_pid_post_connect_task(GF_Filter *filter, GF_FilterPid *pid);
 
 /*internal tasks definitions*/
 void gf_filter_pid_reconfigure_task(GF_FSTask *task);
+void gf_filter_pid_reconfigure_task_discard(GF_FSTask *task);
 void gf_filter_update_arg_task(GF_FSTask *task);
 void gf_filter_pid_disconnect_task(GF_FSTask *task);
 void gf_filter_remove_task(GF_FSTask *task);
@@ -916,7 +1024,7 @@ typedef struct
 #define CAP_MATCH_LOADED_INPUT_ONLY		1
 #define CAP_MATCH_LOADED_OUTPUT_ONLY	1<<1
 
-u32 gf_filter_caps_to_caps_match(const GF_FilterRegister *src, u32 src_bundle_idx, const GF_FilterRegister *dst, GF_Filter *dst_filter, u32 *dst_bundle_idx, s32 for_dst_bundle, u32 *loaded_filter_flags, GF_CapsBundleStore *capstore);
+u32 gf_filter_caps_to_caps_match(const GF_FilterRegister *src, u32 src_bundle_idx, const GF_FilterRegister *dst, GF_Filter *dst_filter, u32 *dst_bundle_idx, u32 for_dst_bundle, u32 *loaded_filter_flags, GF_CapsBundleStore *capstore);
 Bool gf_filter_has_out_caps(const GF_FilterCapability *caps, u32 nb_caps);
 Bool gf_filter_has_in_caps(const GF_FilterCapability *caps, u32 nb_caps);
 
@@ -928,7 +1036,7 @@ void gf_filter_renegociate_output_dst(GF_FilterPid *pid, GF_Filter *filter, GF_F
 GF_Filter *gf_filter_pid_resolve_link(GF_FilterPid *pid, GF_Filter *dst, Bool *filter_reassigned);
 GF_Filter *gf_filter_pid_resolve_link_check_loaded(GF_FilterPid *pid, GF_Filter *dst, Bool *filter_reassigned, GF_List *skip_if_in_filter_list, Bool *skipped);
 
-GF_Filter *gf_filter_pid_resolve_link_for_caps(GF_FilterPid *pid, GF_Filter *dst);
+GF_Filter *gf_filter_pid_resolve_link_for_caps(GF_FilterPid *pid, GF_Filter *dst, Bool check_reconfig_only);
 u32 gf_filter_pid_resolve_link_length(GF_FilterPid *pid, GF_Filter *dst);
 
 Bool gf_filter_pid_caps_match(GF_FilterPid *src_pid, const GF_FilterRegister *freg, GF_Filter *filter_inst, u8 *priority, u32 *dst_bundle_idx, GF_Filter *dst_filter, s32 for_bundle_idx);
@@ -954,7 +1062,9 @@ const GF_PropertyEntry *gf_filter_pid_get_property_entry_str(GF_FilterPid *pid,
 const GF_PropertyValue *gf_filter_pid_get_property_first(GF_FilterPid *pid, u32 prop_4cc);
 const GF_PropertyValue *gf_filter_pid_get_property_str_first(GF_FilterPid *pid, const char *prop_name);
 
+void gf_filter_pid_set_args(GF_Filter *filter, GF_FilterPid *pid);
 
+Bool gf_filter_aggregate_packets(GF_FilterPidInst *dst);
 enum
 {
 	EDGE_STATUS_NONE=0,
@@ -1009,6 +1119,11 @@ void gf_filter_renegociate_output_task(GF_FSTask *task);
 
 void gf_fs_unload_script(GF_FilterSession *fs, void *js_ctx);
 
+
+Bool gf_fs_check_filter_register_cap_ex(const GF_FilterRegister *f_reg, u32 incode, GF_PropertyValue *cap_input, u32 outcode, GF_PropertyValue *cap_output, Bool exact_match_only, Bool out_cap_excluded);
+
+Bool gf_filter_update_arg_apply(GF_Filter *filter, const char *arg_name, const char *arg_value, Bool is_sync_call);
+
 #endif //_GF_FILTER_SESSION_H_
 
 
