@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2020
+ *			Copyright (c) Telecom ParisTech 2020-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / compressed bitstream metadata rewrite filter
@@ -36,22 +36,25 @@ struct _bsrw_pid_ctx
 	GF_FilterPid *ipid, *opid;
 	u32 codec_id;
 	Bool reconfigure;
-	s32 clrp, txchar, mxcoef;
 	GF_Err (*rewrite_pid_config)(GF_BSRWCtx *ctx, BSRWPid *pctx);
 	GF_Err (*rewrite_packet)(GF_BSRWCtx *ctx, BSRWPid *pctx, GF_FilterPacket *pck);
 
-	s32 prev_clrp, prev_txchar, prev_mxcoef, prev_sar;
+	s32 prev_cprim, prev_ctfc, prev_cmx, prev_sar;
 
 	u32 nalu_size_length;
+
+#ifndef GPAC_DISABLE_AV_PARSERS
+	GF_VUIInfo vui;
+#endif
+	Bool rewrite_vui;
 };
 
 struct _bsrw_ctx
 {
 	GF_Fraction sar;
 	s32 m4vpl, prof, lev, pcomp, pidc, pspace, gpcflags;
-	char *clrp, *txchar, *mxcoef;
-	Bool remsei;
-
+	s32 cprim, ctfc, cmx, vidfmt;
+	Bool rmsei, fullrange, novsi;
 
 	GF_List *pids;
 	Bool reconfigure;
@@ -62,7 +65,6 @@ static GF_Err none_rewrite_packet(GF_BSRWCtx *ctx, BSRWPid *pctx, GF_FilterPacke
 	return gf_filter_pck_forward(pck, pctx->opid);
 }
 
-
 static GF_Err m4v_rewrite_pid_config(GF_BSRWCtx *ctx, BSRWPid *pctx)
 {
 	GF_Err e;
@@ -81,7 +83,11 @@ static GF_Err m4v_rewrite_pid_config(GF_BSRWCtx *ctx, BSRWPid *pctx)
 	memcpy(dsi, prop->value.data.ptr, sizeof(u8) * dsi_size);
 
 	if (ctx->sar.num && ctx->sar.den) {
+#ifndef GPAC_DISABLE_AV_PARSERS
 		e = gf_m4v_rewrite_par(&dsi, &dsi_size, ctx->sar.num, ctx->sar.den);
+#else
+		e = GF_NOT_SUPPORTED;
+#endif
 		if (e) {
 			gf_free(dsi);
 			return e;
@@ -89,8 +95,12 @@ static GF_Err m4v_rewrite_pid_config(GF_BSRWCtx *ctx, BSRWPid *pctx)
 		gf_filter_pid_set_property(pctx->opid, GF_PROP_PID_SAR, &PROP_FRAC(ctx->sar) );
 	}
 	if (ctx->m4vpl>=0) {
+#ifndef GPAC_DISABLE_AV_PARSERS
 		gf_m4v_rewrite_pl(&dsi, &dsi_size, (u32) ctx->m4vpl);
 		gf_filter_pid_set_property(pctx->opid, GF_PROP_PID_PROFILE_LEVEL, &PROP_UINT(ctx->m4vpl) );
+#else
+		return GF_NOT_SUPPORTED;
+#endif
 	}
 
 	gf_filter_pid_set_property(pctx->opid, GF_PROP_PID_DECODER_CONFIG, &PROP_DATA_NO_COPY(dsi, dsi_size) );
@@ -98,8 +108,9 @@ static GF_Err m4v_rewrite_pid_config(GF_BSRWCtx *ctx, BSRWPid *pctx)
 	return GF_OK;
 }
 
-static GF_Err avc_rewrite_packet(GF_BSRWCtx *ctx, BSRWPid *pctx, GF_FilterPacket *pck)
+static GF_Err nalu_rewrite_packet(GF_BSRWCtx *ctx, BSRWPid *pctx, GF_FilterPacket *pck, u32 codec_type)
 {
+	Bool is_sei;
 	u32 size, pck_size, final_size;
 	GF_FilterPacket *dst;
 	u8 *output;
@@ -107,6 +118,7 @@ static GF_Err avc_rewrite_packet(GF_BSRWCtx *ctx, BSRWPid *pctx, GF_FilterPacket
 	if (!data)
 		return gf_filter_pck_forward(pck, pctx->opid);
 
+
 	final_size = 0;
 	size=0;
 	while (size<pck_size) {
@@ -120,9 +132,26 @@ static GF_Err avc_rewrite_packet(GF_BSRWCtx *ctx, BSRWPid *pctx, GF_FilterPacket
 			if (!nal_hdr) break;
 			nal_size<<=8;
 		}
-		nal_type = data[size];
-		nal_type = nal_type & 0x1F;
-		if (nal_type != GF_AVC_NALU_SEI) {
+		is_sei = GF_FALSE;
+		//AVC
+		if (codec_type==0) {
+			nal_type = data[size] & 0x1F;
+			if (nal_type == GF_AVC_NALU_SEI) is_sei = GF_TRUE;
+		}
+		//HEVC
+		else if (codec_type==1) {
+			nal_type = (data[size] & 0x7E) >> 1;
+			if ((nal_type == GF_HEVC_NALU_SEI_PREFIX) || (nal_type == GF_HEVC_NALU_SEI_SUFFIX))
+				is_sei = GF_TRUE;
+		}
+		//VVC
+		else if (codec_type==2) {
+			nal_type = data[size+1] >> 3;
+			if ((nal_type == GF_VVC_NALU_SEI_PREFIX) || (nal_type == GF_VVC_NALU_SEI_SUFFIX))
+				is_sei = GF_TRUE;
+		}
+
+		if (!is_sei) {
 			final_size += nal_size+pctx->nalu_size_length;
 		}
 		size += nal_size;
@@ -131,6 +160,8 @@ static GF_Err avc_rewrite_packet(GF_BSRWCtx *ctx, BSRWPid *pctx, GF_FilterPacket
 		return gf_filter_pck_forward(pck, pctx->opid);
 
 	dst = gf_filter_pck_new_alloc(pctx->opid, final_size, &output);
+	if (!dst) return GF_OUT_OF_MEM;
+
 	gf_filter_pck_merge_properties(pck, dst);
 
 	size=0;
@@ -145,9 +176,27 @@ static GF_Err avc_rewrite_packet(GF_BSRWCtx *ctx, BSRWPid *pctx, GF_FilterPacket
 			if (!nal_hdr) break;
 			nal_size<<=8;
 		}
-		nal_type = data[size];
-		nal_type = nal_type & 0x1F;
-		if (nal_type == GF_AVC_NALU_SEI) {
+		is_sei = GF_FALSE;
+
+		//AVC
+		if (codec_type==0) {
+			nal_type = data[size] & 0x1F;
+			if (nal_type == GF_AVC_NALU_SEI) is_sei = GF_TRUE;
+		}
+		//HEVC
+		else if (codec_type==1) {
+			nal_type = (data[size] & 0x7E) >> 1;
+			if ((nal_type == GF_HEVC_NALU_SEI_PREFIX) || (nal_type == GF_HEVC_NALU_SEI_SUFFIX))
+				is_sei = GF_TRUE;
+		}
+		//VVC
+		else if (codec_type==2) {
+			nal_type = data[size+1] >> 3;
+			if ((nal_type == GF_VVC_NALU_SEI_PREFIX) || (nal_type == GF_VVC_NALU_SEI_SUFFIX))
+				is_sei = GF_TRUE;
+		}
+
+		if (is_sei) {
 			size += nal_size;
 			continue;
 		}
@@ -158,6 +207,39 @@ static GF_Err avc_rewrite_packet(GF_BSRWCtx *ctx, BSRWPid *pctx, GF_FilterPacket
 	return gf_filter_pck_send(dst);
 }
 
+static GF_Err avc_rewrite_packet(GF_BSRWCtx *ctx, BSRWPid *pctx, GF_FilterPacket *pck)
+{
+	return nalu_rewrite_packet(ctx, pctx, pck, 0);
+}
+static GF_Err hevc_rewrite_packet(GF_BSRWCtx *ctx, BSRWPid *pctx, GF_FilterPacket *pck)
+{
+	return nalu_rewrite_packet(ctx, pctx, pck, 1);
+}
+static GF_Err vvc_rewrite_packet(GF_BSRWCtx *ctx, BSRWPid *pctx, GF_FilterPacket *pck)
+{
+	return nalu_rewrite_packet(ctx, pctx, pck, 2);
+}
+
+#ifndef GPAC_DISABLE_AV_PARSERS
+static void update_props(BSRWPid *pctx, GF_VUIInfo *vui)
+{
+	if ((vui->ar_num>0) && (vui->ar_den>0))
+		gf_filter_pid_set_property(pctx->opid, GF_PROP_PID_SAR, &PROP_FRAC_INT(vui->ar_num, vui->ar_den) );
+
+	if (vui->fullrange>0)
+		gf_filter_pid_set_property(pctx->opid, GF_PROP_PID_COLR_RANGE, &PROP_BOOL(vui->fullrange) );
+
+	if (!vui->remove_video_info) {
+		if (vui->color_prim>=0)
+			gf_filter_pid_set_property(pctx->opid, GF_PROP_PID_COLR_PRIMARIES, &PROP_UINT(vui->color_prim) );
+		if (vui->color_tfc>=0)
+			gf_filter_pid_set_property(pctx->opid, GF_PROP_PID_COLR_TRANSFER, &PROP_UINT(vui->color_tfc) );
+		if (vui->color_matrix>=0)
+			gf_filter_pid_set_property(pctx->opid, GF_PROP_PID_COLR_MX, &PROP_UINT(vui->color_matrix) );
+	}
+}
+#endif /*GPAC_DISABLE_AV_PARSERS*/
+
 static GF_Err avc_rewrite_pid_config(GF_BSRWCtx *ctx, BSRWPid *pctx)
 {
 	GF_AVCConfig *avcc;
@@ -171,17 +253,25 @@ static GF_Err avc_rewrite_pid_config(GF_BSRWCtx *ctx, BSRWPid *pctx)
 	pctx->reconfigure = GF_FALSE;
 
 	avcc = gf_odf_avc_cfg_read(prop->value.data.ptr, prop->value.data.size);
-	if (ctx->sar.num && ctx->sar.den) {
-		e = gf_media_avc_change_par(avcc, ctx->sar.num, ctx->sar.den);
-		if (!e)
-			gf_filter_pid_set_property(pctx->opid, GF_PROP_PID_SAR, &PROP_FRAC(ctx->sar) );
+
+	if (pctx->rewrite_vui) {
+#ifndef GPAC_DISABLE_AV_PARSERS
+		GF_VUIInfo tmp_vui = pctx->vui;
+		tmp_vui.update = GF_TRUE;
+		e = gf_avc_change_vui(avcc, &tmp_vui);
+		if (!e) {
+			update_props(pctx, &tmp_vui);
+		}
+#else
+		return GF_NOT_SUPPORTED;
+#endif /*GPAC_DISABLE_AV_PARSERS*/
 	}
 
 	if ((ctx->lev>=0) || (ctx->prof>=0) || (ctx->pcomp>=0)) {
 		u32 i, count = gf_list_count(avcc->sequenceParameterSets);
 		for (i=0; i<count; i++) {
-			GF_AVCConfigSlot *sps = gf_list_get(avcc->sequenceParameterSets, i);
-			//first byte is nalu header, then profile_idc (8bits), prof_comp (8buts) a,d level_idc (8bits)
+			GF_NALUFFParam *sps = gf_list_get(avcc->sequenceParameterSets, i);
+			//first byte is nalu header, then profile_idc (8bits), prof_comp (8bits), and level_idc (8bits)
 			if (ctx->prof>=0) {
 				sps->data[1] = (u8) ctx->prof;
 			}
@@ -205,7 +295,7 @@ static GF_Err avc_rewrite_pid_config(GF_BSRWCtx *ctx, BSRWPid *pctx)
 
 	gf_filter_pid_set_property(pctx->opid, GF_PROP_PID_DECODER_CONFIG, &PROP_DATA_NO_COPY(dsi, dsi_size) );
 
-	if (ctx->remsei) {
+	if (ctx->rmsei) {
 		pctx->rewrite_packet = avc_rewrite_packet;
 	} else {
 		pctx->rewrite_packet = none_rewrite_packet;
@@ -215,66 +305,6 @@ static GF_Err avc_rewrite_pid_config(GF_BSRWCtx *ctx, BSRWPid *pctx)
 
 #ifndef GPAC_DISABLE_HEVC
 
-static GF_Err hevc_rewrite_packet(GF_BSRWCtx *ctx, BSRWPid *pctx, GF_FilterPacket *pck)
-{
-	u32 size, pck_size, final_size;
-	GF_FilterPacket *dst;
-	u8 *output;
-	const u8 *data = gf_filter_pck_get_data(pck, &pck_size);
-	if (!data)
-		return gf_filter_pck_forward(pck, pctx->opid);
-
-	final_size = 0;
-	size=0;
-	while (size<pck_size) {
-		u8 nal_type=0;
-		u32 nal_hdr = pctx->nalu_size_length;
-		u32 nal_size = 0;
-		while (nal_hdr) {
-			nal_size |= data[size];
-			size++;
-			nal_hdr--;
-			if (!nal_hdr) break;
-			nal_size<<=8;
-		}
-		nal_type = data[size];
-		nal_type = (nal_type & 0x7E) >> 1;
-		if ((nal_type != GF_HEVC_NALU_SEI_PREFIX) && (nal_type != GF_HEVC_NALU_SEI_SUFFIX) ) {
-			final_size += nal_size+pctx->nalu_size_length;
-		}
-		size += nal_size;
-	}
-	if (final_size == pck_size)
-		return gf_filter_pck_forward(pck, pctx->opid);
-
-	dst = gf_filter_pck_new_alloc(pctx->opid, final_size, &output);
-	gf_filter_pck_merge_properties(pck, dst);
-
-	size=0;
-	while (size<pck_size) {
-		u8 nal_type=0;
-		u32 nal_hdr = pctx->nalu_size_length;
-		u32 nal_size = 0;
-		while (nal_hdr) {
-			nal_size |= data[size];
-			size++;
-			nal_hdr--;
-			if (!nal_hdr) break;
-			nal_size<<=8;
-		}
-		nal_type = data[size];
-		nal_type = (nal_type & 0x7E) >> 1;
-		if ((nal_type == GF_HEVC_NALU_SEI_PREFIX) || (nal_type == GF_HEVC_NALU_SEI_SUFFIX) ) {
-			size += nal_size;
-			continue;
-		}
-		memcpy(output, &data[size-pctx->nalu_size_length], pctx->nalu_size_length+nal_size);
-		output += pctx->nalu_size_length+nal_size;
-		size += nal_size;
-	}
-	return gf_filter_pck_send(dst);
-}
-
 static GF_Err hevc_rewrite_pid_config(GF_BSRWCtx *ctx, BSRWPid *pctx)
 {
 	GF_HEVCConfig *hvcc;
@@ -288,11 +318,19 @@ static GF_Err hevc_rewrite_pid_config(GF_BSRWCtx *ctx, BSRWPid *pctx)
 	pctx->reconfigure = GF_FALSE;
 
 	hvcc = gf_odf_hevc_cfg_read(prop->value.data.ptr, prop->value.data.size, (pctx->codec_id==GF_CODECID_LHVC) ? GF_TRUE : GF_FALSE);
-	if (ctx->sar.num && ctx->sar.den) {
-		e = gf_media_hevc_change_par(hvcc, ctx->sar.num, ctx->sar.den);
-		if (!e)
-			gf_filter_pid_set_property(pctx->opid, GF_PROP_PID_SAR, &PROP_FRAC(ctx->sar) );
+	if (pctx->rewrite_vui) {
+#ifndef GPAC_DISABLE_AV_PARSERS
+		GF_VUIInfo tmp_vui = pctx->vui;
+		tmp_vui.update = GF_TRUE;
+		e = gf_hevc_change_vui(hvcc, &tmp_vui);
+		if (!e) {
+			update_props(pctx, &tmp_vui);
+		}
+#else
+		return GF_NOT_SUPPORTED;
+#endif /*GPAC_DISABLE_AV_PARSERS*/
 	}
+
 	if (ctx->pidc>=0) hvcc->profile_idc = ctx->pidc;
 	if (ctx->pspace>=0) hvcc->profile_space = ctx->pspace;
 	if (ctx->gpcflags>=0) hvcc->general_profile_compatibility_flags = ctx->gpcflags;
@@ -305,7 +343,7 @@ static GF_Err hevc_rewrite_pid_config(GF_BSRWCtx *ctx, BSRWPid *pctx)
 
 	gf_filter_pid_set_property(pctx->opid, GF_PROP_PID_DECODER_CONFIG, &PROP_DATA_NO_COPY(dsi, dsi_size) );
 
-	if (ctx->remsei) {
+	if (ctx->rmsei) {
 		pctx->rewrite_packet = hevc_rewrite_packet;
 	} else {
 		pctx->rewrite_packet = none_rewrite_packet;
@@ -314,6 +352,52 @@ static GF_Err hevc_rewrite_pid_config(GF_BSRWCtx *ctx, BSRWPid *pctx)
 }
 #endif // GPAC_DISABLE_HEVC
 
+
+static GF_Err vvc_rewrite_pid_config(GF_BSRWCtx *ctx, BSRWPid *pctx)
+{
+#ifndef GPAC_DISABLE_AV_PARSERS
+	GF_VVCConfig *vvcc;
+	GF_Err e = GF_OK;
+	u8 *dsi;
+	u32 dsi_size;
+	const GF_PropertyValue *prop;
+
+	prop = gf_filter_pid_get_property(pctx->ipid, GF_PROP_PID_DECODER_CONFIG);
+	if (!prop) return GF_OK;
+	pctx->reconfigure = GF_FALSE;
+
+	vvcc = gf_odf_vvc_cfg_read(prop->value.data.ptr, prop->value.data.size);
+	if (pctx->rewrite_vui) {
+		GF_VUIInfo tmp_vui = pctx->vui;
+		tmp_vui.update = GF_TRUE;
+		e = gf_vvc_change_vui(vvcc, &tmp_vui);
+		if (!e) {
+			update_props(pctx, &tmp_vui);
+		}
+	}
+
+	if (ctx->pidc>=0) vvcc->general_profile_idc = ctx->pidc;
+	if (ctx->lev>=0) vvcc->general_level_idc = ctx->pidc;
+
+
+	gf_odf_vvc_cfg_write(vvcc, &dsi, &dsi_size);
+	pctx->nalu_size_length = vvcc->nal_unit_size;
+	gf_odf_vvc_cfg_del(vvcc);
+	if (e) return e;
+
+	gf_filter_pid_set_property(pctx->opid, GF_PROP_PID_DECODER_CONFIG, &PROP_DATA_NO_COPY(dsi, dsi_size) );
+
+	if (ctx->rmsei) {
+		pctx->rewrite_packet = vvc_rewrite_packet;
+	} else {
+		pctx->rewrite_packet = none_rewrite_packet;
+	}
+	return GF_OK;
+#else
+	return GF_NOT_SUPPORTED;
+#endif /*GPAC_DISABLE_AV_PARSERS*/
+}
+
 static GF_Err none_rewrite_pid_config(GF_BSRWCtx *ctx, BSRWPid *pctx)
 {
 	pctx->reconfigure = GF_FALSE;
@@ -324,57 +408,42 @@ static GF_Err rewrite_pid_config(GF_BSRWCtx *ctx, BSRWPid *pctx)
 {
 	GF_Err e = pctx->rewrite_pid_config(ctx, pctx);
 	if (e) return e;
-	pctx->clrp = pctx->txchar = pctx->mxcoef = -1;
-	pctx->prev_clrp = pctx->prev_txchar = pctx->prev_mxcoef = pctx->prev_sar = -1;
-
-	if (ctx->clrp) {
-		if (!stricmp(ctx->clrp, "BT.709") || !stricmp(ctx->clrp, "BT709")) pctx->clrp = 1;
-		else if (!stricmp(ctx->clrp, "BT.601-625") || !stricmp(ctx->clrp, "BT601-625")) pctx->clrp = 5;
-		else if (!stricmp(ctx->clrp, "BT.601-525") || !stricmp(ctx->clrp, "BT601-525")) pctx->clrp = 6;
-		else if (!stricmp(ctx->clrp, "BT.2020") || !stricmp(ctx->clrp, "BT2020")) pctx->clrp = 9;
-		else if (!stricmp(ctx->clrp, "P3")) pctx->clrp = 11;
-		else if (!stricmp(ctx->clrp, "P3-D65")) pctx->clrp = 12;
-		else {
-			char szCoef[100];
-			pctx->clrp = atoi(ctx->clrp);
-			sprintf(szCoef, "%u", pctx->clrp);
-			if (stricmp(szCoef, ctx->clrp))
-				pctx->clrp = -1;
-		}
-	}
-	if (ctx->txchar) {
-		if (!strcmp(ctx->txchar, "BT-709") || !strcmp(ctx->txchar, "BT709")) pctx->txchar=1;
-		else if (!strcmp(ctx->txchar, "ST-2084") || !strcmp(ctx->txchar, "ST2084")) pctx->txchar=16;
-		else if (!strcmp(ctx->txchar, "STD-B67") || !strcmp(ctx->txchar, "STDB67")) pctx->txchar=18;
-		else {
-			char szTxChar[100];
-			pctx->txchar = atoi(ctx->txchar);
-			sprintf(szTxChar, "%u", pctx->txchar);
-			if (stricmp(szTxChar, ctx->txchar))
-				pctx->txchar = -1;
-		}
-	}
-	if (ctx->mxcoef) {
-		if (!strcmp(ctx->mxcoef, "BT.709") || !strcmp(ctx->mxcoef, "BT709")) pctx->mxcoef = 1;
-		else if (!strcmp(ctx->mxcoef, "BT.601") || !strcmp(ctx->mxcoef, "BT601")) pctx->mxcoef = 6;
-		else if (!strcmp(ctx->mxcoef, "BT.2020") || !strcmp(ctx->mxcoef, "BT2020")) pctx->mxcoef = 9;
-		else {
-			char szMx[100];
-			pctx->mxcoef = atoi(ctx->mxcoef);
-			sprintf(szMx, "%u", pctx->mxcoef);
-			if (stricmp(szMx, ctx->mxcoef))
-				pctx->mxcoef = -1;
-		}
-	}
+	pctx->prev_cprim = pctx->prev_ctfc = pctx->prev_cmx = pctx->prev_sar = -1;
 	return GF_OK;
 }
 
+static void init_vui(GF_BSRWCtx *ctx, BSRWPid *pctx)
+{
+#ifndef GPAC_DISABLE_AV_PARSERS
+	pctx->vui.ar_num = ctx->sar.num;
+	pctx->vui.ar_den = ctx->sar.den;
+	pctx->vui.color_matrix = ctx->cmx;
+	pctx->vui.color_prim = ctx->cprim;
+	pctx->vui.fullrange = ctx->fullrange;
+	pctx->vui.remove_video_info = ctx->novsi;
+	pctx->vui.color_tfc = ctx->ctfc;
+	pctx->vui.video_format = ctx->vidfmt;
+#endif /*GPAC_DISABLE_AV_PARSERS*/
+
+	pctx->rewrite_vui = GF_TRUE;
+	if (ctx->sar.num>=0) return;
+	if ((s32) ctx->sar.den>=0) return;
+	if (ctx->cmx>-1) return;
+	if (ctx->cprim>-1) return;
+	if (ctx->fullrange) return;
+	if (ctx->novsi) return;
+	if (ctx->ctfc>-1) return;
+	if (ctx->vidfmt>-1) return;
+	//all default
+	pctx->rewrite_vui = GF_FALSE;
+}
 
 static GF_Err prores_rewrite_packet(GF_BSRWCtx *ctx, BSRWPid *pctx, GF_FilterPacket *pck)
 {
 	u8 *output;
 	GF_FilterPacket *dst_pck = gf_filter_pck_new_clone(pctx->opid, pck, &output);
-
+	if (!dst_pck) return GF_OUT_OF_MEM;
+	
 	//starting at offset 20 in frame:
 	/*
 	prores_frame->chroma_format = gf_bs_read_int(bs, 2);
@@ -398,7 +467,7 @@ static GF_Err prores_rewrite_packet(GF_BSRWCtx *ctx, BSRWPid *pctx, GF_FilterPac
 		else if (ctx->sar.num * 9 == ctx->sar.den * 16) new_ar = 3;
 		else {
 			if (pctx->prev_sar != new_ar) {
-				GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[BSRW] Aspect ratio %d/%d not registered in ProRes, using 0 (unknown)\n", ctx->sar.num, ctx->sar.den));
+				GF_LOG(GF_LOG_WARNING, GF_LOG_CODING, ("[BSRW] Aspect ratio %d/%d not registered in ProRes, using 0 (unknown)\n", ctx->sar.num, ctx->sar.den));
 			}
 		}
 		new_ar <<= 4;
@@ -409,25 +478,25 @@ static GF_Err prores_rewrite_packet(GF_BSRWCtx *ctx, BSRWPid *pctx, GF_FilterPac
 			pctx->prev_sar = new_ar;
 		}
 	}
-	if (pctx->clrp>=0) {
-		output[22] = (u8) pctx->clrp;
-		if (pctx->prev_clrp != pctx->clrp) {
-			gf_filter_pid_set_property(pctx->opid, GF_PROP_PID_COLR_PRIMARIES, &PROP_UINT(pctx->clrp) );
-			pctx->prev_clrp = pctx->clrp;
+	if (ctx->cprim>=0) {
+		output[22] = (u8) ctx->cprim;
+		if (pctx->prev_cprim != ctx->cprim) {
+			gf_filter_pid_set_property(pctx->opid, GF_PROP_PID_COLR_PRIMARIES, &PROP_UINT(ctx->cprim) );
+			pctx->prev_cprim = ctx->cprim;
 		}
 	}
-	if (pctx->txchar>=0) {
-		output[23] = (u8) pctx->txchar;
-		if (pctx->prev_txchar != pctx->txchar) {
-			gf_filter_pid_set_property(pctx->opid, GF_PROP_PID_COLR_TRANSFER, &PROP_UINT(pctx->txchar) );
-			pctx->prev_txchar = pctx->txchar;
+	if (ctx->ctfc>=0) {
+		output[23] = (u8) ctx->ctfc;
+		if (ctx->ctfc != pctx->prev_ctfc) {
+			gf_filter_pid_set_property(pctx->opid, GF_PROP_PID_COLR_TRANSFER, &PROP_UINT(ctx->ctfc) );
+			pctx->prev_ctfc = ctx->ctfc;
 		}
 	}
-	if (pctx->mxcoef>=0) {
-		output[24] = (u8) pctx->mxcoef;
-		if (pctx->prev_mxcoef != pctx->mxcoef) {
-			gf_filter_pid_set_property(pctx->opid, GF_PROP_PID_COLR_MX, &PROP_UINT(pctx->mxcoef) );
-			pctx->prev_mxcoef = pctx->mxcoef;
+	if (ctx->cmx>=0) {
+		output[24] = (u8) ctx->cmx;
+		if (pctx->prev_cmx != ctx->cmx) {
+			gf_filter_pid_set_property(pctx->opid, GF_PROP_PID_COLR_MX, &PROP_UINT(ctx->cmx) );
+			pctx->prev_cmx = ctx->cmx;
 		}
 	}
 	return gf_filter_pck_send(dst_pck);
@@ -443,8 +512,10 @@ static GF_Err bsrw_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_r
 	//disconnect of src pid (not yet supported)
 	if (is_remove) {
 		if (pctx) {
-			//one in one out, this is simple
-			gf_filter_pid_remove(pctx->opid);
+			if (pctx->opid) {
+				gf_filter_pid_remove(pctx->opid);
+				pctx->opid = NULL;
+			}
 			gf_filter_pid_set_udta(pid, NULL);
 			gf_list_del_item(ctx->pids, pctx);
  			gf_free(pctx);
@@ -464,6 +535,7 @@ static GF_Err bsrw_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_r
 		gf_list_add(ctx->pids, pctx);
 		pctx->opid = gf_filter_pid_new(filter);
 		if (!pctx->opid) return GF_OUT_OF_MEM;
+		init_vui(ctx, pctx);
 	}
 
 	prop = gf_filter_pid_get_property(pid, GF_PROP_PID_CODECID);
@@ -481,6 +553,10 @@ static GF_Err bsrw_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_r
 		pctx->rewrite_pid_config = hevc_rewrite_pid_config;
 		break;
 #endif
+	case GF_CODECID_VVC:
+	case GF_CODECID_VVC_SUBPIC:
+		pctx->rewrite_pid_config = vvc_rewrite_pid_config;
+		break;
 	case GF_CODECID_MPEG4_PART2:
 		pctx->rewrite_pid_config = m4v_rewrite_pid_config;
 		break;
@@ -502,8 +578,11 @@ static GF_Err bsrw_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_r
 
 	gf_filter_pid_copy_properties(pctx->opid, pctx->ipid);
 	pctx->codec_id = prop->value.uint;
-	pctx->reconfigure = GF_TRUE;
+	pctx->reconfigure = GF_FALSE;
 	gf_filter_pid_set_framing_mode(pid, GF_TRUE);
+	//rewrite asap - waiting for first packet could lead to issues further down the chain, especially for movie fragments
+	//were the init segment could have already been flushed at the time we will dispatch the first packet
+	rewrite_pid_config(ctx, pctx);
 	return GF_OK;
 }
 
@@ -525,6 +604,7 @@ static GF_Err bsrw_process(GF_Filter *filter)
 			pctx->reconfigure = GF_TRUE;
 
 		if (pctx->reconfigure) {
+			init_vui(ctx, pctx);
 			GF_Err e = rewrite_pid_config(ctx, pctx);
 			if (e) return e;
 		}
@@ -569,28 +649,31 @@ static void bsrw_finalize(GF_Filter *filter)
 }
 
 #define OFFS(_n)	#_n, offsetof(GF_BSRWCtx, _n)
-static const GF_FilterArgs BSRWArgs[] =
+static GF_FilterArgs BSRWArgs[] =
 {
-	{ OFFS(sar), "aspect ratio to rewrite", GF_PROP_FRACTION, "0/0", NULL, GF_FS_ARG_UPDATE},
+	///do not change order of the first 3
+	{ OFFS(cprim), "color primaries according to ISO/IEC 23001-8 / 23091-2", GF_PROP_CICP_COL_PRIM, "-1", NULL, GF_FS_ARG_UPDATE},
+	{ OFFS(ctfc), "color transfer characteristics according to ISO/IEC 23001-8 / 23091-2", GF_PROP_CICP_COL_TFC, "-1", NULL, GF_FS_ARG_UPDATE},
+	{ OFFS(cmx), "color matrix coeficients according to ISO/IEC 23001-8 / 23091-2", GF_PROP_CICP_COL_MX, "-1", NULL, GF_FS_ARG_UPDATE},
+	{ OFFS(sar), "aspect ratio to rewrite", GF_PROP_FRACTION, "-1/-1", NULL, GF_FS_ARG_UPDATE},
 	{ OFFS(m4vpl), "set ProfileLevel for MPEG-4 video part two", GF_PROP_SINT, "-1", NULL, GF_FS_ARG_UPDATE},
-	{ OFFS(clrp), "color primaries according to ISO/IEC 23001-8 / 23091-2. Value can be the integer value or (case insensitive) `BT709`, `BT601-625`, `BT601-525`, `BT2020`, `P3` or `P3-D65`", GF_PROP_STRING, NULL, NULL, GF_FS_ARG_UPDATE},
-	{ OFFS(txchar), "transfer characteristics according to ISO/IEC 23001-8 / 23091-2. Value can be the integer value or (case insensitive) `BT709`, `ST2084` or `STDB67`", GF_PROP_STRING, NULL, NULL, GF_FS_ARG_UPDATE},
-	{ OFFS(mxcoef), "matrix coeficients according to ISO/IEC 23001-8 / 23091-2. Value can be the integer value or (case insensitive) `BT709`, `BT601` or `BT2020`", GF_PROP_STRING, NULL, NULL, GF_FS_ARG_UPDATE},
-
+	{ OFFS(fullrange), "video full range flag", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_UPDATE},
+	{ OFFS(novsi), "remove video_signal_type from VUI in AVC|H264 and HEVC", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_UPDATE},
 	{ OFFS(prof), "profile indication for AVC|H264", GF_PROP_SINT, "-1", NULL, GF_FS_ARG_UPDATE},
-	{ OFFS(lev), "level indication for AVC|H264", GF_PROP_SINT, "-1", NULL, GF_FS_ARG_UPDATE},
+	{ OFFS(lev), "level indication for AVC|H264, level_idc for VVC", GF_PROP_SINT, "-1", NULL, GF_FS_ARG_UPDATE},
 	{ OFFS(pcomp), "profile compatibility for AVC|H264", GF_PROP_SINT, "-1", NULL, GF_FS_ARG_UPDATE},
-	{ OFFS(pidc), "profile IDC for HEVC", GF_PROP_SINT, "-1", NULL, GF_FS_ARG_UPDATE},
+	{ OFFS(pidc), "profile IDC for HEVC and VVC", GF_PROP_SINT, "-1", NULL, GF_FS_ARG_UPDATE},
 	{ OFFS(pspace), "profile space for HEVC", GF_PROP_SINT, "-1", NULL, GF_FS_ARG_UPDATE},
 	{ OFFS(gpcflags), "general compatibility flags for HEVC", GF_PROP_SINT, "-1", NULL, GF_FS_ARG_UPDATE},
-
-	{ OFFS(remsei), "remove SEI messages from bitstream", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_UPDATE},
-
+	{ OFFS(rmsei), "remove SEI messages from bitstream for AVC|H264, HEVC and VVC", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_UPDATE},
+	{ OFFS(vidfmt), "video format for AVC|H264, HEVC and VVC", GF_PROP_UINT, "-1", "component|pal|ntsc|secam|mac|undef", GF_FS_ARG_UPDATE},
 	{0}
 };
 
 static const GF_FilterCapability BSRWCaps[] =
 {
+	//this is correct but we want the filter to act as passthrough for other media
+#if 0
 	CAP_UINT(GF_CAPS_INPUT_STATIC ,GF_PROP_PID_STREAM_TYPE, GF_STREAM_VISUAL),
 	CAP_BOOL(GF_CAPS_INPUT_EXCLUDED|GF_CAPFLAG_STATIC , GF_PROP_PID_UNFRAMED, GF_TRUE),
 	CAP_UINT(GF_CAPS_INPUT_OUTPUT,GF_PROP_PID_CODECID, GF_CODECID_AVC),
@@ -609,24 +692,45 @@ static const GF_FilterCapability BSRWCaps[] =
 	CAP_UINT(GF_CAPS_INPUT_OUTPUT,GF_PROP_PID_CODECID, GF_CODECID_APCN),
 	CAP_UINT(GF_CAPS_INPUT_OUTPUT,GF_PROP_PID_CODECID, GF_CODECID_APCO),
 	CAP_UINT(GF_CAPS_INPUT_OUTPUT,GF_PROP_PID_CODECID, GF_CODECID_APCS),
+#else
+	CAP_UINT(GF_CAPS_INPUT_EXCLUDED, GF_PROP_PID_STREAM_TYPE, GF_STREAM_FILE),
+	CAP_BOOL(GF_CAPS_INPUT_EXCLUDED, GF_PROP_PID_UNFRAMED, GF_TRUE),
+	CAP_UINT(GF_CAPS_INPUT_EXCLUDED, GF_PROP_PID_CODECID, GF_CODECID_NONE),
+	CAP_UINT(GF_CAPS_OUTPUT_EXCLUDED, GF_PROP_PID_STREAM_TYPE, GF_STREAM_FILE),
+	CAP_UINT(GF_CAPS_OUTPUT_EXCLUDED, GF_PROP_PID_CODECID, GF_CODECID_NONE),
+#endif
+
 };
 
 GF_FilterRegister BSRWRegister = {
 	.name = "bsrw",
 	GF_FS_SET_DESCRIPTION("Compressed bitstream rewriter")
 	GF_FS_SET_HELP("This filter rewrites some metadata of various bitstream formats.\n"
-	"The filter can currently modify the following properties in the bitstream:\n"
-	"- MPEG-4 Visual: aspect ratio and profile/level\n"
-	"- AVC|H264: aspect ratio, profile, level, profile compatibility\n"
-	"- HEVC: aspect ratio\n"
-	"- ProRes: aspect ratio, color primaries, transfer characteristics and matrix coefficients\n"
+	"The filter can currently modify the following properties in video bitstreams:\n"
+	"- MPEG-4 Visual:\n"
+	"  - sample aspect ratio\n"
+	"  - profile and level\n"
+	"- AVC|H264, HEVC and VVC:\n"
+	"  - sample aspect ratio\n"
+	"  - profile, level, profile compatibility\n"
+	"  - video format, video fullrange\n"
+	"  - color primaries, transfer characteristics and matrix coefficients (or remove all info)\n"
+	"- ProRes:\n"
+	"  - sample aspect ratio\n"
+	"  - color primaries, transfer characteristics and matrix coefficients\n"
+	"  \n"
+	"Values are by default initialized to -1, implying to keep the related info (present or not) in the bitstream.\n"
+	"A [-sar]() value of `0/0` will remove sample aspect ratio info from bitstream if possible.\n"
 	"  \n"
 	"The filter can currently modify the following properties in the stream configuration but not in the bitstream:\n"
 	"- HEVC: profile IDC, profile space, general compatibility flags\n"
+	"- VVC: profile IDC, general profile and level indication\n"
+	"  \n"
+	"The filter will work in passthrough mode for all other codecs and media types.\n"
 	)
 	.private_size = sizeof(GF_BSRWCtx),
 	.max_extra_pids = 0xFFFFFFFF,
-	.flags = GF_FS_REG_EXPLICIT_ONLY,
+	.flags = GF_FS_REG_EXPLICIT_ONLY|GF_FS_REG_ALLOW_CYCLIC,
 	.args = BSRWArgs,
 	SETCAPS(BSRWCaps),
 	.initialize = bsrw_initialize,
@@ -638,5 +742,11 @@ GF_FilterRegister BSRWRegister = {
 
 const GF_FilterRegister *bsrw_register(GF_FilterSession *session)
 {
-	return &BSRWRegister;
+	//assign runtime caps on first load
+	if (gf_opts_get_bool("temp", "helponly")) {
+		BSRWArgs[0].min_max_enum = gf_cicp_color_primaries_all_names();
+		BSRWArgs[1].min_max_enum = gf_cicp_color_transfer_all_names();
+		BSRWArgs[2].min_max_enum = gf_cicp_color_matrix_all_names();
+	}
+	return (const GF_FilterRegister *) &BSRWRegister;
 }
