@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2017-2020
+ *			Copyright (c) Telecom ParisTech 2017-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / NVidia Hardware decoder filter
@@ -28,7 +28,7 @@
 #include <gpac/constants.h>
 #include <gpac/filters.h>
 
-#if (defined(WIN32) || defined(GPAC_CONFIG_LINUX) || defined(GPAC_CONFIG_DARWIN)) 
+#if (!defined(GPAC_STATIC_BUILD) && (defined(WIN32) || defined(GPAC_CONFIG_LINUX) || defined(GPAC_CONFIG_DARWIN)) && !defined(GPAC_DISABLE_NVDEC))
 
 #include "dec_nvdec_sdk.h"
 
@@ -69,7 +69,7 @@ typedef struct _nv_dec_ctx
 	u32 num_surfaces;
 
 	GF_FilterPid *ipid, *opid;
-	u32 codec_id;
+	u32 codec_id, cfg_crc;
 	Bool use_gl_texture;
 	u32 width, height, bpp_luma, bpp_chroma;
 	cudaVideoCodec codec_type;
@@ -80,7 +80,7 @@ typedef struct _nv_dec_ctx
 	Bool skip_next_frame;
 	CUresult decode_error, dec_create_error;
 	Bool frame_size_changed;
-	Bool needs_resetup;
+	u32 needs_resetup;
 	unsigned long prefer_dec_mode;
 
 	NVDecInstance *dec_inst;
@@ -89,9 +89,6 @@ typedef struct _nv_dec_ctx
 	GF_List *frames_res;
 	GF_List *src_packets;
 
-	struct __nv_frame *pending_frame;
-
-
 	u8 *xps_buf;
 	u32 xps_buf_size;
 	u32 nal_size_length;
@@ -106,7 +103,10 @@ typedef struct _nv_dec_ctx
 	Bool gl_provider_requested;
 	GLint y_tx_id, uv_tx_id;
 	GLint y_pbo_id, uv_pbo_id;
+	Bool reset_pbo;
 #endif
+	u32 nb_out_frames_pending;
+
 } NVDecCtx;
 
 
@@ -145,6 +145,8 @@ static Bool cuda_ctx_gl = GF_FALSE;
 static CUdevice  cuda_dev = -1;
 #endif
 
+static GF_Err nvdec_flush_frame(NVDecCtx *ctx, NVDecFrame *f);
+
 //#define ENABLE_10BIT_OUTPUT
 
 static GF_Err nvdec_init_decoder(NVDecCtx *ctx)
@@ -242,6 +244,15 @@ Bool load_inactive_dec(NVDecCtx *ctx)
 	return GF_FALSE;
 }
 
+static void nvdec_copy_props(NVDecCtx *ctx)
+{
+	gf_filter_pid_copy_properties(ctx->opid, ctx->ipid);
+	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_DECODER_CONFIG, NULL);
+	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_DECODER_CONFIG_ENHANCEMENT, NULL);
+	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_CODECID, &PROP_UINT(GF_CODECID_RAW));
+	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_UNFRAMED, NULL);
+}
+
 static void nvdec_destroy_decoder(NVDecInstance *inst)
 {
 	if (inst->cu_decoder) {
@@ -268,6 +279,7 @@ static void update_pix_fmt(NVDecCtx *ctx, Bool use_10bits)
 		ctx->pix_fmt = 0;
 		return;
 	}
+	ctx->stride_uv = 0;
 	gf_pixel_get_size_info(ctx->pix_fmt, ctx->width, ctx->height, &ctx->out_size, &ctx->stride, &ctx->stride_uv, &ctx->nb_planes, &ctx->uv_height);
 }
 
@@ -277,8 +289,8 @@ static int CUDAAPI HandleVideoSequence(void *pUserData, CUVIDEOFORMAT *pFormat)
 	Bool skip_output_resize=GF_FALSE;
 	NVDecInstance *inst= (NVDecInstance *)pUserData;
 	NVDecCtx *ctx = inst->ctx;
-
 	u32 w, h;
+
 	w = pFormat->coded_width;
 	h = pFormat->coded_height;
 
@@ -298,6 +310,17 @@ static int CUDAAPI HandleVideoSequence(void *pUserData, CUVIDEOFORMAT *pFormat)
 			return 1;
 		skip_output_resize = GF_TRUE;
 	}
+	ctx->needs_resetup = 1;
+	while (gf_list_count(ctx->frames)) {
+		NVDecFrame *f = gf_list_pop_front(ctx->frames);
+		nvdec_flush_frame(ctx, f);
+	}
+	ctx->needs_resetup = 0;
+
+	if (ctx->nb_out_frames_pending) {
+		GF_LOG(GF_LOG_WARNING, GF_LOG_CODEC, ("[NVDec] Decoder must reset and pending frames not yet drawn: multiple SPS in single config, not supported!\nDisabling GPU frame dispatch\n"));
+		ctx->fmode = NVDEC_COPY;
+	}
 
 	//commented out since this falls back to soft decoding !
 #ifdef ENABLE_10BIT_OUTPUT
@@ -312,7 +335,11 @@ static int CUDAAPI HandleVideoSequence(void *pUserData, CUVIDEOFORMAT *pFormat)
 	ctx->chroma_fmt = pFormat->chroma_format;
 	ctx->stride = pFormat->coded_width;
 
-	//if load_inatcive returns TRUE, we are reusing an existing decoder with the same config, no need to recreate one
+#ifndef GPAC_DISABLE_3D
+	ctx->reset_pbo = GF_TRUE;
+#endif
+
+	//if load_inactive returns TRUE, we are reusing an existing decoder with the same config, no need to recreate one
 	if (load_inactive_dec(ctx)) {
 		GF_LOG(GF_LOG_INFO, GF_LOG_CODEC, ("[NVDec] reusing inactive decoder %dx%d - %d total decoders loaded\n", ctx->width, ctx->height, global_nb_loaded_decoders) );
 		ctx->stride = ctx->dec_inst->stride;
@@ -320,7 +347,7 @@ static int CUDAAPI HandleVideoSequence(void *pUserData, CUVIDEOFORMAT *pFormat)
 		if (!ctx->out_size) ctx->reload_decoder_state = 1;
 
 		update_pix_fmt(ctx, use_10bits);
-		return GF_OK;
+		return 1;
 	}
 	if (!ctx->dec_inst) return GF_OUT_OF_MEM;
 	//if we have an existing decoder but with a different config, let's reload
@@ -333,13 +360,15 @@ static int CUDAAPI HandleVideoSequence(void *pUserData, CUVIDEOFORMAT *pFormat)
 	ctx->dec_inst->codec_type = ctx->codec_type;
 	ctx->dec_inst->chroma_fmt = ctx->chroma_fmt;
 	ctx->dec_inst->ctx = ctx;
-	ctx->stride = use_10bits ? 2*ctx->width : ctx->width;
+	ctx->stride = use_10bits ? 2 * ctx->width : ctx->width;
 
 	update_pix_fmt(ctx, use_10bits);
 
+	nvdec_copy_props(ctx);
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_WIDTH, &PROP_UINT(ctx->width));
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_HEIGHT, &PROP_UINT(ctx->height));
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_STRIDE, &PROP_UINT(ctx->stride));
+	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_STRIDE_UV, NULL);
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_BIT_DEPTH_Y, &PROP_UINT(use_10bits ? ctx->bpp_luma : 8));
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_BIT_DEPTH_UV, &PROP_UINT(use_10bits ? ctx->bpp_chroma : 8));
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_PIXFMT, &PROP_UINT(ctx->pix_fmt));
@@ -407,7 +436,7 @@ static void nvdec_store_paramlist(GF_BitStream *bs, GF_List *psl)
 	u32 i, count;
 	count = gf_list_count(psl);
 	for (i=0; i<count; i++) {
-		GF_AVCConfigSlot *slc = gf_list_get(psl, i);
+		GF_NALUFFParam *slc = gf_list_get(psl, i);
 		gf_bs_write_u32(bs, 1);
 		gf_bs_write_data(bs, slc->data, slc->size);
 	}
@@ -427,7 +456,7 @@ static void nvdec_store_xps(NVDecCtx *ctx, GF_AVCConfig *avc_cfg, GF_HEVCConfig
 		ctx->nal_size_length = hevc_cfg->nal_unit_size;
 		count = gf_list_count(hevc_cfg->param_array);
 		for (i=0; i<count; i++) {
-			GF_HEVCParamArray *pa = gf_list_get(hevc_cfg->param_array, i);
+			GF_NALUFFParamArray *pa = gf_list_get(hevc_cfg->param_array, i);
 			nvdec_store_paramlist(bs, pa->nalus);
 		}
 		gf_odf_hevc_cfg_del(hevc_cfg);
@@ -482,7 +511,7 @@ static GF_Err nvdec_configure_stream(GF_Filter *filter, NVDecCtx *ctx)
 
 	//create a video parser and a video decoder
     memset(&oVideoParserParameters, 0, sizeof(CUVIDPARSERPARAMS));
-	ctx->needs_resetup = GF_FALSE;
+	ctx->needs_resetup = 0;
 	ctx->nal_size_length = 0;
 
 	//this destroys avc_cfg / hevc_cfg
@@ -558,8 +587,13 @@ static GF_Err nvdec_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_
 	NVDecCtx *ctx = (NVDecCtx *) gf_filter_get_udta(filter);
 
 	if (is_remove) {
-		if (ctx->opid) gf_filter_pid_remove(ctx->opid);
-		ctx->opid = NULL;
+		if (ctx->ipid != pid)
+			return GF_OK;
+
+		if (ctx->opid) {
+			gf_filter_pid_remove(ctx->opid);
+			ctx->opid = NULL;
+		}
 		ctx->ipid = NULL;
 
 		if (ctx->unload == 2) {
@@ -587,7 +621,7 @@ static GF_Err nvdec_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_
 	if (ctx->use_gl_texture && (ctx->fmode==NVDEC_GL) && !ctx->gl_provider_requested) {
 		GF_Err e = gf_filter_request_opengl(filter);
 		if (e) {
-			GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[NVDec] failed to request an openGL provider (error %s), will not use OpenGL output\n", gf_error_to_string(e) ));
+			GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[NVDec] failed to request an OpenGL provider (error %s), will not use OpenGL output\n", gf_error_to_string(e) ));
 			ctx->use_gl_texture = GF_FALSE;
 		}
 		ctx->gl_provider_requested = GF_TRUE;
@@ -596,6 +630,9 @@ static GF_Err nvdec_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_
 
 	prop = gf_filter_pid_get_property(pid, GF_PROP_PID_CODECID);
 	if (!prop) return GF_NOT_SUPPORTED;
+	if (ctx->codec_id != prop->value.uint) {
+		ctx->cfg_crc = 0;
+	}
 	ctx->codec_id = prop->value.uint;
 
 	switch (ctx->codec_id) {
@@ -614,14 +651,10 @@ static GF_Err nvdec_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_
 		return GF_NOT_SUPPORTED;
 	}
 
-	if (!ctx->opid)
+	if (!ctx->opid) {
 		ctx->opid = gf_filter_pid_new(filter);
-
-	gf_filter_pid_copy_properties(ctx->opid, ctx->ipid);
-	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_DECODER_CONFIG, NULL);
-	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_DECODER_CONFIG_ENHANCEMENT, NULL);
-	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_CODECID, &PROP_UINT(GF_CODECID_RAW) );
-	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_UNFRAMED, NULL);
+		nvdec_copy_props(ctx);
+	}
 
 	gf_filter_pid_set_framing_mode(ctx->ipid, GF_TRUE);
 
@@ -631,6 +664,11 @@ static GF_Err nvdec_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_
 		prop = gf_filter_pid_get_property(pid, GF_PROP_PID_DECODER_CONFIG);
 		//not ready yet
 		if (!prop) return GF_OK;
+		if (prop->value.data.ptr) {
+			u32 crc = gf_crc_32(prop->value.data.ptr, prop->value.data.size);
+			if (crc == ctx->cfg_crc) return GF_OK;
+			ctx->cfg_crc = crc;
+		}
 		break;
 	}
 
@@ -707,7 +745,7 @@ static GF_Err nvdec_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_
 	}
 
 
-	ctx->needs_resetup = GF_TRUE;
+	ctx->needs_resetup = 1;
 
 	return GF_OK;
 }
@@ -747,20 +785,20 @@ static Bool nvdec_process_event(GF_Filter *filter, const GF_FilterEvent *evt)
 				ctx->dec_inst = NULL;
 				gf_mx_v(global_inst_mutex);
 			}
-			ctx->needs_resetup = GF_TRUE;
+			ctx->needs_resetup = 1;
 			ctx->dec_create_error = CUDA_SUCCESS;
 		} else if (ctx->unload == 1) {
 			if (ctx->dec_inst) {
 				nvdec_destroy_decoder(ctx->dec_inst);
 			}
-			ctx->needs_resetup = GF_TRUE;
+			ctx->needs_resetup = 1;
 			ctx->dec_create_error = CUDA_SUCCESS;
 		}
 		return GF_OK;
 	}
 #endif
 
-static GF_Err nvdec_send_hw_frame(NVDecCtx *ctx);
+static GF_Err nvdec_send_hw_frame(NVDecCtx *ctx, NVDecFrame *f);
 
 static void nvdec_reset_pcks(NVDecCtx *ctx)
 {
@@ -779,6 +817,8 @@ static void nvdec_merge_pck_props(NVDecCtx *ctx, NVDecFrame *f,  GF_FilterPacket
 		src_pck = gf_list_get(ctx->src_packets, i);
 		if (gf_filter_pck_get_cts(src_pck) == f->frame_info.timestamp) {
 			gf_filter_pck_merge_properties(src_pck, dst_pck);
+			gf_filter_pck_set_dependency_flags(dst_pck, 0);
+			gf_filter_pck_set_dts(dst_pck, f->frame_info.timestamp);
 			gf_list_rem(ctx->src_packets, i);
 			gf_filter_pck_unref(src_pck);
 			return;
@@ -786,6 +826,7 @@ static void nvdec_merge_pck_props(NVDecCtx *ctx, NVDecFrame *f,  GF_FilterPacket
 	}
 	//not found !
 	gf_filter_pck_set_cts(dst_pck, f->frame_info.timestamp);
+	gf_filter_pck_set_dts(dst_pck, f->frame_info.timestamp);
 	if (!gf_filter_pck_get_interlaced(dst_pck) && !f->frame_info.progressive_frame) {
 		gf_filter_pck_set_interlaced(dst_pck, f->frame_info.top_field_first ? 1 : 2);
 	}
@@ -794,25 +835,44 @@ static void nvdec_merge_pck_props(NVDecCtx *ctx, NVDecFrame *f,  GF_FilterPacket
 static GF_Err nvdec_process(GF_Filter *filter)
 {
 	NVDecFrame *f;
-    CUdeviceptr map_mem = 0;
-	CUVIDPROCPARAMS params;
-	unsigned int pitch = 0;
 	GF_Err e;
 	u32 pck_size;
 	const u8 *data;
-	u8 *output;
-	GF_FilterPacket *ipck, *dst_pck;
-    CUVIDSOURCEDATAPACKET cu_pkt;
+	GF_FilterPacket *ipck;
+	CUVIDSOURCEDATAPACKET cu_pkt;
 	CUresult res;
-	NVDecCtx *ctx = (NVDecCtx *) gf_filter_get_udta(filter);
+	NVDecCtx *ctx = (NVDecCtx *)gf_filter_get_udta(filter);
 
 	ipck = gf_filter_pid_get_packet(ctx->ipid);
 
 	if (ctx->needs_resetup) {
-		e = nvdec_configure_stream(filter, ctx);
-		if (e<0) return e;
-		//not ready
-		if (e==GF_EOS) return GF_OK;
+		if (gf_list_count(ctx->frames)) {
+#ifndef EMUL_NV_DLL
+			res = cuCtxPushCurrent(cuda_ctx);
+			if (res != CUDA_SUCCESS) {
+				GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[NVDec] failed to push CUDA CTX %s\n", cudaGetErrorEnum(res)));
+			}
+#endif
+			f = gf_list_pop_front(ctx->frames);
+			e = nvdec_flush_frame(ctx, f);
+			cuCtxPopCurrent(NULL);
+			return e;
+		}
+		//send end of sequence
+		if (ctx->dec_inst && (ctx->needs_resetup == 1)) {
+			ctx->needs_resetup = 2;
+			ipck = NULL;
+		}
+		else {
+			if (ctx->nb_out_frames_pending) {
+				gf_filter_ask_rt_reschedule(filter, 1);
+				return GF_OK;
+			}
+			e = nvdec_configure_stream(filter, ctx);
+			if (e < 0) return e;
+			//not ready
+			if (e == GF_EOS) return GF_OK;
+		}
 	}
 
 	memset(&cu_pkt, 0, sizeof(CUVIDSOURCEDATAPACKET));
@@ -820,12 +880,13 @@ static GF_Err nvdec_process(GF_Filter *filter)
 	pck_size = 0;
 	data = NULL;
 	if (!ipck) {
-		if (!gf_filter_pid_is_eos(ctx->ipid))
+		if (!ctx->needs_resetup && !gf_filter_pid_is_eos(ctx->ipid))
 			return GF_OK;
 
 		cu_pkt.flags |= CUVID_PKT_ENDOFSTREAM;
 		ctx->skip_next_frame = GF_FALSE;
-	} else {
+	}
+	else {
 		data = gf_filter_pck_get_data(ipck, &pck_size);
 	}
 
@@ -851,7 +912,7 @@ static GF_Err nvdec_process(GF_Filter *filter)
 
 		while (pck_size) {
 			u32 i, nal_size = 0;
-			for (i = 0; i<ctx->nal_size_length; i++) {
+			for (i = 0; i < ctx->nal_size_length; i++) {
 				nal_size = (nal_size << 8) + ((u8)data[i]);
 			}
 			data += ctx->nal_size_length;
@@ -874,16 +935,17 @@ static GF_Err nvdec_process(GF_Filter *filter)
 	if (ipck) cu_pkt.timestamp = gf_filter_pck_get_cts(ipck);
 
 #ifndef EMUL_NV_DLL
-    res = cuCtxPushCurrent(cuda_ctx);
+	res = cuCtxPushCurrent(cuda_ctx);
 	if (res != CUDA_SUCCESS) {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[NVDec] failed to push CUDA CTX %s\n", cudaGetErrorEnum(res) ) );
+		GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[NVDec] failed to push CUDA CTX %s\n", cudaGetErrorEnum(res)));
 	}
 	if (ctx->skip_next_frame) {
 		ctx->skip_next_frame = GF_FALSE;
-	} else {
+	}
+	else {
 		res = cuvidParseVideoData(ctx->dec_inst->cu_parser, &cu_pkt);
 		if (res != CUDA_SUCCESS) {
-			GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[NVDec] decoder instance %d failed to parse data %s\n", ctx->dec_inst->id, cudaGetErrorEnum(res) ) );
+			GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[NVDec] decoder instance %d failed to parse data %s\n", ctx->dec_inst->id, cudaGetErrorEnum(res)));
 		}
 	}
 #endif //EMUL_NV_DLL
@@ -895,9 +957,10 @@ static GF_Err nvdec_process(GF_Filter *filter)
 	}
 
 	if (ctx->reload_decoder_state) {
-		if (ctx->reload_decoder_state==2) {
+		if (ctx->reload_decoder_state == 2) {
 			nvdec_destroy_decoder(ctx->dec_inst);
-		} else {
+		}
+		else {
 			ctx->skip_next_frame = GF_TRUE;
 		}
 
@@ -908,7 +971,7 @@ static GF_Err nvdec_process(GF_Filter *filter)
 		}
 
 		//need to setup decoder
-		if (! ctx->dec_inst->cu_decoder) {
+		if (!ctx->dec_inst->cu_decoder) {
 			nvdec_init_decoder(ctx);
 		}
 		cuCtxPopCurrent(NULL);
@@ -930,14 +993,27 @@ static GF_Err nvdec_process(GF_Filter *filter)
 		}
 		return GF_OK;
 	}
-	if (ctx->use_gl_texture || (ctx->fmode==NVDEC_SINGLE) ) {
-		assert(!ctx->pending_frame);
-		ctx->pending_frame = f;
-		return nvdec_send_hw_frame(ctx);
+	e = nvdec_flush_frame(ctx, f);
+	cuCtxPopCurrent(NULL);
+	return e;
+}
+
+static GF_Err nvdec_flush_frame(NVDecCtx *ctx, NVDecFrame *f)
+{
+	GF_FilterPacket *dst_pck;
+	CUdeviceptr map_mem = 0;
+	CUVIDPROCPARAMS params;
+	unsigned int pitch = 0;
+	u8 *output;
+	CUresult res;
+	GF_Err e;
+	if (ctx->use_gl_texture || (ctx->fmode==NVDEC_SINGLE))  {
+		return nvdec_send_hw_frame(ctx, f);
 	}
 
 	assert(ctx->out_size);
 	dst_pck = gf_filter_pck_new_alloc(ctx->opid, ctx->out_size, &output);
+	if (!dst_pck) return GF_OUT_OF_MEM;
 
 	memset(&params, 0, sizeof(params));
 	params.progressive_frame = f->frame_info.progressive_frame;
@@ -990,11 +1066,8 @@ static GF_Err nvdec_process(GF_Filter *filter)
 			gf_filter_pck_discard(dst_pck);
 		}
 	}
-	cuCtxPopCurrent(NULL);
-
 	memset(f, 0, sizeof(NVDecFrame));
 	gf_list_add(ctx->frames_res, f);
-
 	return e;
 }
 
@@ -1003,7 +1076,7 @@ void nvframe_release(GF_Filter *filter, GF_FilterPid *pid, GF_FilterPacket *pck)
 	GF_FilterFrameInterface *frame = gf_filter_pck_get_frame_interface(pck);
 	NVDecFrame *f = (NVDecFrame*) frame->user_data;
 	NVDecCtx *ctx = (NVDecCtx *)f->ctx;
-
+	ctx->nb_out_frames_pending--;
 	memset(f, 0, sizeof(NVDecFrame));
 	gf_list_add(ctx->frames_res, f);
 }
@@ -1033,14 +1106,30 @@ GF_Err nvframe_get_gl_texture(GF_FilterFrameInterface *frame, u32 plane_idx, u32
 		GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[NVDec] failed to push CUDA CTX %s\n", cudaGetErrorEnum(res) ) );
 	}
 
+	if (ctx->reset_pbo) {
+		ctx->reset_pbo = GF_FALSE;
+		cuGLUnregisterBufferObject(ctx->y_pbo_id);
+		cuGLUnregisterBufferObject(ctx->uv_pbo_id);
+		glDeleteBuffers(1, &ctx->y_pbo_id);
+		glDeleteBuffers(1, &ctx->uv_pbo_id);
+		ctx->y_pbo_id = ctx->uv_pbo_id = 0;
+		glDeleteTextures(1, &ctx->y_tx_id);
+		glDeleteTextures(1, &ctx->uv_tx_id);
+		ctx->y_tx_id = ctx->uv_tx_id = 0;
+	}
+
 	if (! *gl_tex_id && ! plane_idx && ctx->y_tx_id ) {
 		cuGLUnregisterBufferObject(ctx->y_pbo_id);
+		glDeleteBuffers(1, &ctx->y_pbo_id);
 		ctx->y_pbo_id = 0;
+		glDeleteTextures(1, &ctx->y_tx_id);
 		ctx->y_tx_id = 0;
 	}
 	if (! *gl_tex_id && plane_idx && ctx->uv_tx_id ) {
 		cuGLUnregisterBufferObject(ctx->uv_pbo_id);
+		glDeleteBuffers(1, &ctx->uv_pbo_id);
 		ctx->uv_pbo_id = 0;
+		glDeleteTextures(1, &ctx->uv_tx_id);
 		ctx->uv_tx_id = 0;
 	}
 
@@ -1102,7 +1191,7 @@ GF_Err nvframe_get_gl_texture(GF_FilterFrameInterface *frame, u32 plane_idx, u32
 		gl_fmt = GL_LUMINANCE_ALPHA;
 	}
 
-	cuGLMapBufferObject(&tx_data, &tx_pitch, pbo_id);
+	res = cuGLMapBufferObject(&tx_data, &tx_pitch, pbo_id);
 	if (res != CUDA_SUCCESS) {
 		GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[NVDec] failed to map GL texture data %s\n", cudaGetErrorEnum(res) ) );
 		return GF_IO_ERR;
@@ -1285,14 +1374,9 @@ GF_Err nvframe_get_frame(GF_FilterFrameInterface *frame, u32 plane_idx, const u8
 }
 
 
-GF_Err nvdec_send_hw_frame(NVDecCtx *ctx)
+GF_Err nvdec_send_hw_frame(NVDecCtx *ctx, NVDecFrame *f)
 {
 	GF_FilterPacket *dst_pck;
-	NVDecFrame *f;
-
-	if (!ctx->pending_frame) return GF_BAD_PARAM;
-	f = ctx->pending_frame;
-	ctx->pending_frame = NULL;
 
 	f->gframe.user_data = f;
 	f->gframe.get_plane = nvframe_get_frame;
@@ -1300,6 +1384,8 @@ GF_Err nvdec_send_hw_frame(NVDecCtx *ctx)
 	f->gframe.get_gl_texture = nvframe_get_gl_texture;
 #endif
 
+	ctx->nb_out_frames_pending++;
+
 	if (ctx->frame_size_changed) {
 		ctx->frame_size_changed = GF_FALSE;
 		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_WIDTH, &PROP_UINT(ctx->width));
@@ -1313,7 +1399,8 @@ GF_Err nvdec_send_hw_frame(NVDecCtx *ctx)
 		f->gframe.flags = GF_FRAME_IFCE_BLOCKING;
 
 	dst_pck = gf_filter_pck_new_frame_interface(ctx->opid, &f->gframe, nvframe_release);
-
+	if (!dst_pck) return GF_OUT_OF_MEM;
+	
 	nvdec_merge_pck_props(ctx, f, dst_pck);
 	if (gf_filter_pck_get_seek_flag(dst_pck)) {
 		gf_filter_pck_discard(dst_pck);
@@ -1457,12 +1544,12 @@ static const GF_FilterArgs NVDecArgs[] =
 	{ OFFS(vmode), "video decoder backend\n"
 		"- cuvid: use dedicated video engines directly\n"
 		"- cuda: use a CUDA-based decoder if faster than dedicated engines\n"
-		"- dxva: go through DXVA internally if possible (requires D3D9 interop)", GF_PROP_UINT, "cuvid", "cuvid|cuda|dxva", GF_FS_ARG_HINT_ADVANCED },
+		"- dxva: go through DXVA internally if possible (requires D3D9)", GF_PROP_UINT, "cuvid", "cuvid|cuda|dxva", GF_FS_ARG_HINT_ADVANCED },
 	{ OFFS(fmode), "frame output mode\n"
 		"- copy: each frame is copied and dispatched\n"
-		"- single: frame data is only retrieved when used, single memory space for all frames (not safe if multiple consummers)\n"
+		"- single: frame data is only retrieved when used, single memory space for all frames (not safe if multiple consumers)\n"
 		"- gl: frame data is mapped to an OpenGL texture"
-	, GF_PROP_UINT, "gl", "copy|single|gl|", 0 },
+	, GF_PROP_UINT, "gl", "copy|single|gl", 0 },
 
 	{ 0 }
 };
@@ -1470,7 +1557,7 @@ static const GF_FilterArgs NVDecArgs[] =
 GF_FilterRegister NVDecRegister = {
 	.name = "nvdec",
 	GF_FS_SET_DESCRIPTION("NVidia decoder")
-	GF_FS_SET_HELP("This filter decodes MPEG-2, MPEG-4 Part 2, AVC|H264 and HEVC streams through NVideia decoder. It allows GPU frame dispatch or direct frame copy.")
+	GF_FS_SET_HELP("This filter decodes MPEG-2, MPEG-4 Part 2, AVC|H264 and HEVC streams through NVidia decoder. It allows GPU frame dispatch or direct frame copy.")
 	.private_size = sizeof(NVDecCtx),
 	SETCAPS(NVDecCaps),
 	.flags = GF_FS_REG_CONFIGURE_MAIN_THREAD,
@@ -1485,6 +1572,11 @@ GF_FilterRegister NVDecRegister = {
 
 const GF_FilterRegister *nvdec_register(GF_FilterSession *session)
 {
+	//check if nvdec is not globally blacklisted - if so, do not try to load CUDA SDK which may be time consuming on some devices
+	const char *blacklist = gf_opts_get_key("core", "blacklist");
+	if (blacklist && (blacklist[0]!='-') && strstr(blacklist, "nvdec"))
+		return NULL;
+
 	init_cuda_sdk();
 	//do not register if no SDK
 	if (cuvid_load_state != 2) {
@@ -1503,6 +1595,4 @@ const GF_FilterRegister *nvdec_register(GF_FilterSession *session)
 {
 	return NULL;
 }
-#endif // defined(WIN32) || defined(GPAC_CONFIG_LINUX) || defined(GPAC_CONFIG_DARWIN)
-
-
+#endif // (!defined(GPAC_STATIC_BUILD) && (defined(WIN32) || defined(GPAC_CONFIG_LINUX) || defined(GPAC_CONFIG_DARWIN)) && !defined(GPAC_DISABLE_NVDEC))
