@@ -20,7 +20,7 @@
 #include <gpac/tools.h>
 
 
-#if (defined(WIN32) || defined(GPAC_CONFIG_LINUX) || defined(GPAC_CONFIG_DARWIN))
+#if ((defined(WIN32) || defined(GPAC_CONFIG_LINUX) || defined(GPAC_CONFIG_DARWIN)) && !defined(GPAC_DISABLE_NVDEC))
 
 #include "dec_nvdec_sdk.h"
 
@@ -230,8 +230,8 @@ static CUresult LOAD_LIBRARY_CUDA(CUDADRIVER *pInstance)
 {
     *pInstance = LoadLibrary(__CudaLibName);
     if (*pInstance == NULL) {
-		GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[NVDec] LoadLibrary \"%s\" failed!\n", __CudaLibName));
-        return CUDA_ERROR_UNKNOWN;
+		GF_LOG(GF_LOG_INFO, GF_LOG_CODEC, ("[NVDec] LoadLibrary \"%s\" failed!\n", __CudaLibName));
+        return CUDA_ERROR_SHARED_OBJECT_INIT_FAILED;
     }
     return CUDA_SUCCESS;
 }
@@ -240,7 +240,7 @@ static CUresult LOAD_LIBRARY_CUVID(CUVIDDRIVER *pInstance)
 {
     *pInstance = LoadLibrary(__CuvidLibName);
     if (*pInstance == NULL) {
-		GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[NVDec] LoadLibrary \"%s\" failed!\n", __CuvidLibName));
+		GF_LOG(GF_LOG_INFO, GF_LOG_CODEC, ("[NVDec] LoadLibrary \"%s\" failed!\n", __CuvidLibName));
         return CUDA_ERROR_UNKNOWN;
     }
     return CUDA_SUCCESS;
@@ -249,14 +249,14 @@ static CUresult LOAD_LIBRARY_CUVID(CUVIDDRIVER *pInstance)
 #define GET_PROC_EX(name, alias, required)                     \
     alias = (t##name *)GetProcAddress(curr_lib, #name);               \
     if (alias == NULL && required) {                                    \
-        GF_LOG(GF_LOG_WARNING, GF_LOG_MMIO, ("[NVDec] Failed to find required function \"%s\" \n",       \
+        GF_LOG(GF_LOG_WARNING, GF_LOG_CODEC, ("[NVDec] Failed to find required function \"%s\" \n",       \
                #name));                                  \
         }
 
 #define GET_PROC_EX_V2(name, alias, required)                           \
     alias = (t##name *)GetProcAddress(curr_lib, STRINGIFY(name##_v2));\
     if (alias == NULL && required) {                                    \
-        GF_LOG(GF_LOG_WARNING, GF_LOG_MMIO, ("[NVDec] Failed to find required function \"%s\" \n",       \
+        GF_LOG(GF_LOG_WARNING, GF_LOG_CODEC, ("[NVDec] Failed to find required function \"%s\" \n",       \
                STRINGIFY(name##_v2)));                       \
         }
 
@@ -278,7 +278,7 @@ static CUresult LOAD_LIBRARY_CUDA(CUDADRIVER *pInstance)
 {
     *pInstance = dlopen(__CudaLibName, RTLD_NOW);
     if (*pInstance == NULL) {
-        GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[NVDec] dlopen \"%s\" failed!\n", __CudaLibName));
+        GF_LOG(GF_LOG_INFO, GF_LOG_CODEC, ("[NVDec] dlopen \"%s\" failed!\n", __CudaLibName));
         return CUDA_ERROR_SHARED_OBJECT_INIT_FAILED;
     }
     return CUDA_SUCCESS;
@@ -288,7 +288,7 @@ static CUresult LOAD_LIBRARY_CUVID(CUVIDDRIVER *pInstance)
 {
     *pInstance = dlopen(__CuvidLibName, RTLD_NOW);
     if (*pInstance == NULL) {
-		GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[NVDec]dlopen \"%s\" failed!\n", __CuvidLibName));
+		GF_LOG(GF_LOG_INFO, GF_LOG_CODEC, ("[NVDec]dlopen \"%s\" failed!\n", __CuvidLibName));
         return CUDA_ERROR_UNKNOWN;
     }
     return CUDA_SUCCESS;
@@ -297,14 +297,14 @@ static CUresult LOAD_LIBRARY_CUVID(CUVIDDRIVER *pInstance)
 #define GET_PROC_EX(name, alias, required)                              \
     alias = (t##name *)dlsym(curr_lib, #name);                        \
     if (alias == NULL && required) {                                    \
-        GF_LOG(GF_LOG_WARNING, GF_LOG_MMIO, ("[NVDec] Failed to find required function \"%s\"\n",       \
+        GF_LOG(GF_LOG_WARNING, GF_LOG_CODEC, ("[NVDec] Failed to find required function \"%s\"\n",       \
                #name));                                  \
         }
 
 #define GET_PROC_EX_V2(name, alias, required)                           \
     alias = (t##name *)dlsym(curr_lib, STRINGIFY(name##_v2));         \
     if (alias == NULL && required) {                                    \
-        GF_LOG(GF_LOG_WARNING, GF_LOG_MMIO, ("[NVDec]Failed to find required function \"%s\"\n",       \
+        GF_LOG(GF_LOG_WARNING, GF_LOG_CODEC, ("[NVDec]Failed to find required function \"%s\"\n",       \
                STRINGIFY(name##_v2)));                    \
         }
 
@@ -580,7 +580,7 @@ CUresult CUDAAPI cuInit(unsigned int Flags, int cudaVersion)
         GET_PROC(cuLaunchKernel);
     }
 
-	/*load openGL*/
+	/*load OpenGL*/
     if (cudaVersion >= 2010)
     {
         GET_PROC(cuGLCtxCreate);
@@ -709,4 +709,4 @@ const char *cudaGetErrorEnum(CUresult error)
 	return "<unknown>";
 }
 
-#endif //WIN32
+#endif // ((defined(WIN32) || defined(GPAC_CONFIG_LINUX) || defined(GPAC_CONFIG_DARWIN)) && !defined(GPAC_DISABLE_NVDEC))
