@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2010-2017
+ *			Copyright (c) Telecom ParisTech 2010-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / OpenSVC Decoder filter
@@ -74,7 +74,7 @@ typedef struct
 static GF_Err osvcdec_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_remove)
 {
 	Bool found=GF_TRUE;
-	const GF_PropertyValue *p;
+	const GF_PropertyValue *p, *dsi_enh;
 	u32 i, count, dep_id=0, id=0, cfg_crc=0;
 	s32 res;
 	OPENSVCFRAME Picture;
@@ -83,8 +83,10 @@ static GF_Err osvcdec_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool i
 	if (is_remove) {
 		if (ctx->streams[0].ipid == pid) {
 			memset(ctx->streams, 0, SVC_MAX_STREAMS*sizeof(GF_SVCStream));
-			if (ctx->opid) gf_filter_pid_remove(ctx->opid);
-			ctx->opid = NULL;
+			if (ctx->opid) {
+				gf_filter_pid_remove(ctx->opid);
+				ctx->opid = NULL;
+			}
 			ctx->nb_streams = ctx->active_streams = 0;
 			if (ctx->codec) SVCDecoder_close(ctx->codec);
 			ctx->codec = NULL;
@@ -120,6 +122,7 @@ static GF_Err osvcdec_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool i
 			if ((ctx->streams[i].ipid == pid) && (ctx->streams[i].cfg_crc == cfg_crc)) return GF_OK;
 		}
 	}
+	dsi_enh = gf_filter_pid_get_property(pid, GF_PROP_PID_DECODER_CONFIG_ENHANCEMENT);
 
 	found = GF_FALSE;
 	for (i=0; i<ctx->active_streams; i++) {
@@ -187,6 +190,7 @@ static GF_Err osvcdec_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool i
 			if (SVCDecoder_init(&ctx->codec) == SVC_STATUS_ERROR) return GF_IO_ERR;
 		}
 
+retry:
 		/*decode all NALUs*/
 		count = gf_list_count(cfg->sequenceParameterSets);
 		SetCommandLayer(ctx->layers, 255, 0, &res, 0);//bufindex can be reset without pb
@@ -196,7 +200,7 @@ static GF_Err osvcdec_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool i
 			u32 sid;
 			s32 par_n=0, par_d=0;
 #endif
-			GF_AVCConfigSlot *slc = (GF_AVCConfigSlot*)gf_list_get(cfg->sequenceParameterSets, i);
+			GF_NALUFFParam *slc = (GF_NALUFFParam*)gf_list_get(cfg->sequenceParameterSets, i);
 
 #ifndef GPAC_DISABLE_AV_PARSERS
 			gf_avc_get_sps_info(slc->data, slc->size, &sid, &w, &h, &par_n, &par_d);
@@ -223,7 +227,7 @@ static GF_Err osvcdec_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool i
 
 		count = gf_list_count(cfg->pictureParameterSets);
 		for (i=0; i<count; i++) {
-			GF_AVCConfigSlot *slc = (GF_AVCConfigSlot*)gf_list_get(cfg->pictureParameterSets, i);
+			GF_NALUFFParam *slc = (GF_NALUFFParam*)gf_list_get(cfg->pictureParameterSets, i);
 #ifndef GPAC_DISABLE_AV_PARSERS
 			u32 sps_id, pps_id;
 			gf_avc_get_pps_info(slc->data, slc->size, &pps_id, &sps_id);
@@ -238,6 +242,12 @@ static GF_Err osvcdec_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool i
 
 		}
 		gf_odf_avc_cfg_del(cfg);
+
+		if (dsi_enh) {
+			cfg = gf_odf_avc_cfg_read(dsi_enh->value.data.ptr, dsi_enh->value.data.size);
+			dsi_enh = NULL;
+			if (cfg) goto retry;
+		}
 	} else {
 		if (ctx->nalu_size_length) {
 			return GF_NOT_SUPPORTED;
@@ -374,7 +384,8 @@ static GF_Err osvcdec_process(GF_Filter *filter)
 	if (!src_pck)
 		gf_list_add(ctx->src_packets, pck_ref);
 
-	pic.Width = pic.Height = 0;
+	memset(&pic, 0, sizeof(OPENSVCFRAME));
+
 	for (idx=0; idx<ctx->nb_streams; idx++) {
 		u64 dts, cts;
 #ifndef GPAC_DISABLE_AV_PARSERS
@@ -503,7 +514,7 @@ static GF_Err osvcdec_process(GF_Filter *filter)
 
 		if (got_pic) has_pic = GF_TRUE;
 	}
-	if (!has_pic) return GF_OK;
+	if (!has_pic || !pic.Width || !pic.Height) return GF_OK;
 
 	if ((pic.Width != ctx->width) || (pic.Height!=ctx->height)) {
 		GF_LOG(GF_LOG_INFO, GF_LOG_CODEC, ("[SVC Decoder] Resizing from %dx%d to %dx%d\n", ctx->width, ctx->height, pic.Width, pic.Height ));
@@ -529,6 +540,8 @@ static GF_Err osvcdec_process(GF_Filter *filter)
 	}
 
 	dst_pck = gf_filter_pck_new_alloc(ctx->opid, ctx->out_size, &data);
+	if (!dst_pck) return GF_OUT_OF_MEM;
+
 	memcpy(data, pic.pY[0], ctx->stride*ctx->height);
 	memcpy(data + ctx->stride * ctx->height, pic.pU[0], ctx->stride*ctx->height/4);
 	memcpy(data + 5*ctx->stride * ctx->height/4, pic.pV[0], ctx->stride*ctx->height/4);
@@ -536,7 +549,9 @@ static GF_Err osvcdec_process(GF_Filter *filter)
 	if (src_pck) {
 		gf_filter_pck_merge_properties(src_pck, dst_pck);
 		gf_filter_pck_unref(src_pck);
+		gf_filter_pck_set_dependency_flags(dst_pck, 0);
 	}
+	gf_filter_pck_set_dts(dst_pck, gf_filter_pck_get_cts(dst_pck));
 
 	GF_LOG(GF_LOG_DEBUG, GF_LOG_CODEC, ("[OpenSVC] decoded out frame PTS "LLU"\n", gf_filter_pck_get_cts(dst_pck) ));
 	gf_filter_pck_send(dst_pck);
