@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2000-2017
+ *			Copyright (c) Telecom ParisTech 2000-2021
  *					All rights reserved
  *
  *  This file is part of GPAC / 3GPP/MPEG4 text renderer filter
@@ -212,8 +212,11 @@ static GF_Err ttd_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_re
 	const GF_PropertyValue *p, *dsi;
 
 	if (is_remove) {
-		if (ctx->opid) gf_filter_pid_remove(ctx->opid);
-		ctx->opid = ctx->ipid = NULL;
+		if (ctx->opid) {
+			gf_filter_pid_remove(ctx->opid);
+			ctx->opid = NULL;
+		}
+		ctx->ipid = NULL;
 		return GF_OK;
 	}
 	//TODO: we need to cleanup cap checking upon reconfigure
@@ -252,6 +255,9 @@ static GF_Err ttd_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_re
 			ctx->cfg = NULL;
 			return GF_NON_COMPLIANT_BITSTREAM;
 		}
+		if (!sd->default_style.text_color)
+			sd->default_style.text_color = 0xFFFFFFFF;
+
 		gf_list_add(ctx->cfg->sample_descriptions, sd);
 		ctx->is_tx3g = GF_TRUE;
 	}
@@ -511,6 +517,16 @@ static void ttd_new_text_chunk(GF_TTXTDec *ctx, GF_TextSampleDescriptor *tsd, M_
 		fontSize = tsd->default_style.font_size;
 		styleFlags = tsd->default_style.style_flags;
 		color = tsd->default_style.text_color;
+		if (!color && !tsd->back_color)
+			color = 0xFFFFFFFF;
+		if (!fontSize) {
+			if (ctx->cfg->text_height > 2000)
+				fontSize = 80;
+			else if (ctx->cfg->text_height > 700)
+				fontSize = 40;
+			else
+				fontSize = 20;
+		}
 	} else {
 		fontName = ttd_find_font(tsd, tc->srec->fontID);
 		fontSize = tc->srec->font_size;
@@ -563,6 +579,7 @@ static void ttd_new_text_chunk(GF_TTXTDec *ctx, GF_TextSampleDescriptor *tsd, M_
 	if (!strlen(szStyle)) strcpy(szStyle, "PLAIN");
 	/*also underline for URLs*/
 	if ((styleFlags & GF_TXT_STYLE_UNDERLINED) || (tc->hlink && tc->hlink->URL)) strcat(szStyle, " UNDERLINED");
+	if (styleFlags & GF_TXT_STYLE_STRIKETHROUGH) strcat(szStyle, " STRIKETHROUGH");
 
 	if (tc->is_hilight) {
 		if (tc->hilight_col) {
@@ -639,7 +656,8 @@ static void ttd_new_text_chunk(GF_TTXTDec *ctx, GF_TextSampleDescriptor *tsd, M_
 				memcpy(wsChunk, &utf16_txt[start_char], sizeof(s16)*(i-start_char));
 				wsChunk[i-start_char] = 0;
 				sp = &wsChunk[0];
-				len = (u32) gf_utf8_wcstombs(szLine, 5000, (const unsigned short **) &sp);
+				len = gf_utf8_wcstombs(szLine, 5000, (const unsigned short **) &sp);
+				if (len == GF_UTF8_FAIL) len = 0;
 				szLine[len] = 0;
 
 				gf_sg_vrml_mf_append(&text->string, GF_SG_VRML_MFSTRING, (void **) &st);
@@ -901,7 +919,8 @@ static void ttd_apply_sample(GF_TTXTDec *ctx, GF_TextSample *txt, u32 sample_des
 		char_count = txt->len / 2;
 	} else {
 		char *p = txt->text;
-		char_count = (u32) gf_utf8_mbstowcs(utf16_text, 2500, (const char **) &p);
+		char_count = gf_utf8_mbstowcs(utf16_text, 2500, (const char **) &p);
+		if (char_count == GF_UTF8_FAIL) char_count = 0;
 	}
 
 	chunks = gf_list_new();
@@ -1230,7 +1249,7 @@ static GF_Err ttd_process(GF_Filter *filter)
 
 	//we still process any frame before our clock time even when buffering
 	obj_time = gf_clock_time(ctx->odm->ck);
-	if (cts * 1000 > obj_time * timescale) {
+	if (gf_timestamp_greater(cts, timescale, obj_time, 1000)) {
 		Double ts_offset = (Double) cts;
 		ts_offset /= timescale;
 
