@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2005-2020
+ *			Copyright (c) Telecom ParisTech 2005-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / M2TS demux filter
@@ -59,7 +59,7 @@ typedef struct
 {
 	//opts
 	const char *temi_url;
-	Bool dsmcc, seeksrc;
+	Bool dsmcc, seeksrc, sigfrag;
 
 	GF_Filter *filter;
 	GF_FilterPid *ipid;
@@ -85,6 +85,8 @@ typedef struct
 
 	u32 mux_tune_state;
 	u32 wait_for_progs;
+
+	Bool is_dash;
 } GF_M2TSDmxCtx;
 
 
@@ -160,14 +162,40 @@ static void m2tsdmx_on_event_duration_probe(GF_M2TS_Demuxer *ts, u32 evt_type, v
 	}
 }
 
+static void m2tsdmx_update_sdt(GF_M2TS_Demuxer *ts, void *for_pid)
+{
+	u32 i, count = gf_list_count(ts->programs);
+	for (i=0; i<count; i++) {
+		u32 j, nb_streams;
+		GF_M2TS_Program *prog = gf_list_get(ts->programs, i);
+		GF_M2TS_SDT *sdt = gf_m2ts_get_sdt_info(ts, prog->number);
+		if (!sdt) continue;
+
+		nb_streams = gf_list_count(prog->streams);
+		for (j=0; j<nb_streams; j++) {
+			GF_M2TS_ES *es = gf_list_get(prog->streams, j);
+			if (!es->user) continue;
+			if (for_pid && (es->user != for_pid)) continue;
+			//TODO, translate non standard character maps to UTF8
+			//we for now comment in test mode to avoid non UTF characters in text dumps
+			if (isalnum(sdt->service[0]) || !gf_sys_is_test_mode())
+				gf_filter_pid_set_property((GF_FilterPid *)es->user, GF_PROP_PID_SERVICE_NAME, &PROP_STRING(sdt->service ) );
+
+			if (isalnum(sdt->provider[0]) || !gf_sys_is_test_mode())
+				gf_filter_pid_set_property((GF_FilterPid *)es->user, GF_PROP_PID_SERVICE_PROVIDER, &PROP_STRING( sdt->provider ) );
+		}
+	}
+}
+
 static void m2tsdmx_declare_pid(GF_M2TSDmxCtx *ctx, GF_M2TS_PES *stream, GF_ESD *esd)
 {
-	u32 i, count, codecid=0, stype=0;
+	u32 i, count, codecid=0, stype=0, orig_stype=0;
 	GF_FilterPid *opid;
 	Bool m4sys_stream = GF_FALSE;
 	Bool m4sys_iod_stream = GF_FALSE;
 	Bool has_scal_layer = GF_FALSE;
 	Bool unframed = GF_FALSE;
+	Bool unframed_latm = GF_FALSE;
 	char szName[20];
 	const char *stname;
 	if (stream->user) return;
@@ -206,6 +234,12 @@ static void m2tsdmx_declare_pid(GF_M2TSDmxCtx *ctx, GF_M2TS_PES *stream, GF_ESD
 			if (stream->program->is_scalable)
 				has_scal_layer = GF_TRUE;
 			break;
+		case GF_M2TS_HLS_AVC_CRYPT:
+			stype = GF_STREAM_ENCRYPTED;
+			orig_stype = GF_STREAM_VISUAL;
+			codecid = GF_CODECID_AVC;
+			unframed = GF_TRUE;
+			break;
 		case GF_M2TS_VIDEO_SVC:
 			stype = GF_STREAM_VISUAL;
 			codecid = GF_CODECID_SVC;
@@ -229,6 +263,16 @@ static void m2tsdmx_declare_pid(GF_M2TSDmxCtx *ctx, GF_M2TS_PES *stream, GF_ESD
 			codecid = GF_CODECID_LHVC;
 			has_scal_layer = GF_TRUE;
 			break;
+		case GF_M2TS_VIDEO_VVC:
+		case GF_M2TS_VIDEO_VVC_TEMPORAL:
+			stype = GF_STREAM_VISUAL;
+			codecid = GF_CODECID_VVC;
+			unframed = GF_TRUE;
+			break;
+		case GF_M2TS_VIDEO_VC1:
+			stype = GF_STREAM_VISUAL;
+			codecid = GF_CODECID_SMPTE_VC1;
+			break;
 		case GF_M2TS_AUDIO_MPEG1:
 			stype = GF_STREAM_AUDIO;
 			codecid = GF_CODECID_MPEG_AUDIO;
@@ -239,7 +283,6 @@ static void m2tsdmx_declare_pid(GF_M2TSDmxCtx *ctx, GF_M2TS_PES *stream, GF_ESD
 			codecid = GF_CODECID_MPEG2_PART3;
 			unframed = GF_TRUE;
 			break;
-		case GF_M2TS_AUDIO_LATM_AAC:
 		case GF_M2TS_AUDIO_AAC:
 		case GF_CODECID_AAC_MPEG2_MP:
 		case GF_CODECID_AAC_MPEG2_LCP:
@@ -248,6 +291,19 @@ static void m2tsdmx_declare_pid(GF_M2TSDmxCtx *ctx, GF_M2TS_PES *stream, GF_ESD
 			codecid = GF_CODECID_AAC_MPEG4;
 			unframed = GF_TRUE;
 			break;
+		case GF_M2TS_AUDIO_LATM_AAC:
+			stype = GF_STREAM_AUDIO;
+			codecid = GF_CODECID_AAC_MPEG4;
+			unframed = GF_TRUE;
+			unframed_latm = GF_TRUE;
+			break;
+
+		case GF_M2TS_MHAS_MAIN:
+		case GF_M2TS_MHAS_AUX:
+			stype = GF_STREAM_AUDIO;
+			codecid = GF_CODECID_MHAS;
+			unframed = GF_TRUE;
+			break;
 		case GF_M2TS_AUDIO_AC3:
 			stype = GF_STREAM_AUDIO;
 			codecid = GF_CODECID_AC3;
@@ -257,6 +313,14 @@ static void m2tsdmx_declare_pid(GF_M2TSDmxCtx *ctx, GF_M2TS_PES *stream, GF_ESD
 			stype = GF_STREAM_AUDIO;
 			codecid = GF_CODECID_EAC3;
 			break;
+		case GF_M2TS_AUDIO_DTS:
+			stype = GF_STREAM_AUDIO;
+			codecid = GF_CODECID_DTS_X;
+			break;
+		case GF_M2TS_AUDIO_OPUS:
+			stype = GF_STREAM_AUDIO;
+			codecid = GF_CODECID_OPUS;
+			break;
 		case GF_M2TS_SYSTEMS_MPEG4_SECTIONS:
 			((GF_M2TS_ES*)stream)->flags |= GF_M2TS_ES_SEND_REPEATED_SECTIONS;
 			//fallthrough
@@ -283,13 +347,42 @@ static void m2tsdmx_declare_pid(GF_M2TSDmxCtx *ctx, GF_M2TS_PES *stream, GF_ESD
 			stype = GF_STREAM_AUDIO;
 			codecid = GF_CODECID_EAC3;
 			break;
+
+		case GF_M2TS_HLS_AAC_CRYPT:
+			stype = GF_STREAM_ENCRYPTED;
+			orig_stype = GF_STREAM_AUDIO;
+			codecid = GF_CODECID_AAC_MPEG4;
+			unframed = GF_TRUE;
+			break;
+		case GF_M2TS_HLS_AC3_CRYPT:
+			stype = GF_STREAM_ENCRYPTED;
+			orig_stype = GF_STREAM_AUDIO;
+			codecid = GF_CODECID_AC3;
+			unframed = GF_TRUE;
+			break;
+		case GF_M2TS_HLS_EC3_CRYPT:
+			stype = GF_STREAM_ENCRYPTED;
+			orig_stype = GF_STREAM_AUDIO;
+			codecid = GF_CODECID_EAC3;
+			break;
 		default:
-			GF_LOG(GF_LOG_WARNING, GF_LOG_CONTAINER, ("[M2TSDmx] Stream type %d not supported - ignoring pid\n", stream->stream_type));
+			GF_LOG(GF_LOG_WARNING, GF_LOG_CONTAINER, ("[M2TSDmx] Stream type 0x%02X not supported - ignoring pid\n", stream->stream_type));
 			return;
 		}
 	}
 
-	opid = gf_filter_pid_new(ctx->filter);
+	opid = NULL;
+	for (i=0; i<gf_filter_get_opid_count(ctx->filter); i++) {
+		opid = gf_filter_get_opid(ctx->filter, i);
+		const GF_PropertyValue *p = gf_filter_pid_get_property(opid, GF_PROP_PID_ID);
+		if (p && (p->value.uint == stream->pid))
+			break;
+		opid = NULL;
+	}
+
+	if (!opid)
+		opid = gf_filter_pid_new(ctx->filter);
+
 	stream->user = opid;
 	stream->flags |= GF_M2TS_ES_ALREADY_DECLARED;
 
@@ -298,8 +391,7 @@ static void m2tsdmx_declare_pid(GF_M2TSDmxCtx *ctx, GF_M2TS_PES *stream, GF_ESD
 	gf_filter_pid_set_name(opid, szName);
 
 	gf_filter_pid_set_property(opid, GF_PROP_PID_ID, &PROP_UINT(stream->pid) );
-	if (stream->mpeg4_es_id)
-		gf_filter_pid_set_property(opid, GF_PROP_PID_ESID, &PROP_UINT(stream->mpeg4_es_id) );
+	gf_filter_pid_set_property(opid, GF_PROP_PID_ESID, stream->mpeg4_es_id ? &PROP_UINT(stream->mpeg4_es_id) : NULL);
 
 	if (m4sys_stream) {
 		if (stream->slcfg) gf_free(stream->slcfg);
@@ -322,20 +414,39 @@ static void m2tsdmx_declare_pid(GF_M2TSDmxCtx *ctx, GF_M2TS_PES *stream, GF_ESD
 	} else {
 		gf_filter_pid_set_property(opid, GF_PROP_PID_STREAM_TYPE, &PROP_UINT(stype) );
 		gf_filter_pid_set_property(opid, GF_PROP_PID_CODECID, &PROP_UINT(codecid) );
-		if (unframed)
-			gf_filter_pid_set_property(opid, GF_PROP_PID_UNFRAMED, &PROP_BOOL(GF_TRUE) );
+
+		gf_filter_pid_set_property(opid, GF_PROP_PID_UNFRAMED, unframed ? &PROP_BOOL(GF_TRUE) : NULL);
+		gf_filter_pid_set_property(opid, GF_PROP_PID_UNFRAMED_LATM, unframed_latm ? &PROP_BOOL(GF_TRUE) : NULL );
+
+		if (orig_stype) {
+			gf_filter_pid_set_property(opid, GF_PROP_PID_ORIG_STREAM_TYPE, &PROP_UINT(orig_stype) );
+			gf_filter_pid_set_property(opid, GF_PROP_PID_PROTECTION_SCHEME_TYPE, &PROP_UINT(GF_HLS_SAMPLE_AES_SCHEME) );
+		} else {
+			gf_filter_pid_set_property(opid, GF_PROP_PID_ORIG_STREAM_TYPE, NULL);
+			gf_filter_pid_set_property(opid, GF_PROP_PID_PROTECTION_SCHEME_TYPE, NULL);
+		}
 
 		gf_filter_pid_set_property(opid, GF_PROP_PID_TIMESCALE, &PROP_UINT(90000) );
 		gf_filter_pid_set_property(opid, GF_PROP_PID_CLOCK_ID, &PROP_UINT(stream->program->pcr_pid) );
 	}
-	if (has_scal_layer)
-		gf_filter_pid_set_property(opid, GF_PROP_PID_SCALABLE, &PROP_BOOL(GF_TRUE));
+	gf_filter_pid_set_property(opid, GF_PROP_PID_SCALABLE, has_scal_layer ? &PROP_BOOL(GF_TRUE) : NULL);
 
 	gf_filter_pid_set_property(opid, GF_PROP_PID_SERVICE_ID, &PROP_UINT(stream->program->number) );
 
+	if (stream->lang) {
+		char szLang[4];
+		szLang[0] = (stream->lang>>16) & 0xFF;
+		szLang[1] = (stream->lang>>8) & 0xFF;
+		szLang[2] = stream->lang & 0xFF;
+		szLang[3] = 0;
+		if (szLang[2]==' ') szLang[2] = 0;
+		gf_filter_pid_set_property(opid, GF_PROP_PID_LANGUAGE, &PROP_STRING(szLang) );
+	}
+
+
 	if (ctx->duration.num>1) {
 		gf_filter_pid_set_property(opid, GF_PROP_PID_DURATION, &PROP_FRAC64(ctx->duration) );
-		gf_filter_pid_set_property(opid, GF_PROP_PID_PLAYBACK_MODE, &PROP_UINT(GF_PLAYBACK_MODE_SEEK ) );
+		gf_filter_pid_set_property(opid, GF_PROP_PID_PLAYBACK_MODE, &PROP_UINT(GF_PLAYBACK_MODE_FASTFORWARD ) );
 	}
 	/*indicate our coding dependencies if any*/
 	if (!m4sys_stream) {
@@ -344,8 +455,28 @@ static void m2tsdmx_declare_pid(GF_M2TSDmxCtx *ctx, GF_M2TS_PES *stream, GF_ESD
 			if ((stream->stream_type == GF_M2TS_VIDEO_HEVC_TEMPORAL) || (stream->stream_type == GF_M2TS_VIDEO_HEVC_MCTS)) {
 				gf_filter_pid_set_property(opid, GF_PROP_PID_SUBLAYER, &PROP_BOOL(GF_TRUE) );
 			}
+		} else {
+			gf_filter_pid_set_property(opid, GF_PROP_PID_DEPENDENCY_ID, NULL);
+			gf_filter_pid_set_property(opid, GF_PROP_PID_SUBLAYER, NULL);
 		}
 	}
+
+	if (stream->dv_info[0]) {
+		gf_filter_pid_set_property(opid, GF_PROP_PID_DOLBY_VISION, &PROP_DATA(stream->dv_info, 24) );
+		u32 dvtype=0;
+		if (stream->dv_info[24]) {
+			if (stream->stream_type == GF_M2TS_VIDEO_H264)
+				dvtype = GF_4CC('d','a','v','1');
+			else
+				dvtype = GF_4CC('d','v','h','1');
+		}
+		gf_filter_pid_set_property(opid, GF_PROP_PID_ISOM_SUBTYPE, dvtype ? &PROP_4CC(dvtype) : NULL);
+	} else {
+		gf_filter_pid_set_property(opid, GF_PROP_PID_DOLBY_VISION, NULL);
+	}
+
+	m2tsdmx_update_sdt(ctx->ts, opid);
+
 	gf_m2ts_set_pes_framing((GF_M2TS_PES *)stream, GF_M2TS_PES_FRAMING_DEFAULT);
 }
 
@@ -354,18 +485,6 @@ static void m2tsdmx_setup_program(GF_M2TSDmxCtx *ctx, GF_M2TS_Program *prog)
 	u32 i, count;
 
 	count = gf_list_count(prog->streams);
-#ifdef FILTER_FIXME
-	if (ctx->ts->tuner) {
-		Bool found = 0;
-		for (i=0; i<count; i++) {
-			GF_M2TS_PES *pes = gf_list_get(prog->streams, i);
-			if (pes->pid==ctx->ts->tuner->vpid) found = 1;
-			else if (pes->pid==ctx->ts->tuner->apid) found = 1;
-		}
-		if (!found) return;
-	}
-#endif
-
 	for (i=0; i<count; i++) {
 		GF_M2TS_PES *es = gf_list_get(prog->streams, i);
 		if (es->pid==prog->pmt_pid) continue;
@@ -394,7 +513,7 @@ static void m2tsdmx_setup_program(GF_M2TSDmxCtx *ctx, GF_M2TS_Program *prog)
 	}
 }
 
-static void m2tdmx_merge_temi(GF_M2TS_ES *stream, GF_FilterPacket *pck)
+static void m2tdmx_merge_temi(GF_FilterPid *pid, GF_M2TS_ES *stream, GF_FilterPacket *pck)
 {
 	if (stream->props) {
 		char szID[100];
@@ -407,6 +526,12 @@ static void m2tdmx_merge_temi(GF_M2TS_ES *stream, GF_FilterPacket *pck)
 		}
 		gf_list_del(stream->props);
 		stream->props = NULL;
+
+		if (!(stream->flags & GF_M2TS_ES_TEMI_INFO)) {
+			stream->flags |= GF_M2TS_ES_TEMI_INFO;
+			gf_filter_pid_set_property(pid, GF_PROP_PID_HAS_TEMI, &PROP_BOOL(GF_TRUE) );
+		}
+		
 	}
 }
 
@@ -422,6 +547,8 @@ static void m2tsdmx_send_packet(GF_M2TSDmxCtx *ctx, GF_M2TS_PES_PCK *pck)
 	opid = pck->stream->user;
 
 	dst_pck = gf_filter_pck_new_alloc(opid, pck->data_len, &data);
+	if (!dst_pck) return;
+
 	memcpy(data, pck->data, pck->data_len);
 	//we don't have end of frame signaling
 	gf_filter_pck_set_framing(dst_pck, (pck->flags & GF_M2TS_PES_PCK_AU_START) ? GF_TRUE : GF_FALSE, GF_FALSE);
@@ -433,7 +560,12 @@ static void m2tsdmx_send_packet(GF_M2TSDmxCtx *ctx, GF_M2TS_PES_PCK *pck)
 		}
 		gf_filter_pck_set_sap(dst_pck, (pck->flags & GF_M2TS_PES_PCK_RAP) ? GF_FILTER_SAP_1 : GF_FILTER_SAP_NONE);
 	}
-	m2tdmx_merge_temi((GF_M2TS_ES *)pck->stream, dst_pck);
+	m2tdmx_merge_temi(opid, (GF_M2TS_ES *)pck->stream, dst_pck);
+
+	if (pck->stream->is_seg_start) {
+		pck->stream->is_seg_start = GF_FALSE;
+		gf_filter_pck_set_property(dst_pck, GF_PROP_PCK_CUE_START, &PROP_BOOL(GF_TRUE));
+	}
 	gf_filter_pck_send(dst_pck);
 }
 
@@ -474,6 +606,8 @@ static GFINLINE void m2tsdmx_send_sl_packet(GF_M2TSDmxCtx *ctx, GF_M2TS_SL_PCK *
 	}
 
 	dst_pck = gf_filter_pck_new_alloc(opid, pck->data_len - slh_len, &data);
+	if (!dst_pck) return;
+
 	memcpy(data, pck->data + slh_len, pck->data_len - slh_len);
 	start = end = GF_FALSE;
 	if (slc->useAccessUnitStartFlag && slh.accessUnitStartFlag) start = GF_TRUE;
@@ -491,7 +625,11 @@ static GFINLINE void m2tsdmx_send_sl_packet(GF_M2TSDmxCtx *ctx, GF_M2TS_SL_PCK *
 
 	gf_filter_pck_set_carousel_version(dst_pck, pck->version_number);
 
-	m2tdmx_merge_temi(pck->stream, dst_pck);
+	m2tdmx_merge_temi(opid, pck->stream, dst_pck);
+	if (pck->stream->is_seg_start) {
+		pck->stream->is_seg_start = GF_FALSE;
+		gf_filter_pck_set_property(dst_pck, GF_PROP_PCK_CUE_START, &PROP_BOOL(GF_TRUE));
+	}
 	gf_filter_pck_send(dst_pck);
 
 	if (pck->version_number + 1 == pck->stream->slcfg->carousel_version)
@@ -545,7 +683,7 @@ static GFINLINE void m2tsdmx_send_sl_packet(GF_M2TSDmxCtx *ctx, GF_M2TS_SL_PCK *
 	}
 }
 
-#ifdef FILTER_FIXME
+#if 0  //unused
 static void m2tsdmx_declare_epg_pid(GF_M2TSDmxCtx *ctx)
 {
 	assert(ctx->eit_pid == NULL);
@@ -558,6 +696,7 @@ static void m2tsdmx_declare_epg_pid(GF_M2TSDmxCtx *ctx)
 }
 #endif
 
+
 static void m2tsdmx_on_event(GF_M2TS_Demuxer *ts, u32 evt_type, void *param)
 {
 	u32 i, count;
@@ -596,35 +735,17 @@ static void m2tsdmx_on_event(GF_M2TS_Demuxer *ts, u32 evt_type, void *param)
 	case GF_M2TS_EVT_SDT_FOUND:
 	case GF_M2TS_EVT_SDT_UPDATE:
 //	case GF_M2TS_EVT_SDT_REPEAT:
-		count = gf_list_count(ts->programs);
-		for (i=0; i<count; i++) {
-			GF_M2TS_Program *prog = gf_list_get(ts->programs, i);
-			GF_M2TS_SDT *sdt = gf_m2ts_get_sdt_info(ts, prog->number);
-			if (sdt) {
-				u32 j, nb_streams;
-				nb_streams = gf_list_count(prog->streams);
-				for (j=0; j<nb_streams; j++) {
-					GF_M2TS_ES *es = gf_list_get(prog->streams, j);
-					if (es->user) {
-						//TODO, translate non standard character maps to UTF8
-						//we for now comment in test mode to avoid non UTF characters in text dumps
-						if (isalnum(sdt->service[0]) || !gf_sys_is_test_mode())
-							gf_filter_pid_set_property((GF_FilterPid *)es->user, GF_PROP_PID_SERVICE_NAME, &PROP_NAME( sdt->service ) );
-
-						if (isalnum(sdt->provider[0]) || !gf_sys_is_test_mode())
-							gf_filter_pid_set_property((GF_FilterPid *)es->user, GF_PROP_PID_SERVICE_PROVIDER, &PROP_NAME( sdt->provider ) );
-					}
-				}
-			}
-		}
+		m2tsdmx_update_sdt(ts, NULL);
 		break;
 	case GF_M2TS_EVT_DVB_GENERAL:
 		if (ctx->eit_pid) {
 			GF_M2TS_SL_PCK *pck = (GF_M2TS_SL_PCK *)param;
 			u8 *data;
 			GF_FilterPacket *dst_pck = gf_filter_pck_new_alloc(ctx->eit_pid, pck->data_len, &data);
-			memcpy(data, pck->data, pck->data_len);
-			gf_filter_pck_send(dst_pck);
+			if (dst_pck) {
+				memcpy(data, pck->data, pck->data_len);
+				gf_filter_pck_send(dst_pck);
+			}
 		}
 		break;
 	case GF_M2TS_EVT_PES_PCK:
@@ -660,8 +781,14 @@ static void m2tsdmx_on_event(GF_M2TS_Demuxer *ts, u32 evt_type, void *param)
 			if (!stream->user) continue;
 
 			dst_pck = gf_filter_pck_new_shared(stream->user, NULL, 0, NULL);
+			if (!dst_pck) continue;
+
 			gf_filter_pck_set_cts(dst_pck, pcr);
 			gf_filter_pck_set_clock_type(dst_pck, discontinuity ? GF_FILTER_CLOCK_PCR_DISC : GF_FILTER_CLOCK_PCR);
+			if (pck->stream->is_seg_start) {
+				pck->stream->is_seg_start = GF_FALSE;
+				gf_filter_pck_set_property(dst_pck, GF_PROP_PCK_CUE_START, &PROP_BOOL(GF_TRUE));
+			}
 			gf_filter_pck_send(dst_pck);
 
 			if (map_time) {
@@ -692,7 +819,7 @@ static void m2tsdmx_on_event(GF_M2TS_Demuxer *ts, u32 evt_type, void *param)
 					gf_filter_pid_set_info(stream->user, GF_PROP_PID_UTC_TIMESTAMP, & PROP_LONGUINT(prog->last_pcr_value / 300) );
 				}
 			}
-			GF_LOG(GF_LOG_INFO, GF_LOG_CONTAINER, ("[M2TS In] Mapping TDT Time %04d/%02d/%02d %02d:%02d:%02d and PCR time "LLD" on program %d\n",
+			GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[M2TS In] Mapping TDT Time %04d/%02d/%02d %02d:%02d:%02d and PCR time "LLD" on program %d\n",
 				                                       tdt->year, tdt->month, tdt->day, tdt->hour, tdt->minute, tdt->second, prog->last_pcr_value/300, prog->number));
 		}
 	}
@@ -750,7 +877,10 @@ static void m2tsdmx_on_event(GF_M2TS_Demuxer *ts, u32 evt_type, void *param)
 		gf_bs_write_int(bs, temi_l->is_splicing, 1);
 		gf_bs_write_int(bs, temi_l->reload_external, 1);
 		gf_bs_write_int(bs, 0, 5);
-		gf_bs_write_double(bs, temi_l->activation_countdown);
+		if (temi_l->is_announce) {
+			gf_bs_write_u32(bs, temi_l->activation_countdown.den);
+			gf_bs_write_u32(bs, temi_l->activation_countdown.num);
+		}
 		gf_bs_get_content(bs, &t->data, &t->len);
 		gf_bs_del(bs);
 
@@ -798,6 +928,19 @@ static void m2tsdmx_on_event(GF_M2TS_Demuxer *ts, u32 evt_type, void *param)
 		gf_list_add(es->props, t);
 	}
 	break;
+	case GF_M2TS_EVT_STREAM_REMOVED:
+	{
+		GF_M2TS_ES *es = (GF_M2TS_ES *)param;
+		if (es && es->props) {
+			while (gf_list_count(es->props)) {
+				GF_TEMIInfo *t = gf_list_pop_back(es->props);
+				gf_free(t->data);
+				gf_free(t);
+			}
+			gf_list_del(es->props);
+		}
+	}
+		break;
 	}
 }
 
@@ -809,7 +952,18 @@ static GF_Err m2tsdmx_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool i
 
 	if (is_remove) {
 		ctx->ipid = NULL;
-//		gf_filter_pid_remove(ctx->opid);
+		u32 i, count = gf_list_count(ctx->ts->programs);
+		for (i=0; i<count; i++) {
+			GF_M2TS_Program *prog = gf_list_get(ctx->ts->programs, i);
+			u32 j, count2 = gf_list_count(prog->streams);
+			for (j=0; j<count2; j++) {
+				GF_M2TS_ES *es = gf_list_get(prog->streams, j);
+				if (es->user) {
+					GF_FilterPid *opid = es->user;
+					gf_filter_pid_remove(opid);
+				}
+			}
+		}
 		return GF_OK;
 	}
 	if (! gf_filter_pid_check_caps(pid))
@@ -819,7 +973,7 @@ static GF_Err m2tsdmx_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool i
 	ctx->mux_tune_state = DMX_TUNE_DONE;
 
 	p = gf_filter_pid_get_property(pid, GF_PROP_PID_FILEPATH);
-	if (p && p->value.string && !ctx->duration.num) {
+	if (p && p->value.string && !ctx->duration.num && strncmp(p->value.string, "gmem://", 7)) {
 		stream = gf_fopen(p->value.string, "rb");
 	}
 
@@ -942,12 +1096,14 @@ static Bool m2tsdmx_process_event(GF_Filter *filter, const GF_FilterEvent *com)
 			}
 			return GF_FALSE;
 		}
+		if (com->play.no_byterange_forward)
+			ctx->is_dash = GF_TRUE;
 		/*mark pcr as not initialized*/
 		if (pes->program->pcr_pid==pes->pid) pes->program->first_dts=0;
 		gf_m2ts_set_pes_framing(pes, GF_M2TS_PES_FRAMING_DEFAULT);
 		GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[M2TSDmx] Setting default reframing for PID %d\n", pes->pid));
 
-		/*this is a multplex, only trigger the play command for the first stream activated*/
+		/*this is a multiplex, only trigger the play command for the first activated stream*/
 		ctx->nb_playing++;
 		if (ctx->nb_playing>1) return GF_TRUE;
 
@@ -1041,15 +1197,18 @@ static void m2tsdmx_finalize(GF_Filter *filter)
 static GF_Err m2tsdmx_process(GF_Filter *filter)
 {
 	GF_M2TSDmxCtx *ctx = gf_filter_get_udta(filter);
-	GF_FilterPacket *pck = gf_filter_pid_get_packet(ctx->ipid);
+	GF_FilterPacket *pck;
+	Bool check_block = GF_TRUE;
 	const char *data;
 	u32 size;
 
+restart:
+	pck = gf_filter_pid_get_packet(ctx->ipid);
 	if (!pck) {
 		if (gf_filter_pid_is_eos(ctx->ipid)) {
 			u32 i, nb_streams = gf_filter_get_opid_count(filter);
 
-			gf_m2ts_flush_all(ctx->ts);
+			gf_m2ts_flush_all(ctx->ts, ctx->is_dash);
 			for (i=0; i<nb_streams; i++) {
 				GF_FilterPid *opid = gf_filter_get_opid(filter, i);
 				gf_filter_pid_set_eos(opid);
@@ -1058,12 +1217,19 @@ static GF_Err m2tsdmx_process(GF_Filter *filter)
 		}
 		return GF_OK;
 	}
+	if (ctx->sigfrag) {
+		Bool is_start;
+		gf_filter_pck_get_framing(pck, &is_start, NULL);
+		if (is_start) {
+			gf_m2ts_mark_seg_start(ctx->ts);
+		}
+	}
 	//we process even if no stream playing: since we use unframed dispatch we may need to send packets to configure reframers
 	//which will in turn connect to the sink which will send the PLAY event marking stream(s) as playing
 	if (ctx->in_seek) {
 		gf_m2ts_reset_parsers(ctx->ts);
 		ctx->in_seek = GF_FALSE;
-	} else {
+	} else if (check_block) {
 		u32 i, nb_streams, would_block = 0;
 		nb_streams = gf_filter_get_opid_count(filter);
 		for (i=0; i<nb_streams; i++) {
@@ -1074,6 +1240,8 @@ static GF_Err m2tsdmx_process(GF_Filter *filter)
 		}
 		if (would_block && (would_block==nb_streams))
 			return GF_OK;
+
+		check_block = GF_FALSE;
 	}
 
 	data = gf_filter_pck_get_data(pck, &size);
@@ -1088,6 +1256,8 @@ static GF_Err m2tsdmx_process(GF_Filter *filter)
 		gf_filter_pid_send_event(ctx->ipid, &fevt);
 		ctx->mux_tune_state = DMX_TUNE_DONE;
 		gf_m2ts_reset_parsers(ctx->ts);
+	} else {
+		goto restart;
 	}
 	return GF_OK;
 }
@@ -1103,14 +1273,15 @@ static const char *m2tsdmx_probe_data(const u8 *data, u32 size, GF_FilterProbeSc
 
 static const GF_FilterCapability M2TSDmxCaps[] =
 {
-	CAP_UINT(GF_CAPS_INPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_FILE),
-	CAP_STRING(GF_CAPS_INPUT, GF_PROP_PID_FILE_EXT, "ts|m2t|mts|dmb|trp"),
-	CAP_STRING(GF_CAPS_INPUT, GF_PROP_PID_MIME, "video/mpeg-2|video/mp2t|video/mpeg"),
+	CAP_UINT(GF_CAPS_INPUT_STATIC, GF_PROP_PID_STREAM_TYPE, GF_STREAM_FILE),
+	CAP_STRING(GF_CAPS_INPUT_STATIC, GF_PROP_PID_FILE_EXT, "ts|m2t|mts|dmb|trp"),
+	CAP_STRING(GF_CAPS_INPUT_STATIC, GF_PROP_PID_MIME, "video/mpeg-2|video/mp2t|video/mpeg|audio/mpeg-2|audio/mp2t"),
 	CAP_UINT(GF_CAPS_OUTPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_AUDIO),
 	CAP_UINT(GF_CAPS_OUTPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_VISUAL),
 	CAP_UINT(GF_CAPS_OUTPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_SCENE),
 	CAP_UINT(GF_CAPS_OUTPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_OD),
 	CAP_UINT(GF_CAPS_OUTPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_PRIVATE_SCENE),
+	CAP_UINT(GF_CAPS_OUTPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_ENCRYPTED),
 	CAP_UINT(GF_CAPS_OUTPUT_EXCLUDED, GF_PROP_PID_CODECID, GF_CODECID_RAW),
 };
 
@@ -1120,13 +1291,14 @@ static const GF_FilterArgs M2TSDmxArgs[] =
 	{ OFFS(temi_url), "force TEMI URL", GF_PROP_NAME, NULL, NULL, GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(dsmcc), "enable DSMCC receiver", GF_PROP_BOOL, "no", NULL, GF_FS_ARG_HINT_EXPERT},
 	{ OFFS(seeksrc), "seek local source file back to origin once all programs are setup", GF_PROP_BOOL, "true", NULL, GF_FS_ARG_HINT_EXPERT},
+	{ OFFS(sigfrag), "signal segment boundaries on output packets for DASH or HLS sources", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_ADVANCED},
 	{0}
 };
 
 
 GF_FilterRegister M2TSDmxRegister = {
 	.name = "m2tsdmx",
-	GF_FS_SET_DESCRIPTION("MPEG-2 TS demuxer")
+	GF_FS_SET_DESCRIPTION("MPEG-2 TS demultiplexer")
 	GF_FS_SET_HELP("This filter demultiplexes MPEG-2 Transport Stream files/data into a set of media PIDs and frames.")
 	.private_size = sizeof(GF_M2TSDmxCtx),
 	.initialize = m2tsdmx_initialize,
