@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2005-2020
+ *			Copyright (c) Telecom ParisTech 2005-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / NHML demuxer filter
@@ -33,6 +33,8 @@
 #include <gpac/isomedia.h>
 #include <gpac/base_coding.h>
 
+#ifndef GPAC_DISABLE_CORE_TOOLS
+
 #ifndef GPAC_DISABLE_AV_PARSERS
 #include <gpac/avparse.h>
 #endif
@@ -59,7 +61,7 @@ typedef struct
 	u32 sample_num;
 
 	FILE *mdia;
-	char szMedia[GF_MAX_PATH];
+	char *media_file;
 
 	GF_DOMParser *parser;
 	GF_XMLNode *root;
@@ -97,8 +99,7 @@ GF_Err nhmldmx_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_remov
 
 	if (is_remove) {
 		ctx->ipid = NULL;
-		//gf_filter_pid_remove(st->opid);
-
+		gf_filter_pid_remove(ctx->opid);
 		return GF_OK;
 	}
 	if (! gf_filter_pid_check_caps(pid))
@@ -114,6 +115,11 @@ GF_Err nhmldmx_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_remov
 		if (p && p->value.string && strstr(p->value.string, "dims")) ctx->is_dims = GF_TRUE;
 	}
 
+	//check multithreaded FileIO restrictions
+	p = gf_filter_pid_get_property(ctx->ipid, GF_PROP_PID_FILEPATH);
+	if (p && p->value.string && gf_fileio_is_main_thread(p->value.string)) {
+		gf_filter_force_main_thread(filter, GF_TRUE);
+	}
 	return GF_OK;
 }
 
@@ -209,6 +215,122 @@ static Bool nhmldmx_process_event(GF_Filter *filter, const GF_FilterEvent *evt)
 	return GF_FALSE;
 }
 
+static GF_XMLNode *nhmldmx_get_props(GF_NHMLDmxCtx *ctx, char *ID)
+{
+	u32 i=0;
+	GF_XMLNode *node;
+	while ((node = (GF_XMLNode *) gf_list_enum(ctx->root->content, &i))) {
+		GF_XMLAttribute *att;
+		GF_XMLNode *childnode;
+		u32 j;
+		if (node->type) continue;
+		if (!stricmp(node->name, "Properties")) {
+			j=0;
+			while ( (att = (GF_XMLAttribute *)gf_list_enum(node->attributes, &j))) {
+				if (!strcmp(att->name, "id") && !strcmp(att->value, ID)) return node;
+			}
+			continue;
+		}
+		if (stricmp(node->name, "NHNTSample")) continue;
+		j=0;
+		while ((childnode = (GF_XMLNode *) gf_list_enum(node->content, &j))) {
+			u32 k=0;
+			if (childnode->type) continue;
+			if (stricmp(childnode->name, "Properties")) continue;
+			while ( (att = (GF_XMLAttribute *)gf_list_enum(childnode->attributes, &k))) {
+				if (!strcmp(att->name, "id") && !strcmp(att->value, ID)) return childnode;
+			}
+		}
+	}
+	return NULL;
+}
+static void nhmldmx_set_props(GF_NHMLDmxCtx *ctx, GF_XMLNode *props, GF_FilterPacket *pck)
+{
+	GF_XMLNode *childnode;
+	u32 i=0;
+	GF_XMLAttribute *att;
+
+	i=0;
+	while ( (att = (GF_XMLAttribute *)gf_list_enum(props->attributes, &i))) {
+		if (strcmp(att->name, "ref")) continue;;
+		GF_XMLNode *ref_props = nhmldmx_get_props(ctx, att->value);
+		if (ref_props) {
+			props = ref_props;
+			break;
+		}
+	}
+
+	i=0;
+	while ((childnode = (GF_XMLNode *) gf_list_enum(props->content, &i))) {
+		u32 j;
+		GF_XMLNode *bs_child;
+		Bool has_bs = GF_FALSE;
+		if (childnode->type) continue;
+		if (stricmp(childnode->name, "P")) continue;
+
+		char *ptype=NULL;
+		char *pname=NULL;
+		char *pval=NULL;
+		j=0;
+		while ( (att = (GF_XMLAttribute *)gf_list_enum(childnode->attributes, &j))) {
+			if (!strcmp(att->name, "type")) ptype = att->value;
+			else if (!strcmp(att->name, "name")) pname = att->value;
+			else if (!strcmp(att->name, "value")) pval = att->value;
+			else if (!pname) {
+				pname = att->name;
+				pval = att->value;
+			}
+		}
+
+		j=0;
+		while ( (bs_child = (GF_XMLNode *)gf_list_enum(childnode->content, &j))) {
+			if (bs_child->type) continue;
+			if (!stricmp(bs_child->name, "BS")) has_bs = GF_TRUE;
+		}
+
+
+		Bool reset=GF_TRUE;
+		u32 prop_type = 0;
+		u32 p4cc = gf_props_get_id(pname);
+		if (p4cc) {
+			prop_type = gf_props_4cc_get_type(p4cc);
+			pname = NULL;
+			if (prop_type!=GF_PROP_DATA)
+				has_bs = GF_FALSE;
+		} else if (ptype) {
+			prop_type = gf_props_parse_type(ptype);
+		} else {
+			prop_type = GF_PROP_STRING;
+		}
+		GF_PropertyValue prop_val;
+		if (has_bs) {
+			GF_BitStream *bs = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
+			prop_val.type = GF_PROP_DATA_NO_COPY;
+			gf_xml_parse_bit_sequence_bs(childnode, ctx->src_url, bs);
+			gf_bs_get_content(bs, &prop_val.value.data.ptr, &prop_val.value.data.size);
+			gf_bs_del(bs);
+			reset = GF_FALSE;
+		} else if (pval) {
+			prop_val = gf_props_parse_value(prop_type, pname, pval, NULL, ',');
+		} else {
+			continue;
+		}
+
+		if (p4cc) {
+			if (pck)
+				gf_filter_pck_set_property(pck, p4cc, &prop_val );
+			else
+				gf_filter_pid_set_property(ctx->opid, p4cc, &prop_val );
+		} else {
+			if (pck)
+				gf_filter_pck_set_property_dyn(pck, pname, &prop_val );
+			else
+				gf_filter_pid_set_property_dyn(ctx->opid, pname, &prop_val );
+		}
+		if (reset)
+			gf_props_reset_single(&prop_val);
+	}
+}
 
 typedef struct
 {
@@ -444,7 +566,7 @@ static GF_Err compress_sample_data(GF_NHMLDmxCtx *ctx, u32 compress_type, char *
 	}\
 
 
-static GF_Err nhmldmx_init_parsing(GF_Filter *filter, GF_NHMLDmxCtx *ctx)
+static GF_Err nhmldmx_config_output(GF_Filter *filter, GF_NHMLDmxCtx *ctx, GF_XMLNode *root)
 {
 	GF_Err e;
 	Bool inRootOD;
@@ -454,76 +576,32 @@ static GF_Err nhmldmx_init_parsing(GF_Filter *filter, GF_NHMLDmxCtx *ctx)
 
 	u32 dims_profile, dims_level, dims_pathComponents, dims_fullRequestHost, dims_streamType, dims_containsRedundant;
 	char *textEncoding, *contentEncoding, *dims_content_script_types, *mime_type, *xml_schema_loc, *xmlns;
-	FILE *nhml;
-	const GF_PropertyValue *p;
 	char *auxiliary_mime_types = NULL;
-	char *ext, szName[1000], szInfo[GF_MAX_PATH], szXmlFrom[1000], szXmlHeaderEnd[1000];
+	char*init_name=NULL, szXmlFrom[1000], szXmlHeaderEnd[1000];
 	u8 *specInfo;
 	char compressor_name[100];
 	GF_XMLNode *node;
 	FILE *finfo;
-	u64 media_size, last_dts;
-	char *szRootName, *szSampleName, *szImpName;
+	u64 media_size;
+	char *szImpName;
 
-	szRootName = ctx->is_dims ? "DIMSStream" : "NHNTStream";
-	szSampleName = ctx->is_dims ? "DIMSUnit" : "NHNTSample";
 	szImpName = ctx->is_dims ? "DIMS" : "NHML";
 
-	p = gf_filter_pid_get_property(ctx->ipid, GF_PROP_PID_FILEPATH);
-	if (!p) {
-		gf_filter_pid_drop_packet(ctx->ipid);
-		return GF_NOT_SUPPORTED;
-	}
-	ctx->src_url = p->value.string;
-	nhml = gf_fopen(ctx->src_url, "rt");
-	if (!nhml) {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[NHMLDmx] Cannot find %s file %s", szImpName, ctx->src_url));
-		return GF_URL_ERROR;
-	}
-
-	szName[0] = 0;
-	if (!strncmp(ctx->src_url, "gfio://", 7)) {
-		char *base = gf_file_basename( gf_fileio_translate_url(ctx->src_url) );
-		if (base) strcpy(szName, base);
-	} else {
-		strcpy(szName, ctx->src_url);
-	}
-	ext = gf_file_ext_start(szName);
-	if (ext) ext[0] = 0;
-	strcpy(ctx->szMedia, szName);
-	strcpy(szInfo, szName);
-	strcat(ctx->szMedia, ".media");
-	strcat(szInfo, ".info");
-
-	ctx->parser = gf_xml_dom_new();
-	e = gf_xml_dom_parse(ctx->parser, p->value.string, NULL, NULL);
-	if (e) {
-		gf_fclose(nhml);
-		GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[NHMLDmx] Error parsing %s file: Line %d - %s", szImpName, gf_xml_dom_get_line(ctx->parser), gf_xml_dom_get_error(ctx->parser) ));
-		return GF_NON_COMPLIANT_BITSTREAM;
-	}
-	gf_fclose(nhml);
-
-	ctx->root = gf_xml_dom_get_root(ctx->parser);
-	if (!ctx->root) {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[NHMLDmx] Error parsing %s file - no root node found", szImpName ));
-		return GF_NON_COMPLIANT_BITSTREAM;
-	}
-
 	ctx->dts_inc = 0;
 	inRootOD = GF_FALSE;
 	ctx->compress_type = 0;
 	specInfo = NULL;
 
+	gf_dynstrcat(&init_name, ctx->media_file, NULL);
+	char *ext = gf_file_ext_start(init_name);
+	if (ext) ext[0] = 0;
+	gf_dynstrcat(&init_name, ".info", NULL);
+
+
 #ifndef GPAC_DISABLE_ZLIB
 	ctx->use_dict = GF_FALSE;
 #endif
 
-	if (stricmp(ctx->root->name, szRootName)) {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[NHMLDmx] Error parsing %s file - \"%s\" root expected, got \"%s\"", szImpName, szRootName, ctx->root->name));
-		return GF_NON_COMPLIANT_BITSTREAM;
-	}
-
 	tkID = mtype = streamType = codecid = par_den = par_num = 0;
 	ctx->timescale = 1000;
 	i=0;
@@ -538,19 +616,32 @@ static GF_Err nhmldmx_init_parsing(GF_Filter *filter, GF_NHMLDmxCtx *ctx)
 	dims_streamType = GF_TRUE;
 	dims_containsRedundant = 1;
 
-	while ((att = (GF_XMLAttribute *)gf_list_enum(ctx->root->attributes, &i))) {
+	while ((att = (GF_XMLAttribute *)gf_list_enum(root->attributes, &i))) {
 		if (!stricmp(att->name, "streamType")) {
 			NHML_SCAN_INT("%u", streamType)
+			if (!streamType)
+				streamType = gf_stream_type_by_name(att->value);
 		} else if (!stricmp(att->name, "mediaType") && (strlen(att->value)==4)) {
 			mtype = GF_4CC(att->value[0], att->value[1], att->value[2], att->value[3]);
 		} else if (!stricmp(att->name, "mediaSubType") && (strlen(att->value)==4)) {
 			codec_tag = GF_4CC(att->value[0], att->value[1], att->value[2], att->value[3]);
 		} else if (!stricmp(att->name, "objectTypeIndication")) {
 			NHML_SCAN_INT("%u", codecid)
-		} else if (!stricmp(att->name, "codecID") && (strlen(att->value)==4)) {
-			codecid = GF_4CC(att->value[0], att->value[1], att->value[2], att->value[3]);
+		} else if (!stricmp(att->name, "codecID")) {
+			if (strlen(att->value)==4) {
+				codecid = GF_4CC(att->value[0], att->value[1], att->value[2], att->value[3]);
+			} else {
+				codecid = gf_codecid_parse(att->value);
+			}
 		} else if (!stricmp(att->name, "timeScale")) {
+			u32 old_timescale = ctx->timescale;
 			NHML_SCAN_INT("%u", ctx->timescale)
+			if (old_timescale && ctx->timescale && (root != ctx->root)) {
+				if (ctx->dts_inc)
+					ctx->dts_inc = (u32) gf_timestamp_rescale(ctx->dts_inc, old_timescale, ctx->timescale);
+				if (ctx->last_dts != GF_FILTER_NO_TS)
+					ctx->last_dts = gf_timestamp_rescale(ctx->last_dts, old_timescale, ctx->timescale);
+			}
 		} else if (!stricmp(att->name, "width")) {
 			NHML_SCAN_INT("%u", width)
 		} else if (!stricmp(att->name, "height")) {
@@ -565,12 +656,12 @@ static GF_Err nhmldmx_init_parsing(GF_Filter *filter, GF_NHMLDmxCtx *ctx)
 			NHML_SCAN_INT("%u", nb_channels)
 		} else if (!stricmp(att->name, "baseMediaFile")) {
 			char *url = gf_url_concatenate(ctx->src_url, att->value);
-			strcpy(ctx->szMedia, url ? url : att->value);
-			if (url) gf_free(url);
+			if (ctx->media_file) gf_free(ctx->media_file);
+			ctx->media_file = url;
 		} else if (!stricmp(att->name, "specificInfoFile")) {
 			char *url = gf_url_concatenate(ctx->src_url, att->value);
-			strcpy(szInfo, url ? url : att->value);
-			if (url) gf_free(url);
+			if (init_name) gf_free(init_name);
+			init_name = url;
 		} else if (!stricmp(att->name, "headerEnd")) {
 			NHML_SCAN_INT("%u", ctx->header_end)
 		} else if (!stricmp(att->name, "trackID")) {
@@ -686,8 +777,8 @@ static GF_Err nhmldmx_init_parsing(GF_Filter *filter, GF_NHMLDmxCtx *ctx)
 		codecid = GF_CODECID_DIMS;
 		streamType = GF_STREAM_SCENE;
 	}
-	if (gf_file_exists_ex(ctx->szMedia, ctx->src_url))
-		ctx->mdia = gf_fopen_ex(ctx->szMedia, ctx->src_url, "rb");
+	if (gf_file_exists_ex(ctx->media_file, ctx->src_url))
+		ctx->mdia = gf_fopen_ex(ctx->media_file, ctx->src_url, "rb", GF_FALSE);
 
 	specInfoSize = 0;
 	if (!streamType && !mtype && !codec_tag) {
@@ -696,13 +787,16 @@ static GF_Err nhmldmx_init_parsing(GF_Filter *filter, GF_NHMLDmxCtx *ctx)
 	}
 
 	finfo = NULL;
-	if (gf_file_exists_ex(szInfo, ctx->src_url))
-		finfo = gf_fopen_ex(szInfo, ctx->src_url, "rb");
+	if (gf_file_exists_ex(init_name, ctx->src_url))
+		finfo = gf_fopen_ex(init_name, ctx->src_url, "rb", GF_FALSE);
 
 	if (finfo) {
 		e = gf_file_load_data_filep(finfo, (u8 **)&specInfo, &specInfoSize);
 		gf_fclose(finfo);
-		if (e) return e;
+		if (e) {
+			if (init_name) gf_free(init_name);
+			return e;
+		}
 	} else if (ctx->header_end) {
 		/* for text based streams, the decoder specific info can be at the beginning of the file */
 		specInfoSize = ctx->header_end;
@@ -714,9 +808,10 @@ static GF_Err nhmldmx_init_parsing(GF_Filter *filter, GF_NHMLDmxCtx *ctx)
 		/* for XML based streams, the decoder specific info can be up to some element in the file */
 		strcpy(szXmlFrom, "doc.start");
 		ctx->samp_buffer_size = 0;
-		e = nhml_sample_from_xml(ctx, ctx->szMedia, szXmlFrom, szXmlHeaderEnd);
+		e = nhml_sample_from_xml(ctx, ctx->media_file, szXmlFrom, szXmlHeaderEnd);
 		if (e) {
 			GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[NHMLDmx] failed to load XML header: %s", gf_error_to_string(e) ));
+			if (init_name) gf_free(init_name);
 			return e;
 		}
 
@@ -726,25 +821,30 @@ static GF_Err nhmldmx_init_parsing(GF_Filter *filter, GF_NHMLDmxCtx *ctx)
 		specInfo[specInfoSize] = 0;
 	}
 
-	i=0;
-	while ((node = (GF_XMLNode *) gf_list_enum(ctx->root->content, &i))) {
+	i = 0;
+	GF_XMLNode *par = (root != ctx->root) ? root : ctx->root;
+	while ((node = (GF_XMLNode *) gf_list_enum(par->content, &i))) {
 		if (node->type) continue;
+		if (!stricmp(node->name, ctx->is_dims ? "DIMSUnit" : "NHNTSample") ) break;
 		if (stricmp(node->name, "DecoderSpecificInfo") ) continue;
 
 		e = gf_xml_parse_bit_sequence(node, ctx->src_url, &specInfo, &specInfoSize);
 		if (e) {
 			if (specInfo) gf_free(specInfo);
+			if (init_name) gf_free(init_name);
 			return e;
 		}
 		break;
 	}
 
-	ctx->opid = gf_filter_pid_new(filter);
+	if (!ctx->opid)
+		ctx->opid = gf_filter_pid_new(filter);
+
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_STREAM_TYPE, &PROP_UINT(streamType) );
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_CODECID, &PROP_UINT(codecid) );
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_TIMESCALE, &PROP_UINT(ctx->timescale) );
 	if (ctx->reframe)
-	   gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_UNFRAMED, &PROP_UINT(GF_TRUE) );
+	   gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_UNFRAMED, &PROP_BOOL(GF_TRUE) );
 
 #ifndef GPAC_DISABLE_AV_PARSERS
 	if (!width && !height && specInfo && (codecid==GF_CODECID_MPEG4_PART2)) {
@@ -852,13 +952,7 @@ static GF_Err nhmldmx_init_parsing(GF_Filter *filter, GF_NHMLDmxCtx *ctx)
 		specInfoSize = 0;
 	}
 
-	if (inRootOD) gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_IN_IOD, &PROP_BOOL(GF_TRUE) );
-
-	ctx->media_done = 0;
-	ctx->current_child_idx = 0;
-	ctx->last_dts = GF_FILTER_NO_TS;
-
-	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_FILEPATH, & PROP_STRING(ctx->szMedia));
+	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_FILEPATH, & PROP_STRING(ctx->media_file));
 
 	if (ctx->mdia) {
 		media_size = gf_fsize(ctx->mdia);
@@ -867,6 +961,93 @@ static GF_Err nhmldmx_init_parsing(GF_Filter *filter, GF_NHMLDmxCtx *ctx)
 
 	if (specInfo) gf_free(specInfo);
 	if (auxiliary_mime_types) gf_free(auxiliary_mime_types);
+	if (init_name) gf_free(init_name);
+
+	if (inRootOD) gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_IN_IOD, &PROP_BOOL(GF_TRUE) );
+
+
+	i = 0;
+	par = (root != ctx->root) ? root : ctx->root;
+	while ((node = (GF_XMLNode *) gf_list_enum(par->content, &i))) {
+		if (node->type) continue;
+		if (!stricmp(node->name, ctx->is_dims ? "DIMSUnit" : "NHNTSample") ) break;
+
+		if (stricmp(node->name, "Properties")) continue;
+
+		nhmldmx_set_props(ctx, node, NULL);
+		break;
+	}
+
+	return GF_OK;
+}
+
+static GF_Err nhmldmx_init_parsing(GF_Filter *filter, GF_NHMLDmxCtx *ctx)
+{
+	GF_Err e;
+	u32 i;
+	GF_XMLAttribute *att;
+	FILE *nhml;
+	const GF_PropertyValue *p;
+	char *ext;
+	GF_XMLNode *node;
+	u64 last_dts;
+	char *szRootName, *szSampleName, *szImpName;
+
+	szRootName = ctx->is_dims ? "DIMSStream" : "NHNTStream";
+	szSampleName = ctx->is_dims ? "DIMSUnit" : "NHNTSample";
+	szImpName = ctx->is_dims ? "DIMS" : "NHML";
+
+	p = gf_filter_pid_get_property(ctx->ipid, GF_PROP_PID_FILEPATH);
+	if (!p) {
+		gf_filter_pid_drop_packet(ctx->ipid);
+		return GF_NOT_SUPPORTED;
+	}
+	ctx->src_url = p->value.string;
+	nhml = gf_fopen(ctx->src_url, "rt");
+	if (!nhml) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[NHMLDmx] Cannot find %s file %s", szImpName, ctx->src_url));
+		return GF_URL_ERROR;
+	}
+
+	if (ctx->media_file) gf_free(ctx->media_file);
+	ctx->media_file = NULL;
+
+	if (!strncmp(ctx->src_url, "gfio://", 7)) {
+		char *base = gf_file_basename( gf_fileio_translate_url(ctx->src_url) );
+		if (base) gf_dynstrcat(&ctx->media_file, base, NULL);
+	} else {
+		gf_dynstrcat(&ctx->media_file, ctx->src_url, NULL);
+	}
+	if (!ctx->media_file) return GF_OUT_OF_MEM;
+	ext = gf_file_ext_start(ctx->media_file);
+	if (ext) ext[0] = 0;
+	gf_dynstrcat(&ctx->media_file, ".media", NULL);
+
+	ctx->parser = gf_xml_dom_new();
+	e = gf_xml_dom_parse(ctx->parser, p->value.string, NULL, NULL);
+	if (e) {
+		gf_fclose(nhml);
+		GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[NHMLDmx] Error parsing %s file: Line %d - %s", szImpName, gf_xml_dom_get_line(ctx->parser), gf_xml_dom_get_error(ctx->parser) ));
+		return GF_NON_COMPLIANT_BITSTREAM;
+	}
+	gf_fclose(nhml);
+
+	ctx->root = gf_xml_dom_get_root(ctx->parser);
+	if (!ctx->root) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[NHMLDmx] Error parsing %s file - no root node found", szImpName ));
+		return GF_NON_COMPLIANT_BITSTREAM;
+	}
+
+	if (stricmp(ctx->root->name, szRootName)) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[NHMLDmx] Error parsing %s file - \"%s\" root expected, got \"%s\"", szImpName, szRootName, ctx->root->name));
+		return GF_NON_COMPLIANT_BITSTREAM;
+	}
+
+	nhmldmx_config_output(filter, ctx, ctx->root);
+
+	ctx->media_done = 0;
+	ctx->current_child_idx = 0;
+	ctx->last_dts = GF_FILTER_NO_TS;
 
 	//compute duration
 	ctx->duration.den = ctx->timescale;
@@ -880,7 +1061,12 @@ static GF_Err nhmldmx_init_parsing(GF_Filter *filter, GF_NHMLDmxCtx *ctx)
 		s32 cts_offset=0;
 		u64 sample_duration = 0;
 		if (node->type) continue;
-		if (stricmp(node->name, szSampleName) ) continue;
+		if (!stricmp(node->name, szSampleName) ) {
+		} else if (!stricmp(node->name, "NHNTReconfig") ) {
+			break;
+		} else {
+			continue;
+		}
 
 		while ( (att = (GF_XMLAttribute *)gf_list_enum(node->attributes, &j))) {
 			if (!stricmp(att->name, "DTS") || !stricmp(att->name, "time")) {
@@ -896,6 +1082,7 @@ static GF_Err nhmldmx_init_parsing(GF_Filter *filter, GF_NHMLDmxCtx *ctx)
 			else if (!stricmp(att->name, "duration") ) sscanf(att->value, ""LLU, &sample_duration);
 			else if (!stricmp(att->name, "isRAP") ) ctx->has_sap = GF_TRUE;
 		}
+
 		last_dts = ctx->duration.num;
 		if (dts) ctx->duration.num = (u32) (dts + cts_offset);
 		if (sample_duration) {
@@ -909,29 +1096,201 @@ static GF_Err nhmldmx_init_parsing(GF_Filter *filter, GF_NHMLDmxCtx *ctx)
 	if (last_dts) {
 		ctx->duration.num += (u32) (ctx->duration.num - last_dts);
 	}
+
+	p = gf_filter_pid_get_property(ctx->opid, GF_PROP_PID_STREAM_TYPE);
 	//assume image, one sec (default for old arch)
-	if ((streamType==4) && !ctx->duration.num) {
+	if (p && (p->value.uint==4) && !ctx->duration.num) {
 		ctx->is_img = GF_TRUE;
 		ctx->duration.num =ctx->duration.den;
 	}
 
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_DURATION, & PROP_FRAC64(ctx->duration) );
-
+	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_PLAYBACK_MODE, &PROP_UINT(GF_PLAYBACK_MODE_FASTFORWARD ) );
 	return e;
 }
 
+
+
 void nhml_get_bs(GF_BitStream **bs, char *data, u32 size, u32 mode)
 {
 	if (*bs) gf_bs_reassign_buffer(*bs, data, size);
 	else  (*bs) = gf_bs_new(data, size, mode);
 }
 
+static void nhmldmx_set_subs(GF_NHMLDmxCtx *ctx, GF_XMLNode *node, GF_FilterPacket *pck)
+{
+	GF_XMLNode *childnode, *subs;
+	u32 i=0;
+	while ((childnode = (GF_XMLNode *) gf_list_enum(node->content, &i))) {
+		u32 j;
+		GF_XMLAttribute *att;
+		GF_BitStream *bs;
+
+		if (childnode->type) continue;
+		if (stricmp(childnode->name, "SubSamples")) continue;
+
+		u32 flags=0;
+		j=0;
+		while ( (att = (GF_XMLAttribute *)gf_list_enum(childnode->attributes, &j))) {
+			if (!strcmp(att->name, "flags")) flags = atoi(att->value);
+		}
+
+		bs = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
+		j=0;
+		while ((subs = (GF_XMLNode *) gf_list_enum(childnode->content, &j))) {
+			if (subs->type) continue;
+			if (stricmp(subs->name, "SubSample")) continue;
+
+			Bool discardable=0;
+			u32 priority=0;
+			u32 size=0;
+			u32 codec_info=0;
+			char *fsize = NULL;
+			Bool ftext=GF_FALSE;
+
+			u32 k = 0;
+			while ((att = (GF_XMLAttribute *)gf_list_enum(subs->attributes, &k))) {
+				if (!stricmp(att->name, "size")) size = atoi(att->value);
+				else if (!stricmp(att->name, "fsize")) fsize = att->value;
+				else if (!stricmp(att->name, "textmode")) {
+					if (!stricmp(att->value, "yes") || !stricmp(att->value, "true") || !stricmp(att->value, "1")) {
+						ftext = GF_TRUE;
+					}
+				}
+				else if (!stricmp(att->name, "discardable")) {
+					if (!stricmp(att->value, "yes") || !stricmp(att->value, "true") || !stricmp(att->value, "1")) {
+						discardable = GF_TRUE;
+					}
+				}
+				else if (!stricmp(att->name, "priority")) priority = atoi(att->value);
+				else if (!stricmp(att->name, "codec_info")) {
+					if (!strnicmp(att->value, "0x", 2)) sscanf(att->value+2, "%x", &codec_info);
+					else sscanf(att->value, "%u", &codec_info);
+				}
+			}
+
+			if (fsize) {
+				char *sub_file_url = gf_url_concatenate(ctx->src_url, fsize);
+				if (sub_file_url) {
+					FILE *f = gf_fopen(sub_file_url, ftext ? "rt" : "r");
+					if (f) {
+						size += (u32) gf_fsize(f);
+						gf_fclose(f);
+					}
+					gf_free(sub_file_url);
+				}
+			}
+
+			gf_bs_write_u32(bs, flags); //flags
+			gf_bs_write_u32(bs, size);
+			gf_bs_write_u32(bs, codec_info); //codec_specific_parameters
+			gf_bs_write_u8(bs, priority); //priority
+			gf_bs_write_u8(bs, discardable); //discardable
+		}
+
+		u8 *subs_data;
+		u32 subs_size;
+		gf_bs_get_content(bs, &subs_data, &subs_size);
+		gf_bs_del(bs);
+		gf_filter_pck_set_property(pck, GF_PROP_PCK_SUBS, &PROP_DATA_NO_COPY(subs_data, subs_size) );
+		break;
+	}
+}
+
+static GF_XMLNode *nhmldmx_find_sai(GF_NHMLDmxCtx *ctx, char *id)
+{
+	GF_XMLNode *node;
+	u32 idx=0;
+	while ((node = (GF_XMLNode *) gf_list_enum(ctx->root->content, &idx))) {
+		GF_XMLNode *child;
+		u32 j=0;
+		if (node->type) continue;
+		if (stricmp(node->name, ctx->is_dims ? "DIMSUnit" : "NHNTSample") ) continue;
+
+		while ((child = (GF_XMLNode *) gf_list_enum(node->content, &j))) {
+			u32 k=0;
+			GF_XMLAttribute *att;
+			if (child->type) continue;
+			if (stricmp(child->name, "SAI") ) continue;
+			while ((att = (GF_XMLAttribute *) gf_list_enum(child->attributes, &k))) {
+				if (!strcmp(att->name, "id") && !strcmp(att->value, id))
+					return child;
+			}
+		}
+	}
+	return NULL;
+}
+
+static void nhmldmx_set_sai(GF_NHMLDmxCtx *ctx, GF_XMLNode *node, GF_FilterPacket *pck)
+{
+	GF_XMLNode *childnode;
+	u32 i=0;
+	while ((childnode = (GF_XMLNode *) gf_list_enum(node->content, &i))) {
+		u32 j;
+		char szPName[30];
+		GF_XMLAttribute *att;
+		GF_BitStream *bs;
+
+		if (childnode->type) continue;
+		if (stricmp(childnode->name, "SAI")) continue;
+
+		Bool is_ref=GF_FALSE;
+		u32 type, aux_info;
+		Bool is_sample_group;
+		Bool valid = GF_TRUE;
+
+retry_sai:
+		type=0;
+		aux_info=0;
+		is_sample_group = GF_FALSE;
+		j=0;
+		while ( (att = (GF_XMLAttribute *)gf_list_enum(childnode->attributes, &j))) {
+			if (!strcmp(att->name, "type")) type = gf_4cc_parse(att->value);
+			else if (!strcmp(att->name, "ref")) {
+				if (!is_ref) {
+					GF_XMLNode *ref_sai = nhmldmx_find_sai(ctx, att->value);
+					if (ref_sai) {
+						is_ref = GF_TRUE;
+						childnode = ref_sai;
+						goto retry_sai;
+					} else {
+						GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[NHMLDmx] Cannot locate SAI with id %s, ignoring SAI", att->value));
+						valid=GF_FALSE;
+					}
+				}
+			}
+			else if (!strcmp(att->name, "aux_info")) aux_info = atoi(att->value);
+			else if (!strcmp(att->name, "group")) {
+				if (!strcmp(att->value, "yes") || !strcmp(att->value, "true") || !strcmp(att->value, "1")) is_sample_group = GF_TRUE;
+			}
+		}
+		if (!valid) continue;
+
+		bs = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
+		gf_xml_parse_bit_sequence_bs(childnode, ctx->src_url, bs);
+
+		u8 *sai_data;
+		u32 sai_size;
+		gf_bs_get_content(bs, &sai_data, &sai_size);
+		gf_bs_del(bs);
+
+		char *prefix = is_sample_group ? "grp" : "sai";
+		if (aux_info) {
+			sprintf(szPName, "%s_%s_%d", prefix, gf_4cc_to_str(type), aux_info);
+		} else {
+			sprintf(szPName, "%s_%s", prefix, gf_4cc_to_str(type));
+		}
+
+		gf_filter_pck_set_property_dyn(pck, szPName, &PROP_DATA_NO_COPY(sai_data, sai_size) );
+	}
+}
+
+
 static GF_Err nhmldmx_send_sample(GF_Filter *filter, GF_NHMLDmxCtx *ctx)
 {
 	GF_XMLNode *node, *childnode;
 	u64 sample_duration = 0;
 	char szMediaTemp[GF_MAX_PATH], szXmlFrom[1000], szXmlTo[1000];
-	char *szSubSampleName = ctx->is_dims ? "DIMSSubUnit" : "NHNTSubSample";
 
 	while ((node = (GF_XMLNode *) gf_list_enum(ctx->root->content, &ctx->current_child_idx))) {
 		u8 *data;
@@ -943,12 +1302,16 @@ static GF_Err nhmldmx_send_sample(GF_Filter *filter, GF_NHMLDmxCtx *ctx)
 		GF_Err e=GF_OK;
 		s32 cts_offset;
 		u64 offset=0, byte_offset = GF_FILTER_NO_BO;
-		u32 nb_subsamples = 0;
-		Bool redundant_rap, append, has_subbs, first_subsample_is_first = GF_FALSE;
+		Bool redundant_rap, append, has_subbs;
 		u32 compress_type;
 		char *base_data = NULL;
 		if (node->type) continue;
-		if (stricmp(node->name, ctx->is_dims ? "DIMSUnit" : "NHNTSample") ) continue;
+		if (stricmp(node->name, ctx->is_dims ? "DIMSUnit" : "NHNTSample") ) {
+			if (!strcmp(node->name, "NHNTReconfig")) {
+				nhmldmx_config_output(filter, ctx, node);
+			}
+			continue;
+		}
 
 		strcpy(szMediaTemp, "");
 		strcpy(szXmlFrom, "");
@@ -985,7 +1348,10 @@ static GF_Err nhmldmx_send_sample(GF_Filter *filter, GF_NHMLDmxCtx *ctx)
 			}
 			else if (!stricmp(att->name, "CTSOffset")) cts_offset = atoi(att->value);
 			else if (!stricmp(att->name, "isRAP") ) {
-				sap_type = (!stricmp(att->value, "yes")) ? GF_FILTER_SAP_1 : GF_FILTER_SAP_NONE;
+				if ((att->value[0]>='0') && (att->value[0]<='9'))
+					sap_type = atoi(att->value);
+				else
+					sap_type = (!stricmp(att->value, "yes")) ? GF_FILTER_SAP_1 : GF_FILTER_SAP_NONE;
 			}
 			else if (!stricmp(att->name, "isSyncShadow")) redundant_rap = !stricmp(att->value, "yes") ? 1 : 0;
 			else if (!stricmp(att->name, "SAPType") ) sap_type = atoi(att->value);
@@ -999,14 +1365,27 @@ static GF_Err nhmldmx_send_sample(GF_Filter *filter, GF_NHMLDmxCtx *ctx)
 					} else {
 						base_data = att->value;
 					}
+				} else if (!strnicmp(att->value, "gmem://", 7)) {
+					GF_LOG(GF_LOG_WARNING, GF_LOG_PARSER, ("[NHMLDmx] Invalid url %s for NHML import\n", att->value));
 				} else {
 					char *url = gf_url_concatenate(ctx->src_url, att->value);
-					strcpy(szMediaTemp, url ? url : att->value);
-					if (url) gf_free(url);
+					if (!url) {
+						GF_LOG(GF_LOG_WARNING, GF_LOG_PARSER, ("[NHMLDmx] Failed to get full url for %s\n", att->value));
+					} else {
+						strncpy(szMediaTemp, url, GF_MAX_PATH-1);
+						szMediaTemp[GF_MAX_PATH-1] = 0;
+						gf_free(url);
+					}
 				}
 			}
-			else if (!stricmp(att->name, "xmlFrom")) strcpy(szXmlFrom, att->value);
-			else if (!stricmp(att->name, "xmlTo")) strcpy(szXmlTo, att->value);
+			else if (!stricmp(att->name, "xmlFrom")) {
+				strncpy(szXmlFrom, att->value, 999);
+				szXmlFrom[999]=0;
+			}
+			else if (!stricmp(att->name, "xmlTo")) {
+				strncpy(szXmlTo, att->value, 999);
+				szXmlTo[999]=0;
+			}
 			/*DIMS flags*/
 			else if (!stricmp(att->name, "is-Scene") && !stricmp(att->value, "yes"))
 				dims_flags |= GF_DIMS_UNIT_S;
@@ -1033,25 +1412,41 @@ static GF_Err nhmldmx_send_sample(GF_Filter *filter, GF_NHMLDmxCtx *ctx)
 
 		if (ctx->is_img) sample_duration = ctx->duration.den;
 
+		GF_XMLNode *sample_child=NULL;
+		Bool has_subs_child = GF_FALSE;
+		Bool has_sai_child = GF_FALSE;
 		has_subbs = GF_FALSE;
+		GF_XMLNode *props = NULL;
 		j=0;
 		while ((childnode = (GF_XMLNode *) gf_list_enum(node->content, &j))) {
 			if (childnode->type) continue;
+			if (!stricmp(childnode->name, "Media")) {
+				sample_child = childnode;
+				has_subbs = GF_TRUE;
+			}
+			if (!stricmp(childnode->name, "SubSamples")) {
+				has_subs_child = GF_TRUE;
+			}
+			if (!stricmp(childnode->name, "SAI")) {
+				has_sai_child = GF_TRUE;
+			}
 			if (!stricmp(childnode->name, "BS")) {
 				has_subbs = GF_TRUE;
-				break;
+			}
+			if (!stricmp(childnode->name, "Properties")) {
+				props = childnode;
 			}
 		}
 
 		if (strlen(szXmlFrom) && strlen(szXmlTo)) {
 			char *xml_file;
 			if (strlen(szMediaTemp)) xml_file = szMediaTemp;
-			else xml_file = ctx->szMedia;
+			else xml_file = ctx->media_file;
 			ctx->samp_buffer_size = 0;
 			e = nhml_sample_from_xml(ctx, xml_file, szXmlFrom, szXmlTo);
 		} else if (ctx->is_dims && !strlen(szMediaTemp)) {
 
-			char *content = gf_xml_dom_serialize(node, GF_TRUE);
+			char *content = gf_xml_dom_serialize(node, GF_TRUE, GF_FALSE);
 
 			ctx->samp_buffer_size = 3 + (u32) strlen(content);
 			if (ctx->samp_buffer_alloc < ctx->samp_buffer_size) {
@@ -1067,14 +1462,6 @@ static GF_Err nhmldmx_send_sample(GF_Filter *filter, GF_NHMLDmxCtx *ctx)
 			/*same DIMS unit*/
 			if (ctx->last_dts == dts)
 				append = GF_TRUE;
-
-		} else if (has_subbs) {
-			gf_bs_reassign_buffer(ctx->bs_w, ctx->samp_buffer, ctx->samp_buffer_alloc);
-			gf_xml_parse_bit_sequence_bs(node, ctx->src_url, ctx->bs_w);
-			gf_bs_get_content(ctx->bs_w, &ctx->samp_buffer, &ctx->samp_buffer_size);
-			if (ctx->samp_buffer_size > ctx->samp_buffer_alloc)
-				ctx->samp_buffer_alloc = ctx->samp_buffer_size;
-
 		} else if (base_data) {
 			char *start = strchr(base_data, ',');
 			if (start) {
@@ -1089,18 +1476,10 @@ static GF_Err nhmldmx_send_sample(GF_Filter *filter, GF_NHMLDmxCtx *ctx)
 			Bool close = GF_FALSE;
 			FILE *f = ctx->mdia;
 
-			j = 0;
-			while ((childnode = (GF_XMLNode *)gf_list_enum(node->content, &j))) {
-				if (childnode->type) continue;
-				if (!stricmp(childnode->name, szSubSampleName)) {
-					nb_subsamples++;
-				}
-			}
-
 			if (strlen(szMediaTemp)) {
 				f = gf_fopen(szMediaTemp, "rb");
 				if (!f) {
-					GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[NHMLDmx] import failure in sample %d: file %s not found", ctx->sample_num, close ? szMediaTemp : ctx->szMedia));
+					GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[NHMLDmx] import failure in sample %d: file %s not found", ctx->sample_num, close ? szMediaTemp : ctx->media_file));
 					return GF_NON_COMPLIANT_BITSTREAM;
 				}
 				close = GF_TRUE;
@@ -1149,14 +1528,28 @@ static GF_Err nhmldmx_send_sample(GF_Filter *filter, GF_NHMLDmxCtx *ctx)
 					}
 				}
 				if (close) gf_fclose(f);
-			} else if (!nb_subsamples) {
-				GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[NHMLDmx] No media file associated with sample %d!\n", ctx->sample_num));
-				e = GF_URL_ERROR;
+			} else {
+				e = GF_OK;
 			}
 		}
 
 		if (e) return e;
 
+		//child BS present, parse them
+		if (has_subbs) {
+			nhml_get_bs(&ctx->bs_w, ctx->samp_buffer, ctx->samp_buffer_size, GF_BITSTREAM_WRITE);
+			if (ctx->samp_buffer_size) {
+				gf_bs_seek(ctx->bs_w, ctx->samp_buffer_size);
+			}
+			gf_xml_parse_bit_sequence_bs(sample_child ? sample_child : node, ctx->src_url, ctx->bs_w);
+			gf_bs_get_content(ctx->bs_w, &ctx->samp_buffer, &ctx->samp_buffer_size);
+			if (ctx->samp_buffer_size > ctx->samp_buffer_alloc)
+				ctx->samp_buffer_alloc = ctx->samp_buffer_size;
+		}
+		if (!ctx->samp_buffer_size) {
+			GF_LOG(GF_LOG_WARNING, GF_LOG_PARSER, ("[NHMLDmx] No media file associated with sample %d!\n", ctx->sample_num));
+		}
+
 		//override DIMS flags
 		if (ctx->is_dims) {
 			if (strstr(ctx->samp_buffer + 3, "svg ")) dims_flags |= GF_DIMS_UNIT_S;
@@ -1185,8 +1578,10 @@ static GF_Err nhmldmx_send_sample(GF_Filter *filter, GF_NHMLDmxCtx *ctx)
 		}
 		if (ctx->samp_buffer_size) {
 			pck = gf_filter_pck_new_alloc(ctx->opid, ctx->samp_buffer_size, &data);
+			if (!pck) return GF_OUT_OF_MEM;
+
 			memcpy(data, ctx->samp_buffer, ctx->samp_buffer_size);
-			gf_filter_pck_set_framing(pck, append ? GF_FALSE : GF_TRUE, nb_subsamples ? GF_FALSE : GF_TRUE);
+			gf_filter_pck_set_framing(pck, append ? GF_FALSE : GF_TRUE, GF_FALSE);
 			if (!append) {
 				gf_filter_pck_set_sap(pck, sap_type);
 				gf_filter_pck_set_dts(pck, dts);
@@ -1206,87 +1601,16 @@ static GF_Err nhmldmx_send_sample(GF_Filter *filter, GF_NHMLDmxCtx *ctx)
 						gf_filter_pck_set_seek_flag(pck, GF_TRUE);
 				}
 			}
+			if (has_subs_child) {
+				nhmldmx_set_subs(ctx, node, pck);
+			}
+			if (has_sai_child) {
+				nhmldmx_set_sai(ctx, node, pck);
+			}
+			if (props) {
+				nhmldmx_set_props(ctx, props, pck);
+			}
 			gf_filter_pck_send(pck);
-		} else {
-			first_subsample_is_first = GF_TRUE;
-		}
-
-		if (nb_subsamples) {
-			if (ctx->samp_buffer_alloc<14*nb_subsamples) {
-				ctx->samp_buffer_alloc = 14*nb_subsamples;
-				ctx->samp_buffer = gf_realloc(ctx->samp_buffer, ctx->samp_buffer_alloc);
-			}
-			assert(ctx->samp_buffer);
-			nhml_get_bs(&ctx->bs_w, ctx->samp_buffer, ctx->samp_buffer_alloc, GF_BITSTREAM_WRITE);
-		}
-
-		j = 0;
-		while (!append && nb_subsamples && (childnode = (GF_XMLNode *)gf_list_enum(node->content, &j))) {
-			if (childnode->type) continue;
-			if (!stricmp(childnode->name, szSubSampleName)) {
-				u32 k = 0;
-				while ((att = (GF_XMLAttribute *)gf_list_enum(childnode->attributes, &k))) {
-					if (!stricmp(att->name, "mediaFile")) {
-						u64 subsMediaFileSize = 0;
-						FILE *f = NULL;
-						char *sub_file_url = gf_url_concatenate(ctx->src_url, att->value);
-						if (sub_file_url) {
-							f = gf_fopen(sub_file_url, "rb");
-							gf_free(sub_file_url);
-						}
-
-						if (!f) {
-							GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[NHMLDmx] Error: mediaFile \"%s\" not found for subsample in sample %d. Abort.\n", att->value, ctx->sample_num));
-							return GF_URL_ERROR;
-						}
-						subsMediaFileSize = gf_fsize(f);
-						assert(subsMediaFileSize < 0x80000000);
-
-						//send continuation frame
-						pck = gf_filter_pck_new_alloc(ctx->opid, (u32) subsMediaFileSize, &data);
-						subsMediaFileSize = (u32) gf_fread(data, (u32) subsMediaFileSize, f);
-						gf_fclose(f);
-
-						nb_subsamples--;
-						if (first_subsample_is_first) {
-							gf_filter_pck_set_framing(pck, GF_TRUE, nb_subsamples ? GF_FALSE : GF_TRUE);
-
-							gf_filter_pck_set_sap(pck, sap_type);
-							gf_filter_pck_set_dts(pck, dts);
-							gf_filter_pck_set_cts(pck, dts+cts_offset);
-							if (redundant_rap && !ctx->is_dims) gf_filter_pck_set_dependency_flags(pck, 0x1);
-
-							if (sample_duration || ctx->dts_inc)
-								gf_filter_pck_set_duration(pck, sample_duration ? (u32) sample_duration : ctx->dts_inc);
-
-							if (ctx->in_seek) {
-								if (dts+cts_offset >= ctx->start_range * ctx->timescale)
-									ctx->in_seek = GF_FALSE;
-								else
-									gf_filter_pck_set_seek_flag(pck, GF_TRUE);
-							}
-
-							first_subsample_is_first = GF_FALSE;
-						} else {
-							gf_filter_pck_set_framing(pck, GF_FALSE, nb_subsamples ? GF_FALSE : GF_TRUE);
-						}
-
-
-						gf_bs_write_u32(ctx->bs_w, 0); //flags
-						gf_bs_write_u32(ctx->bs_w, (u32) subsMediaFileSize);
-						gf_bs_write_u32(ctx->bs_w, 0); //reserved
-						gf_bs_write_u8(ctx->bs_w, 0); //priority
-						gf_bs_write_u8(ctx->bs_w, 0); //discardable
-
-						if (!nb_subsamples) {
-							u32 subs_size = (u32) gf_bs_get_position(ctx->bs_w);
-							gf_filter_pck_set_property(pck, GF_PROP_PCK_SUBS, &PROP_DATA(ctx->samp_buffer, subs_size) );
-						}
-
-						gf_filter_pck_send(pck);
-					}
-				}
-			}
 		}
 
 		ctx->last_dts = dts;
@@ -1367,13 +1691,14 @@ void nhmldmx_finalize(GF_Filter *filter)
 	if (ctx->bs_w) gf_bs_del(ctx->bs_w);
 	if (ctx->samp_buffer) gf_free(ctx->samp_buffer);
 	if (ctx->zlib_buffer) gf_free(ctx->zlib_buffer);
+	if (ctx->media_file) gf_free(ctx->media_file);
 }
 
 
 #define OFFS(_n)	#_n, offsetof(GF_NHMLDmxCtx, _n)
 static const GF_FilterArgs GF_NHMLDmxArgs[] =
 {
-	{ OFFS(reframe), "force reparsing of referenced content", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_ADVANCED},
+	{ OFFS(reframe), "force re-parsing of referenced content", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(index), "indexing window length", GF_PROP_DOUBLE, "1.0", NULL, 0},
 	{0}
 };
@@ -1384,14 +1709,13 @@ static const GF_FilterCapability NHMLDmxCaps[] =
 	CAP_UINT(GF_CAPS_INPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_FILE),
 	CAP_STRING(GF_CAPS_INPUT, GF_PROP_PID_FILE_EXT, "nhml|dims|dml"),
 	CAP_STRING(GF_CAPS_INPUT, GF_PROP_PID_MIME, "application/x-nhml|application/dims"),
-	CAP_UINT(GF_CAPS_OUTPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_AUDIO),
-	CAP_UINT(GF_CAPS_OUTPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_VISUAL),
-	CAP_UINT(GF_CAPS_OUTPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_SCENE),
+	CAP_UINT(GF_CAPS_OUTPUT_EXCLUDED, GF_PROP_PID_STREAM_TYPE, GF_STREAM_UNKNOWN),
+	CAP_UINT(GF_CAPS_OUTPUT_EXCLUDED, GF_PROP_PID_CODECID, GF_CODECID_NONE)
 };
 
 GF_FilterRegister NHMLDmxRegister = {
 	.name = "nhmlr",
-	GF_FS_SET_DESCRIPTION("NHML parser")
+	GF_FS_SET_DESCRIPTION("NHML reader")
 	GF_FS_SET_HELP("This filter reads NHML files/data to produce a media PID and frames.\n"
 	"NHML documentation is available at https://wiki.gpac.io/NHML-Format\n")
 	.private_size = sizeof(GF_NHMLDmxCtx),
@@ -1409,3 +1733,5 @@ const GF_FilterRegister *nhmldmx_register(GF_FilterSession *session)
 	return &NHMLDmxRegister;
 }
 
+#endif /*GPAC_DISABLE_CORE_TOOLS*/
+
