@@ -4,7 +4,7 @@
  *			Authors: Jean Le Feuvre
  *					 Yacine Mathurin Boubacar Aziakou
  *					 Samir Mustapha
- *			Copyright (c) Telecom ParisTech 2019
+ *			Copyright (c) Telecom ParisTech 2019-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / HEVC tile merger filter
@@ -155,33 +155,33 @@ static void hevcmerge_rewrite_pps(GF_HEVCMergeCtx *ctx, char *in_PPS, u32 in_PPS
 
 	//Read and write NAL header bits
 	gf_bs_write_int(ctx->bs_nal_out, gf_bs_read_int(ctx->bs_nal_in, 16), 16);
-	gf_bs_set_ue(ctx->bs_nal_out, gf_bs_get_ue(ctx->bs_nal_in)); //pps_pic_parameter_set_id
-	gf_bs_set_ue(ctx->bs_nal_out, gf_bs_get_ue(ctx->bs_nal_in)); //pps_seq_parameter_set_id
+	gf_bs_write_ue(ctx->bs_nal_out, gf_bs_read_ue(ctx->bs_nal_in)); //pps_pic_parameter_set_id
+	gf_bs_write_ue(ctx->bs_nal_out, gf_bs_read_ue(ctx->bs_nal_in)); //pps_seq_parameter_set_id
 	gf_bs_write_int(ctx->bs_nal_out, gf_bs_read_int(ctx->bs_nal_in, 7), 7); //from dependent_slice_segments_enabled_flag to cabac_init_present_flag
-	gf_bs_set_ue(ctx->bs_nal_out, gf_bs_get_ue(ctx->bs_nal_in)); //num_ref_idx_l0_default_active_minus1
-	gf_bs_set_ue(ctx->bs_nal_out, gf_bs_get_ue(ctx->bs_nal_in)); //num_ref_idx_l1_default_active_minus1
-	gf_bs_set_se(ctx->bs_nal_out, gf_bs_get_se(ctx->bs_nal_in)); //init_qp_minus26
+	gf_bs_write_ue(ctx->bs_nal_out, gf_bs_read_ue(ctx->bs_nal_in)); //num_ref_idx_l0_default_active_minus1
+	gf_bs_write_ue(ctx->bs_nal_out, gf_bs_read_ue(ctx->bs_nal_in)); //num_ref_idx_l1_default_active_minus1
+	gf_bs_write_se(ctx->bs_nal_out, gf_bs_read_se(ctx->bs_nal_in)); //init_qp_minus26
 	gf_bs_write_int(ctx->bs_nal_out, gf_bs_read_int(ctx->bs_nal_in, 2), 2); //from constrained_intra_pred_flag to transform_skip_enabled_flag
 	cu_qp_delta_enabled_flag = gf_bs_read_int(ctx->bs_nal_in, 1); //cu_qp_delta_enabled_flag
 	gf_bs_write_int(ctx->bs_nal_out, cu_qp_delta_enabled_flag, 1); //
 	if (cu_qp_delta_enabled_flag)
-		gf_bs_set_ue(ctx->bs_nal_out, gf_bs_get_ue(ctx->bs_nal_in)); // diff_cu_qp_delta_depth
-	gf_bs_set_se(ctx->bs_nal_out, gf_bs_get_se(ctx->bs_nal_in)); // pps_cb_qp_offset
-	gf_bs_set_se(ctx->bs_nal_out, gf_bs_get_se(ctx->bs_nal_in)); // pps_cr_qp_offset
+		gf_bs_write_ue(ctx->bs_nal_out, gf_bs_read_ue(ctx->bs_nal_in)); // diff_cu_qp_delta_depth
+	gf_bs_write_se(ctx->bs_nal_out, gf_bs_read_se(ctx->bs_nal_in)); // pps_cb_qp_offset
+	gf_bs_write_se(ctx->bs_nal_out, gf_bs_read_se(ctx->bs_nal_in)); // pps_cr_qp_offset
 	gf_bs_write_int(ctx->bs_nal_out, gf_bs_read_int(ctx->bs_nal_in, 4), 4); // from pps_slice_chroma_qp_offsets_present_flag to transquant_bypass_enabled_flag
 
 
 	gf_bs_read_int(ctx->bs_nal_in, 1);
 	gf_bs_write_int(ctx->bs_nal_out, 1, 1);
 	gf_bs_write_int(ctx->bs_nal_out, gf_bs_read_int(ctx->bs_nal_in, 1), 1);//entropy_coding_sync_enabled_flag
-	gf_bs_set_ue(ctx->bs_nal_out, ctx->nb_cols-1);//write num_tile_columns_minus1
+	gf_bs_write_ue(ctx->bs_nal_out, ctx->nb_cols-1);//write num_tile_columns_minus1
 	//num_tile_rows_minus1
 	if (ctx->enable_multi_rows) {
 		u32 nb_rows = ctx->nb_rows - 1;
 		assert(ctx->nb_rows);
-		gf_bs_set_ue(ctx->bs_nal_out, nb_rows);
+		gf_bs_write_ue(ctx->bs_nal_out, nb_rows);
 	} else {
-		gf_bs_set_ue(ctx->bs_nal_out, 0);//num_tile_rows_minus1
+		gf_bs_write_ue(ctx->bs_nal_out, 0);//num_tile_rows_minus1
 	}
 	gf_bs_write_int(ctx->bs_nal_out, 0, 1);  //uniform_spacing_flag
 
@@ -189,7 +189,7 @@ static void hevcmerge_rewrite_pps(GF_HEVCMergeCtx *ctx, char *in_PPS, u32 in_PPS
 	{
 		u32 i;
 		for (i = 0; i < ctx->nb_cols-1; i++)
-			gf_bs_set_ue(ctx->bs_nal_out, (ctx->grid[i].width / ctx->max_CU_width - 1));
+			gf_bs_write_ue(ctx->bs_nal_out, (ctx->grid[i].width / ctx->max_CU_width - 1));
 
 		//if multi row is possible, declare the row height
 		if (ctx->enable_multi_rows && (ctx->nb_rows>1))  {
@@ -201,7 +201,7 @@ static void hevcmerge_rewrite_pps(GF_HEVCMergeCtx *ctx, char *in_PPS, u32 in_PPS
 				HEVCTilePidCtx *tile = gf_list_get(ctx->ordered_pids, i);
 				//only check height in the first column
 				if (tile->pos_col) continue;
-				gf_bs_set_ue(ctx->bs_nal_out, (tile->height / ctx->max_CU_width - 1)); // row_height_minus1[i]
+				gf_bs_write_ue(ctx->bs_nal_out, (tile->height / ctx->max_CU_width - 1)); // row_height_minus1[i]
 				nb_rows--;
 				if (!nb_rows) break;
 			}
@@ -301,8 +301,8 @@ u32 hevcmerge_rewrite_slice(GF_HEVCMergeCtx *ctx, HEVCTilePidCtx *tile_pid, char
 		gf_bs_write_int(ctx->bs_nal_out, gf_bs_read_int(ctx->bs_nal_in, 1), 1);
 	}
 
-	pps_id = gf_bs_get_ue(ctx->bs_nal_in);
-	gf_bs_set_ue(ctx->bs_nal_out, pps_id);
+	pps_id = gf_bs_read_ue(ctx->bs_nal_in);
+	gf_bs_write_ue(ctx->bs_nal_out, pps_id);
 
 	pps = &hevc->pps[pps_id];
 	sps = &hevc->sps[pps->sps_id];
@@ -331,15 +331,15 @@ u32 hevcmerge_rewrite_slice(GF_HEVCMergeCtx *ctx, HEVCTilePidCtx *tile_pid, char
 	}
 	//compute new qp delta
 	new_slice_qp_delta = hevc->s_info.pps->pic_init_qp_minus26 + hevc->s_info.slice_qp_delta - ctx->base_pps_init_qp_delta_minus26;
-	gf_bs_set_se(ctx->bs_nal_out, new_slice_qp_delta);
-	gf_bs_get_se(ctx->bs_nal_in);
+	gf_bs_write_se(ctx->bs_nal_out, new_slice_qp_delta);
+	gf_bs_read_se(ctx->bs_nal_in);
 
 	//copy over until num_entry_points
 	while (num_entry_point_start != (gf_bs_get_position(ctx->bs_nal_in) - 1) * 8 + gf_bs_get_bit_position(ctx->bs_nal_in)) {
 		gf_bs_write_int(ctx->bs_nal_out, gf_bs_read_int(ctx->bs_nal_in, 1), 1);
 	}
 	//write num_entry_points to 0 (always present since we use tiling)
-	gf_bs_set_ue(ctx->bs_nal_out, 0);
+	gf_bs_write_ue(ctx->bs_nal_out, 0);
 
 	//write slice extension to 0
 	if (pps->slice_segment_header_extension_present_flag)
@@ -354,7 +354,7 @@ u32 hevcmerge_rewrite_slice(GF_HEVCMergeCtx *ctx, HEVCTilePidCtx *tile_pid, char
 	//read byte_alignment() is bit=1 + x bit=0
 	al = gf_bs_read_int(ctx->bs_nal_in, 1);
 	if (al != 1) {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[HEVCMerge] source slice header not properly aligned\n"));
+		GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[HEVCMerge] source slice header not properly aligned\n"));
 	}
 	gf_bs_align(ctx->bs_nal_in);
 
@@ -400,10 +400,10 @@ static GF_Err hevcmerge_rewrite_config(GF_HEVCMergeCtx *ctx, GF_FilterPid *opid,
 
 	// for all the list objects in param_array
 	for (i = 0; i < gf_list_count(hvcc->param_array); i++) { // hvcc->param_array:list object
-		GF_HEVCParamArray *ar = (GF_HEVCParamArray *) gf_list_get(hvcc->param_array, i); // ar contains the i-th item in param_array
+		GF_NALUFFParamArray *ar = (GF_NALUFFParamArray *) gf_list_get(hvcc->param_array, i); // ar contains the i-th item in param_array
 		for (j = 0; j < gf_list_count(ar->nalus); j++) { // for all the nalus the i-th param got
 			/*! used for storing AVC sequenceParameterSetNALUnit and pictureParameterSetNALUnit*/
-			GF_AVCConfigSlot *sl = (GF_AVCConfigSlot *)gf_list_get(ar->nalus, j); // store j-th nalus in *sl
+			GF_NALUFFParam *sl = (GF_NALUFFParam *)gf_list_get(ar->nalus, j); // store j-th nalus in *sl
 
 			if (ar->type == GF_HEVC_NALU_SEQ_PARAM) {
 				char *outSPS=NULL;
@@ -512,7 +512,7 @@ void hevcmerge_build_srdmap(GF_HEVCMergeCtx *ctx, Bool use_abs_pos)
 				x = (srd_x * ctx->out_width) / srd_ref->value.vec2i.x;
 				w = (srd_w * ctx->out_width) / srd_ref->value.vec2i.x;
 			} else {
-				GF_LOG(GF_LOG_WARNING, GF_LOG_FILTER, ("[HEVCMerge] width=0 in source pid SRD referential, cannot output SRD map\n" ));
+				GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[HEVCMerge] width=0 in source pid SRD referential, cannot output SRD map\n" ));
 				srd_map_valid = GF_FALSE;
 				break;
 			}
@@ -520,7 +520,7 @@ void hevcmerge_build_srdmap(GF_HEVCMergeCtx *ctx, Bool use_abs_pos)
 				y = (srd_y * ctx->out_height) / srd_ref->value.vec2i.y;
 				h = (srd_h * ctx->out_height) / srd_ref->value.vec2i.y;
 			} else {
-				GF_LOG(GF_LOG_WARNING, GF_LOG_FILTER, ("[HEVCMerge] height=0 in source pid SRD referential, undefined results\n" ));
+				GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[HEVCMerge] height=0 in source pid SRD referential, undefined results\n" ));
 				srd_map_valid = GF_FALSE;
 				break;
 			}
@@ -530,7 +530,7 @@ void hevcmerge_build_srdmap(GF_HEVCMergeCtx *ctx, Bool use_abs_pos)
 			w = (s32)(tile->width);
 			h = (s32)(tile->height);
 		} else {
-			GF_LOG(GF_LOG_WARNING, GF_LOG_FILTER, ("[HEVCMerge] SRD, SRD_REF and CROP_POS are not defined for source pid, undefined results\n" ));
+			GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[HEVCMerge] SRD, SRD_REF and CROP_POS are not defined for source pid, undefined results\n" ));
 			srd_map_valid = GF_FALSE;
 			break;
 		}
@@ -598,15 +598,15 @@ static GF_Err hevcmerge_rebuild_grid(GF_HEVCMergeCtx *ctx,  GF_FilterPid *pid)
 		}
 	}
 	if ((nb_cols>1) && (nb_rows>1)) {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[HEVCMerge] Cannot merge more than one tile not a multiple of CTUs in both width and height, not possible in standard\n"));
+		GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[HEVCMerge] Cannot merge more than one tile not a multiple of CTUs in both width and height, not possible in standard\n"));
 		return GF_BAD_PARAM;
 	}
 	if (nb_has_pos && nb_no_pos) {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[HEVCMerge] Cannot merge tiles with explicit positioning and tiles with implicit positioning, not supported\n"));
+		GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[HEVCMerge] Cannot merge tiles with explicit positioning and tiles with implicit positioning, not supported\n"));
 		return GF_BAD_PARAM;
 	}
 	if (nb_rel_pos && nb_abs_pos) {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[HEVCMerge] Cannot merge tiles with both relative explicit positioning and absolute explicit positioning, not supported\n"));
+		GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[HEVCMerge] Cannot merge tiles with both relative explicit positioning and absolute explicit positioning, not supported\n"));
 		return GF_BAD_PARAM;
 	}
 	if (ctx->grid) gf_free(ctx->grid);
@@ -648,7 +648,7 @@ static GF_Err hevcmerge_rebuild_grid(GF_HEVCMergeCtx *ctx,  GF_FilterPid *pid)
 				}
 
 				if (overlap) {
-					GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[HEVCMerge] Overlapping tiles detected, cannot merge\n"));
+					GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[HEVCMerge] Overlapping tiles detected, cannot merge\n"));
 					return GF_BAD_PARAM;
 				}
 			}
@@ -727,7 +727,7 @@ static GF_Err hevcmerge_rebuild_grid(GF_HEVCMergeCtx *ctx,  GF_FilterPid *pid)
 			}
 		}
 		if (!ctx->grid) {
-			GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[HEVCMerge] Failed to create grid\n"));
+			GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[HEVCMerge] Failed to create grid\n"));
 			return GF_OUT_OF_MEM;
 		}
 		// pass on the grid to insert empty columns
@@ -763,7 +763,7 @@ static GF_Err hevcmerge_rebuild_grid(GF_HEVCMergeCtx *ctx,  GF_FilterPid *pid)
 			}
 			//check non-last columns are multiple of max CU width
 			if ((j+1<max_cols) && (ctx->grid[j].width % ctx->max_CU_height) ) {
-				GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[HEVCMerge] Invalid grid specification, column %d width %d not a multiple of max CU width and not the last one\n", j+1, ctx->grid[j].width));
+				GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[HEVCMerge] Invalid grid specification, column %d width %d not a multiple of max CU width and not the last one\n", j+1, ctx->grid[j].width));
 				return GF_BAD_PARAM;
 			}
 			for (i=0; i<nb_pids; i++) {
@@ -771,7 +771,7 @@ static GF_Err hevcmerge_rebuild_grid(GF_HEVCMergeCtx *ctx,  GF_FilterPid *pid)
 				if (nb_rel_pos) {
 					if (-tile->pos_x != ctx->grid[j].pos_x) continue;
 					if (ctx->grid[j].width != tile->width) {
-						GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[HEVCMerge] Invalid relative positioning in the same column of tiles with different width %d vs %d\n", tile->width, ctx->grid[j].width));
+						GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[HEVCMerge] Invalid relative positioning in the same column of tiles with different width %d vs %d\n", tile->width, ctx->grid[j].width));
 						return GF_BAD_PARAM;
 					}
 					tile->pos_col = j;
@@ -782,7 +782,7 @@ static GF_Err hevcmerge_rebuild_grid(GF_HEVCMergeCtx *ctx,  GF_FilterPid *pid)
 					tile->pos_col = j;
 					tile->pos_row = tile->pos_y / ctx->max_CU_height;
 					if (tile->pos_row * ctx->max_CU_height != tile->pos_y) {
-						GF_LOG(GF_LOG_WARNING, GF_LOG_CODEC, ("[HEVCMerge] Tile y %d not a multiple of CU height %d, adjusting to next boundary\n", tile->pos_y, ctx->max_CU_height));
+						GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[HEVCMerge] Tile y %d not a multiple of CU height %d, adjusting to next boundary\n", tile->pos_y, ctx->max_CU_height));
 						tile->pos_row++;
 					}
 					if (tile->pos_y + tile->height > ctx->grid[j].height)
@@ -799,7 +799,7 @@ static GF_Err hevcmerge_rebuild_grid(GF_HEVCMergeCtx *ctx,  GF_FilterPid *pid)
 				HEVCTilePidCtx *tile = gf_list_get(ctx->pids, i);
 				if (tile->pos_col != j) continue;
 				if ((tile->pos_row < ctx->grid[j].max_row_pos) && (tile->height % ctx->max_CU_height)) {
-					GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[HEVCMerge] Invalid grid specification, row %d in column %d height %d not a multiple of max CU height and not the last one\n", tile->pos_row, j+1, tile->height));
+					GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[HEVCMerge] Invalid grid specification, row %d in column %d height %d not a multiple of max CU height and not the last one\n", tile->pos_row, j+1, tile->height));
 					return GF_BAD_PARAM;
 				}
 			}
@@ -897,12 +897,12 @@ static GF_Err hevcmerge_rebuild_grid(GF_HEVCMergeCtx *ctx,  GF_FilterPid *pid)
 	ctx->nb_cols = nb_cols;
 
 	//recompute slice addresses
-	GF_LOG(GF_LOG_INFO, GF_LOG_CODEC, ("[HEVCMerge] Grid reconfigured, output size %dx%d, %d input pids:\n", ctx->out_width, ctx->out_height, nb_pids));
+	GF_LOG(GF_LOG_INFO, GF_LOG_MEDIA, ("[HEVCMerge] Grid reconfigured, output size %dx%d, %d input pids:\n", ctx->out_width, ctx->out_height, nb_pids));
 	for (i=0; i<nb_pids; i++) {
 		HEVCTilePidCtx *pidctx = gf_list_get(ctx->pids, i);
 		pidctx->slice_segment_address = hevcmerge_compute_address(ctx, pidctx, nb_abs_pos ? GF_TRUE : GF_FALSE);
 
-		GF_LOG(GF_LOG_INFO, GF_LOG_CODEC, ("- pid %s (pos %dx%d) size %dx%d new address %d\n",
+		GF_LOG(GF_LOG_INFO, GF_LOG_MEDIA, ("- pid %s (pos %dx%d) size %dx%d new address %d\n",
 				gf_filter_pid_get_name(pidctx->pid),
 				nb_has_pos ? pidctx->pos_x : pidctx->pos_col,
 				nb_has_pos ? pidctx->pos_y : pidctx->pos_row,
@@ -1006,7 +1006,7 @@ static GF_Err hevcmerge_check_sps_pps(GF_HEVCMergeCtx *ctx, HEVCTilePidCtx *pid_
 
 #define CHECK_SPS_VAL(__name)	\
 			if (sps_base->__name != sps_o->__name ) { \
-				GF_LOG(GF_LOG_WARNING, GF_LOG_CODEC, ("[HEVCMerge] "#__name" differs in SPS between %s and %s, undefined results\n", src_base, src_o));\
+				GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[HEVCMerge] "#__name" differs in SPS between %s and %s, undefined results\n", src_base, src_o));\
 				all_ok = GF_FALSE;\
 			}\
 
@@ -1099,7 +1099,7 @@ static GF_Err hevcmerge_check_sps_pps(GF_HEVCMergeCtx *ctx, HEVCTilePidCtx *pid_
 
 #define CHECK_PPS_VAL(__name)	\
 			if (pps_base->__name != pps_o->__name ) { \
-				GF_LOG(GF_LOG_WARNING, GF_LOG_CODEC, ("[HEVCMerge] "#__name" differs in PPS, undefined results\n"));\
+				GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[HEVCMerge] "#__name" differs in PPS, undefined results\n"));\
 				all_ok = GF_FALSE;\
 			}\
 
@@ -1171,9 +1171,10 @@ static GF_Err hevcmerge_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool
 		gf_list_del_item(ctx->pids, tile_pid);
 		gf_free(tile_pid);
 		if (!gf_list_count(ctx->pids)) {
-			if (ctx->opid)
+			if (ctx->opid) {
 				gf_filter_pid_remove(ctx->opid);
-
+				ctx->opid = NULL;
+			}
 			return GF_OK;
 		}
 		grid_config_changed = GF_TRUE;
@@ -1228,35 +1229,35 @@ static GF_Err hevcmerge_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool
 	ctx->hevc_nalu_size_length = 4;
 	e = GF_OK;
 	for (i = 0; i < gf_list_count(hvcc->param_array); i++) {
-		GF_HEVCParamArray *ar = (GF_HEVCParamArray *) gf_list_get(hvcc->param_array, i);
+		GF_NALUFFParamArray *ar = (GF_NALUFFParamArray *) gf_list_get(hvcc->param_array, i);
 		for (j = 0; j < gf_list_count(ar->nalus); j++) {
-			GF_AVCConfigSlot *sl = (GF_AVCConfigSlot *)gf_list_get(ar->nalus, j);
+			GF_NALUFFParam *sl = (GF_NALUFFParam *)gf_list_get(ar->nalus, j);
 			s32 idx = 0;
 
 			if (ar->type == GF_HEVC_NALU_SEQ_PARAM) {
-				idx = gf_media_hevc_read_sps(sl->data, sl->size, &tile_pid->hevc_state);
+				idx = gf_hevc_read_sps(sl->data, sl->size, &tile_pid->hevc_state);
 				if (idx>=0) {
 					if (!ctx->max_CU_width) {
 						ctx->max_CU_width = tile_pid->hevc_state.sps[idx].max_CU_width;
 					} else if (ctx->max_CU_width != tile_pid->hevc_state.sps[idx].max_CU_width) {
-						GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[HEVCMerge] Cannot merge tiles not using the same max CU width (%d vs %d)\n", ctx->max_CU_width, tile_pid->hevc_state.sps[idx].max_CU_width));
+						GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[HEVCMerge] Cannot merge tiles not using the same max CU width (%d vs %d)\n", ctx->max_CU_width, tile_pid->hevc_state.sps[idx].max_CU_width));
 						e = GF_BAD_PARAM;
 						break;
 					}
 					if (!ctx->max_CU_height) {
 						ctx->max_CU_height = tile_pid->hevc_state.sps[idx].max_CU_height;
 					} else if (ctx->max_CU_height != tile_pid->hevc_state.sps[idx].max_CU_height) {
-						GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[HEVCMerge] Cannot merge tiles not using the same max CU height (%d vs %d)\n", ctx->max_CU_height, tile_pid->hevc_state.sps[idx].max_CU_height));
+						GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[HEVCMerge] Cannot merge tiles not using the same max CU height (%d vs %d)\n", ctx->max_CU_height, tile_pid->hevc_state.sps[idx].max_CU_height));
 						e = GF_BAD_PARAM;
 						break;
 					}
 				}
 			}
 			else if (ar->type == GF_HEVC_NALU_VID_PARAM) {
-				idx = gf_media_hevc_read_vps(sl->data, sl->size, &tile_pid->hevc_state);
+				idx = gf_hevc_read_vps(sl->data, sl->size, &tile_pid->hevc_state);
 			}
 			else if (ar->type == GF_HEVC_NALU_PIC_PARAM) {
-				idx = gf_media_hevc_read_pps(sl->data, sl->size, &tile_pid->hevc_state);
+				idx = gf_hevc_read_pps(sl->data, sl->size, &tile_pid->hevc_state);
 			}
 			if (idx < 0) {
 				// WARNING
@@ -1300,7 +1301,7 @@ static GF_Err hevcmerge_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool
 		if (p->value.vec2i.x>0) {
 			pos_x = p->value.vec2i.x / ctx->max_CU_width;
 			if (pos_x * ctx->max_CU_width != p->value.vec2i.x) {
-				GF_LOG(GF_LOG_WARNING, GF_LOG_CODEC, ("[HEVCMerge] CropOrigin X %d is not a multiple of max CU width %d, adjusting to next boundary\n", p->value.vec2i.x, ctx->max_CU_width));
+				GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[HEVCMerge] CropOrigin X %d is not a multiple of max CU width %d, adjusting to next boundary\n", p->value.vec2i.x, ctx->max_CU_width));
 				pos_x++;
 			}
 			pos_x *= ctx->max_CU_width;
@@ -1311,7 +1312,7 @@ static GF_Err hevcmerge_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool
 		if (p->value.vec2i.y>0) {
 			pos_y = p->value.vec2i.y / ctx->max_CU_height;
 			if (pos_y * ctx->max_CU_height != p->value.vec2i.y) {
-				GF_LOG(GF_LOG_WARNING, GF_LOG_CODEC, ("[HEVCMerge] CropOrigin Y %d is not a multiple of max CU height %d, adjusting to next boundary\n", p->value.vec2i.y, ctx->max_CU_height));
+				GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[HEVCMerge] CropOrigin Y %d is not a multiple of max CU height %d, adjusting to next boundary\n", p->value.vec2i.y, ctx->max_CU_height));
 				pos_y++;
 			}
 			pos_y *= ctx->max_CU_height;
@@ -1407,7 +1408,7 @@ static GF_Err hevcmerge_process(GF_Filter *filter)
 		}
 		dts = gf_filter_pck_get_dts(pck_src);
 
-		if (dts * min_dts_timescale < min_dts * tile_pid->timescale) {
+		if (!min_dts_timescale || gf_timestamp_less(dts, tile_pid->timescale, min_dts, min_dts_timescale)) {
 			min_dts = dts;
 			min_dts_timescale = tile_pid->timescale;
 		}
@@ -1429,13 +1430,13 @@ static GF_Err hevcmerge_process(GF_Filter *filter)
 		if (nb_eos) {
 			if (pck_src) {
 				tile_pid->nb_pck++;
-				GF_LOG(GF_LOG_WARNING, GF_LOG_CODEC, ("[HEVCMerge] pids of unequal duration, skipping packet %d on pid %d\n", tile_pid->nb_pck, i+1));
+				GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[HEVCMerge] pids of unequal duration, skipping packet %d on pid %d\n", tile_pid->nb_pck, i+1));
 				gf_filter_pid_drop_packet(tile_pid->pid);
 			}
 			continue;
 		}
 		if (!pck_src) {
-			GF_LOG(GF_LOG_WARNING, GF_LOG_CODEC, ("[HEVCMerge] no data on pid %d while merging, eos detected %d\n", i+1, gf_filter_pid_is_eos(tile_pid->pid) ));
+			GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[HEVCMerge] no data on pid %d while merging, eos detected %d\n", i+1, gf_filter_pid_is_eos(tile_pid->pid) ));
 			continue;
 		}
 
@@ -1459,14 +1460,14 @@ static GF_Err hevcmerge_process(GF_Filter *filter)
 
 			nal_length = gf_bs_read_int(ctx->bs_au_in, tile_pid->nalu_size_length * 8);
 			pos = (u32) gf_bs_get_position(ctx->bs_au_in);
-			gf_media_hevc_parse_nalu(data + pos, nal_length, &tile_pid->hevc_state, &nal_unit_type, &temporal_id, &layer_id);
+			gf_hevc_parse_nalu(data + pos, nal_length, &tile_pid->hevc_state, &nal_unit_type, &temporal_id, &layer_id);
 			gf_bs_skip_bytes(ctx->bs_au_in, nal_length); //skip nal in bs
 
 			//VCL nal, rewrite slice header
 			if (nal_unit_type < 32) {
 				if (!i) current_poc = tile_pid->hevc_state.s_info.poc;
 				else if (current_poc != tile_pid->hevc_state.s_info.poc) {
-					GF_LOG(GF_LOG_WARNING, GF_LOG_CODEC, ("[HEVCMerge] merging AU %u with different POC (%d vs %d), undefined results.\n", tile_pid->nb_pck, current_poc, tile_pid->hevc_state.s_info.poc));
+					GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[HEVCMerge] merging AU %u with different POC (%d vs %d), undefined results.\n", tile_pid->nb_pck, current_poc, tile_pid->hevc_state.s_info.poc));
 				}
 
 				nal_pck_size = hevcmerge_rewrite_slice(ctx, tile_pid, data + pos, nal_length);
@@ -1474,7 +1475,7 @@ static GF_Err hevcmerge_process(GF_Filter *filter)
 			}
 			//NON-vcl, copy for SEI or drop (we should not have any SPS/PPS/VPS in the bitstream, they are in the decoder config prop)
 			else {
-				gf_media_hevc_parse_nalu(data + pos, nal_length, &tile_pid->hevc_state, &nal_unit_type, &temporal_id, &layer_id);
+				gf_hevc_parse_nalu(data + pos, nal_length, &tile_pid->hevc_state, &nal_unit_type, &temporal_id, &layer_id);
 				// Copy SEI_PREFIX only for the first sample.
 				if (nal_unit_type == GF_HEVC_NALU_SEI_PREFIX && !found_sei_prefix) {
 					found_sei_prefix = GF_TRUE;
@@ -1496,6 +1497,8 @@ static GF_Err hevcmerge_process(GF_Filter *filter)
 			}
 			if (!output_pck) {
 				output_pck = gf_filter_pck_new_alloc(ctx->opid, ctx->hevc_nalu_size_length + nal_pck_size, &output_nal);
+				if (!output_pck) return GF_OUT_OF_MEM;
+				
 				// todo: might need to rewrite crypto info
 				gf_filter_pck_merge_properties(pck_src, output_pck);
 			}
@@ -1530,7 +1533,7 @@ static GF_Err hevcmerge_process(GF_Filter *filter)
 
 static GF_Err hevcmerge_initialize(GF_Filter *filter)
 {
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_CODEC, ("[HEVCMerge] hevcmerge_initialize started.\n"));
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[HEVCMerge] hevcmerge_initialize started.\n"));
 	GF_HEVCMergeCtx *ctx = (GF_HEVCMergeCtx *)gf_filter_get_udta(filter);
 	ctx->bs_au_in = gf_bs_new((char *)ctx, 1, GF_BITSTREAM_READ);
 	ctx->bs_nal_in = gf_bs_new((char *)ctx, 1, GF_BITSTREAM_READ);
@@ -1541,7 +1544,7 @@ static GF_Err hevcmerge_initialize(GF_Filter *filter)
 
 static void hevcmerge_finalize(GF_Filter *filter)
 {
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_CODEC, ("[HEVCMerge] hevcmerge_finalize.\n"));
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[HEVCMerge] hevcmerge_finalize.\n"));
 	GF_HEVCMergeCtx *ctx = (GF_HEVCMergeCtx *)gf_filter_get_udta(filter);
 	if (ctx->buffer_nal) gf_free(ctx->buffer_nal);
 	if (ctx->buffer_nal_no_epb) gf_free(ctx->buffer_nal_no_epb);
@@ -1576,7 +1579,7 @@ static const GF_FilterCapability HEVCMergeCaps[] =
 
 static const GF_FilterArgs HEVCMergeArgs[] =
 {
-	{ OFFS(strict), "strict comparison of SPS and PPS of input pids - see filter help", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_ADVANCED},
+	{ OFFS(strict), "strict comparison of SPS and PPS of input PIDs", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(mrows), "signal multiple rows in tile grid when possible", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_ADVANCED},
 	{0}
 };
@@ -1589,36 +1592,37 @@ GF_FilterRegister HEVCMergeRegister = {
 		"If [-mrows]() is set and tiles properly align on the final grid, multiple rows will be declared in the PPS.\n"
 		"Positioning of tiles can be automatic (implicit) or explicit.\n"
 		"The filter will check the SPS and PPS configurations of input PID and warn if they are not aligned but will still process them unless [-strict]() is set.\n"
-		"The filter assumes that all input PIDs are synchronized (frames share the same timestamp) and will reassemble frames with the same dts. If pids are of unequal duration, the filter will drop frames as soon as one pid is over.\n"
+		"The filter assumes that all input PIDs are synchronized (frames share the same timestamp) and will reassemble frames with the same decode time. If PIDs are of unequal duration, the filter will drop frames as soon as one PID is over.\n"
 		"## Implicit Positioning\n"
-		"In implicit positioning, results may vary based on the order of input pids declaration.\n"
+		"In implicit positioning, results may vary based on the order of input PIDs declaration.\n"
 		"In this mode the filter will automatically allocate new columns for tiles with height not a multiple of max CU height.\n"
 		"## Explicit Positioning\n"
 		"In explicit positioning, the `CropOrigin` property on input PIDs is used to setup the tile grid. In this case, tiles shall not overlap in the final output.\n"
 		"If `CropOrigin` is used, it shall be set on all input sources.\n"
 		"If positive coordinates are used, they specify absolute positioning in pixels of the tiles. The coordinates are automatically adjusted to the next multiple of max CU width and height.\n"
-		"If negative coordinates are used, they specify relative positioning (eg `0x-1` indicates to place the tile below the tile 0x0).\n"
+		"If negative coordinates are used, they specify relative positioning (e.g. `0x-1` indicates to place the tile below the tile 0x0).\n"
 		"In this mode, it is the caller responsibility to set coordinates so that all tiles in a column have the same width and only the last row/column uses non-multiple of max CU width/height values. The filter will complain and abort if this is not respected.\n"
 		"- If an horizontal blank is detected in the layout, an empty column in the tiling grid will be inserted.\n"
 		"- If a vertical blank is detected in the layout, it is ignored.\n"
 		"  \n"
 		"## Spatial Relationship Description (SRD)\n"
 		"\n"
-		"The filter will create an `SRDMap` property in the output PID if `SRDRef` and `SRD`or `CropOrigin` are set on all input PIDs.\n"
+		"The filter will create an `SRDMap` property in the output PID if `SRDRef` and `SRD` or `CropOrigin` are set on all input PIDs.\n"
 		"The `SRDMap` allows forwarding the logical sources `SRD` in the merged PID.\n"
-		"The output pid `SRDRef` is set to the output video size.\n"
+		"The output PID `SRDRef` is set to the output video size.\n"
 		"The input `SRDRef` and `SRD` are usually specified in DASH MPD, but can be manually assigned to inputs.\n"
 		"- `SRDRef` gives the size of the referential used for the input `SRD` (usually matches the original video size, but not always)\n"
 		"- `SRD` gives the size and position of the input in the original video, expressed in `SRDRef` referential of the input.\n"
-		"The inputs do not need to have matching `SRDRef`."
+		"The inputs do not need to have matching `SRDRef`\n."
 		"EX src1:SRD=0x0x640x480:SRDRef=1280x720\n"
 		"This indicates that `src1` contains a video located at 0,0, with a size of 640x480 pixels in a virtual source of 1280x720 pixels.\n"
 		"EX src2:SRD=640x0x640x480:SRDRef=1280x720\n"
 		"This indicates that `src1` contains a video located at 640,0, with a size of 640x480 pixels in a virtual source of 1280x720 pixels.\n"
 		" \n"
-		"Each merged input is described by 8 intergers in the output `SRDMap`:\n"
+		"Each merged input is described by 8 integers in the output `SRDMap`:\n"
 		"- the source `SRD` is rescaled in the output `SRDRef` to form the first part (4 integers) of the `SRDMap` (i.e. __where was the input ?__)\n"
 		"- the source location in the reconstructed video forms the second part (4 integers) of the `SRDMap` (i.e. __where are the input pixels in the output ?__)\n"
+		" \n"
 		"Assuming the two sources are encoded at 320x240 and merged as src2 above src1, the output will be a 320x480 video with a `SRDMap` of {0,160,160,240,0,0,320,240,0,0,160,240,0,240,320,240}\n"
 		"Note: merged inputs are always listed in `SRDMap` in their tile order in the output bitstream.\n"
 		"\n"
@@ -1626,7 +1630,7 @@ GF_FilterRegister HEVCMergeRegister = {
 		"- the `CropOrigin` gives the location in the source\n"
 		"- the input size gives the size in the source, and no rescaling of referential is done\n"
 		"EX src1:CropOrigin=0x0  src1:CropOrigin=640x0 \n"
-		"Assuming the two sources are encoded at 320x240 and merged as src1 above src2, the output will be a 320x480 video with a `SRDMap` of {0,0,320,240,0,0,320,240,640,0,320,240,0,240,320,240}\n"
+		"Assuming the two sources are encoded at 320x240 and merged as src1 above src2, the output will be a 320x480 video with a `SRDMap` of `{0,0,320,240,0,0,320,240,640,0,320,240,0,240,320,240}`\n"
 	)
 	.private_size = sizeof(GF_HEVCMergeCtx),
 	SETCAPS(HEVCMergeCaps),
