@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2000-2017
+ *			Copyright (c) Telecom ParisTech 2000-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / RTP/RTSP input filter
@@ -34,7 +34,6 @@ static void rtpin_reset(GF_RTPIn *ctx, Bool is_finalized)
 		GF_RTPInStream *st = (GF_RTPInStream *)gf_list_get(ctx->streams, 0);
 		gf_list_rem(ctx->streams, 0);
 		if (!is_finalized && st->opid) gf_filter_pid_remove(st->opid);
-		st->opid = NULL;
 		rtpin_stream_del(st);
 	}
 
@@ -68,11 +67,12 @@ static GF_FilterProbeScore rtpin_probe_url(const char *url, const char *mime)
 }
 
 //simplified version of RTSP_UnpackURL for SAT>IP
-static void rtpin_satip_get_server_ip(const char *sURL, char *Server)
+void rtpin_satip_get_server_ip(const char *sURL, char *Server)
 {
 	char schema[10], *test, text[1024], *retest;
 	u32 i, len;
 	Bool is_ipv6;
+	if (!sURL) return;
 
 	strcpy(Server, "");
 
@@ -134,7 +134,7 @@ static GF_Err rtpin_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_
 	GF_RTPIn *ctx = gf_filter_get_udta(filter);
 
 	if (ctx->src) {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[RTPIn] Configure pid called on filter instanciated with SRC %s\n", ctx->src));
+		GF_LOG(GF_LOG_ERROR, GF_LOG_RTP, ("[RTPIn] Configure pid called on filter instanciated with SRC %s\n", ctx->src));
 		return GF_BAD_PARAM;
 	}
 
@@ -193,21 +193,14 @@ static void gf_rtp_switch_quality(GF_RTPIn *rtp, Bool switch_up)
 	if (!cur_stream) return;
 
 	if (switch_up) {
-		/*this is the highest stream*/
-		if (!cur_stream->next_stream) {
-			cur_stream->status = RTP_Running;
-			return;
-		} else {
-			for (i = 0; i < count; i++) {
-				stream = (GF_RTPInStream *) gf_list_get(rtp->streams, i);
-				if (stream->mid == cur_stream->next_stream) {
-					/*resume streaming next channel*/
-					rtpin_stream_init(stream, GF_FALSE);
-					stream->status = RTP_Running;
-					rtp->cur_mid = stream->mid;
-					break;
-				}
-
+		for (i = 0; i < count; i++) {
+			stream = (GF_RTPInStream *) gf_list_get(rtp->streams, i);
+			if (stream->prev_stream == cur_stream->mid) {
+				/*resume streaming next channel*/
+				rtpin_stream_init(stream, GF_FALSE);
+				stream->status = RTP_Running;
+				rtp->cur_mid = stream->mid;
+				break;
 			}
 		}
 	} else {
@@ -220,9 +213,16 @@ static void gf_rtp_switch_quality(GF_RTPIn *rtp, Bool switch_up)
 				stream = (GF_RTPInStream *) gf_list_get(rtp->streams, i);
 				if (stream->mid == cur_stream->prev_stream) {
 					/*stop streaming current channel*/
+					if (cur_stream->rtp_ch->rtp)
+						gf_sk_group_unregister(rtp->sockgroup, cur_stream->rtp_ch->rtp);
+					if (cur_stream->rtp_ch->rtcp)
+						gf_sk_group_unregister(rtp->sockgroup, cur_stream->rtp_ch->rtcp);
+
 					gf_rtp_stop(cur_stream->rtp_ch);
 					cur_stream->status = RTP_Connected;
 					rtp->cur_mid = stream->mid;
+					if (cur_stream->opid)
+						gf_filter_pid_set_eos(cur_stream->opid);
 					break;
 				}
 			}
@@ -233,7 +233,7 @@ static void gf_rtp_switch_quality(GF_RTPIn *rtp, Bool switch_up)
 }
 
 
-#ifdef FILTER_FIXME
+#if 0 //unused
 static void rtpin_send_data_base64(GF_RTPInStream *stream)
 {
 	u32 size;
@@ -252,6 +252,7 @@ static void rtpin_send_data_base64(GF_RTPInStream *stream)
 	size = gf_base64_decode(data, (u32) strlen(data), stream->buffer, stream->rtpin->block_size);
 
 	pck = gf_filter_pck_new_alloc(stream->opid, size, &pck_data);
+	if (!pck) return;
 	memcpy(pck_data, stream->buffer, size);
 	gf_filter_pck_set_cts(pck, (u64) (stream->current_start * stream->ts_res));
 	gf_filter_pck_set_sap(pck, GF_FILTER_SAP_1);
@@ -271,6 +272,9 @@ static void rtpin_check_setup(GF_RTPInStream *stream)
 		rtpin_stream_ack_connect(stream, GF_OK);
 		return;
 	default:
+		if (stream->rtsp && ! (stream->rtsp->flags & RTSP_AGG_CONTROL) && stream->session_id)  {
+			return;
+		}
 		break;
 	}
 	memset(&ch_desc, 0, sizeof(RTPIn_StreamDescribe));
@@ -305,6 +309,9 @@ static Bool rtpin_process_event(GF_Filter *filter, const GF_FilterEvent *evt)
 
 		if ((stream->status==RTP_Running) && ((ctx->last_start_range >= 0) && (ctx->last_start_range==evt->play.start_range)))
 		 	return GF_TRUE;
+		//stream was canceled due to setup failure
+		if (stream->status == RTP_Unavailable)
+			return GF_TRUE;
 
 		GF_LOG(GF_LOG_DEBUG, GF_LOG_RTP, ("[RTP] Processing play on channel @%08x - %s\n", stream, stream->rtsp ? "RTSP control" : "No control (RTP)" ));
 		/*is this RTSP or direct RTP?*/
@@ -340,8 +347,7 @@ static Bool rtpin_process_event(GF_Filter *filter, const GF_FilterEvent *evt)
 		} else {
 			ctx->last_start_range = evt->play.start_range;
 			stream->status = RTP_Running;
-			if (!stream->next_stream)
-				ctx->cur_mid = stream->mid;
+			ctx->cur_mid = stream->mid;
 
 			if (stream->rtp_ch) {
 				//wait for RTCP to perform stream sync
@@ -398,6 +404,10 @@ static Bool rtpin_process_event(GF_Filter *filter, const GF_FilterEvent *evt)
 		assert(stream->rtsp);
 		rtpin_rtsp_usercom_send(stream->rtsp, stream, evt);
 		break;
+	case GF_FEVT_CONNECT_FAIL:
+		//stream canceled due to setup failure, prevent any further setup on PLAY
+		stream->status = RTP_Unavailable;
+		break;
 	default:
 		break;
 	}
@@ -599,11 +609,22 @@ static GF_Err rtpin_process(GF_Filter *filter)
 			ctx->session->connect_error = GF_OK;
 		}
 	}
+
+	//we had data, ask for immediate re-process
+	if (tot_read) {
+		gf_filter_post_process_task(filter);
+		return GF_OK;
+	}
+
 	if (ctx->max_sleep<0)
 		gf_filter_ask_rt_reschedule(filter, (u32) ((-ctx->max_sleep) *1000) );
+	else if (!ctx->min_frame_dur_ms) {
+		gf_filter_ask_rt_reschedule(filter, 1000);
+	}
 	else {
 		assert(ctx->min_frame_dur_ms <= (u32) ctx->max_sleep);
-		gf_filter_ask_rt_reschedule(filter, ctx->min_frame_dur_ms*1000);
+		//reschedule in half the frame dur
+		gf_filter_ask_rt_reschedule(filter, ctx->min_frame_dur_ms*500);
 	}
 	return GF_OK;
 }
@@ -623,6 +644,8 @@ static GF_Err rtpin_initialize(GF_Filter *filter)
 	ctx->last_start_range = -1.0;
 
 	ctx->sockgroup = gf_sk_group_new();
+	//prevent blocking so that we are always called even if output is full
+	gf_filter_prevent_blocking(filter, GF_TRUE);
 
 	//sdp mode, we will have a configure_pid
 	if (!ctx->src) return GF_OK;
@@ -651,7 +674,7 @@ static GF_Err rtpin_initialize(GF_Filter *filter)
 		} else {
 			ip = gf_strdup(ip);
 		}
-		stream = rtpin_stream_new_standalone(ctx, ip, port);
+		stream = rtpin_stream_new_standalone(ctx, ip, port, GF_FALSE);
 		gf_free(ip);
 		if (!stream)
 			e = GF_OUT_OF_MEM;
@@ -745,31 +768,34 @@ static const GF_FilterCapability RTPInCaps[] =
 static const GF_FilterArgs RTPInArgs[] =
 {
 	{ OFFS(src), "location of source content (SDP, RTP or RTSP URL)", GF_PROP_NAME, NULL, NULL, 0},
-	{ OFFS(firstport), "default first port number to use. 0 lets the filter decide", GF_PROP_UINT, "0", NULL, GF_FS_ARG_HINT_ADVANCED},
+	{ OFFS(firstport), "default first port number to use (0 lets the filter decide)", GF_PROP_UINT, "0", NULL, GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(ifce), "default interface IP to use for multicast. If NULL, the default system interface will be used", GF_PROP_STRING, NULL, NULL, GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(ttl), "multicast TTL", GF_PROP_UINT, "127", "0-127", GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(reorder_len), "reorder length in packets", GF_PROP_UINT, "1000", NULL, GF_FS_ARG_HINT_ADVANCED},
-	{ OFFS(reorder_delay), "max delay in RTP reorderer, packets will be dispatched after that", GF_PROP_UINT, "50", NULL, GF_FS_ARG_HINT_ADVANCED},
-	{ OFFS(block_size), "buffer size fur RTP/UDP or RTSP when interleaved", GF_PROP_UINT, "0x200000", NULL, GF_FS_ARG_HINT_ADVANCED},
+	{ OFFS(reorder_delay), "max delay in RTP re-orderer, packets will be dispatched after that", GF_PROP_UINT, "50", NULL, GF_FS_ARG_HINT_ADVANCED},
+	{ OFFS(block_size), "buffer size for RTP/UDP or RTSP when interleaved", GF_PROP_UINT, "0x200000", NULL, GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(disable_rtcp), "disable RTCP reporting", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(nat_keepalive), "delay in ms of NAT keepalive, disabled by default (except for SatIP, set to 30s by default)", GF_PROP_UINT, "0", NULL, GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(force_mcast), "force multicast on indicated IP in RTSP setup", GF_PROP_STRING, NULL, NULL, GF_FS_ARG_HINT_ADVANCED},
-	{ OFFS(use_client_ports), "force using client ports  (hack for some RTSP servers overriding client ports)", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_ADVANCED},
+	{ OFFS(use_client_ports), "force using client ports (hack for some RTSP servers overriding client ports)", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(bandwidth), "set bandwidth param for RTSP requests", GF_PROP_UINT, "0", NULL, GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(default_port), "set default RTSP port", GF_PROP_UINT, "554", "0-65535", 0},
 	{ OFFS(satip_port), "set default port for SATIP", GF_PROP_UINT, "1400", "0-65535", 0},
 	{ OFFS(interleave), "set RTP over RTSP", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_EXPERT},
 	{ OFFS(udp_timeout), "default timeout before considering UDP is down", GF_PROP_UINT, "10000", NULL, 0},
 	{ OFFS(rtsp_timeout), "default timeout before considering RTSP is down", GF_PROP_UINT, "3000", NULL, 0},
-	{ OFFS(rtcp_timeout), "default timeout for RTCP trafic in ms. After this timeout, playback will start unsync. If 0 always wait for RTCP", GF_PROP_UINT, "5000", NULL, GF_FS_ARG_HINT_ADVANCED},
+	{ OFFS(rtcp_timeout), "default timeout for RTCP traffic in ms. After this timeout, playback will start out of sync. If 0 always wait for RTCP", GF_PROP_UINT, "5000", NULL, GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(autortsp), "automatically reconfig RTSP interleaving if UDP timeout", GF_PROP_BOOL, "true", NULL, GF_FS_ARG_HINT_ADVANCED},
-	{ OFFS(first_packet_drop), "set number of first RTP packet to drop - 0 if no drop", GF_PROP_UINT, "0", NULL, GF_FS_ARG_HINT_EXPERT},
-	{ OFFS(frequency_drop), "drop 1 out of N packet - 0 disable droping", GF_PROP_UINT, "0", NULL, GF_FS_ARG_HINT_EXPERT},
+	{ OFFS(first_packet_drop), "set number of first RTP packet to drop (0 if no drop)", GF_PROP_UINT, "0", NULL, GF_FS_ARG_HINT_EXPERT},
+	{ OFFS(frequency_drop), "drop 1 out of N packet (0 disable dropping)", GF_PROP_UINT, "0", NULL, GF_FS_ARG_HINT_EXPERT},
 	{ OFFS(user_agent), "user agent string, by default solved from GPAC preferences", GF_PROP_STRING, "$GUA", NULL, 0},
 	{ OFFS(languages), "user languages, by default solved from GPAC preferences", GF_PROP_STRING, "$GLANG", NULL, 0},
-	{ OFFS(stats), "update statistics to the user every given MS, 0 disables reporting", GF_PROP_UINT, "500", NULL, GF_FS_ARG_HINT_ADVANCED},
-	{ OFFS(max_sleep), "set max sleep in milliseconds. A negative value -N means to always sleep for N ms, a positive value N means to sleep at most N ms but will sleep less if frame duration is shorter", GF_PROP_SINT, "1000", NULL, GF_FS_ARG_HINT_EXPERT},
+	{ OFFS(stats), "update statistics to the user every given MS (0 disables reporting)", GF_PROP_UINT, "500", NULL, GF_FS_ARG_HINT_ADVANCED},
+	{ OFFS(max_sleep), "set max sleep in milliseconds:\n"
+	"- a negative value `-N` means to always sleep for `N` ms\n"
+	"- a positive value `N` means to sleep at most `N` ms but will sleep less if frame duration is shorter", GF_PROP_SINT, "1000", NULL, GF_FS_ARG_HINT_EXPERT},
 	{ OFFS(rtcpsync), "use RTCP to adjust synchronization", GF_PROP_BOOL, "true", NULL, GF_FS_ARG_HINT_EXPERT},
+	{ OFFS(forceagg), "force RTSP control aggregation (patch for buggy servers)", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_EXPERT},
 	{0}
 };
 
@@ -782,7 +808,7 @@ GF_FilterRegister RTPInRegister = {
 	"- RTP direct url through `rtp://` protocol scheme\n"
 	"- RTSP session processing through `rtsp://` and `satip://` protocol schemes\n"
 	" \n"
-	"The filter produces either media PIDs and compressed media frames, or file PIDs and multiplex data (e.g., MPEG-2 TS).")
+	"The filter produces either PIDs with media frames, or file PIDs with multiplexed data (e.g. MPEG-2 TS).")
 	.private_size = sizeof(GF_RTPIn),
 	.args = RTPInArgs,
 	.initialize = rtpin_initialize,
