@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2000-2017
+ *			Copyright (c) Telecom ParisTech 2000-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / RTP/RTSP input filter
@@ -164,6 +164,8 @@ static void rtp_sl_packet_cbk(void *udta, u8 *payload, u32 size, GF_SLHeader *hd
 	}
 
 	pck = gf_filter_pck_new_alloc(stream->opid, size, &pck_data);
+	if (!pck) return;
+	
 	memcpy(pck_data, payload, size);
 	if (hdr->decodingTimeStampFlag)
 		gf_filter_pck_set_dts(pck, hdr->decodingTimeStamp);
@@ -218,47 +220,7 @@ static void rtp_sl_packet_cbk(void *udta, u8 *payload, u32 size, GF_SLHeader *hd
 	hdr->decodingTimeStamp = dts;
 }
 
-GF_RTPInStream *rtpin_stream_new_satip(GF_RTPIn *rtp, const char *server_ip)
-{
-	GF_RTSPTransport trans;
-	GF_RTPInStream *tmp;
-	GF_SAFEALLOC(tmp, GF_RTPInStream);
-	if (!tmp) return NULL;
-	tmp->rtpin = rtp;
-	tmp->buffer = gf_malloc(sizeof(char) * rtp->block_size);
-
-	/*create an RTP channel*/
-	tmp->rtp_ch = gf_rtp_new();
-	tmp->control = gf_strdup("*");
-
-	memset(&trans, 0, sizeof(GF_RTSPTransport));
-	trans.Profile = "RTP/AVP";
-	trans.source = (char *) server_ip;
-	trans.IsUnicast = GF_TRUE;
-	trans.client_port_first = 0;
-	trans.client_port_last = 0;
-	trans.port_first = 0;
-	trans.port_last = 0;
-
-	if (gf_rtp_setup_transport(tmp->rtp_ch, &trans, NULL) != GF_OK) {
-		rtpin_stream_del(tmp);
-		return NULL;
-	}
-
-	gf_rtp_setup_payload(tmp->rtp_ch, 33, 90000);
-
-	if (rtp->disable_rtcp) tmp->flags |= RTP_ENABLE_RTCP;
-
-	/*setup NAT keep-alive*/
-	gf_rtp_enable_nat_keepalive(tmp->rtp_ch, rtp->nat_keepalive ? rtp->nat_keepalive : 30000);
-
-	tmp->range_start = 0;
-	tmp->range_end = 0;
-
-	return tmp;
-}
-
-GF_RTPInStream *rtpin_stream_new_standalone(GF_RTPIn *rtp, const char *flow_ip, u32 port)
+GF_RTPInStream *rtpin_stream_new_standalone(GF_RTPIn *rtp, const char *flow_ip, u32 port, Bool for_satip)
 {
 	GF_RTSPTransport trans;
 	GF_RTPInStream *tmp;
@@ -273,25 +235,35 @@ GF_RTPInStream *rtpin_stream_new_standalone(GF_RTPIn *rtp, const char *flow_ip,
 	memset(&trans, 0, sizeof(GF_RTSPTransport));
 	trans.Profile = "RTP/AVP";
 	trans.source = (char *) flow_ip;
-	trans.IsUnicast = gf_sk_is_multicast_address(flow_ip);
-	trans.client_port_first = port;
-	trans.client_port_last = port+1;
-	trans.port_first = port;
-	trans.port_last = port+1;
+	if (for_satip) {
+		tmp->control = gf_strdup("*");
+
+		trans.IsUnicast = GF_TRUE;
+		//ports not set for satip
+	} else {
+		trans.IsUnicast = gf_sk_is_multicast_address(flow_ip);
+		trans.client_port_first = port;
+		trans.client_port_last = port+1;
+		trans.port_first = port;
+		trans.port_last = port+1;
+	}
 
 	if (gf_rtp_setup_transport(tmp->rtp_ch, &trans, NULL) != GF_OK) {
 		rtpin_stream_del(tmp);
 		return NULL;
 	}
 
-//	gf_rtp_setup_payload(tmp->rtp_ch, 33, 90000);
+	if (for_satip)
+		gf_rtp_setup_payload(tmp->rtp_ch, 33, 90000);
 
-	if (rtp->disable_rtcp) tmp->flags |= RTP_ENABLE_RTCP;
+	if (rtp->disable_rtcp)
+		tmp->flags |= RTP_ENABLE_RTCP;
 
 	/*setup NAT keep-alive*/
 	gf_rtp_enable_nat_keepalive(tmp->rtp_ch, rtp->nat_keepalive ? rtp->nat_keepalive : 30000);
 	tmp->range_start = 0;
 	tmp->range_end = 0;
+
 	return tmp;
 }
 
@@ -338,9 +310,13 @@ GF_RTPInStream *rtpin_stream_new(GF_RTPIn *rtp, GF_SDPMedia *media, GF_SDPInfo *
 			sscanf(att->Value, "%*d lay L%d %*s %2999s", &base_stream, buf);
 			if (!strlen(buf))
 				sscanf(att->Value, "%*d lay %2999s", buf);
+			//we only check first indicated dependency
+			char *sep = strchr(buf, ':');
+			if (sep) sep[0] = 0;
 			sscanf(buf, "L%d", &prev_stream);
 		}
 	}
+	if (!base_stream && mid) base_stream = mid;
 
 	if (range) {
 		Start = range->start;
@@ -470,7 +446,7 @@ GF_RTPInStream *rtpin_stream_new(GF_RTPIn *rtp, GF_SDPMedia *media, GF_SDPInfo *
 		if (rvc_data) {
 			if (is_gz) {
 #ifdef GPAC_DISABLE_ZLIB
-				GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("Error: no zlib support - RVC not supported in RTP\n"));
+				GF_LOG(GF_LOG_WARNING, GF_LOG_RTP, ("Error: no zlib support - RVC not supported in RTP\n"));
 				gf_free(rvc_data);
 				return NULL;
 #else
@@ -655,8 +631,7 @@ static void rtpin_adjust_sync(GF_RTPIn *ctx)
 
 		ntp_diff_us = max_ntp_us;
 		ntp_diff_us -= stream->init_ntp_us;
-		ntp_diff_us *= stream->rtp_ch->TimeScale;
-		ntp_diff_us /= 1000000;
+		ntp_diff_us = gf_timestamp_rescale(ntp_diff_us, 1000000, stream->rtp_ch->TimeScale);
 		stream->ts_adjust = ntp_diff_us;
 	}
 }
@@ -685,8 +660,7 @@ static void rtpin_stream_on_rtcp_pck(GF_RTPInStream *stream, char *pck, u32 size
 
 		//ntp time at origin: ntp(now) - ntp(first_pck) = rtpts(now) - rtpts(orig) -> ntp_first = ntp - rtp_diff
 		rtp_diff_us = stream->rtp_ch->last_SR_rtp_time - (stream->first_rtp_ts - 1);
-		rtp_diff_us *= 1000000;
-		rtp_diff_us /= stream->rtp_ch->TimeScale;
+		rtp_diff_us = gf_timestamp_rescale(rtp_diff_us, stream->rtp_ch->TimeScale, 1000000);
 
 		stream->init_ntp_us = ntp_clock_us - rtp_diff_us;
 
@@ -739,10 +713,10 @@ u32 rtpin_stream_read(GF_RTPInStream *stream)
 			rtpin_stream_on_rtp_pck(stream, stream->buffer, size);
 		}
 	}
-	if (!tot_size) return 0;
 
 	/*and send the report*/
-	if (stream->flags & RTP_ENABLE_RTCP) gf_rtp_send_rtcp_report(stream->rtp_ch);
+	if (tot_size && (stream->flags & RTP_ENABLE_RTCP))
+		gf_rtp_send_rtcp_report(stream->rtp_ch);
 
 	/*detect timeout*/
 	if (stream->rtpin->udp_timeout) {
@@ -751,12 +725,14 @@ u32 rtpin_stream_read(GF_RTPInStream *stream)
 		} else if (stream->rtp_ch->net_info.IsUnicast) {
 			u32 diff = gf_sys_clock() - stream->last_udp_time;
 			if (diff >= stream->rtpin->udp_timeout) {
-				char szMessage[1024];
-				GF_LOG(GF_LOG_WARNING, GF_LOG_RTP, ("[RTP] UDP Timeout after %d ms\n", diff));
-				sprintf(szMessage, "No data received in %d ms%s", diff, stream->rtpin->autortsp ? ", retrying using TCP" : "");
-				rtpin_send_message(stream->rtpin, GF_IP_UDP_TIMEOUT, szMessage);
-				if (stream->rtpin->autortsp)
+				GF_LOG(GF_LOG_WARNING, GF_LOG_RTP, ("[RTP] UDP Timeout after %d ms on stream %d%s\n", diff, stream->ES_ID,
+					(stream->rtpin->session && stream->rtpin->autortsp) ? ", retrying using TCP" : ""));
+
+				if (stream->rtpin->autortsp && stream->rtpin->session && !stream->rtpin->retry_tcp) {
 					stream->rtpin->retry_tcp = GF_TRUE;
+				} else {
+					gf_filter_pid_set_eos(stream->opid);
+				}
 			}
 		}
 	}
