@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2018
+ *			Copyright (c) Telecom ParisTech 2018-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / audio output filter
@@ -37,7 +37,7 @@ typedef struct
 	Bool clock;
 	GF_Fraction64 dur;
 	Double speed, start;
-	u32 vol, pan, buffer;
+	u32 vol, pan, buffer, mbuffer, rbuffer;
 	GF_Fraction adelay;
 	
 	GF_FilterPid *pid;
@@ -51,16 +51,21 @@ typedef struct
 
 	u32 pck_offset;
 	u64 first_cts;
+	u64 last_cts;
 	Bool aborted;
-	Bool speed_set;
+	u32 speed_set;
 	GF_Filter *filter;
-	Bool is_eos;
+	Bool is_eos, in_error;
 	Bool first_write_done;
 
-	s32 pid_delay;
+	s64 pid_delay;
 
 	Bool buffer_done, no_buffering;
 	u64 hwdelay_us, totaldelay_us;
+
+	u64 rebuffer;
+	Bool do_seek;
+	u64 last_clock;
 } GF_AudioOutCtx;
 
 
@@ -81,6 +86,9 @@ void aout_reconfig(GF_AudioOutCtx *ctx)
 		gf_filter_pid_get_packet(ctx->pid);
 		return;
 	}
+	//we only support packed audio at output
+	if (afmt>GF_AUDIO_FMT_LAST_PACKED)
+		afmt -= GF_AUDIO_FMT_LAST_PACKED;
 
 	e = ctx->audio_out->Configure(ctx->audio_out, &sr, &nb_ch, &afmt, ch_cfg);
 	if (e) {
@@ -89,7 +97,7 @@ void aout_reconfig(GF_AudioOutCtx *ctx)
 		sr = 44100;
 		nb_ch = 2;
 	}
-	if (ctx->speed == FIX_ONE) ctx->speed_set = GF_TRUE;
+	if (ctx->speed == FIX_ONE) ctx->speed_set = 1;
 
 	if (ctx->vol<=100) {
 		if (ctx->audio_out->SetVolume)
@@ -114,7 +122,7 @@ void aout_reconfig(GF_AudioOutCtx *ctx)
 		gf_filter_pid_negociate_property(ctx->pid, GF_PROP_PID_AUDIO_FORMAT, &PROP_UINT(afmt));
 		gf_filter_pid_negociate_property(ctx->pid, GF_PROP_PID_NUM_CHANNELS, &PROP_UINT(nb_ch));
 		gf_filter_pid_negociate_property(ctx->pid, GF_PROP_PID_AUDIO_SPEED, &PROP_DOUBLE(ctx->speed));
-		ctx->speed_set = GF_TRUE;
+		ctx->speed_set = (ctx->speed==1.0) ? 1 : 2;
 		ctx->needs_recfg = GF_FALSE;
 		//drop all packets until next reconfig
 		ctx->wait_recfg = GF_TRUE;
@@ -125,6 +133,12 @@ void aout_reconfig(GF_AudioOutCtx *ctx)
 	} else if (e==GF_OK) {
 		ctx->needs_recfg = GF_FALSE;
 		ctx->wait_recfg = GF_FALSE;
+	} else {
+		if (!ctx->in_error) {
+			ctx->in_error = GF_TRUE;
+			gf_filter_abort(ctx->filter);
+		}
+		return;
 	}
 	ctx->bytes_per_sample = gf_audio_fmt_bit_depth(afmt) * nb_ch / 8;
 	ctx->hwdelay_us = 0;
@@ -173,6 +187,27 @@ static u32 aout_fill_output(void *ptr, u8 *buffer, u32 buffer_size)
 
 	memset(buffer, 0, buffer_size);
 	if (!ctx->pid || ctx->aborted) return 0;
+	if (!ctx->speed) return 0;
+
+	if (ctx->do_seek) {
+		GF_FilterEvent evt;
+
+		GF_FEVT_INIT(evt, GF_FEVT_STOP, ctx->pid);
+		gf_filter_pid_send_event(ctx->pid, &evt);
+
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("[AudioOut] Seek request to %f speed %f\n", ctx->start, ctx->speed));
+		gf_filter_pid_init_play_event(ctx->pid, &evt, ctx->start, ctx->speed, "VideoOut");
+		gf_filter_pid_send_event(ctx->pid, &evt);
+
+		//reinit clock
+		ctx->first_write_done = GF_FALSE;
+		ctx->buffer_done = GF_FALSE;
+		ctx->do_seek = GF_FALSE;
+		ctx->pck_offset = 0;
+		ctx->last_cts = 0;
+		return 0;
+	}
+
 
 	if (!ctx->buffer_done) {
 		u32 size;
@@ -186,7 +221,11 @@ static u32 aout_fill_output(void *ptr, u8 *buffer, u32 buffer_size)
 		/*the compositor sends empty packets after its reconfiguration to check when the config is active
 		we therefore probe the first packet before probing the buffer fullness*/
 		pck = gf_filter_pid_get_packet(ctx->pid);
-		if (!pck) return 0;
+		if (!pck) {
+			if (gf_filter_pid_is_eos(ctx->pid))
+				ctx->is_eos = GF_TRUE;
+			return 0;
+		}
 
 		if (! gf_filter_pck_is_blocking_ref(pck)) {
 			if ((dur < ctx->buffer * 1000) && !gf_filter_pid_is_eos(ctx->pid))
@@ -201,7 +240,27 @@ static u32 aout_fill_output(void *ptr, u8 *buffer, u32 buffer_size)
 				return 0;
 		}
 		ctx->buffer_done = GF_TRUE;
+		if (ctx->rebuffer) {
+			GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[AudioOut] rebuffer done in "LLU" ms\n", (u32) ( (gf_sys_clock_high_res() - ctx->rebuffer) /1000) ));
+			ctx->rebuffer = 0;
+		}
+	} else if (ctx->rbuffer && !ctx->rebuffer) {
+		//query full buffer duration in us
+		u64 dur = gf_filter_pid_query_buffer_duration(ctx->pid, GF_FALSE);
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("[AudioOut] buffer %d / %d ms\r", dur/1000, ctx->buffer));
+		if ((dur < ctx->rbuffer * 1000) && !gf_filter_pid_has_seen_eos(ctx->pid)) {
+			GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[AudioOut] buffer %u less than min threshold %u, rebuffering\n", (u32) (dur/1000), ctx->rbuffer));
+			ctx->rebuffer = gf_sys_clock_high_res();
+			ctx->buffer_done = GF_FALSE;
+			return GF_OK;
+		}
+#ifndef GPAC_DISABLE_LOG
+	} else if (gf_log_tool_level_on(GF_LOG_MMIO, GF_LOG_DEBUG)) {
+		u64 dur = gf_filter_pid_query_buffer_duration(ctx->pid, GF_FALSE);
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("[AudioOut] buffer %d / %d ms\r", dur/1000, ctx->buffer));
+#endif
 	}
+
 	//do not throw underflow log util first packet is fetched
 	if (ctx->first_write_done)
 		is_first_pck = GF_FALSE;
@@ -227,7 +286,7 @@ static u32 aout_fill_output(void *ptr, u8 *buffer, u32 buffer_size)
 
 		delay = ctx->pid_delay;
 		if (ctx->adelay.den)
-			delay += ctx->adelay.num * (s32)ctx->timescale / (s32)ctx->adelay.den;
+			delay += gf_timestamp_rescale(ctx->adelay.num, ctx->adelay.den, ctx->timescale);
 
 		cts = gf_filter_pck_get_cts(pck);
 		if (delay >= 0) {
@@ -239,10 +298,24 @@ static u32 aout_fill_output(void *ptr, u8 *buffer, u32 buffer_size)
 			cts -= (u64) -delay;
 		}
 
+		if (ctx->last_cts && (cts != GF_FILTER_NO_TS) && (cts>ctx->last_cts)) {
+			u64 now = gf_sys_clock_high_res();
+			u64 diff = cts - ctx->last_cts;
+			//diff too high and no discontinuity, wait
+			if ((diff > ctx->timescale/5) && (gf_filter_pid_get_clock_info(ctx->pid, NULL, NULL) != GF_FILTER_CLOCK_PCR_DISC) ) {
+				diff = gf_timestamp_rescale(diff, ctx->timescale, 1000000);
+				if (now < ctx->last_clock + diff) {
+					GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("[AudioOut] Frame too early by "LLU" us\n", ctx->last_clock + diff - now));
+					return 0;
+				}
+			}
+		}
+
+
 		if (ctx->dur.num>0) {
 			if (!ctx->first_cts) ctx->first_cts = cts+1;
 
-			if ((cts - ctx->first_cts + 1) * ctx->dur.den > (u64) ctx->dur.num*ctx->timescale) {
+			if (gf_timestamp_greater(cts - ctx->first_cts + 1, ctx->timescale, ctx->dur.num, ctx->dur.den)) {
 				gf_filter_pid_drop_packet(ctx->pid);
 				if (!ctx->aborted) {
 					GF_FilterEvent evt;
@@ -260,19 +333,23 @@ static u32 aout_fill_output(void *ptr, u8 *buffer, u32 buffer_size)
 		if (!done && ctx->clock && data && size) {
 			GF_Fraction64 timestamp;
 			timestamp.num = cts;
-			if (ctx->pck_offset)
-				timestamp.num += ctx->pck_offset/ctx->bytes_per_sample;
+			if (ctx->pck_offset) {
+				u32 nb_samp = ctx->pck_offset/ctx->bytes_per_sample;
+				if (ctx->timescale != ctx->sr) {
+					nb_samp = (u32) gf_timestamp_rescale(nb_samp, ctx->sr, ctx->timescale);
+				}
+				timestamp.num += nb_samp;
+			}
 
-			timestamp.num -= (ctx->hwdelay_us*ctx->timescale)/1000000;
+			timestamp.num -= gf_timestamp_rescale(ctx->hwdelay_us, 1000000, ctx->timescale);
 			if (timestamp.num<0) timestamp.num = 0;
 			timestamp.den = ctx->timescale;
 			gf_filter_hint_single_clock(ctx->filter, gf_sys_clock_high_res(), timestamp);
-			GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("[AudioOut] At %d ms audio frame CTS "LLU" (compensated time %g s)\n", gf_sys_clock(), cts, ((Double)timestamp.num)/timestamp.den ));
+			GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("[AudioOut] At %d ms audio frame CTS "LLU" (compensated time %g s, HW delay "LLU" us)\n", gf_sys_clock(), cts, ((Double)timestamp.num)/timestamp.den, ctx->hwdelay_us ));
 		}
 		
-		if (data && !ctx->wait_recfg) {
+		if (data && !ctx->wait_recfg && (size >= ctx->pck_offset)) {
 			u32 nb_copy;
-			assert(size >= ctx->pck_offset);
 			
 			nb_copy = (size - ctx->pck_offset);
 			if (nb_copy + done > buffer_size) nb_copy = buffer_size - done;
@@ -285,6 +362,8 @@ static u32 aout_fill_output(void *ptr, u8 *buffer, u32 buffer_size)
 				gf_filter_update_status(ctx->filter, -1, szStatus);
 			}
 
+			ctx->last_cts = cts;
+			ctx->last_clock = gf_sys_clock_high_res();
 
 			done += nb_copy;
 			ctx->first_write_done = GF_TRUE;
@@ -323,7 +402,9 @@ static GF_Err aout_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_r
 		return GF_OK;
 	}
 	assert(!ctx->pid || (ctx->pid==pid));
-	gf_filter_pid_check_caps(pid);
+
+	if (!gf_filter_pid_check_caps(pid))
+		return GF_NOT_SUPPORTED;
 
 	sr = afmt = nb_ch = timescale = 0;
 	ch_cfg = 0;
@@ -354,30 +435,35 @@ static GF_Err aout_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_r
 
 	if (ctx->first_cts && (ctx->timescale != timescale)) {
 		ctx->first_cts-=1;
-		ctx->first_cts *= timescale;
-		ctx->first_cts /= ctx->timescale;
+		gf_timestamp_rescale(ctx->first_cts, ctx->timescale, timescale);
 		ctx->first_cts+=1;
+
+		gf_timestamp_rescale(ctx->last_cts, ctx->timescale, timescale);
 	}
 	ctx->timescale = timescale;
 
-	p = gf_filter_pid_get_property_str(pid, "BufferLength");
-	ctx->no_buffering = (p && !p->value.sint) ? GF_TRUE : GF_FALSE;
-	if (ctx->no_buffering) ctx->buffer_done = GF_TRUE;
-
-	if ((ctx->sr==sr) && (ctx->afmt == afmt) && (ctx->nb_ch == nb_ch) && (ctx->ch_cfg == ch_cfg)) {
-		ctx->needs_recfg = GF_FALSE;
-		ctx->wait_recfg = GF_FALSE;
-		return GF_OK;
+	p = gf_filter_pid_get_property(pid, GF_PROP_PID_PLAY_BUFFER);
+	ctx->no_buffering = (p && !p->value.uint) ? GF_TRUE : GF_FALSE;
+	if (ctx->no_buffering) {
+		ctx->buffer_done = GF_TRUE;
+		ctx->clock = GF_FALSE;
+	}
+	Bool buffer_req_changed = GF_FALSE;
+	if (p && p->value.uint) {
+		if (ctx->buffer < p->value.uint) {
+			ctx->buffer = p->value.uint;
+			buffer_req_changed = GF_TRUE;
+		}
 	}
 
-	//whenever change of sample rate / format / channel, force buffer requirements and speed setup
 	if ((ctx->sr!=sr) || (ctx->afmt != afmt) || (ctx->nb_ch != nb_ch)) {
+		buffer_req_changed = GF_TRUE;
+	}
+	if (buffer_req_changed) {
 		GF_FilterEvent evt;
-		ctx->speed_set = GF_FALSE;
-
 		//set buffer reqs to bdur or 100 ms - we don't "buffer" in the filter, but this will allow dispatching
 		//several input frames in the buffer (default being 1 pck / 1000 us max in buffers). Not doing so could cause
-		//the session to end because input is blocked (no tasks posted) and output still holds a packet 
+		//the session to end because input is blocked (no tasks posted) and output still holds a packet
 		GF_FEVT_INIT(evt, GF_FEVT_BUFFER_REQ, pid);
 		evt.buffer_req.max_buffer_us = ctx->buffer * 1000;
 		if (ctx->bdur) {
@@ -386,9 +472,32 @@ static GF_Err aout_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_r
 			if (evt.buffer_req.max_buffer_us < b)
 				evt.buffer_req.max_buffer_us = (u32) b;
 		}
-		evt.buffer_req.pid_only = GF_TRUE;
-
+		//we have a max buffer, move our computed max to playout and setup max buffer
+		if (ctx->mbuffer > evt.buffer_req.max_buffer_us / 1000 ) {
+			evt.buffer_req.max_playout_us = evt.buffer_req.max_buffer_us;
+			evt.buffer_req.max_buffer_us = ctx->mbuffer * 1000;
+		}
+		//more than 200ms, use regular buffer
+		else if (ctx->buffer>200) {
+			evt.buffer_req.max_playout_us = evt.buffer_req.max_buffer_us;
+		}
+		//we don't have a max buffer, set buffer requirements to PID only
+		else {
+			evt.buffer_req.pid_only = GF_TRUE;
+		}
 		gf_filter_pid_send_event(pid, &evt);
+	}
+
+	if ((ctx->sr==sr) && (ctx->afmt == afmt) && (ctx->nb_ch == nb_ch) && (ctx->ch_cfg == ch_cfg)) {
+		ctx->needs_recfg = GF_FALSE;
+		ctx->wait_recfg = GF_FALSE;
+		return GF_OK;
+	}
+
+	//whenever change of sample rate / format / channel, force speed setup
+	if ((ctx->sr!=sr) || (ctx->afmt != afmt) || (ctx->nb_ch != nb_ch)) {
+		GF_FilterEvent evt;
+		ctx->speed_set = 0;
 
 		gf_filter_pid_init_play_event(pid, &evt, ctx->start, ctx->speed, "AudioOut");
 		gf_filter_pid_send_event(pid, &evt);
@@ -406,7 +515,7 @@ static GF_Err aout_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_r
 	if (p) aout_set_priority(ctx, p->value.uint);
 
 	p = gf_filter_pid_get_property(pid, GF_PROP_PID_DELAY);
-	ctx->pid_delay = p ? p->value.sint : 0;
+	ctx->pid_delay = p ? p->value.longsint : 0;
 
 	ctx->needs_recfg = GF_TRUE;
 	
@@ -462,7 +571,7 @@ static GF_Err aout_initialize(GF_Filter *filter)
 	}
 	if (ctx->audio_out->SelfThreaded) {
 	} else if (ctx->threaded) {
-		ctx->th = gf_th_new("AudioOutput");
+		ctx->th = gf_th_new("gf_aout");
 		gf_th_run(ctx->th, aout_th_proc, ctx);
 	}
 
@@ -487,6 +596,7 @@ static void aout_finalize(GF_Filter *filter)
 			GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("[AudioOut] audio thread stopped\n"));
 			gf_th_del(ctx->th);
 		} else {
+			ctx->aborted = GF_TRUE;
 			ctx->audio_out->Shutdown(ctx->audio_out);
 		}
 		gf_modules_close_interface((GF_BaseInterface *)ctx->audio_out);
@@ -498,6 +608,9 @@ static GF_Err aout_process(GF_Filter *filter)
 {
 	GF_AudioOutCtx *ctx = (GF_AudioOutCtx *) gf_filter_get_udta(filter);
 
+	if (ctx->in_error)
+		return GF_IO_ERR;
+
 	if (!ctx->th && ctx->needs_recfg) {
 		aout_reconfig(ctx);
 	}
@@ -531,22 +644,55 @@ static Bool aout_process_event(GF_Filter *filter, const GF_FilterEvent *evt)
 	return GF_TRUE;
 }
 
+GF_Err aout_update_arg(GF_Filter *filter, const char *arg_name, const GF_PropertyValue *new_val)
+{
+	GF_AudioOutCtx *ctx = (GF_AudioOutCtx *) gf_filter_get_udta(filter);
+
+	if (!strcmp(arg_name, "start")) {
+		if (!ctx->pid) return GF_OK;
+		ctx->do_seek = GF_TRUE;
+	}
+	else if (!strcmp(arg_name, "speed")) {
+		if (ctx->speed != new_val->value.number) {
+			if (new_val->value.number==0) return GF_OK;
+			if ((new_val->value.number==1) && ((ctx->speed==0) || (ctx->speed==1)))
+				return GF_OK;
+			if (ctx->speed_set != 2) {
+				ctx->needs_recfg = GF_TRUE;
+				ctx->speed_set = 0;
+			} else {
+				GF_FilterEvent evt;
+				GF_FEVT_INIT(evt, GF_FEVT_SET_SPEED, ctx->pid)
+				evt.play.speed = new_val->value.number;
+				gf_filter_pid_send_event(ctx->pid, &evt);
+			}
+		}
+	}
+	return GF_OK;
+}
+
+
+
 #define OFFS(_n)	#_n, offsetof(GF_AudioOutCtx, _n)
 
 static const GF_FilterArgs AudioOutArgs[] =
 {
 	{ OFFS(drv), "audio driver name", GF_PROP_NAME, NULL, NULL, 0},
-	{ OFFS(bnum), "number of audio buffers - 0 for auto", GF_PROP_UINT, "2", NULL, 0},
-	{ OFFS(bdur), "total duration of all buffers in ms - 0 for auto. The longer the audio buffer is, the longer the audio latency will be (pause/resume). The quality of fast forward audio playback will also be degradated when using large audio buffers", GF_PROP_UINT, "100", NULL, 0},
+	{ OFFS(bnum), "number of audio buffers (0 for auto)", GF_PROP_UINT, "2", NULL, 0},
+	{ OFFS(bdur), "total duration of all buffers in ms (0 for auto)", GF_PROP_UINT, "100", NULL, 0},
 	{ OFFS(threaded), "force dedicated thread creation if sound card driver is not threaded", GF_PROP_BOOL, "true", NULL, GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(dur), "only play the specified duration", GF_PROP_FRACTION, "0", NULL, GF_FS_ARG_HINT_ADVANCED},
-	{ OFFS(clock), "hint audio clock for this stream (reports system time and CTS), for other filters to use", GF_PROP_BOOL, "true", NULL, GF_FS_ARG_HINT_ADVANCED},
-	{ OFFS(speed), "set playback speed. If speed is negative and start is 0, start is set to -1", GF_PROP_DOUBLE, "1.0", NULL, 0},
-	{ OFFS(start), "set playback start offset. Negative value means percent of media dur with -1 <=> dur", GF_PROP_DOUBLE, "0.0", NULL, 0},
+	{ OFFS(clock), "hint audio clock for this stream", GF_PROP_BOOL, "true", NULL, GF_FS_ARG_HINT_ADVANCED},
+	{ OFFS(speed), "set playback speed. If speed is negative and start is 0, start is set to -1", GF_PROP_DOUBLE, "1.0", NULL, GF_FS_ARG_UPDATE},
+	{ OFFS(start), "set playback start offset. A negative value means percent of media duration with -1 equal to duration", GF_PROP_DOUBLE, "0.0", NULL, GF_FS_ARG_UPDATE},
 	{ OFFS(vol), "set default audio volume, as a percentage between 0 and 100", GF_PROP_UINT, "100", "0-100", GF_FS_ARG_UPDATE},
 	{ OFFS(pan), "set stereo pan, as a percentage between 0 and 100, 50 being centered", GF_PROP_UINT, "50", "0-100", GF_FS_ARG_UPDATE},
-	{ OFFS(buffer), "set buffer in ms", GF_PROP_UINT, "200", NULL, 0},
+	{ OFFS(buffer), "set playout buffer in ms", GF_PROP_UINT, "200", NULL, 0},
+	{ OFFS(mbuffer), "set max buffer occupancy in ms. If less than buffer, use buffer", GF_PROP_UINT, "0", NULL, 0},
+	{ OFFS(rbuffer), "rebuffer trigger in ms. If 0 or more than buffer, disable rebuffering", GF_PROP_UINT, "0", NULL, GF_FS_ARG_UPDATE},
 	{ OFFS(adelay), "set audio delay in sec", GF_PROP_FRACTION, "0", NULL, GF_FS_ARG_HINT_ADVANCED|GF_FS_ARG_UPDATE},
+	{ OFFS(buffer_done), "buffer done indication (readonly, for user app)", GF_PROP_BOOL, NULL, NULL, GF_ARG_HINT_EXPERT},
+	{ OFFS(rebuffer), "system time in us at which last rebuffer started, 0 if not rebuffering (readonly, for user app)", GF_PROP_LUINT, NULL, NULL, GF_ARG_HINT_EXPERT},
 	{0}
 };
 
@@ -561,7 +707,11 @@ static const GF_FilterCapability AudioOutCaps[] =
 GF_FilterRegister AudioOutRegister = {
 	.name = "aout",
 	GF_FS_SET_DESCRIPTION("Audio output")
-	GF_FS_SET_HELP("This filter outputs a single uncompressed audio PID to a soundcard.")
+	GF_FS_SET_HELP("This filter writes a single uncompressed audio input PID to a sound card or other audio output device.\n"
+	"\n"
+	"The longer the audio buffering [-bdur]() is, the longer the audio latency will be (pause/resume). The quality of fast forward audio playback will also be degraded when using large audio buffers.\n"
+	"\n"
+	"If [-clock]() is set, the filter will report system time (in us) and corresponding packet CTS for other filters to use for AV sync.\n")
 	.private_size = sizeof(GF_AudioOutCtx),
 	.args = AudioOutArgs,
 	SETCAPS(AudioOutCaps),
@@ -569,7 +719,8 @@ GF_FilterRegister AudioOutRegister = {
 	.finalize = aout_finalize,
 	.configure_pid = aout_configure_pid,
 	.process = aout_process,
-	.process_event = aout_process_event
+	.process_event = aout_process_event,
+	.update_arg = aout_update_arg
 };
 
 const GF_FilterRegister *aout_register(GF_FilterSession *session)
