@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2019-2020
+ *			Copyright (c) Telecom ParisTech 2019-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / rtsp output filter
@@ -57,7 +57,7 @@ typedef struct
 {
 	//options
 	char *dst, *user_agent;
-	GF_List *mounts;
+	GF_PropStringList mounts;
 	u32 port, firstport;
 	Bool xps;
 	u32 mtu;
@@ -197,18 +197,12 @@ static GF_Err rtspout_send_sdp(GF_RTSPOutSession *sess)
 	return GF_OK;
 }
 
-
 static void rtspout_del_stream(GF_RTPOutStream *st)
 {
-	if (st->rtp) gf_rtp_streamer_del(st->rtp);
-	if (st->pck) gf_filter_pid_drop_packet(st->pid);
-	if (st->avcc)
-		gf_odf_avc_cfg_del(st->avcc);
-	if (st->hvcc)
-		gf_odf_hevc_cfg_del(st->hvcc);
-	gf_free(st);
+	rtpout_del_stream(st);
 }
 
+
 static void rtspout_del_session(GF_RTSPOutSession *sess)
 {
 	//server mode, cleanup
@@ -321,7 +315,7 @@ static GF_Err rtspout_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool i
 
 	if (ctx->loop) {
 		p = gf_filter_pid_get_property(pid, GF_PROP_PID_PLAYBACK_MODE);
-		if (!p || (p->value.uint<GF_PLAYBACK_MODE_SEEK)) {
+		if (!p || (p->value.uint<GF_PLAYBACK_MODE_FASTFORWARD)) {
 			GF_LOG(GF_LOG_ERROR, GF_LOG_RTP, ("[RTSPOut] PID %s cannot be seek, disabling loop\n", gf_filter_pid_get_name(pid) ));
 
 			sess->loop_disabled = GF_TRUE;
@@ -382,7 +376,7 @@ static GF_Err rtspout_initialize(GF_Filter *filter)
 	ip = ctx->ifce;
 
 	if (!ctx->dst) {
-		if (!ctx->mounts) {
+		if (! ctx->mounts.nb_items) {
 			GF_LOG(GF_LOG_ERROR, GF_LOG_RTP, ("[RTSPOut] No root dir for server, cannot run\n" ));
 			return GF_BAD_PARAM;
 		}
@@ -469,15 +463,12 @@ static Bool rtspout_init_clock(GF_RTSPOutCtx *ctx, GF_RTSPOutSession *sess)
 
 		if (dts==GF_FILTER_NO_TS) dts=0;
 
-		dts *= 1000000;
-		dts /= stream->timescale;
+		dts = gf_timestamp_rescale(dts, stream->timescale, 1000000);
 		if (min_dts > dts)
 			min_dts = dts;
 
 		if (ctx->tso>0) {
-			u64 offset = ctx->tso;
-			offset *= stream->timescale;
-			offset /= 1000000;
+			u64 offset = gf_timestamp_rescale(ctx->tso, 1000000, stream->timescale);
 			stream->rtp_ts_offset = (u32) offset;
 		}
 		stream->current_cts = gf_filter_pck_get_cts(pck);
@@ -594,21 +585,27 @@ static GF_Err rtspout_interleave_packet(void *cbk1, void *cbk2, Bool is_rtcp, u8
 	return gf_rtsp_session_write_interleaved(sess->rtsp, idx, pck, pck_size);
 }
 
-void rtspout_on_filter_setup_error(GF_Filter *f, void *on_setup_error_udta, GF_Err e)
+Bool rtspout_on_filter_setup_error(GF_Filter *f, void *on_setup_error_udta, GF_Err e)
 {
 	GF_RTSPOutSession *sess = (GF_RTSPOutSession *)on_setup_error_udta;
 
 	gf_list_del_item(sess->filter_srcs, f);
-	if (gf_list_count(sess->filter_srcs)) return;
+	//we don't notify error at our session level if the request fails
+	if (gf_list_count(sess->filter_srcs)) return GF_TRUE;
 
 	if (sess->sdp_state != SDP_LOADED) {
 		sess->sdp_state = SDP_LOADED;
 		gf_rtsp_response_reset(sess->response);
 		sess->response->ResponseCode = NC_RTSP_Internal_Server_Error;
+		if ((e == GF_URL_ERROR) || (e == GF_URL_REMOVED) || (e == GF_IP_CONNECTION_FAILURE)) {
+			sess->response->ResponseCode = NC_RTSP_Not_Found;
+		}
 		sess->response->CSeq = sess->command->CSeq;
 		rtspout_send_response(sess->ctx, sess);
 	}
 	rtspout_del_session(sess);
+	//we don't notify error at our session level if the request fails
+	return GF_TRUE;
 }
 
 static GF_Err rtspout_load_media_service(GF_Filter *filter, GF_RTSPOutCtx *ctx, GF_RTSPOutSession *sess, char *src_url)
@@ -721,8 +718,8 @@ static char *rtspout_get_local_res_path(GF_RTSPOutCtx *ctx, char *res_path)
 {
 	u32 i;
 	char *src_url=NULL;
-	for (i=0; i<gf_list_count(ctx->mounts); i++) {
-		char *mpoint = gf_list_get(ctx->mounts, i);
+	for (i=0; i<ctx->mounts.nb_items; i++) {
+		char *mpoint = ctx->mounts.vals[i];
 
 		gf_dynstrcat(&src_url, mpoint, NULL);
 		gf_dynstrcat(&src_url, res_path, "/");
@@ -886,7 +883,7 @@ static GF_Err rtspout_process_session_signaling(GF_Filter *filter, GF_RTSPOutCtx
 						else
 							rsp_code = NC_RTSP_Not_Found;
 					} else {
-						if (gf_filter_is_supported_source(filter, src_url, NULL)) {
+						if (gf_filter_is_supported_source(filter, res_path, NULL)) {
 							src_url = gf_strdup(res_path);
 							gf_list_add(paths, src_url);
 						} else {
@@ -949,7 +946,7 @@ static GF_Err rtspout_process_session_signaling(GF_Filter *filter, GF_RTSPOutCtx
 		return GF_OK;
 	}
 
-	//forbid any access to the streams, ony describe is allowed
+	//forbid any access to the streams, only describe is allowed
 	if (sess->mcast_mirror) {
 		gf_rtsp_response_reset(sess->response);
 		sess->response->ResponseCode = NC_RTSP_Unauthorized;
@@ -1142,10 +1139,10 @@ static GF_Err rtspout_process_session_signaling(GF_Filter *filter, GF_RTSPOutCtx
 				sess->loop = GF_TRUE;
 
 			if ((sess->play_state==2) && !sess->command->Range) {
-				u64 ellapsed_us = gf_sys_clock_high_res() - sess->pause_sys_clock;
+				u64 elapsed_us = gf_sys_clock_high_res() - sess->pause_sys_clock;
 				sess->pause_sys_clock = 0;
 				sess->play_state = 1;
-				sess->sys_clock_at_init += ellapsed_us;
+				sess->sys_clock_at_init += elapsed_us;
 
 				gf_rtsp_response_reset(sess->response);
 				sess->response->ResponseCode = NC_RTSP_OK;
@@ -1261,18 +1258,18 @@ static const GF_FilterCapability RTSPOutCaps[] =
 #define OFFS(_n)	#_n, offsetof(GF_RTSPOutCtx, _n)
 static const GF_FilterArgs RTSPOutArgs[] =
 {
-	{ OFFS(dst), "location of destination file - see filter help ", GF_PROP_NAME, NULL, NULL, 0},
+	{ OFFS(dst), "location of destination resource", GF_PROP_NAME, NULL, NULL, 0},
 	{ OFFS(port), "server port", GF_PROP_UINT, "554", NULL, 0},
 	{ OFFS(firstport), "port for first stream in session", GF_PROP_UINT, "6000", NULL, GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(mtu), "size of RTP MTU in bytes", GF_PROP_UINT, "1460", NULL, 0},
-	{ OFFS(ttl), "time-to-live for muticast packets. A value of 0 uses client requested TTL, or 1", GF_PROP_UINT, "0", NULL, GF_FS_ARG_HINT_ADVANCED},
-	{ OFFS(ifce), "default network inteface to use", GF_PROP_STRING, NULL, NULL, GF_FS_ARG_HINT_ADVANCED},
-	{ OFFS(payt), "payload type to use for dynamic configs", GF_PROP_UINT, "96", "96-127", GF_FS_ARG_HINT_EXPERT},
-	{ OFFS(mpeg4), "send all streams using MPEG-4 generic payload format if posible", GF_PROP_BOOL, "false", NULL, 0},
+	{ OFFS(ttl), "time-to-live for multicast packets (a value of 0 uses client requested TTL, or 1)", GF_PROP_UINT, "0", NULL, GF_FS_ARG_HINT_ADVANCED},
+	{ OFFS(ifce), "default network interface to use", GF_PROP_STRING, NULL, NULL, GF_FS_ARG_HINT_ADVANCED},
+	{ OFFS(payt), "payload type to use for dynamic decoder configurations", GF_PROP_UINT, "96", "96-127", GF_FS_ARG_HINT_EXPERT},
+	{ OFFS(mpeg4), "send all streams using MPEG-4 generic payload format if possible", GF_PROP_BOOL, "false", NULL, 0},
 	{ OFFS(delay), "send delay for packet (negative means send earlier)", GF_PROP_SINT, "0", NULL, GF_FS_ARG_HINT_ADVANCED},
-	{ OFFS(tt), "time tolerance in microseconds. Whenever schedule time minus realtime is below this value, the packet is sent right away", GF_PROP_UINT, "1000", NULL, GF_FS_ARG_HINT_ADVANCED},
-	{ OFFS(runfor), "run the session for the given time in ms. Negative value means run for ever (if loop) or source duration, 0 only outputs the sdp", GF_PROP_SINT, "-1", NULL, GF_FS_ARG_HINT_EXPERT},
-	{ OFFS(tso), "set timestamp offset in microsecs. Negative value means random initial timestamp", GF_PROP_SINT, "-1", NULL, GF_FS_ARG_HINT_EXPERT},
+	{ OFFS(tt), "time tolerance in microsecond (whenever schedule time minus realtime is below this value, the packet is sent right away)", GF_PROP_UINT, "1000", NULL, GF_FS_ARG_HINT_ADVANCED},
+	{ OFFS(runfor), "run the session for the given time in ms. A negative value means run for ever if loop or source duration, value 0 only outputs the sdp", GF_PROP_SINT, "-1", NULL, GF_FS_ARG_HINT_EXPERT},
+	{ OFFS(tso), "set timestamp offset in microseconds (negative value means random initial timestamp)", GF_PROP_SINT, "-1", NULL, GF_FS_ARG_HINT_EXPERT},
 	{ OFFS(xps), "force parameter set injection at each SAP. If not set, only inject if different from SDP ones", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_EXPERT},
 	{ OFFS(latm), "use latm for AAC payload format", GF_PROP_BOOL, "false", NULL, 0},
 	{ OFFS(mounts), "list of directories to expose in server mode", GF_PROP_STRING_LIST, NULL, NULL, 0},
@@ -1280,8 +1277,8 @@ static const GF_FilterArgs RTSPOutArgs[] =
 	{ OFFS(maxc), "maximum number of connections", GF_PROP_UINT, "100", NULL, GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(user_agent), "user agent string, by default solved from GPAC preferences", GF_PROP_STRING, "$GUA", NULL, 0},
 	{ OFFS(close), "close RTSP connection after each request, except when RTP over RTSP is used", GF_PROP_BOOL, "true", NULL, GF_FS_ARG_HINT_EXPERT},
-	{ OFFS(loop), "loop all streams in session (not always possible depending on source type) - see filter help", GF_PROP_BOOL, "true", NULL, GF_FS_ARG_HINT_EXPERT},
-	{ OFFS(dynurl), "allow dynamic service assembly - see filter help", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_EXPERT},
+	{ OFFS(loop), "loop all streams in session (not always possible depending on source type)", GF_PROP_BOOL, "true", NULL, GF_FS_ARG_HINT_EXPERT},
+	{ OFFS(dynurl), "allow dynamic service assembly", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_EXPERT},
 	{ OFFS(mcast), "control multicast setup of a session\n"
 				"- off: clients are never allowed to create a multicast\n"
 				"- on: clients can create multicast sessions\n"
@@ -1295,31 +1292,31 @@ static const GF_FilterArgs RTSPOutArgs[] =
 GF_FilterRegister RTSPOutRegister = {
 	.name = "rtspout",
 	GF_FS_SET_DESCRIPTION("RTSP Server")
-	GF_FS_SET_HELP("The RTSP server partially implements RTSP 1.0, with support for OPTIONS, DESCRIBE, SETUP, PLAY, PAUSE and TEARDOWN.\n"\
+	GF_FS_SET_HELP("The RTSP server partially implements RTSP 1.0, with support for OPTIONS, DESCRIBE, SETUP, PLAY, PAUSE and TEARDOWN.\n"
 		"Multiple PLAY ranges are not supported, PLAY range end is not supported, PAUSE range is not supported.\n"
-		"Only aggregated control is supported for PLAY and PAUSE, PAUSE/PLAY on single stream is not supported.\n"\
-		"The server only runs on TCP, and handles request in sequence (will not probe for commands until previous response was sent).\n"\
-		"The server supports both RTP over UDP delivery and RTP interleaved over RTSP delivery.\n"\
-		"\n"\
-		"The filter can work as a simple output filter by specifying the [-dst]() option:\n"\
-		"EX gpac -i source -o rtsp://myip/sessionname\n"\
-		"EX gpac -i source dst=rtsp://myip/sessionname\n"\
-		"In this mode, only one session is possible. It is possible to [-loop]() the input source(s).\n"\
-		"\n"\
-		"The filter can work as a regular RTSP server by specifying the [-mounts]() option to indicate paths of media file to be served:\n"\
-		"EX gpac rtspout:mounts=mydir1,mydir2\n"\
-		"In server mode, it is possible to load any source supported by gpac by setting the option [-dynurl]().\n"\
-		"The expected syntax of the dynamic RTSP URLs is `rtsp://servername/?URL1[&URLN]` or `rtsp://servername/@URL1[@URLN]` \n"\
-		"Each URL can be absolute or local, in which case it is resolved against the mount point(s).\n"\
-		"EX gpac -i rtsp://localhost/?pipe://mynamepipe&myfile.mp4 [dst filters]\n"\
-		"The server will resolve this URL in a new session containing streams from myfile.mp4 and streams from pipe mynamepipe.\n"\
-		"When setting [-runfor]() in server mode, the server will exit at the end of the last session being closed.\n"\
-		"\n"\
-		"In both modes, clients can setup multicast if the [-mcast]() option is `on` or `mirror`.\n"\
-		"When [-mcast]() is set to `mirror` mode, any DESCRIBE command on a resource already delivered through a multicast session will use that multicast.\n"\
-		"Consequently, only DESCRIBE methods are processed for such sessions, other methods will return Unauthorized.\n"\
-		"\n"\
-		"The scheduling algorithm and RTP options are the same as the RTP output filter, see [gpac -h rtpout](rtpout)\n"\
+		"Only aggregated control is supported for PLAY and PAUSE, PAUSE/PLAY on single stream is not supported.\n"
+		"The server only runs on TCP, and handles request in sequence: it will not probe for commands until previous response is sent.\n"
+		"The server supports both RTP over UDP delivery and RTP interleaved over RTSP delivery.\n"
+		"\n"
+		"The filter can work as a simple output filter by specifying the [-dst]() option:\n"
+		"EX gpac -i source -o rtsp://myip/sessionname\n"
+		"EX gpac -i source -o rtsp://myip/sessionname\n"
+		"In this mode, only one session is possible. It is possible to [-loop]() the input source(s).\n"
+		"\n"
+		"The filter can work as a regular RTSP server by specifying the [-mounts]() option to indicate paths of media file to be served:\n"
+		"EX gpac rtspout:mounts=mydir1,mydir2\n"
+		"In server mode, it is possible to load any source supported by gpac by setting the option [-dynurl]().\n"
+		"The expected syntax of the dynamic RTSP URLs is `rtsp://servername/?URL1[&URLN]` or `rtsp://servername/@URL1[@URLN]` \n"
+		"Each URL can be absolute or local, in which case it is resolved against the mount point(s).\n"
+		"EX gpac -i rtsp://localhost/?pipe://mynamepipe&myfile.mp4 [dst filters]\n"
+		"The server will resolve this URL in a new session containing streams from `myfile.mp4` and streams from pipe `mynamepipe`.\n"
+		"When setting [-runfor]() in server mode, the server will exit at the end of the last session being closed.\n"
+		"\n"
+		"In both modes, clients can setup multicast if the [-mcast]() option is `on` or `mirror`.\n"
+		"When [-mcast]() is set to `mirror` mode, any DESCRIBE command on a resource already delivered through a multicast session will use that multicast.\n"
+		"Consequently, only DESCRIBE methods are processed for such sessions, other methods will return Unauthorized.\n"
+		"\n"
+		"The scheduling algorithm and RTP options are the same as the RTP output filter, see [gpac -h rtpout](rtpout)\n"
 	)
 	.private_size = sizeof(GF_RTSPOutCtx),
 	.max_extra_pids = -1,
