@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Romain Bouqueau, Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2018-2019
+ *			Copyright (c) Telecom ParisTech 2018-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / AV1 IVF/OBU/annexB reframer filter
@@ -50,8 +50,10 @@ typedef struct
 	GF_Fraction fps;
 	Double index;
 	Bool importer;
-	Bool deps;
+	Bool deps, notime, temporal_delim;
 	
+	u32 bsdbg;
+
 	//only one input pid declared
 	GF_FilterPid *ipid;
 	//only one output pid declared
@@ -80,6 +82,7 @@ typedef struct
 	Bool is_vp9;
 	Bool is_vpX;
 	u32 codecid;
+	u32 num_frames;
 	GF_VPConfig *vp_cfg;
 
 	Bool is_playing;
@@ -96,6 +99,10 @@ typedef struct
 
 	Bool pts_from_file;
 	u64 cumulated_dur, last_pts;
+	u32 bitrate;
+
+	u32 clli_crc, mdcv_crc;
+	Bool copy_props;
 } GF_AV1DmxCtx;
 
 
@@ -106,7 +113,10 @@ GF_Err av1dmx_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_remove
 
 	if (is_remove) {
 		ctx->ipid = NULL;
-		gf_filter_pid_remove(ctx->opid);
+		if (ctx->opid) {
+			gf_filter_pid_remove(ctx->opid);
+			ctx->opid = NULL;
+		}
 		return GF_OK;
 	}
 	if (! gf_filter_pid_check_caps(pid))
@@ -121,10 +131,16 @@ GF_Err av1dmx_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_remove
 		gf_filter_pid_copy_properties(ctx->opid, ctx->ipid);
 		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_UNFRAMED, NULL);
 	}
-	if (ctx->timescale) {
+
+	//if source has no timescale, recompute time
+	if (!ctx->timescale) {
+		ctx->notime = GF_TRUE;
+	} else {
 		//if we have a FPS prop, use it
 		p = gf_filter_pid_get_property(pid, GF_PROP_PID_FPS);
 		if (p) ctx->cur_fps = p->value.frac;
+
+		ctx->copy_props = GF_TRUE;
 	}
 	return GF_OK;
 }
@@ -153,10 +169,10 @@ GF_Err av1dmx_check_format(GF_Filter *filter, GF_AV1DmxCtx *ctx, GF_BitStream *b
 	ctx->pts_from_file = GF_FALSE;
 	if (gf_media_probe_ivf(bs)) {
 		u32 width = 0, height = 0;
-		u32 codec_fourcc = 0, frame_rate = 0, time_scale = 0, num_frames = 0;
+		u32 codec_fourcc = 0, timebase_den = 0, timebase_num = 0, num_frames = 0;
 		ctx->bsmode = IVF;
 
-		e = gf_media_parse_ivf_file_header(bs, &width, &height, &codec_fourcc, &frame_rate, &time_scale, &num_frames);
+		e = gf_media_parse_ivf_file_header(bs, &width, &height, &codec_fourcc, &timebase_num, &timebase_den, &num_frames);
 		if (e) return e;
 
 		switch (codec_fourcc) {
@@ -176,11 +192,11 @@ GF_Err av1dmx_check_format(GF_Filter *filter, GF_AV1DmxCtx *ctx, GF_BitStream *b
 		case GF_4CC('V', 'P', '1', '0'):
 			ctx->codecid = GF_CODECID_VP10;
 			ctx->vp_cfg = gf_odf_vp_cfg_new();
-			GF_LOG(GF_LOG_WARNING, GF_LOG_PARSER, ("[IVF] %s parsing not implemented, import might be uncomplete or broken\n", gf_4cc_to_str(codec_fourcc) ));
+			GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[IVF] %s parsing not implemented, import might be uncomplete or broken\n", gf_4cc_to_str(codec_fourcc) ));
 			break;
 		default:
 			ctx->codecid = codec_fourcc;
-			GF_LOG(GF_LOG_WARNING, GF_LOG_PARSER, ("[IVF] Unsupported codec FourCC %s\n", gf_4cc_to_str(codec_fourcc) ));
+			GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[IVF] Unsupported codec FourCC %s\n", gf_4cc_to_str(codec_fourcc) ));
 			return GF_NON_COMPLIANT_BITSTREAM;
 		}
 		if (ctx->vp_cfg && !ctx->is_vp9) {
@@ -193,38 +209,43 @@ GF_Err av1dmx_check_format(GF_Filter *filter, GF_AV1DmxCtx *ctx, GF_BitStream *b
 
 		ctx->state.width = ctx->state.width < width ? width : ctx->state.width;
 		ctx->state.height = ctx->state.height < height ? height : ctx->state.height;
-		ctx->state.tb_num = frame_rate; //time_base.numerator
-		ctx->state.tb_den = time_scale; //time_base.denominator
+		ctx->state.tb_num = timebase_num;
+		ctx->state.tb_den = timebase_den;
+		ctx->num_frames = num_frames;
 
 		if ((!ctx->fps.num || !ctx->fps.den) && ctx->state.tb_num && ctx->state.tb_den && ! ( (ctx->state.tb_num<=1) && (ctx->state.tb_den<=1) ) ) {
 			ctx->cur_fps.num = ctx->state.tb_num;
 			ctx->cur_fps.den = ctx->state.tb_den;
-			GF_LOG(GF_LOG_DEBUG, GF_LOG_PARSER, ("[AV1Dmx] Detected IVF format FPS %d/%d\n", ctx->cur_fps.num, ctx->cur_fps.den));
+			GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[AV1Dmx] Detected IVF format FPS %d/%d\n", ctx->cur_fps.num, ctx->cur_fps.den));
 			ctx->pts_from_file = GF_TRUE;
 		} else {
-			GF_LOG(GF_LOG_DEBUG, GF_LOG_PARSER, ("[AV1Dmx] Detected IVF format\n"));
+			GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[AV1Dmx] Detected IVF format\n"));
 		}
 		ctx->file_hdr_size = (u32) gf_bs_get_position(bs);
 		if (last_obu_end) (*last_obu_end) = (u32) gf_bs_get_position(bs);
 		return GF_OK;
 	} else if (gf_media_aom_probe_annexb(bs)) {
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[AV1Dmx] Detected Annex B format\n"));
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[AV1Dmx] Detected Annex B format\n"));
 		ctx->bsmode = AnnexB;
 	} else {
 		gf_bs_seek(bs, 0);
 		e = aom_av1_parse_temporal_unit_from_section5(bs, &ctx->state);
 		if (e && !gf_list_count(ctx->state.frame_state.frame_obus) ) {
+			if (e==GF_BUFFER_TOO_SMALL) {
+				gf_av1_reset_state(&ctx->state, GF_FALSE);
+				return GF_BUFFER_TOO_SMALL;
+			}
 			gf_filter_setup_failure(filter, e);
 			ctx->bsmode = UNSUPPORTED;
 			return e;
 		}
 		if (ctx->state.obu_type != OBU_TEMPORAL_DELIMITER) {
-			GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[AV1Dmx] Error OBU stream start with %s, not a temporal delimiter - NOT SUPPORTED\n", gf_av1_get_obu_name(ctx->state.obu_type) ));
+			GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[AV1Dmx] Error OBU stream start with %s, not a temporal delimiter - NOT SUPPORTED\n", gf_av1_get_obu_name(ctx->state.obu_type) ));
 			gf_filter_setup_failure(filter, e);
 			ctx->bsmode = UNSUPPORTED;
 			return e;
 		}
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[AV1Dmx] Detected OBUs Section 5 format\n"));
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[AV1Dmx] Detected OBUs Section 5 format\n"));
 		ctx->bsmode = OBUs;
 
 		gf_av1_reset_state(&ctx->state, GF_FALSE);
@@ -242,7 +263,7 @@ static void av1dmx_check_dur(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 	FILE *stream;
 	GF_Err e;
 	GF_BitStream *bs;
-	u64 duration, cur_dur, last_cdur;
+	u64 duration, cur_dur, last_cdur, rate, max_pts, last_pts;
 	AV1State av1state;
 	const char *filepath=NULL;
 	const GF_PropertyValue *p;
@@ -261,18 +282,26 @@ static void av1dmx_check_dur(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 	ctx->is_file = GF_TRUE;
 
 	if (ctx->index<0) {
-		p = gf_filter_pid_get_property(ctx->ipid, GF_PROP_PID_DOWN_SIZE);
-		if (!p || (p->value.longuint > 100000000)) {
-			GF_LOG(GF_LOG_INFO, GF_LOG_PARSER, ("[AV1/VP9] Source file larger than 100M, skipping indexing\n"));
+		if (gf_opts_get_bool("temp", "force_indexing")) {
+			ctx->index = 1.0;
 		} else {
-			ctx->index = -ctx->index;
+			p = gf_filter_pid_get_property(ctx->ipid, GF_PROP_PID_DOWN_SIZE);
+			if (!p || (p->value.longuint > 20000000)) {
+				GF_LOG(GF_LOG_INFO, GF_LOG_MEDIA, ("[AV1/VP9] Source file larger than 20M, skipping indexing\n"));
+			} else {
+				ctx->index = -ctx->index;
+			}
 		}
 	}
 	if (ctx->index<=0)
 		return;
 
-	stream = gf_fopen(filepath, "rb");
-	if (!stream) return;
+	stream = gf_fopen_ex(filepath, NULL, "rb", GF_TRUE);
+	if (!stream) {
+		if (gf_fileio_is_main_thread(p->value.string))
+			ctx->file_loaded = GF_TRUE;
+		return;
+	}
 
 	ctx->index_size = 0;
 
@@ -285,6 +314,7 @@ static void av1dmx_check_dur(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 	av1state.skip_frames = GF_TRUE;
 	av1state.config = gf_odf_av1_cfg_new();
 
+	max_pts = last_pts = 0;
 	duration = 0;
 	cur_dur = last_cdur = 0;
 	while (gf_bs_available(bs)) {
@@ -318,8 +348,14 @@ static void av1dmx_check_dur(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 		 	break;
 
 		if (pts != GF_FILTER_NO_TS) {
+			if (pts + max_pts < last_pts) {
+				max_pts = last_pts + ctx->cur_fps.den;
+			}
+			pts += max_pts;
 			duration = pts;
 			cur_dur = pts - last_cdur;
+
+			last_pts = pts;
 		} else {
 			duration += ctx->cur_fps.den;
 			cur_dur += ctx->cur_fps.den;
@@ -340,6 +376,7 @@ static void av1dmx_check_dur(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 			cur_dur = 0;
 		}
 	}
+	rate = gf_bs_get_position(bs);
 	gf_bs_del(bs);
 	gf_fclose(stream);
 	gf_odf_av1_cfg_del(av1state.config);
@@ -350,11 +387,13 @@ static void av1dmx_check_dur(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 		ctx->duration.den = ctx->cur_fps.num;
 
 		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_DURATION, & PROP_FRAC64(ctx->duration));
-	}
 
-	//currently not supported because of OBU size field rewrite - could work on some streams but we would
-	//need to analyse all OBUs in the stream for that
-	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_CAN_DATAREF, & PROP_BOOL(GF_FALSE) );
+		if (ctx->duration.num && (!gf_sys_is_test_mode() || gf_opts_get_bool("temp", "force_indexing"))) {
+			rate *= 8 * ctx->duration.den;
+			rate /= ctx->duration.num;
+			ctx->bitrate = (u32) rate;
+		}
+	}
 }
 
 
@@ -384,7 +423,7 @@ static Bool av1dmx_process_event(GF_Filter *filter, const GF_FilterEvent *evt)
 				ctx->index = -ctx->index;
 				ctx->file_loaded = GF_FALSE;
 				ctx->duration.den = ctx->duration.num = 0;
-				GF_LOG(GF_LOG_INFO, GF_LOG_PARSER, ("[AV1/VP9Demx] Play request from %d, building index\n", ctx->start_range));
+				GF_LOG(GF_LOG_INFO, GF_LOG_MEDIA, ("[AV1/VP9Demx] Play request from %d, building index\n", ctx->start_range));
 				av1dmx_check_dur(filter, ctx);
 			}
 
@@ -434,7 +473,7 @@ static GFINLINE void av1dmx_update_cts(GF_AV1DmxCtx *ctx)
 	assert(ctx->cur_fps.num);
 	assert(ctx->cur_fps.den);
 
-	if (ctx->timescale) {
+	if (!ctx->notime) {
 		u64 inc = ctx->cur_fps.den;
 		inc *= ctx->timescale;
 		inc /= ctx->cur_fps.num;
@@ -444,6 +483,39 @@ static GFINLINE void av1dmx_update_cts(GF_AV1DmxCtx *ctx)
 	}
 }
 
+static void av1dmx_set_mdcv(GF_AV1DmxCtx *ctx, GF_FilterPid *pid, GF_FilterPacket *pck)
+{
+	u32 i;
+	u64 val;
+	u8 rw_mdcv[24];
+	GF_BitStream *bs_r = gf_bs_new(ctx->state.mdcv_data, 24, GF_BITSTREAM_READ);
+	GF_BitStream *bs_w = gf_bs_new(rw_mdcv, 24, GF_BITSTREAM_WRITE);
+
+	//3x{display_primaries_x, display_primaries_y} + whitePoint_x + whitePoint_y
+	//translate from AV1 representation 0.16 float to MPEG in increments of 0.00002 (1/50000)
+	for (i=0; i<8; i++) {
+		val = gf_bs_read_u16(bs_r);
+		val = (50000 * val) / 65536;
+		gf_bs_write_u16(bs_w, (u32) val);
+	}
+	//max_display_mastering_luminance: 24.8 fixed point in AV1 vs increments of 0.0001 (1/10000) candelas per square metre in MPEG
+	val = gf_bs_read_u32(bs_r);
+	val = (10000 * val) / 256;
+	gf_bs_write_u32(bs_w, (u32) val);
+
+	//min_display_mastering_luminance: 18.14 fixed point in AV1 vs increments of 0.0001 (1/10000) candelas per square metre in MPEG
+	val = gf_bs_read_u32(bs_r);
+	val = (10000 * val) / 16384;
+	gf_bs_write_u32(bs_w, (u32) val);
+	gf_bs_del(bs_r);
+	gf_bs_del(bs_w);
+
+	if (pid) {
+		gf_filter_pid_set_property(pid, GF_PROP_PID_MASTER_DISPLAY_COLOUR, &PROP_DATA(rw_mdcv, 24));
+	} else if (pck) {
+		gf_filter_pck_set_property(pck, GF_PROP_PID_MASTER_DISPLAY_COLOUR, &PROP_DATA(rw_mdcv, 24));
+	}
+}
 static void av1dmx_check_pid(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 {
 	u8 *dsi;
@@ -459,8 +531,10 @@ static void av1dmx_check_pid(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 	dsi = NULL;
 	dsi_size = 0;
 
+	crc = 0;
 	if (ctx->vp_cfg) {
 		gf_odf_vp_cfg_write(ctx->vp_cfg, &dsi, &dsi_size, ctx->vp_cfg->codec_initdata_size ? GF_TRUE : GF_FALSE);
+		crc = gf_crc_32(dsi, dsi_size);
 	} else if (ctx->is_av1) {
 		//first or config changed, compute dsi
 		while (gf_list_count(ctx->state.config->obu_array)) {
@@ -474,6 +548,9 @@ static void av1dmx_check_pid(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 			GF_AV1_OBUArrayEntry *a = (GF_AV1_OBUArrayEntry*) gf_list_get(ctx->state.frame_state.header_obus, 0);
 			gf_list_add(ctx->state.config->obu_array, a);
 			gf_list_rem(ctx->state.frame_state.header_obus, 0);
+			if (a->obu_type == OBU_SEQUENCE_HEADER) {
+				crc = gf_crc_32(a->obu, (u32) a->obu_length);
+			}
 		}
 		gf_odf_av1_cfg_write(ctx->state.config, &dsi, &dsi_size);
 
@@ -481,11 +558,13 @@ static void av1dmx_check_pid(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 			ctx->cur_fps.num = ctx->state.tb_num;
 			ctx->cur_fps.den = ctx->state.tb_den;
 		}
-
+		if (!crc) {
+			gf_free(dsi);
+			return;
+		}
 	}
-	crc = gf_crc_32(dsi, dsi_size);
 
-	if (crc == ctx->dsi_crc) {
+	if ((crc == ctx->dsi_crc) && !ctx->copy_props) {
 		gf_free(dsi);
 		return;
 	}
@@ -493,31 +572,72 @@ static void av1dmx_check_pid(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 
 	//copy properties at init or reconfig
 	gf_filter_pid_copy_properties(ctx->opid, ctx->ipid);
+	ctx->copy_props = GF_FALSE;
+
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_STREAM_TYPE, & PROP_UINT(GF_STREAM_VISUAL));
 
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_CODECID, & PROP_UINT(ctx->codecid));
 
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_TIMESCALE, & PROP_UINT(ctx->cur_fps.num));
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_FPS, & PROP_FRAC(ctx->cur_fps));
-	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_WIDTH, & PROP_UINT(ctx->state.width));
-	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_HEIGHT, & PROP_UINT(ctx->state.height));
+	if (ctx->state.sequence_width && ctx->state.sequence_height) {
+		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_WIDTH, & PROP_UINT(ctx->state.sequence_width));
+		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_HEIGHT, & PROP_UINT(ctx->state.sequence_height));
+	} else {
+		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_WIDTH, & PROP_UINT(ctx->state.width));
+		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_HEIGHT, & PROP_UINT(ctx->state.height));
+	}
 
 	if (ctx->duration.num)
 		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_DURATION, & PROP_FRAC64(ctx->duration));
 
+	//currently not supported because of OBU size field rewrite - could work on some streams but we would
+	//need to analyse all OBUs in the stream for that
+	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_CAN_DATAREF, NULL );
+
+	if (ctx->bitrate) {
+		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_BITRATE, & PROP_UINT(ctx->bitrate));
+	}
+
 	if (dsi && dsi_size)
 		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_DECODER_CONFIG, & PROP_DATA_NO_COPY(dsi, dsi_size));
 
 	if (ctx->is_file && ctx->index) {
 		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_PLAYBACK_MODE, & PROP_UINT(GF_PLAYBACK_MODE_FASTFORWARD) );
 	}
+	if (ctx->num_frames) {
+		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_NB_FRAMES, & PROP_UINT(ctx->num_frames) );
+	}
 
+	ctx->clli_crc = 0;
+	ctx->mdcv_crc = 0;
 	if (ctx->is_av1) {
 		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_COLR_PRIMARIES, & PROP_UINT(ctx->state.color_primaries) );
 		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_COLR_TRANSFER, & PROP_UINT(ctx->state.transfer_characteristics) );
 		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_COLR_MX, & PROP_UINT(ctx->state.matrix_coefficients) );
 		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_COLR_RANGE, & PROP_BOOL(ctx->state.color_range) );
+
+
+		if (ctx->state.clli_valid) {
+			gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_CONTENT_LIGHT_LEVEL, &PROP_DATA(ctx->state.clli_data, 4));
+			ctx->clli_crc = gf_crc_32(ctx->state.clli_data, 4);
+		}
+		if (ctx->state.mdcv_valid) {
+			av1dmx_set_mdcv(ctx, ctx->opid, NULL);
+			ctx->mdcv_crc = gf_crc_32(ctx->state.mdcv_data, 24);
+		}
+
 	}
+	//disabled for the time being, matchin `colr` box will be injected by mp43mx if needed
+	//check vpX specs to see if always needed
+#if 0
+	else if (ctx->vp_cfg) {
+		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_COLR_PRIMARIES, & PROP_UINT(ctx->vp_cfg->colour_primaries) );
+		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_COLR_TRANSFER, & PROP_UINT(ctx->vp_cfg->transfer_characteristics) );
+		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_COLR_MX, & PROP_UINT(ctx->vp_cfg->matrix_coefficients) );
+		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_COLR_RANGE, & PROP_BOOL(ctx->vp_cfg->video_fullRange_flag) );
+	}
+#endif
 }
 
 GF_Err av1dmx_parse_ivf(GF_Filter *filter, GF_AV1DmxCtx *ctx)
@@ -543,7 +663,7 @@ GF_Err av1dmx_parse_ivf(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 		pts += ctx->cumulated_dur;
 		if (ctx->last_pts && (ctx->last_pts>pts)) {
 			pts -= ctx->cumulated_dur;
-			GF_LOG(GF_LOG_WARNING, GF_LOG_PARSER, ("[IVF/AV1] Corrupted timestamp "LLU" less than previous timestamp "LLU", assuming concatenation\n", pts, ctx->last_pts));
+			GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[IVF/AV1] Corrupted timestamp "LLU" less than previous timestamp "LLU", assuming concatenation\n", pts, ctx->last_pts));
 			ctx->cumulated_dur = ctx->last_pts + ctx->cur_fps.den;
 			ctx->cumulated_dur -= pts;
 			pts = ctx->cumulated_dur;
@@ -606,6 +726,10 @@ GF_Err av1dmx_parse_vp9(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 	pos_ivf_hdr = gf_bs_get_position(ctx->bs);
 	e = gf_media_parse_ivf_frame_header(ctx->bs, &frame_size, &pts);
 	if (e) return e;
+	if (!frame_size) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[IVF/VP9] Corrupted frame header !\n"));
+		return GF_NON_COMPLIANT_BITSTREAM;
+	}
 
 	pos = gf_bs_get_position(ctx->bs);
 	if (gf_bs_available(ctx->bs) < frame_size) {
@@ -617,7 +741,7 @@ GF_Err av1dmx_parse_vp9(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 		pts += ctx->cumulated_dur;
 		if (ctx->last_pts && (ctx->last_pts>pts)) {
 			pts -= ctx->cumulated_dur;
-			GF_LOG(GF_LOG_WARNING, GF_LOG_PARSER, ("[IVF/VP9] Corrupted timestamp "LLU" less than previous timestamp "LLU", assuming concatenation\n", pts, ctx->last_pts));
+			GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[IVF/VP9] Corrupted timestamp "LLU" less than previous timestamp "LLU", assuming concatenation\n", pts, ctx->last_pts));
 			ctx->cumulated_dur = ctx->last_pts + ctx->cur_fps.den;
 			ctx->cumulated_dur -= pts;
 			pts = ctx->cumulated_dur;
@@ -626,35 +750,35 @@ GF_Err av1dmx_parse_vp9(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 	}
 
 	/*check if it is a superframe*/
-	e = gf_media_vp9_parse_superframe(ctx->bs, frame_size, &num_frames_in_superframe, frame_sizes, &superframe_index_size);
+	e = gf_vp9_parse_superframe(ctx->bs, frame_size, &num_frames_in_superframe, frame_sizes, &superframe_index_size);
 	if (e) {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[VP9Dmx] Error parsing superframe structure\n"));
+		GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[VP9Dmx] Error parsing superframe structure\n"));
 		return e;
 	}
 
 	for (i = 0; i < num_frames_in_superframe; ++i) {
 		u64 pos2 = gf_bs_get_position(ctx->bs);
-		if (gf_media_vp9_parse_sample(ctx->bs, ctx->vp_cfg, &key_frame, &width, &height, &renderWidth, &renderHeight) != GF_OK) {
-			GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[VP9Dmx] Error parsing frame\n"));
+		if (gf_vp9_parse_sample(ctx->bs, ctx->vp_cfg, &key_frame, &width, &height, &renderWidth, &renderHeight) != GF_OK) {
+			GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[VP9Dmx] Error parsing frame\n"));
 			return e;
 		}
 		e = gf_bs_seek(ctx->bs, pos2 + frame_sizes[i]);
 		if (e) {
-			GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[VP9Dmx] Seek bad param (offset "LLU") (1)", pos2 + frame_sizes[i]));
+			GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[VP9Dmx] Seek bad param (offset "LLU") (1)", pos2 + frame_sizes[i]));
 			return e;
 		}
 	}
 	if (gf_bs_get_position(ctx->bs) + superframe_index_size != pos + frame_size) {
-		GF_LOG(GF_LOG_WARNING, GF_LOG_PARSER, ("[VP9Dmx] Inconsistent IVF frame size of "LLU" bytes.\n", frame_size));
-		GF_LOG(GF_LOG_WARNING, GF_LOG_PARSER, ("      Detected %d frames (+ %d bytes for the superframe index):\n", num_frames_in_superframe, superframe_index_size));
+		GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[VP9Dmx] Inconsistent IVF frame size of "LLU" bytes.\n", frame_size));
+		GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("      Detected %d frames (+ %d bytes for the superframe index):\n", num_frames_in_superframe, superframe_index_size));
 		for (i = 0; i < num_frames_in_superframe; ++i) {
-			GF_LOG(GF_LOG_WARNING, GF_LOG_PARSER, ("         superframe %d, size is %u bytes\n", i, frame_sizes[i]));
+			GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("         superframe %d, size is %u bytes\n", i, frame_sizes[i]));
 		}
-		GF_LOG(GF_LOG_WARNING, GF_LOG_PARSER, ("\n"));
+		GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("\n"));
 	}
 	e = gf_bs_seek(ctx->bs, pos + frame_size);
 	if (e) {
-		GF_LOG(GF_LOG_WARNING, GF_LOG_PARSER, ("[VP9Dmx] Seek bad param (offset "LLU") (2)", pos + frame_size));
+		GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[VP9Dmx] Seek bad param (offset "LLU") (2)", pos + frame_size));
 		return e;
 	}
 
@@ -714,19 +838,25 @@ GF_Err av1dmx_parse_vp9(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 
 static GF_Err av1dmx_parse_flush_sample(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 {
-	u32 pck_size;
-	GF_FilterPacket *pck;
-	u8 *output;
+	u32 pck_size = 0;
+	GF_FilterPacket *pck = NULL;
+	u8 *output = NULL;
 
+	if (!ctx->opid)
+		return GF_NON_COMPLIANT_BITSTREAM;
+		
 	gf_bs_get_content_no_truncate(ctx->state.bs, &ctx->state.frame_obus, &pck_size, &ctx->state.frame_obus_alloc);
 
 	if (!pck_size) {
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[AV1Dmx] no frame OBU, skipping OBU\n"));
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[AV1Dmx] no frame OBU, skipping OBU\n"));
 		return GF_OK;
 	}
 
 	pck = gf_filter_pck_new_alloc(ctx->opid, pck_size, &output);
-	if (ctx->src_pck) gf_filter_pck_merge_properties(ctx->src_pck, pck);
+	if (!pck) return GF_OUT_OF_MEM;
+
+	if (ctx->src_pck)
+		gf_filter_pck_merge_properties(ctx->src_pck, pck);
 
 	gf_filter_pck_set_cts(pck, ctx->cts);
 	gf_filter_pck_set_sap(pck, ctx->state.frame_state.key_frame ? GF_FILTER_SAP_1 : 0);
@@ -746,6 +876,19 @@ static GF_Err av1dmx_parse_flush_sample(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 	 	gf_filter_pck_set_dependency_flags(pck, flags);
 	}
 
+	if (ctx->state.clli_valid) {
+		u32 crc = gf_crc_32(ctx->state.clli_data, 4);
+		if (crc != ctx->clli_crc) {
+			gf_filter_pck_set_property(pck, GF_PROP_PID_CONTENT_LIGHT_LEVEL, &PROP_DATA(ctx->state.clli_data, 4));
+		}
+	}
+	if (ctx->state.mdcv_valid) {
+		u32 crc = gf_crc_32(ctx->state.mdcv_data, 24);
+		if (crc != ctx->mdcv_crc) {
+			av1dmx_set_mdcv(ctx, NULL, pck);
+		}
+	}
+
 	gf_filter_pck_send(pck);
 
 	av1dmx_update_cts(ctx);
@@ -806,7 +949,7 @@ GF_Err av1dmx_parse_av1(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 
 	if (!ctx->opid) {
 		if (ctx->state.obu_type != OBU_TEMPORAL_DELIMITER) {
-			GF_LOG(GF_LOG_WARNING, GF_LOG_CONTAINER, ("[AV1Dmx] output pid not configured (no sequence header yet ?), skipping OBU\n"));
+			GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[AV1Dmx] output pid not configured (no sequence header yet ?), skipping OBU\n"));
 		}
 		gf_av1_reset_state(&ctx->state, GF_FALSE);
 		return GF_OK;
@@ -822,6 +965,27 @@ GF_Err av1dmx_parse_av1(GF_Filter *filter, GF_AV1DmxCtx *ctx)
 
 }
 
+GF_Err gf_bs_set_logger(GF_BitStream *bs, void (*on_bs_log)(void *udta, const char *field_name, u32 nb_bits, u64 field_val, s32 idx1, s32 idx2, s32 idx3), void *udta);
+static void av1dmx_bs_log(void *udta, const char *field_name, u32 nb_bits, u64 field_val, s32 idx1, s32 idx2, s32 idx3)
+{
+	GF_AV1DmxCtx *ctx = (GF_AV1DmxCtx *) udta;
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, (" %s", field_name));
+	if (idx1>=0) {
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("_%d", idx1));
+		if (idx2>=0) {
+			GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("_%d", idx2));
+			if (idx3>=0) {
+				GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("_%d", idx3));
+			}
+		}
+	}
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("=\""LLD, field_val));
+	if ((ctx->bsdbg==2) && ((s32) nb_bits > 1) )
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("(%u)", nb_bits));
+
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("\" "));
+}
+
 GF_Err av1dmx_process_buffer(GF_Filter *filter, GF_AV1DmxCtx *ctx, const char *data, u32 data_size, Bool is_copy)
 {
 	u32 last_obu_end = 0;
@@ -830,9 +994,15 @@ GF_Err av1dmx_process_buffer(GF_Filter *filter, GF_AV1DmxCtx *ctx, const char *d
 	if (!ctx->bs) ctx->bs = gf_bs_new(data, data_size, GF_BITSTREAM_READ);
 	else gf_bs_reassign_buffer(ctx->bs, data, data_size);
 
+#ifndef GPAC_DISABLE_LOG
+	if (ctx->bsdbg && gf_log_tool_level_on(GF_LOG_MEDIA, GF_LOG_DEBUG))
+		gf_bs_set_logger(ctx->bs, av1dmx_bs_log, ctx);
+#endif
+
 	//check ivf vs obu vs annexB
 	e = av1dmx_check_format(filter, ctx, ctx->bs, &last_obu_end);
-	if (e) return e;
+	if (e==GF_BUFFER_TOO_SMALL) return GF_OK;
+	else if (e) return e;
 
 	while (gf_bs_available(ctx->bs)) {
 
@@ -913,10 +1083,9 @@ GF_Err av1dmx_process(GF_Filter *filter)
 
 	data = (char *) gf_filter_pck_get_data(pck, &pck_size);
 
-	//input pid sets some timescale - we flushed pending data , update cts
+	//input pid is muxed - we flushed pending data , update cts unless recomputing all times
 	if (ctx->timescale) {
 		Bool start, end;
-		u64 cts;
 
 		e = GF_OK;
 
@@ -945,10 +1114,12 @@ GF_Err av1dmx_process(GF_Filter *filter)
 			if (e) return e;
 		}
 
-		//begining of a new frame
-		cts = gf_filter_pck_get_cts(pck);
-		if (cts != GF_FILTER_NO_TS)
-			ctx->cts = cts;
+		//beginning of a new frame
+		if (!ctx->notime) {
+			u64 cts = gf_filter_pck_get_cts(pck);
+			if (cts != GF_FILTER_NO_TS)
+				ctx->cts = cts;
+		}
 		if (ctx->src_pck) gf_filter_pck_unref(ctx->src_pck);
 		ctx->src_pck = pck;
 		gf_filter_pck_ref_props(&ctx->src_pck);
@@ -984,6 +1155,16 @@ GF_Err av1dmx_process(GF_Filter *filter)
 	return e;
 }
 
+static GF_Err av1dmx_initialize(GF_Filter *filter)
+{
+	GF_AV1DmxCtx *ctx = gf_filter_get_udta(filter);
+	gf_av1_init_state(&ctx->state);
+	if (ctx->temporal_delim)
+		ctx->state.keep_temporal_delim = GF_TRUE;
+
+	return GF_OK;
+}
+
 static void av1dmx_finalize(GF_Filter *filter)
 {
 	GF_AV1DmxCtx *ctx = gf_filter_get_udta(filter);
@@ -1014,9 +1195,11 @@ static const char * av1dmx_probe_data(const u8 *data, u32 size, GF_FilterProbeSc
 		mime = "video/x-ivf";
 	} else {
 		res = gf_media_aom_probe_annexb(bs);
-		if (res) *score = GF_FPROBE_SUPPORTED;
-		else {
+		if (res) {
+			*score = GF_FPROBE_SUPPORTED;
+		} else {
 			AV1State state;
+			Bool has_seq_header = GF_FALSE;
 			GF_Err e;
 			u32 nb_units = 0;
 
@@ -1026,23 +1209,40 @@ static const char * av1dmx_probe_data(const u8 *data, u32 size, GF_FilterProbeSc
 				e = aom_av1_parse_temporal_unit_from_section5(bs, &state);
 				if ((e==GF_OK) || (nb_units && (e==GF_BUFFER_TOO_SMALL) ) ) {
 					if (!nb_units || gf_list_count(state.frame_state.header_obus) || gf_list_count(state.frame_state.frame_obus)) {
+						if (gf_list_count(state.frame_state.header_obus)) {
+							has_seq_header = GF_TRUE;
+						}
 						nb_units++;
 						if (e==GF_BUFFER_TOO_SMALL)
 							nb_units++;
-					} else
+					} else {
+						//we got one frame + seq header without errors, assume maybe supported
+						if (has_seq_header) {
+							res = GF_TRUE;
+							*score = GF_FPROBE_MAYBE_SUPPORTED;
+						}
 						break;
+					}
+				}
+				//very large frame
+				else if (!nb_units && (e==GF_BUFFER_TOO_SMALL)) {
+					if (gf_list_count(state.frame_state.header_obus) && state.width && state.height) {
+						res = GF_TRUE;
+						*score = GF_FPROBE_MAYBE_SUPPORTED;
+					}
+					break;
 				} else {
 					break;
 				}
 				gf_av1_reset_state(&state, GF_FALSE);
-				if (nb_units>2) break;
+				if (nb_units>2) {
+					res = GF_TRUE;
+					*score = GF_FPROBE_SUPPORTED;
+					break;
+				}
 			}
 			gf_odf_av1_cfg_del(state.config);
 			gf_av1_reset_state(&state, GF_TRUE);
-			if (nb_units>2) {
-				res = GF_TRUE;
-				*score = GF_FPROBE_MAYBE_SUPPORTED;
-			}
 		}
 	}
 
@@ -1077,10 +1277,18 @@ static const GF_FilterCapability AV1DmxCaps[] =
 static const GF_FilterArgs AV1DmxArgs[] =
 {
 	{ OFFS(fps), "import frame rate (0 default to FPS from bitstream or 25 Hz)", GF_PROP_FRACTION, "0/1000", NULL, 0},
-	{ OFFS(index), "indexing window length. If 0, bitstream is not probed for duration. A negative value skips the indexing if the source file is larger than 100M (slows down importers) unless a play with start range > 0 is issued, otherwise uses the positive value", GF_PROP_DOUBLE, "-1.0", NULL, 0},
+	{ OFFS(index), "indexing window length. If 0, bitstream is not probed for duration. A negative value skips the indexing if the source file is larger than 100M (slows down importers) unless a play with start range > 0 is issued", GF_PROP_DOUBLE, "-1.0", NULL, 0},
 
 	{ OFFS(importer), "compatibility with old importer", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_ADVANCED},
-	{ OFFS(deps), "import samples dependencies information", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_EXPERT},
+	{ OFFS(deps), "import sample dependency information", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_EXPERT},
+	{ OFFS(notime), "ignore input timestamps, rebuild from 0", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_ADVANCED},
+	{ OFFS(temporal_delim), "keep temporal delimiters in reconstructed frames", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_EXPERT},
+
+
+	{ OFFS(bsdbg), "debug NAL parsing in `parser@debug logs\n"
+		"- off: not enabled\n"
+		"- on: enabled\n"
+		"- full: enable with number of bits dumped", GF_PROP_UINT, "off", "off|on|full", GF_FS_ARG_HINT_EXPERT},
 	{0}
 };
 
@@ -1091,6 +1299,7 @@ GF_FilterRegister AV1DmxRegister = {
 	GF_FS_SET_HELP("This filter parses AV1 OBU, AV1 AnnexB or IVF with AV1 or VP9 files/data and outputs corresponding visual PID and frames.")
 	.private_size = sizeof(GF_AV1DmxCtx),
 	.args = AV1DmxArgs,
+	.initialize = av1dmx_initialize,
 	.finalize = av1dmx_finalize,
 	SETCAPS(AV1DmxCaps),
 	.configure_pid = av1dmx_configure_pid,
