@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2000-2017
+ *			Copyright (c) Telecom ParisTech 2000-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / MP3 reframer filter
@@ -39,7 +39,7 @@ typedef struct
 {
 	//filter args
 	Double index;
-	Bool expart;
+	Bool expart, forcemp3;
 
 	//only one input pid declared
 	GF_FilterPid *ipid;
@@ -74,6 +74,7 @@ typedef struct
 	u32 id3_buffer_size, id3_buffer_alloc;
 
 	GF_FilterPid *vpid;
+	Bool copy_props;
 } GF_MP3DmxCtx;
 
 
@@ -86,7 +87,10 @@ GF_Err mp3_dmx_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_remov
 
 	if (is_remove) {
 		ctx->ipid = NULL;
-		gf_filter_pid_remove(ctx->opid);
+		if (ctx->opid) {
+			gf_filter_pid_remove(ctx->opid);
+			ctx->opid = NULL;
+		}
 		return GF_OK;
 	}
 	if (! gf_filter_pid_check_caps(pid))
@@ -105,6 +109,7 @@ GF_Err mp3_dmx_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_remov
 		gf_filter_pid_copy_properties(ctx->opid, ctx->ipid);
 		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_UNFRAMED, NULL);
 	}
+	if (ctx->timescale) ctx->copy_props = GF_TRUE;
 	return GF_OK;
 }
 
@@ -129,8 +134,12 @@ static void mp3_dmx_check_dur(GF_Filter *filter, GF_MP3DmxCtx *ctx)
 	}
 	ctx->is_file = GF_TRUE;
 
-	stream = gf_fopen(p->value.string, "rb");
-	if (!stream) return;
+	stream = gf_fopen_ex(p->value.string, NULL, "rb", GF_TRUE);
+	if (!stream) {
+		if (gf_fileio_is_main_thread(p->value.string))
+			ctx->file_loaded = GF_TRUE;
+		return;
+	}
 
 	ctx->index_size = 0;
 
@@ -180,7 +189,6 @@ static void mp3_dmx_check_dur(GF_Filter *filter, GF_MP3DmxCtx *ctx)
 
 	p = gf_filter_pid_get_property(ctx->ipid, GF_PROP_PID_FILE_CACHED);
 	if (p && p->value.boolean) ctx->file_loaded = GF_TRUE;
-	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_CAN_DATAREF, & PROP_BOOL(GF_TRUE ) );
 }
 
 
@@ -189,10 +197,10 @@ static void id3dmx_set_string(GF_FilterPid *apid, char *name, u8 *buf, Bool is_d
 {
 	if ((buf[0]==0xFF) || (buf[0]==0xFE)) {
 		const u16 *sptr = (u16 *) (buf+2);
-		s32 len = (s32) ( UTF8_MAX_BYTES_PER_CHAR * gf_utf8_wcslen(sptr) );
+		u32 len = UTF8_MAX_BYTES_PER_CHAR * gf_utf8_wcslen(sptr);
 		char *tmp = gf_malloc(len+1);
-		len = (s32) gf_utf8_wcstombs(tmp, len, &sptr);
-		if (len>=0) {
+		len = gf_utf8_wcstombs(tmp, len, &sptr);
+		if (len != GF_UTF8_FAIL) {
 			tmp[len] = 0;
 			if (is_dyn) {
 				gf_filter_pid_set_property_dyn(apid, name, &PROP_STRING(tmp) );
@@ -200,7 +208,7 @@ static void id3dmx_set_string(GF_FilterPid *apid, char *name, u8 *buf, Bool is_d
 				gf_filter_pid_set_property_str(apid, name, &PROP_STRING(tmp) );
 			}
 		} else {
-			GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[MP3Dmx] Corrupted ID3 text frame %s\n", name));
+			GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[MP3Dmx] Corrupted ID3 text frame %s\n", name));
 		}
 		gf_free(tmp);
 	} else {
@@ -216,7 +224,7 @@ void id3dmx_flush(GF_Filter *filter, u8 *id3_buf, u32 id3_buf_size, GF_FilterPid
 {
 	GF_BitStream *bs = gf_bs_new(id3_buf, id3_buf_size, GF_BITSTREAM_READ);
 	char *sep_desc;
-	char *buf=NULL;
+	char *_buf=NULL;
 	u32 buf_alloc=0;
 	gf_bs_skip_bytes(bs, 3);
 	/*u8 major = */gf_bs_read_u8(bs);
@@ -232,6 +240,7 @@ void id3dmx_flush(GF_Filter *filter, u8 *id3_buf, u32 id3_buf_size, GF_FilterPid
 	}
 
 	while (size && (gf_bs_available(bs)>=10) ) {
+		char *buf;
 		char szTag[1024];
 		char *sep;
 		s32 tag_idx;
@@ -246,24 +255,23 @@ void id3dmx_flush(GF_Filter *filter, u8 *id3_buf, u32 id3_buf_size, GF_FilterPid
 			break;
 
 		if (size<fsize) {
-			GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[MP3Dmx] Broken ID3 frame tag %s, size %d but remaining bytes %d\n", gf_4cc_to_str(ftag), fsize, size));
+			GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[MP3Dmx] Broken ID3 frame tag %s, size %d but remaining bytes %d\n", gf_4cc_to_str(ftag), fsize, size));
 			break;
 		}
 
 		if (buf_alloc<=fsize) {
-			buf = gf_realloc(buf, fsize+2);
-			buf_alloc = fsize+2;
+			_buf = gf_realloc(_buf, fsize+3);
+			buf_alloc = fsize+3;
 		}
-
-		gf_bs_read_data(bs, buf, fsize);
-		buf[fsize]=0;
-		buf[fsize+1]=0;
+		//read into _buf+1 so that buf+1 is always %2 mem aligned as it can be loaded as unsigned short
+		gf_bs_read_data(bs, _buf+1, fsize);
+		_buf[fsize+1]=0;
+		_buf[fsize+2]=0;
+		buf = _buf+1;
 
 		tag_idx = gf_itags_find_by_id3tag(ftag);
-		if (tag_idx>=0) {
-			const char *tag_name = gf_itags_get_name((u32) tag_idx);
-			id3dmx_set_string(audio_pid, (char *) tag_name, buf+1, GF_FALSE);
-		} else if (ftag==GF_ID3V2_FRAME_TXXX) {
+
+		if (ftag==GF_ID3V2_FRAME_TXXX) {
 			sep = memchr(buf, 0, fsize);
 			if (sep) {
 				if (!stricmp(buf+1, "comment")) {
@@ -289,7 +297,7 @@ void id3dmx_flush(GF_Filter *filter, u8 *id3_buf, u32 id3_buf_size, GF_FilterPid
 				if (video_pid_p) {
 					e = gf_filter_pid_raw_new(filter, NULL, NULL, buf+1, NULL, sep_desc+1, pic_size, GF_FALSE, video_pid_p);
 					if (e) {
-						GF_LOG(GF_LOG_ERROR, GF_LOG_PARSER, ("[MP3Dmx] error setting up video pid for cover art: %s\n", gf_error_to_string(e) ));
+						GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[MP3Dmx] error setting up video pid for cover art: %s\n", gf_error_to_string(e) ));
 					}
 					if (*video_pid_p) {
 						u8 *out_buffer;
@@ -297,9 +305,11 @@ void id3dmx_flush(GF_Filter *filter, u8 *id3_buf, u32 id3_buf_size, GF_FilterPid
 						gf_filter_pid_set_name(*video_pid_p, "CoverArt");
 						gf_filter_pid_set_property(*video_pid_p, GF_PROP_PID_COVER_ART, &PROP_BOOL(GF_TRUE));
 						dst_pck = gf_filter_pck_new_alloc(*video_pid_p, pic_size, &out_buffer);
-						gf_filter_pck_set_framing(dst_pck, GF_TRUE, GF_TRUE);
-						memcpy(out_buffer, sep_desc+1, pic_size);
-						gf_filter_pck_send(dst_pck);
+						if (dst_pck) {
+							gf_filter_pck_set_framing(dst_pck, GF_TRUE, GF_TRUE);
+							memcpy(out_buffer, sep_desc+1, pic_size);
+							gf_filter_pck_send(dst_pck);
+						}
 
 						gf_filter_pid_set_eos(*video_pid_p);
 					}
@@ -307,6 +317,9 @@ void id3dmx_flush(GF_Filter *filter, u8 *id3_buf, u32 id3_buf_size, GF_FilterPid
 					gf_filter_pid_set_property(audio_pid, GF_PROP_PID_COVER_ART, &PROP_DATA(sep_desc+1, pic_size) );
 				}
 			}
+		} else if (tag_idx>=0) {
+			const char *tag_name = gf_itags_get_name((u32) tag_idx);
+			id3dmx_set_string(audio_pid, (char *) tag_name, buf+1, GF_FALSE);
 		} else {
 			sprintf(szTag, "tag_%s", gf_4cc_to_str(ftag));
 			if ((ftag>>24) == 'T') {
@@ -314,12 +327,11 @@ void id3dmx_flush(GF_Filter *filter, u8 *id3_buf, u32 id3_buf_size, GF_FilterPid
 			} else {
 				gf_filter_pid_set_property_dyn(audio_pid, szTag, &PROP_DATA(buf, fsize) );
 			}
-			break;
 		}
 		size -= fsize;
 	}
 	gf_bs_del(bs);
-	if (buf) gf_free(buf);
+	if (_buf) gf_free(_buf);
 }
 static void mp3_dmx_flush_id3(GF_Filter *filter, GF_MP3DmxCtx *ctx)
 {
@@ -338,9 +350,11 @@ static void mp3_dmx_check_pid(GF_Filter *filter, GF_MP3DmxCtx *ctx)
 
 	if ((ctx->sr == gf_mp3_sampling_rate(ctx->hdr)) && (ctx->nb_ch == gf_mp3_num_channels(ctx->hdr) )
 		&& (ctx->codecid == gf_mp3_object_type_indication(ctx->hdr) )
+		&& !ctx->copy_props
 	)
 		return;
 
+	ctx->copy_props = GF_FALSE;
 	//copy properties at init or reconfig
 	gf_filter_pid_copy_properties(ctx->opid, ctx->ipid);
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_STREAM_TYPE, & PROP_UINT( GF_STREAM_AUDIO));
@@ -350,18 +364,29 @@ static void mp3_dmx_check_pid(GF_Filter *filter, GF_MP3DmxCtx *ctx)
 	}
 	if (ctx->duration.num)
 		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_DURATION, & PROP_FRAC64(ctx->duration));
+	if (!ctx->timescale)
+		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_CAN_DATAREF, & PROP_BOOL(GF_TRUE ) );
 
 	if (!ctx->timescale) gf_filter_pid_set_name(ctx->opid, "audio");
 
 	ctx->nb_ch = gf_mp3_num_channels(ctx->hdr);
 	ctx->codecid = gf_mp3_object_type_indication(ctx->hdr);
+	switch (gf_mp3_layer(ctx->hdr)) {
+	case 3:
+		if (ctx->forcemp3) {
+			ctx->codecid = GF_CODECID_MPEG_AUDIO;
+		}
+		break;
+	case 1:
+		ctx->codecid = GF_CODECID_MPEG_AUDIO_L1;
+		break;
+	}
 	sr = gf_mp3_sampling_rate(ctx->hdr);
 
 	if (!ctx->timescale) {
 		//we change sample rate, change cts
 		if (ctx->cts && ctx->sr && (ctx->sr != sr)) {
-			ctx->cts *= sr;
-			ctx->cts /= ctx->sr;
+			ctx->cts = gf_timestamp_rescale(ctx->cts, ctx->sr, sr);
 		}
 	}
 	ctx->sr = sr;
@@ -369,9 +394,13 @@ static void mp3_dmx_check_pid(GF_Filter *filter, GF_MP3DmxCtx *ctx)
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_TIMESCALE, & PROP_UINT(ctx->timescale ? ctx->timescale : sr));
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_SAMPLE_RATE, & PROP_UINT(sr));
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_NUM_CHANNELS, & PROP_UINT(ctx->nb_ch) );
-	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_CODECID, & PROP_UINT(gf_mp3_object_type_indication(ctx->hdr) ) );
+	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_CODECID, & PROP_UINT(ctx->codecid ) );
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_SAMPLES_PER_FRAME, & PROP_UINT(gf_mp3_window_size(ctx->hdr) ) );
 
+	if (!gf_sys_is_test_mode() ) {
+		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_BITRATE, & PROP_UINT(gf_mp3_bit_rate(ctx->hdr) ) );
+	}
+
 	if (ctx->id3_buffer_size)
 		mp3_dmx_flush_id3(filter, ctx);
 
@@ -462,6 +491,7 @@ GF_Err mp3_dmx_process(GF_Filter *filter)
 {
 	GF_MP3DmxCtx *ctx = gf_filter_get_udta(filter);
 	GF_FilterPacket *pck, *dst_pck;
+	Bool is_eos=GF_FALSE;
 	u8 *data, *output;
 	u8 *start;
 	u32 pck_size, remain, prev_pck_size;
@@ -484,6 +514,7 @@ GF_Err mp3_dmx_process(GF_Filter *filter)
 				ctx->src_pck = NULL;
 				return GF_EOS;
 			}
+			is_eos = GF_TRUE;
 		} else {
 			return GF_OK;
 		}
@@ -592,7 +623,7 @@ GF_Err mp3_dmx_process(GF_Filter *filter)
 				if ((sync[size]=='T') && (sync[size+1]=='A') && (sync[size+2]=='G')) {
 					skip_id3v1=GF_TRUE;
 				} else {
-					GF_LOG(GF_LOG_WARNING, GF_LOG_PARSER, ("[MP3Dmx] invalid frame, resyncing\n"));
+					GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[MP3Dmx] invalid frame, resyncing\n"));
 					goto drop_byte;
 				}
 			}
@@ -627,6 +658,7 @@ GF_Err mp3_dmx_process(GF_Filter *filter)
 
 		if (!ctx->in_seek) {
 			dst_pck = gf_filter_pck_new_alloc(ctx->opid, size, &output);
+			if (!dst_pck) break;
 			memcpy(output, sync, size);
 
 			gf_filter_pck_set_cts(dst_pck, ctx->cts);
@@ -648,7 +680,9 @@ GF_Err mp3_dmx_process(GF_Filter *filter)
 
 		//truncated last frame
 		if (bytes_to_drop>remain) {
-			GF_LOG(GF_LOG_WARNING, GF_LOG_PARSER, ("[MP3Dmx] truncated frame!\n"));
+			if (!is_eos) {
+				GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[MP3Dmx] truncated frame!\n"));
+			}
 			bytes_to_drop=remain;
 		}
 
@@ -686,6 +720,14 @@ drop_byte:
 	return GF_OK;
 }
 
+static GF_Err mp3_dmx_initialize(GF_Filter *filter)
+{
+	GF_MP3DmxCtx *ctx = gf_filter_get_udta(filter);
+	//in test mode we keep strict signaling, eg MPEG-2 layer 3 is still mpeg2 audio
+	if (gf_sys_is_test_mode()) ctx->forcemp3 = GF_FALSE;
+	return GF_OK;
+}
+
 static void mp3_dmx_finalize(GF_Filter *filter)
 {
 	GF_MP3DmxCtx *ctx = gf_filter_get_udta(filter);
@@ -702,6 +744,10 @@ static const char *mp3_dmx_probe_data(const u8 *data, u32 size, GF_FilterProbeSc
 	u32 nb_frames=0;
 	u32 pos=0;
 	u32 prev_pos=0;
+	s32 prev_sr_idx=-1;
+	s32 prev_ch=-1;
+	s32 prev_layer=-1;
+	s32 init_pos = -1;
 	Bool has_id3 = GF_FALSE;
 
 	/* Check for ID3 */
@@ -724,10 +770,13 @@ static const char *mp3_dmx_probe_data(const u8 *data, u32 size, GF_FilterProbeSc
 		u32 hdr = gf_mp3_get_next_header_mem(data, size, &pos);
 		if (!hdr) break;
 
+		if (init_pos<0) init_pos = pos;
+
 		if (gf_mp3_version(hdr) > 3)
 			break;
-		u8 sampleRateIndex = (hdr >> 10) & 0x3;
-		if (sampleRateIndex>2)
+		//check sample rate
+		u8 val = (hdr >> 10) & 0x3;
+		if (val>2)
 			break;
 		u32 fsize = gf_mp3_frame_size(hdr);
 		if (prev_pos && pos) {
@@ -735,6 +784,32 @@ static const char *mp3_dmx_probe_data(const u8 *data, u32 size, GF_FilterProbeSc
 			break;
 		}
 
+		if (prev_sr_idx>=0) {
+			if ((u8) prev_sr_idx != val) {
+				nb_frames=0;
+				break;
+			}
+		}
+		prev_sr_idx = val;
+
+		val = gf_mp3_num_channels(hdr);
+		if (prev_ch>=0) {
+			if ((u8) prev_ch != val) {
+				nb_frames=0;
+				break;
+			}
+		}
+		prev_ch = val;
+
+		val = gf_mp3_layer(hdr);
+		if (prev_layer>=0) {
+			if ((u8) prev_layer != val) {
+				nb_frames=0;
+				break;
+			}
+		}
+		prev_layer = val;
+
 		if (fsize + pos > size) {
 			nb_frames++;
 			break;
@@ -749,7 +824,7 @@ static const char *mp3_dmx_probe_data(const u8 *data, u32 size, GF_FilterProbeSc
 	}
 
 	if (nb_frames>=2) {
-		*score = GF_FPROBE_SUPPORTED;
+		*score = (init_pos==0) ? GF_FPROBE_SUPPORTED : GF_FPROBE_MAYBE_NOT_SUPPORTED;
 		return "audio/mp3";
 	}
 	if (nb_frames && has_id3) {
@@ -767,12 +842,14 @@ static const GF_FilterCapability MP3DmxCaps[] =
 	CAP_UINT(GF_CAPS_OUTPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_AUDIO),
 	CAP_UINT(GF_CAPS_OUTPUT, GF_PROP_PID_CODECID, GF_CODECID_MPEG_AUDIO),
 	CAP_UINT(GF_CAPS_OUTPUT, GF_PROP_PID_CODECID, GF_CODECID_MPEG2_PART3),
+	CAP_UINT(GF_CAPS_OUTPUT, GF_PROP_PID_CODECID, GF_CODECID_MPEG_AUDIO_L1),
 	CAP_BOOL(GF_CAPS_OUTPUT_EXCLUDED, GF_PROP_PID_UNFRAMED, GF_TRUE),
 	{0},
 	CAP_UINT(GF_CAPS_INPUT_OUTPUT,GF_PROP_PID_STREAM_TYPE, GF_STREAM_AUDIO),
 	CAP_BOOL(GF_CAPS_INPUT,GF_PROP_PID_UNFRAMED, GF_TRUE),
 	CAP_UINT(GF_CAPS_INPUT_OUTPUT,GF_PROP_PID_CODECID, GF_CODECID_MPEG_AUDIO),
 	CAP_UINT(GF_CAPS_INPUT_OUTPUT,GF_PROP_PID_CODECID, GF_CODECID_MPEG2_PART3),
+	CAP_UINT(GF_CAPS_INPUT_OUTPUT,GF_PROP_PID_CODECID, GF_CODECID_MPEG_AUDIO_L1),
 	CAP_BOOL(GF_CAPS_OUTPUT_EXCLUDED, GF_PROP_PID_UNFRAMED, GF_TRUE),
 	{0},
 	//also declare generic file output for embedded files (cover art & co), but explicit to skip this cap in chain resolution
@@ -786,7 +863,8 @@ static const GF_FilterCapability MP3DmxCaps[] =
 static const GF_FilterArgs MP3DmxArgs[] =
 {
 	{ OFFS(index), "indexing window length", GF_PROP_DOUBLE, "1.0", NULL, 0},
-	{ OFFS(expart), "expose pictures as a dedicated video pid", GF_PROP_BOOL, "false", NULL, 0},
+	{ OFFS(expart), "expose pictures as a dedicated video PID", GF_PROP_BOOL, "false", NULL, 0},
+	{ OFFS(forcemp3), "force mp3 signaling for MPEG-2 Audio layer 3", GF_PROP_BOOL, "true", NULL, GF_ARG_HINT_EXPERT},
 	{0}
 };
 
@@ -797,6 +875,7 @@ GF_FilterRegister MP3DmxRegister = {
 	GF_FS_SET_HELP("This filter parses MPEG-1/2 audio files/data and outputs corresponding audio PID and frames.")
 	.private_size = sizeof(GF_MP3DmxCtx),
 	.args = MP3DmxArgs,
+	.initialize = mp3_dmx_initialize,
 	.finalize = mp3_dmx_finalize,
 	SETCAPS(MP3DmxCaps),
 	.configure_pid = mp3_dmx_configure_pid,
