@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2000-2019
+ *			Copyright (c) Telecom ParisTech 2000-2021
  *					All rights reserved
  *
  *  This file is part of GPAC / ISO Media File Format sub-project
@@ -241,6 +241,7 @@ GF_Err ftab_box_size(GF_Box *s)
 GF_Box *text_box_new()
 {
 	ISOM_DECL_BOX_ALLOC(GF_TextSampleEntryBox, GF_ISOM_BOX_TYPE_TEXT);
+	gf_isom_sample_entry_init((GF_SampleEntryBox *)tmp);
 	return (GF_Box *) tmp;
 }
 
@@ -257,6 +258,7 @@ void text_box_del(GF_Box *s)
 GF_Box *tx3g_box_new()
 {
 	ISOM_DECL_BOX_ALLOC(GF_Tx3gSampleEntryBox, GF_ISOM_BOX_TYPE_TX3G);
+	gf_isom_sample_entry_init((GF_SampleEntryBox *)tmp);
 	return (GF_Box *) tmp;
 }
 
@@ -304,13 +306,12 @@ void gpp_read_style(GF_BitStream *bs, GF_StyleRecord *rec)
 	rec->text_color = gpp_read_rgba(bs);
 }
 
-GF_Err tx3g_on_child_box(GF_Box *s, GF_Box *a)
+GF_Err tx3g_on_child_box(GF_Box *s, GF_Box *a, Bool is_rem)
 {
 	GF_Tx3gSampleEntryBox *ptr = (GF_Tx3gSampleEntryBox*)s;
 	switch (a->type) {
 	case GF_ISOM_BOX_TYPE_FTAB:
-		if (ptr->font_table) ERROR_ON_DUPLICATED_BOX(a, ptr)
-		ptr->font_table = (GF_FontTableBox *)a;
+		BOX_FIELD_ASSIGN(font_table, GF_FontTableBox)
 		break;
 	default:
 		return GF_OK;
@@ -336,7 +337,7 @@ GF_Err tx3g_box_read(GF_Box *s, GF_BitStream *bs)
 	gpp_read_style(bs, &ptr->default_style);
 
 
-	return gf_isom_box_array_read(s, bs, tx3g_on_child_box);
+	return gf_isom_box_array_read(s, bs);
 }
 
 /*this is a quicktime specific box - see apple documentation*/
@@ -346,11 +347,22 @@ GF_Err text_box_read(GF_Box *s, GF_BitStream *bs)
 	u16 pSize;
 	GF_TextSampleEntryBox *ptr = (GF_TextSampleEntryBox*)s;
 
-	ISOM_DECREASE_SIZE(ptr, 51);
-
+	ISOM_DECREASE_SIZE(ptr, 8);
 	e = gf_isom_base_sample_entry_read((GF_SampleEntryBox *)ptr, bs);
 	if (e) return e;
 
+	ptr->textJustification = 1;
+
+	//some weird text entries are not QT text nor 3gpp, cf issue #1030
+	if (!ptr->size) {
+		return GF_OK;
+	}
+	if (ptr->size < 43) {
+		GF_LOG(GF_LOG_WARNING, GF_LOG_CONTAINER, ("[iso file] Broken text box (%d bytes but min 43 required), skiping parsing.\n", ptr->size));
+		return GF_OK;
+	}
+	ISOM_DECREASE_SIZE(ptr, 43);
+
 	ptr->displayFlags = gf_bs_read_u32(bs);			/*Display flags*/
 	ptr->textJustification = gf_bs_read_u32(bs);	/*Text justification*/
 	gf_bs_read_data(bs, ptr->background_color, 6);	/*Background color*/
@@ -411,7 +423,14 @@ GF_Err text_box_read(GF_Box *s, GF_BitStream *bs)
 		ptr->textName[pSize] = '\0';				/*Font name*/
 	}
 	ISOM_DECREASE_SIZE(ptr, pSize);
-	return gf_isom_box_array_read(s, bs, NULL);
+
+	u32 next_size = gf_bs_peek_bits(bs, 32, 0);
+	if (next_size > ptr->size) {
+		GF_LOG(GF_LOG_WARNING, GF_LOG_CONTAINER, ("[iso file] Broken text box detected, skiping parsing.\n"));
+		ptr->textJustification = 1;
+		return GF_OK;
+	}
+	return gf_isom_box_array_read(s, bs);
 }
 
 void gpp_write_rgba(GF_BitStream *bs, u32 col)
@@ -471,6 +490,7 @@ GF_Err text_box_write(GF_Box *s, GF_BitStream *bs)
 	if (e) return e;
 	gf_bs_write_data(bs, ptr->reserved, 6);
 	gf_bs_write_u16(bs, ptr->dataReferenceIndex);
+
 	gf_bs_write_u32(bs, ptr->displayFlags);			/*Display flags*/
 	gf_bs_write_u32(bs, ptr->textJustification);	/*Text justification*/
 	gf_bs_write_data(bs, ptr->background_color, 6);	/*Background color*/
@@ -502,8 +522,9 @@ GF_Err text_box_size(GF_Box *s)
 {
 	GF_TextSampleEntryBox *ptr = (GF_TextSampleEntryBox*)s;
 
+	s->size += 8;
 	/*base + this + string length*/
-	s->size += 51 + 1;
+	s->size += 43 + 1;
 	if (ptr->textName)
 		s->size += strlen(ptr->textName);
 	return GF_OK;
@@ -531,7 +552,7 @@ GF_Err styl_box_read(GF_Box *s, GF_BitStream *bs)
 	ISOM_DECREASE_SIZE(ptr, 2);
 	ptr->entry_count = gf_bs_read_u16(bs);
 
-	if (ptr->size<ptr->entry_count * GPP_STYLE_SIZE)
+	if (ptr->size / GPP_STYLE_SIZE < ptr->entry_count)
 		return GF_ISOM_INVALID_FILE;
 
 	if (ptr->entry_count) {
@@ -667,7 +688,7 @@ GF_Err krok_box_read(GF_Box *s, GF_BitStream *bs)
 	ISOM_DECREASE_SIZE(ptr, 6)
 	ptr->highlight_starttime = gf_bs_read_u32(bs);
 	ptr->nb_entries = gf_bs_read_u16(bs);
-	if (ptr->size < ptr->nb_entries * 8)
+	if (ptr->size / 8 < ptr->nb_entries)
 		return GF_ISOM_INVALID_FILE;
 
 	if (ptr->nb_entries) {
@@ -1148,6 +1169,7 @@ GF_Err diST_box_size(GF_Box *s)
 GF_Box *dims_box_new()
 {
 	ISOM_DECL_BOX_ALLOC(GF_DIMSSampleEntryBox, GF_ISOM_BOX_TYPE_DIMS);
+	gf_isom_sample_entry_init((GF_SampleEntryBox *)tmp);
 	return (GF_Box*)tmp;
 }
 void dims_box_del(GF_Box *s)
@@ -1156,17 +1178,15 @@ void dims_box_del(GF_Box *s)
 	gf_free(s);
 }
 
-static GF_Err dims_on_child_box(GF_Box *s, GF_Box *a)
+GF_Err dims_on_child_box(GF_Box *s, GF_Box *a, Bool is_rem)
 {
 	GF_DIMSSampleEntryBox *ptr = (GF_DIMSSampleEntryBox  *)s;
 	switch (a->type) {
 	case GF_ISOM_BOX_TYPE_DIMC:
-		if (ptr->config) ERROR_ON_DUPLICATED_BOX(a, ptr)
-			ptr->config = (GF_DIMSSceneConfigBox*)a;
+		BOX_FIELD_ASSIGN(config, GF_DIMSSceneConfigBox)
 		break;
 	case GF_ISOM_BOX_TYPE_DIST:
-		if (ptr->scripts) ERROR_ON_DUPLICATED_BOX(a, ptr)
-			ptr->scripts = (GF_DIMSScriptTypesBox*)a;
+		BOX_FIELD_ASSIGN(scripts, GF_DIMSScriptTypesBox)
 		break;
 	}
 	return GF_OK;
@@ -1180,7 +1200,7 @@ GF_Err dims_box_read(GF_Box *s, GF_BitStream *bs)
 	if (e) return e;
 
 	ISOM_DECREASE_SIZE(p, 8);
-	return gf_isom_box_array_read(s, bs, dims_on_child_box);
+	return gf_isom_box_array_read(s, bs);
 }
 
 #ifndef GPAC_DISABLE_ISOM_WRITE
