@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Author: Romain Bouqueau, Jean Le Feuvre
- *			Copyright (c) Romain Bouqueau 2012- Telecom Paris 2019-
+ *			Copyright (c) Romain Bouqueau 2012- Telecom Paris 2019-2022
  *				All rights reserved
  *
  *          Note: this development was kindly sponsorized by Vizion'R (http://vizionr.com)
@@ -79,7 +79,8 @@ GF_Err abst_box_read(GF_Box *s, GF_BitStream *bs)
 	int i;
 	u32 tmp_strsize;
 	char *tmp_str;
-	GF_Err e;
+	Bool zfound=GF_FALSE;
+	GF_Err e = GF_OK;
 
 	ISOM_DECREASE_SIZE(ptr, 25)
 	ptr->bootstrapinfo_version = gf_bs_read_u32(bs);
@@ -94,116 +95,166 @@ GF_Err abst_box_read(GF_Box *s, GF_BitStream *bs)
 	i=0;
 	if (ptr->size<8) return GF_ISOM_INVALID_FILE;
 	tmp_strsize =(u32)ptr->size;
-	tmp_str = gf_malloc(sizeof(char)*tmp_strsize);
+	tmp_str = gf_malloc(sizeof(char)*(tmp_strsize+1));
 	if (!tmp_str) return GF_OUT_OF_MEM;
-	memset(tmp_str, 0, sizeof(char)*tmp_strsize);
+	memset(tmp_str, 0, sizeof(char)*(tmp_strsize+1) );
 
 	while (tmp_strsize) {
-		ISOM_DECREASE_SIZE(ptr, 1)
+		ISOM_DECREASE_SIZE_GOTO_EXIT(ptr, 1)
 		tmp_str[i] = gf_bs_read_u8(bs);
 		tmp_strsize--;
-		if (!tmp_str[i])
+		if (!tmp_str[i]) {
+			zfound = GF_TRUE;
 			break;
+		}
 		i++;
 	}
+	if (!zfound) {
+		e = GF_ISOM_INVALID_FILE;
+		goto exit;
+	}
 	if (i) {
 		ptr->movie_identifier = gf_strdup(tmp_str);
 	}
 
-	ISOM_DECREASE_SIZE(ptr, 1)
+	ISOM_DECREASE_SIZE_GOTO_EXIT(ptr, 1)
 	ptr->server_entry_count = gf_bs_read_u8(bs);
 	for (i=0; i<ptr->server_entry_count; i++) {
 		int j=0;
+		zfound = GF_FALSE;
 		tmp_strsize=(u32)ptr->size;
 		while (tmp_strsize) {
-			ISOM_DECREASE_SIZE(ptr, 1)
+			ISOM_DECREASE_SIZE_GOTO_EXIT(ptr, 1)
 			tmp_str[j] = gf_bs_read_u8(bs);
 			tmp_strsize--;
-			if (!tmp_str[j])
+			if (!tmp_str[j]) {
+				zfound = GF_TRUE;
 				break;
+			}
 			j++;
 		}
+		if (!zfound) {
+			e = GF_ISOM_INVALID_FILE;
+			goto exit;
+		}
 		if (j) {
 			gf_list_insert(ptr->server_entry_table, gf_strdup(tmp_str), i);
 		}
 	}
+	if (ptr->server_entry_count != gf_list_count(ptr->server_entry_table)) {
+		e = GF_ISOM_INVALID_FILE;
+		goto exit;
+	}
 
-	ISOM_DECREASE_SIZE(ptr, 1)
+	ISOM_DECREASE_SIZE_GOTO_EXIT(ptr, 1)
 	ptr->quality_entry_count = gf_bs_read_u8(bs);
 	for (i=0; i<ptr->quality_entry_count; i++) {
 		int j=0;
+		zfound = GF_FALSE;
 		tmp_strsize=(u32)ptr->size;
 		while (tmp_strsize) {
-			ISOM_DECREASE_SIZE(ptr, 1)
+			ISOM_DECREASE_SIZE_GOTO_EXIT(ptr, 1)
 			tmp_str[j] = gf_bs_read_u8(bs);
 			tmp_strsize--;
-			if (!tmp_str[j])
+			if (!tmp_str[j]) {
+				zfound = GF_TRUE;
 				break;
+			}
 			j++;
 		}
 
+		if (!zfound) {
+			e = GF_ISOM_INVALID_FILE;
+			goto exit;
+		}
 		if (j) {
 			gf_list_insert(ptr->quality_entry_table, gf_strdup(tmp_str), i);
 		}
 	}
+	if (ptr->quality_entry_count != gf_list_count(ptr->quality_entry_table)) {
+		e = GF_ISOM_INVALID_FILE;
+		goto exit;
+	}
 
 	i=0;
 	tmp_strsize=(u32)ptr->size;
+	zfound = GF_FALSE;
 	while (tmp_strsize) {
-		ISOM_DECREASE_SIZE(ptr, 1)
+		ISOM_DECREASE_SIZE_GOTO_EXIT(ptr, 1)
 		tmp_str[i] = gf_bs_read_u8(bs);
 		tmp_strsize--;
-		if (!tmp_str[i])
+		if (!tmp_str[i]) {
+			zfound = GF_TRUE;
 			break;
+		}
 		i++;
 	}
+	if (!zfound) {
+		e = GF_ISOM_INVALID_FILE;
+		goto exit;
+	}
+
 	if (i) {
 		ptr->drm_data = gf_strdup(tmp_str);
 	}
 
 	i=0;
 	tmp_strsize=(u32)ptr->size;
+	zfound = GF_FALSE;
 	while (tmp_strsize) {
-		ISOM_DECREASE_SIZE(ptr, 1)
+		ISOM_DECREASE_SIZE_GOTO_EXIT(ptr, 1)
 		tmp_str[i] = gf_bs_read_u8(bs);
 		tmp_strsize--;
-		if (!tmp_str[i])
+		if (!tmp_str[i]) {
+			zfound = GF_TRUE;
 			break;
+		}
 		i++;
 	}
+	if (!zfound) {
+		e = GF_ISOM_INVALID_FILE;
+		goto exit;
+	}
+
 	if (i) {
 		ptr->meta_data = gf_strdup(tmp_str);
 	}
 
-	ISOM_DECREASE_SIZE(ptr, 1)
+	ISOM_DECREASE_SIZE_GOTO_EXIT(ptr, 1)
 	ptr->segment_run_table_count = gf_bs_read_u8(bs);
 	for (i=0; i<ptr->segment_run_table_count; i++) {
 		GF_AdobeSegmentRunTableBox *asrt = NULL;
 		e = gf_isom_box_parse((GF_Box **)&asrt, bs);
 		if (e) {
 			if (asrt) gf_isom_box_del((GF_Box*)asrt);
-			gf_free(tmp_str);
-			return e;
+			goto exit;
 		}
 		gf_list_add(ptr->segment_run_table_entries, asrt);
 	}
+	if (ptr->segment_run_table_count != gf_list_count(ptr->segment_run_table_entries)) {
+		e = GF_ISOM_INVALID_FILE;
+		goto exit;
+	}
 
-	ISOM_DECREASE_SIZE(ptr, 1)
+	ISOM_DECREASE_SIZE_GOTO_EXIT(ptr, 1)
 	ptr->fragment_run_table_count = gf_bs_read_u8(bs);
 	for (i=0; i<ptr->fragment_run_table_count; i++) {
 		GF_AdobeFragmentRunTableBox *afrt = NULL;
 		e = gf_isom_box_parse((GF_Box **)&afrt, bs);
 		if (e) {
 			if (afrt) gf_isom_box_del((GF_Box*)afrt);
-			gf_free(tmp_str);
-			return e;
+			goto exit;
 		}
 		gf_list_add(ptr->fragment_run_table_entries, afrt);
 	}
+	if (ptr->fragment_run_table_count != gf_list_count(ptr->fragment_run_table_entries)) {
+		e = GF_ISOM_INVALID_FILE;
+		goto exit;
+	}
 
+exit:
 	gf_free(tmp_str);
-
-	return GF_OK;
+	return e;
 }
 
 GF_Box *abst_box_new()
@@ -351,12 +402,13 @@ GF_Err afra_box_read(GF_Box *s, GF_BitStream *bs)
 	ptr->time_scale = gf_bs_read_u32(bs);
 
 	ptr->entry_count = gf_bs_read_u32(bs);
-	if (ptr->size < ptr->entry_count * (ptr->long_offsets ? 16 : 12))
+	if (ptr->size / ( (ptr->long_offsets ? 16 : 12) ) < ptr->entry_count)
 		return GF_ISOM_INVALID_FILE;
 
 	for (i=0; i<ptr->entry_count; i++) {
 		GF_AfraEntry *ae = gf_malloc(sizeof(GF_AfraEntry));
 		if (!ae) return GF_OUT_OF_MEM;
+		gf_list_insert(ptr->local_access_entries, ae, i);
 
 		ISOM_DECREASE_SIZE(ptr, 8)
 		ae->time = gf_bs_read_u64(bs);
@@ -367,8 +419,6 @@ GF_Err afra_box_read(GF_Box *s, GF_BitStream *bs)
 			ISOM_DECREASE_SIZE(ptr, 4)
 			ae->offset = gf_bs_read_u32(bs);
 		}
-
-		gf_list_insert(ptr->local_access_entries, ae, i);
 	}
 
 	if (ptr->global_entries) {
@@ -377,6 +427,8 @@ GF_Err afra_box_read(GF_Box *s, GF_BitStream *bs)
 		for (i=0; i<ptr->global_entry_count; i++) {
 			GF_GlobalAfraEntry *ae = gf_malloc(sizeof(GF_GlobalAfraEntry));
 			if (!ae) return GF_OUT_OF_MEM;
+			gf_list_insert(ptr->global_access_entries, ae, i);
+
 			ISOM_DECREASE_SIZE(ptr, 8)
 			ae->time = gf_bs_read_u64(bs);
 			if (ptr->long_ids) {
@@ -397,8 +449,6 @@ GF_Err afra_box_read(GF_Box *s, GF_BitStream *bs)
 				ae->afra_offset = gf_bs_read_u32(bs);
 				ae->offset_from_afra = gf_bs_read_u32(bs);
 			}
-
-			gf_list_insert(ptr->global_access_entries, ae, i);
 		}
 	}
 
@@ -512,22 +562,24 @@ GF_Err asrt_box_read(GF_Box *s, GF_BitStream *bs)
 	for (i=0; i<ptr->quality_entry_count; i++) {
 		int j=0;
 		u32 tmp_strsize=(u32)ptr->size;
-		char *tmp_str = (char*) gf_malloc(tmp_strsize);
+		if (!tmp_strsize) return GF_ISOM_INVALID_FILE;
+		char *tmp_str = (char*) gf_malloc(tmp_strsize+1);
 		if (!tmp_str) return GF_OUT_OF_MEM;
+		tmp_str[tmp_strsize]=0;
 		while (tmp_strsize) {
 			tmp_str[j] = gf_bs_read_u8(bs);
+			ISOM_DECREASE_SIZE(ptr, 1)
 			tmp_strsize--;
 			if (!tmp_str[j])
 				break;
 			j++;
 		}
-		ISOM_DECREASE_SIZE(ptr, j)
 		gf_list_insert(ptr->quality_segment_url_modifiers, tmp_str, i);
 	}
 
 	ISOM_DECREASE_SIZE(ptr, 4)
 	ptr->segment_run_entry_count = gf_bs_read_u32(bs);
-	if (ptr->size < ptr->segment_run_entry_count*8)
+	if (ptr->size / 8 < ptr->segment_run_entry_count)
 		return GF_ISOM_INVALID_FILE;
 
 	for (i=0; i<ptr->segment_run_entry_count; i++) {
@@ -627,28 +679,32 @@ GF_Err afrt_box_read(GF_Box *s, GF_BitStream *bs)
 	if (ptr->size < ptr->quality_entry_count)
 		return GF_ISOM_INVALID_FILE;
 
+
 	for (i=0; i<ptr->quality_entry_count; i++) {
 		int j=0;
-		u32 tmp_strsize=(u32)ptr->size-8;
-		char *tmp_str = (char*) gf_malloc(tmp_strsize);
+		u32 tmp_strsize = (u32) ptr->size;
+		if (!tmp_strsize) return GF_ISOM_INVALID_FILE;
+		char *tmp_str = (char*) gf_malloc(tmp_strsize+1);
 		if (!tmp_str) return GF_OUT_OF_MEM;
+		tmp_str[tmp_strsize]=0;
 		while (tmp_strsize) {
 			tmp_str[j] = gf_bs_read_u8(bs);
+			ISOM_DECREASE_SIZE(ptr, 1)
 			tmp_strsize--;
 			if (!tmp_str[j])
 				break;
 			j++;
 		}
-		ISOM_DECREASE_SIZE(ptr, j)
 		gf_list_insert(ptr->quality_segment_url_modifiers, tmp_str, i);
 	}
 
+	ISOM_DECREASE_SIZE(ptr, 4)
 	ptr->fragment_run_entry_count = gf_bs_read_u32(bs);
-	if (ptr->size < ptr->fragment_run_entry_count*16)
+	if (ptr->size / 16 < ptr->fragment_run_entry_count)
 		return GF_ISOM_INVALID_FILE;
 	for (i=0; i<ptr->fragment_run_entry_count; i++) {
 		GF_AdobeFragmentRunEntry *fre = gf_malloc(sizeof(GF_AdobeFragmentRunEntry));
-		if (!fre) return GF_ISOM_INVALID_FILE;
+		if (!fre) return GF_OUT_OF_MEM;
 		ISOM_DECREASE_SIZE(ptr, 16)
 		fre->first_fragment = gf_bs_read_u32(bs);
 		fre->first_fragment_timestamp = gf_bs_read_u64(bs);
