@@ -2,7 +2,7 @@
  *          GPAC - Multimedia Framework C SDK
  *
  *          Authors: Cyril Concolato
- *          Copyright (c) Telecom ParisTech 2000-2012
+ *          Copyright (c) Telecom ParisTech 2000-2021
  *                  All rights reserved
  *
  *  This file is part of GPAC / ISO Media File Format sub-project
@@ -106,12 +106,6 @@ GF_Box *vtte_box_new() {
 	return (GF_Box *)tmp;
 }
 
-GF_Box *wvtt_box_new()
-{
-	ISOM_DECL_BOX_ALLOC(GF_WebVTTSampleEntryBox, GF_ISOM_BOX_TYPE_WVTT);
-	return (GF_Box *)tmp;
-}
-
 void boxstring_box_del(GF_Box *s)
 {
 	GF_StringBox *box = (GF_StringBox *)s;
@@ -129,8 +123,16 @@ void vtte_box_del(GF_Box *s)
 	gf_free(s);
 }
 
+GF_Box *wvtt_box_new()
+{
+	ISOM_DECL_BOX_ALLOC(GF_WebVTTSampleEntryBox, GF_ISOM_BOX_TYPE_WVTT);
+	gf_isom_sample_entry_init((GF_SampleEntryBox *)tmp);
+	return (GF_Box *)tmp;
+}
+
 void wvtt_box_del(GF_Box *s)
 {
+	gf_isom_sample_entry_predestroy((GF_SampleEntryBox *)s);
 	gf_free(s);
 }
 
@@ -143,21 +145,22 @@ GF_Err boxstring_box_read(GF_Box *s, GF_BitStream *bs)
 	return GF_OK;
 }
 
-static GF_Err vtcu_Add(GF_Box *s, GF_Box *box)
+
+GF_Err vtcu_on_child_box(GF_Box *s, GF_Box *a, Bool is_rem)
 {
-	GF_VTTCueBox *cuebox = (GF_VTTCueBox *)s;
-	switch(box->type) {
+	GF_VTTCueBox *ptr = (GF_VTTCueBox *)s;
+	switch (a->type) {
 	case GF_ISOM_BOX_TYPE_CTIM:
-		cuebox->time = (GF_StringBox *)box;
+		BOX_FIELD_ASSIGN(time, GF_StringBox);
 		break;
 	case GF_ISOM_BOX_TYPE_IDEN:
-		cuebox->id = (GF_StringBox *)box;
+		BOX_FIELD_ASSIGN(id, GF_StringBox);
 		break;
 	case GF_ISOM_BOX_TYPE_STTG:
-		cuebox->settings = (GF_StringBox *)box;
+		BOX_FIELD_ASSIGN(settings, GF_StringBox);
 		break;
 	case GF_ISOM_BOX_TYPE_PAYL:
-		cuebox->payload = (GF_StringBox *)box;
+		BOX_FIELD_ASSIGN(payload, GF_StringBox);
 		break;
 	}
 	return GF_OK;
@@ -165,20 +168,20 @@ static GF_Err vtcu_Add(GF_Box *s, GF_Box *box)
 
 GF_Err vtcu_box_read(GF_Box *s, GF_BitStream *bs)
 {
-	return gf_isom_box_array_read(s, bs, vtcu_Add);
+	return gf_isom_box_array_read(s, bs);
 }
 
 GF_Err vtte_box_read(GF_Box *s, GF_BitStream *bs)
 {
-	return gf_isom_box_array_read(s, bs, NULL);
+	return gf_isom_box_array_read(s, bs);
 }
 
-static GF_Err wvtt_Add(GF_Box *s, GF_Box *box)
+GF_Err wvtt_on_child_box(GF_Box *s, GF_Box *a, Bool is_rem)
 {
-	GF_WebVTTSampleEntryBox *wvtt = (GF_WebVTTSampleEntryBox *)s;
-	switch(box->type) {
+	GF_WebVTTSampleEntryBox *ptr = (GF_WebVTTSampleEntryBox *)s;
+	switch (a->type) {
 	case GF_ISOM_BOX_TYPE_VTTC_CONFIG:
-		wvtt->config = (GF_StringBox *)box;
+		BOX_FIELD_ASSIGN(config, GF_StringBox);
 		break;
 	}
 	return GF_OK;
@@ -192,7 +195,7 @@ GF_Err wvtt_box_read(GF_Box *s, GF_BitStream *bs)
 	if (e) return e;
 
 	wvtt->size -= 8;
-	return gf_isom_box_array_read(s, bs, wvtt_Add);
+	return gf_isom_box_array_read(s, bs);
 }
 
 #ifndef GPAC_DISABLE_ISOM_WRITE
@@ -263,7 +266,7 @@ GF_Err wvtt_box_size(GF_Box *s)
 	return GF_OK;
 }
 
-static GF_Err webvtt_write_cue(GF_BitStream *bs, GF_WebVTTCue *cue)
+static GF_Err wvtt_write_cue(GF_BitStream *bs, GF_WebVTTCue *cue)
 {
 	GF_Err e;
 	GF_VTTCueBox *cuebox;
@@ -303,7 +306,7 @@ GF_ISOSample *gf_isom_webvtt_to_sample(void *s)
 		GF_WebVTTCue *cue;
 		i=0;
 		while ((cue = (GF_WebVTTCue *)gf_list_enum(samp->cues, &i))) {
-			e = webvtt_write_cue(bs, cue);
+			e = wvtt_write_cue(bs, cue);
 			if (e) break;
 		}
 		if (e) {
@@ -362,10 +365,9 @@ GF_Err boxstring_box_dump(GF_Box *a, FILE * trace)
 		break;
 	}
 	gf_isom_box_dump_start(a, szName, trace);
-	gf_fprintf(trace, "><![CDATA[\n");
-	if (sbox->string)
-		gf_fprintf(trace, "%s", sbox->string);
-	gf_fprintf(trace, "\n]]>");
+	gf_fprintf(trace, ">");
+	if (sbox->string && strlen(sbox->string))
+		gf_fprintf(trace, "<![CDATA[\n%s\n]]>", sbox->string);
 	gf_isom_box_dump_done(szName, a, trace);
 	return GF_OK;
 }
@@ -423,11 +425,12 @@ struct _webvtt_parser {
 	void (*on_cue_read)(void *, GF_WebVTTCue *);
 };
 
-
+#if !defined(GPAC_DISABLE_MEDIA_IMPORT) && !defined(GPAC_DISABLE_ISOM)
 static Bool gf_webvtt_timestamp_is_zero(GF_WebVTTTimestamp *ts)
 {
 	return (ts->hour == 0 && ts->min == 0 && ts->sec == 0 && ts->ms == 0) ? GF_TRUE : GF_FALSE;
 }
+#endif
 
 #ifndef GPAC_DISABLE_MEDIA_IMPORT
 
@@ -491,8 +494,9 @@ static GF_Err gf_webvtt_cue_add_property(GF_WebVTTCue *cue, GF_WebVTTCueProperty
 	}
 	if (*prop) {
 		len = (u32) strlen(*prop);
-		*prop = (char*)gf_realloc(*prop, sizeof(char) * (len + text_len + 1) );
-		strcpy(*prop + len, text_data);
+		(*prop) = (char*)gf_realloc((*prop), sizeof(char) * (len + text_len + 1) );
+		strncpy((*prop) + len, text_data, text_len);
+		(*prop)[len+text_len] = 0;
 	} else {
 		*prop = gf_strdup(text_data);
 	}
@@ -1097,13 +1101,13 @@ exit:
 }
 
 GF_EXPORT
-GF_List *gf_webvtt_parse_iso_cues(GF_ISOSample *iso_sample, u64 start)
+GF_List *gf_webvtt_parse_iso_cues(GF_ISOSample *iso_sample, u64 start, u64 end)
 {
-	return gf_webvtt_parse_cues_from_data(iso_sample->data, iso_sample->dataLength, start);
+	return gf_webvtt_parse_cues_from_data(iso_sample->data, iso_sample->dataLength, start, end);
 }
 
 GF_EXPORT
-GF_List *gf_webvtt_parse_cues_from_data(const u8 *data, u32 dataLength, u64 start)
+GF_List *gf_webvtt_parse_cues_from_data(const u8 *data, u32 dataLength, u64 start, u64 end)
 {
 	GF_List *cues;
 	GF_WebVTTCue *cue;
@@ -1130,6 +1134,7 @@ GF_List *gf_webvtt_parse_cues_from_data(const u8 *data, u32 dataLength, u64 star
 			}
 			gf_list_add(cues, cue);
 			gf_webvtt_timestamp_set(&cue->start, start);
+			gf_webvtt_timestamp_set(&cue->end, end);
 			if (cuebox->id) {
 				gf_webvtt_cue_add_property(cue, WEBVTT_ID, cuebox->id->string, (u32) strlen(cuebox->id->string));
 			}
@@ -1251,13 +1256,15 @@ GF_Err gf_webvtt_merge_cues(GF_WebVTTParser *parser, u64 start, GF_List *cues)
 	return GF_OK;
 }
 
-GF_Err gf_webvtt_parse_iso_sample(GF_WebVTTParser *parser, u32 timescale, GF_ISOSample *iso_sample, Bool merge, Bool box_mode)
+static GF_Err gf_webvtt_parse_iso_sample(GF_WebVTTParser *parser, u32 timescale, GF_ISOSample *iso_sample, u32 duration, Bool merge, Bool box_mode)
 {
 	if (merge) {
 		u64             start;
+		u64             end;
 		GF_List         *cues;
 		start = (iso_sample->DTS * 1000) / timescale;
-		cues = gf_webvtt_parse_iso_cues(iso_sample, start);
+		end = (iso_sample->DTS + duration) * 1000 / timescale;
+		cues = gf_webvtt_parse_iso_cues(iso_sample, start, end);
 		gf_webvtt_merge_cues(parser, start, cues);
 		gf_list_del(cues);
 	} else {
@@ -1503,12 +1510,14 @@ GF_Err gf_webvtt_dump_iso_track(GF_MediaExporter *dumper, u32 track, Bool merge,
 
 	count = gf_isom_get_sample_count(dumper->file, track);
 	for (i=0; i<count; i++) {
+		u32 sdur;
 		GF_ISOSample *samp = gf_isom_get_sample(dumper->file, track, i+1, &di);
 		if (!samp) {
 			e = gf_isom_last_error(dumper->file);
 			goto exit;
 		}
-		e = gf_webvtt_parse_iso_sample(parser, timescale, samp, merge, box_dump);
+		sdur = gf_isom_get_sample_duration(dumper->file, track, i+1);
+		e = gf_webvtt_parse_iso_sample(parser, timescale, samp, sdur, merge, box_dump);
 		if (e) {
 			goto exit;
 		}
