@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2000-2012
+ *			Copyright (c) Telecom ParisTech 2000-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / MPEG-4 ObjectDescriptor sub-project
@@ -477,7 +477,7 @@ void gf_odf_avc_cfg_del(GF_AVCConfig *cfg)
 {
 	if (!cfg) return;
 	while (gf_list_count(cfg->sequenceParameterSets)) {
-		GF_AVCConfigSlot *sl = (GF_AVCConfigSlot *)gf_list_get(cfg->sequenceParameterSets, 0);
+		GF_NALUFFParam *sl = (GF_NALUFFParam *)gf_list_get(cfg->sequenceParameterSets, 0);
 		gf_list_rem(cfg->sequenceParameterSets, 0);
 		if (sl->data) gf_free(sl->data);
 		gf_free(sl);
@@ -486,7 +486,7 @@ void gf_odf_avc_cfg_del(GF_AVCConfig *cfg)
 	cfg->sequenceParameterSets = NULL;
 
 	while (gf_list_count(cfg->pictureParameterSets)) {
-		GF_AVCConfigSlot *sl = (GF_AVCConfigSlot *)gf_list_get(cfg->pictureParameterSets, 0);
+		GF_NALUFFParam *sl = (GF_NALUFFParam *)gf_list_get(cfg->pictureParameterSets, 0);
 		gf_list_rem(cfg->pictureParameterSets, 0);
 		if (sl->data) gf_free(sl->data);
 		gf_free(sl);
@@ -496,7 +496,7 @@ void gf_odf_avc_cfg_del(GF_AVCConfig *cfg)
 
 	if (cfg->sequenceParameterSetExtensions) {
 		while (gf_list_count(cfg->sequenceParameterSetExtensions)) {
-			GF_AVCConfigSlot *sl = (GF_AVCConfigSlot *)gf_list_get(cfg->sequenceParameterSetExtensions, 0);
+			GF_NALUFFParam *sl = (GF_NALUFFParam *)gf_list_get(cfg->sequenceParameterSetExtensions, 0);
 			gf_list_rem(cfg->sequenceParameterSetExtensions, 0);
 			if (sl->data) gf_free(sl->data);
 			gf_free(sl);
@@ -527,7 +527,7 @@ GF_Err gf_odf_avc_cfg_write_bs(GF_AVCConfig *cfg, GF_BitStream *bs)
 		gf_bs_write_int(bs, count, 5);
 	}
 	for (i=0; i<count; i++) {
-		GF_AVCConfigSlot *sl = (GF_AVCConfigSlot *)gf_list_get(cfg->sequenceParameterSets, i);
+		GF_NALUFFParam *sl = (GF_NALUFFParam *)gf_list_get(cfg->sequenceParameterSets, i);
 		if (!cfg->write_annex_b) {
 			gf_bs_write_u16(bs, sl->size);
 		} else {
@@ -540,7 +540,7 @@ GF_Err gf_odf_avc_cfg_write_bs(GF_AVCConfig *cfg, GF_BitStream *bs)
 		gf_bs_write_int(bs, count, 8);
 	}
 	for (i=0; i<count; i++) {
-		GF_AVCConfigSlot *sl = (GF_AVCConfigSlot *)gf_list_get(cfg->pictureParameterSets, i);
+		GF_NALUFFParam *sl = (GF_NALUFFParam *)gf_list_get(cfg->pictureParameterSets, i);
 		if (!cfg->write_annex_b) {
 			gf_bs_write_u16(bs, sl->size);
 		} else {
@@ -548,7 +548,7 @@ GF_Err gf_odf_avc_cfg_write_bs(GF_AVCConfig *cfg, GF_BitStream *bs)
 		}
 		gf_bs_write_data(bs, sl->data, sl->size);
 	}
-	if (gf_avc_is_rext_profile(cfg->AVCProfileIndication)) {
+	if (gf_avcc_use_extensions(cfg->AVCProfileIndication)) {
 		if (!cfg->write_annex_b) {
 			gf_bs_write_int(bs, 0xFF, 6);
 			gf_bs_write_int(bs, cfg->chroma_format, 2);
@@ -562,7 +562,7 @@ GF_Err gf_odf_avc_cfg_write_bs(GF_AVCConfig *cfg, GF_BitStream *bs)
 			gf_bs_write_u8(bs, count);
 		}
 		for (i=0; i<count; i++) {
-			GF_AVCConfigSlot *sl = (GF_AVCConfigSlot *) gf_list_get(cfg->sequenceParameterSetExtensions, i);
+			GF_NALUFFParam *sl = (GF_NALUFFParam *) gf_list_get(cfg->sequenceParameterSetExtensions, i);
 			if (!cfg->write_annex_b) {
 				gf_bs_write_u16(bs, sl->size);
 			} else {
@@ -601,21 +601,51 @@ GF_AVCConfig *gf_odf_avc_cfg_read(u8 *dsi, u32 dsi_size)
 	gf_bs_read_int(bs, 3);
 	count = gf_bs_read_int(bs, 5);
 	for (i=0; i<count; i++) {
-		GF_AVCConfigSlot *sl = (GF_AVCConfigSlot *)gf_malloc(sizeof(GF_AVCConfigSlot));
-		sl->size = gf_bs_read_int(bs, 16);
+		GF_NALUFFParam *sl;
+		u32 size = gf_bs_read_int(bs, 16);
+		if ((size>gf_bs_available(bs)) || (size<2)) {
+			GF_LOG(GF_LOG_ERROR, GF_LOG_CODING, ("[AVC] Wrong param set size %d\n", size));
+			gf_odf_avc_cfg_del(avcc);
+			return NULL;
+		}
+		GF_SAFEALLOC(sl, GF_NALUFFParam );
+		if (!sl) {
+			gf_odf_avc_cfg_del(avcc);
+			return NULL;
+		}
+		sl->size = size;
 		sl->data = (char*)gf_malloc(sizeof(char)*sl->size);
+		if (!sl->data) {
+			gf_odf_avc_cfg_del(avcc);
+			return NULL;
+		}
 		gf_bs_read_data(bs, sl->data, sl->size);
 		gf_list_add(avcc->sequenceParameterSets, sl);
 	}
 	count = gf_bs_read_int(bs, 8);
 	for (i=0; i<count; i++) {
-		GF_AVCConfigSlot *sl = (GF_AVCConfigSlot *)gf_malloc(sizeof(GF_AVCConfigSlot));
-		sl->size = gf_bs_read_int(bs, 16);
+		GF_NALUFFParam *sl;
+		u32 size = gf_bs_read_int(bs, 16);
+		if ((size>gf_bs_available(bs)) || (size<2)) {
+			GF_LOG(GF_LOG_ERROR, GF_LOG_CODING, ("[AVC] Wrong param set size %d\n", size));
+			gf_odf_avc_cfg_del(avcc);
+			return NULL;
+		}
+		GF_SAFEALLOC(sl, GF_NALUFFParam );
+		if (!sl) {
+			gf_odf_avc_cfg_del(avcc);
+			return NULL;
+		}
+		sl->size = size;
 		sl->data = (char*)gf_malloc(sizeof(char)*sl->size);
+		if (!sl->data) {
+			gf_odf_avc_cfg_del(avcc);
+			return NULL;
+		}
 		gf_bs_read_data(bs, sl->data, sl->size);
 		gf_list_add(avcc->pictureParameterSets, sl);
 	}
-	if (gf_avc_is_rext_profile(avcc->AVCProfileIndication)) {
+	if (gf_avcc_use_extensions(avcc->AVCProfileIndication)) {
 		gf_bs_read_int(bs, 6);
 		avcc->chroma_format = gf_bs_read_int(bs, 2);
 		gf_bs_read_int(bs, 5);
@@ -627,9 +657,24 @@ GF_AVCConfig *gf_odf_avc_cfg_read(u8 *dsi, u32 dsi_size)
 		if (count) {
 			avcc->sequenceParameterSetExtensions = gf_list_new();
 			for (i=0; i<count; i++) {
-				GF_AVCConfigSlot *sl = (GF_AVCConfigSlot *)gf_malloc(sizeof(GF_AVCConfigSlot));
-				sl->size = gf_bs_read_u16(bs);
-				sl->data = (char *)gf_malloc(sizeof(char) * sl->size);
+				GF_NALUFFParam *sl;
+				u32 size = gf_bs_read_int(bs, 16);
+				if ((size>gf_bs_available(bs)) || (size<2)) {
+					GF_LOG(GF_LOG_ERROR, GF_LOG_CODING, ("[AVC] Wrong param set size %d\n", size));
+					gf_odf_avc_cfg_del(avcc);
+					return NULL;
+				}
+				GF_SAFEALLOC(sl, GF_NALUFFParam );
+				if (!sl) {
+					gf_odf_avc_cfg_del(avcc);
+					return NULL;
+				}
+				sl->size = size;
+				sl->data = (char*)gf_malloc(sizeof(char)*sl->size);
+				if (!sl->data) {
+					gf_odf_avc_cfg_del(avcc);
+					return NULL;
+				}
 				gf_bs_read_data(bs, sl->data, sl->size);
 				gf_list_add(avcc->sequenceParameterSetExtensions, sl);
 			}
@@ -874,11 +919,11 @@ void gf_odf_hevc_cfg_del(GF_HEVCConfig *cfg)
 {
 	if (!cfg) return;
 	while (gf_list_count(cfg->param_array)) {
-		GF_HEVCParamArray *pa = (GF_HEVCParamArray*)gf_list_get(cfg->param_array, 0);
+		GF_NALUFFParamArray *pa = (GF_NALUFFParamArray*)gf_list_get(cfg->param_array, 0);
 		gf_list_rem(cfg->param_array, 0);
 
 		while (gf_list_count(pa->nalus)) {
-			GF_AVCConfigSlot *n = (GF_AVCConfigSlot*)gf_list_get(pa->nalus, 0);
+			GF_NALUFFParam *n = (GF_NALUFFParam*)gf_list_get(pa->nalus, 0);
 			gf_list_rem(pa->nalus, 0);
 			if (n->data) gf_free(n->data);
 			gf_free(n);
@@ -943,7 +988,7 @@ GF_Err gf_odf_hevc_cfg_write_bs(GF_HEVCConfig *cfg, GF_BitStream *bs)
 
 	for (i=0; i<count; i++) {
 		u32 nalucount, j;
-		GF_HEVCParamArray *ar = (GF_HEVCParamArray*)gf_list_get(cfg->param_array, i);
+		GF_NALUFFParamArray *ar = (GF_NALUFFParamArray*)gf_list_get(cfg->param_array, i);
 
 		nalucount = gf_list_count(ar->nalus);
 		if (!cfg->write_annex_b) {
@@ -954,7 +999,7 @@ GF_Err gf_odf_hevc_cfg_write_bs(GF_HEVCConfig *cfg, GF_BitStream *bs)
 		}
 
 		for (j=0; j<nalucount; j++) {
-			GF_AVCConfigSlot *sl = (GF_AVCConfigSlot *)gf_list_get(ar->nalus, j);
+			GF_NALUFFParam *sl = (GF_NALUFFParam *)gf_list_get(ar->nalus, j);
 			if (!cfg->write_annex_b) {
 				gf_bs_write_int(bs, sl->size, 16);
 			} else {
@@ -1034,8 +1079,8 @@ GF_HEVCConfig *gf_odf_hevc_cfg_read_bs(GF_BitStream *bs, Bool is_lhvc)
 	count = gf_bs_read_int(bs, 8);
 	for (i=0; i<count; i++) {
 		u32 nalucount, j;
-		GF_HEVCParamArray *ar;
-		GF_SAFEALLOC(ar, GF_HEVCParamArray);
+		GF_NALUFFParamArray *ar;
+		GF_SAFEALLOC(ar, GF_NALUFFParamArray);
 		if (!ar) {
 			gf_odf_hevc_cfg_del(cfg);
 			return NULL;
@@ -1048,14 +1093,14 @@ GF_HEVCConfig *gf_odf_hevc_cfg_read_bs(GF_BitStream *bs, Bool is_lhvc)
 		ar->type = gf_bs_read_int(bs, 6);
 		nalucount = gf_bs_read_int(bs, 16);
 		for (j=0; j<nalucount; j++) {
-			GF_AVCConfigSlot *sl;
+			GF_NALUFFParam *sl;
 			u32 size = gf_bs_read_int(bs, 16);
-			if (size>gf_bs_available(bs)) {
-				GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[iso file] Wrong param set size %d\n", size));
+			if ((size>gf_bs_available(bs)) || (size<2)) {
+				GF_LOG(GF_LOG_ERROR, GF_LOG_CODING, ("[HEVC] Wrong param set size %d\n", size));
 				gf_odf_hevc_cfg_del(cfg);
 				return NULL;
 			}
-			GF_SAFEALLOC(sl, GF_AVCConfigSlot );
+			GF_SAFEALLOC(sl, GF_NALUFFParam );
 			if (!sl) {
 				gf_odf_hevc_cfg_del(cfg);
 				return NULL;
@@ -1079,6 +1124,292 @@ GF_HEVCConfig *gf_odf_hevc_cfg_read(u8 *dsi, u32 dsi_size, Bool is_lhvc)
 	return cfg;
 }
 
+GF_EXPORT
+GF_VVCConfig *gf_odf_vvc_cfg_new()
+{
+	GF_VVCConfig *cfg;
+	GF_SAFEALLOC(cfg, GF_VVCConfig);
+	if (!cfg) return NULL;
+	cfg->param_array = gf_list_new();
+	cfg->nal_unit_size = 4;
+	cfg->chroma_format = 1;
+	cfg->bit_depth = 8;
+	return cfg;
+}
+
+GF_EXPORT
+void gf_odf_vvc_cfg_del(GF_VVCConfig *cfg)
+{
+	if (!cfg) return;
+	while (gf_list_count(cfg->param_array)) {
+		GF_NALUFFParamArray *pa = (GF_NALUFFParamArray*)gf_list_get(cfg->param_array, 0);
+		gf_list_rem(cfg->param_array, 0);
+
+		while (gf_list_count(pa->nalus)) {
+			GF_NALUFFParam *n = (GF_NALUFFParam*)gf_list_get(pa->nalus, 0);
+			gf_list_rem(pa->nalus, 0);
+			if (n->data) gf_free(n->data);
+			gf_free(n);
+		}
+		gf_list_del(pa->nalus);
+		gf_free(pa);
+	}
+	gf_list_del(cfg->param_array);
+	if (cfg->general_constraint_info)
+		gf_free(cfg->general_constraint_info);
+	if (cfg->sub_profiles_idc)
+		gf_free(cfg->sub_profiles_idc);
+	gf_free(cfg);
+}
+
+GF_EXPORT
+GF_Err gf_odf_vvc_cfg_write_bs(GF_VVCConfig *cfg, GF_BitStream *bs)
+{
+	u32 i, count;
+
+	count = gf_list_count(cfg->param_array);
+
+	if (!cfg->write_annex_b) {
+
+		gf_bs_write_int(bs, 0xFF, 5);
+		gf_bs_write_int(bs, cfg->nal_unit_size - 1, 2);
+		gf_bs_write_int(bs, cfg->ptl_present, 1);
+
+		if (cfg->ptl_present) {
+			s32 idx;
+
+			gf_bs_write_int(bs, cfg->ols_idx, 9);
+			gf_bs_write_int(bs, cfg->numTemporalLayers, 3);
+			gf_bs_write_int(bs, cfg->constantFrameRate, 2);
+			gf_bs_write_int(bs, cfg->chroma_format, 2);
+			gf_bs_write_int(bs, cfg->bit_depth - 8, 3);
+			gf_bs_write_int(bs, 0xFF, 5);
+
+			if (!cfg->general_constraint_info)
+				cfg->num_constraint_info = 0;
+
+			//write PTL
+			gf_bs_write_int(bs, 0, 2);
+			gf_bs_write_int(bs, cfg->num_constraint_info, 6);
+			gf_bs_write_int(bs, cfg->general_profile_idc, 7);
+			gf_bs_write_int(bs, cfg->general_tier_flag, 1);
+			gf_bs_write_u8(bs, cfg->general_level_idc);
+			gf_bs_write_int(bs, cfg->ptl_frame_only_constraint, 1);
+			gf_bs_write_int(bs, cfg->ptl_multilayer_enabled, 1);
+
+			if (cfg->num_constraint_info) {
+				gf_bs_write_data(bs, cfg->general_constraint_info, cfg->num_constraint_info - 1);
+				gf_bs_write_int(bs, cfg->general_constraint_info[cfg->num_constraint_info - 1], 6);
+			} else {
+				gf_bs_write_int(bs, 0, 6);
+			}
+
+			for (idx=cfg->numTemporalLayers-2; idx>=0; idx--) {
+				u8 val = cfg->ptl_sublayer_present_mask & (1<<idx);
+				gf_bs_write_int(bs, val ? 1 : 0, 1);
+			}
+			for (idx=cfg->numTemporalLayers; idx<=8 && cfg->numTemporalLayers>1; idx++) {
+				gf_bs_write_int(bs, 0, 1);
+			}
+			for (idx=cfg->numTemporalLayers-2; idx>=0; idx--) {
+				if (cfg->ptl_sublayer_present_mask & (1<<idx))
+					gf_bs_write_u8(bs, cfg->sublayer_level_idc[idx]);
+			}
+			if (!cfg->sub_profiles_idc) cfg->num_sub_profiles = 0;
+			gf_bs_write_u8(bs, cfg->num_sub_profiles);
+			for (idx=0; idx<cfg->num_sub_profiles; idx++) {
+				gf_bs_write_u32(bs, cfg->sub_profiles_idc[idx]);
+			}
+			//end PTL
+
+			gf_bs_write_u16(bs, cfg->maxPictureWidth);
+			gf_bs_write_u16(bs, cfg->maxPictureHeight);
+			gf_bs_write_u16(bs, cfg->avgFrameRate);
+		}
+		gf_bs_write_int(bs, count, 8);
+	}
+
+	for (i=0; i<count; i++) {
+		u32 nalucount, j;
+		GF_NALUFFParamArray *ar = (GF_NALUFFParamArray*)gf_list_get(cfg->param_array, i);
+
+		nalucount = gf_list_count(ar->nalus);
+		if (!cfg->write_annex_b) {
+			gf_bs_write_int(bs, ar->array_completeness, 1);
+			gf_bs_write_int(bs, 0, 2);
+			gf_bs_write_int(bs, ar->type, 5);
+
+			if ((ar->type != GF_VVC_NALU_DEC_PARAM) && (ar->type != GF_VVC_NALU_OPI))
+				gf_bs_write_int(bs, nalucount, 16);
+			else
+				nalucount = 1;
+		}
+
+		for (j=0; j<nalucount; j++) {
+			GF_NALUFFParam *sl = (GF_NALUFFParam *)gf_list_get(ar->nalus, j);
+			if (!cfg->write_annex_b) {
+				gf_bs_write_int(bs, sl->size, 16);
+			} else {
+				gf_bs_write_u32(bs, 1);
+			}
+			gf_bs_write_data(bs, sl->data, sl->size);
+		}
+	}
+	return GF_OK;
+}
+
+GF_EXPORT
+GF_Err gf_odf_vvc_cfg_write(GF_VVCConfig *cfg, u8 **outData, u32 *outSize)
+{
+	GF_Err e;
+	GF_BitStream *bs = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
+	*outSize = 0;
+	*outData = NULL;
+	e = gf_odf_vvc_cfg_write_bs(cfg, bs);
+	if (e==GF_OK)
+		gf_bs_get_content(bs, outData, outSize);
+
+	gf_bs_del(bs);
+	return e;
+}
+
+GF_EXPORT
+GF_VVCConfig *gf_odf_vvc_cfg_read_bs(GF_BitStream *bs)
+{
+	u32 i, count;
+	GF_VVCConfig *cfg = gf_odf_vvc_cfg_new();
+
+	gf_bs_read_int(bs, 5);
+	cfg->nal_unit_size = 1 + gf_bs_read_int(bs, 2);
+	cfg->ptl_present = gf_bs_read_int(bs, 1);
+
+	if (cfg->ptl_present) {
+		s32 j;
+
+		cfg->ols_idx = gf_bs_read_int(bs, 9);
+		cfg->numTemporalLayers = gf_bs_read_int(bs, 3);
+		cfg->constantFrameRate = gf_bs_read_int(bs, 2);
+		cfg->chroma_format = gf_bs_read_int(bs, 2);
+		cfg->bit_depth = 8 + gf_bs_read_int(bs, 3);
+		gf_bs_read_int(bs, 5);
+
+		//parse PTL
+		gf_bs_read_int(bs, 2);
+		cfg->num_constraint_info = gf_bs_read_int(bs, 6);
+		cfg->general_profile_idc = gf_bs_read_int(bs, 7);
+		cfg->general_tier_flag = gf_bs_read_int(bs, 1);
+		cfg->general_level_idc = gf_bs_read_u8(bs);
+		cfg->ptl_frame_only_constraint = gf_bs_read_int(bs, 1);
+		cfg->ptl_multilayer_enabled = gf_bs_read_int(bs, 1);
+
+		if (cfg->num_constraint_info) {
+			cfg->general_constraint_info = gf_malloc(sizeof(u8)*cfg->num_constraint_info);
+			if (!cfg->general_constraint_info) {
+				gf_free(cfg);
+				GF_LOG(GF_LOG_ERROR, GF_LOG_CODING, ("[VVC] alloc failed while parsing vvc config\n"));
+				return NULL;
+			}
+			gf_bs_read_data(bs, cfg->general_constraint_info, cfg->num_constraint_info - 1);
+			cfg->general_constraint_info[cfg->num_constraint_info-1] =  gf_bs_read_int(bs, 6);
+		} else {
+			//forbidden in spec!
+			gf_bs_read_int(bs, 6);
+		}
+
+		cfg->ptl_sublayer_present_mask = 0;
+		for (j=cfg->numTemporalLayers-2; j>=0; j--) {
+			u32 val = gf_bs_read_int(bs, 1);
+			cfg->ptl_sublayer_present_mask |= val << j;
+		}
+		for (j=cfg->numTemporalLayers; j<=8 && cfg->numTemporalLayers>1; j++) {
+			gf_bs_read_int(bs, 1);
+		}
+		for (j=cfg->numTemporalLayers-2; j>=0; j--) {
+			if (cfg->ptl_sublayer_present_mask & (1<<j)) {
+				cfg->sublayer_level_idc[j] = gf_bs_read_u8(bs);
+			}
+		}
+		cfg->num_sub_profiles = gf_bs_read_u8(bs);
+		if (cfg->num_sub_profiles) {
+			cfg->sub_profiles_idc = gf_malloc(sizeof(u32)*cfg->num_sub_profiles);
+			if (!cfg->sub_profiles_idc) {
+				gf_free(cfg->general_constraint_info);
+				gf_free(cfg);
+				GF_LOG(GF_LOG_ERROR, GF_LOG_CODING, ("[VVC] alloc failed while parsing vvc config\n"));
+				return NULL;
+			}
+		}
+		for (i=0; i<cfg->num_sub_profiles; i++) {
+			cfg->sub_profiles_idc[i] = gf_bs_read_u32(bs);
+		}
+
+		//end PTL
+
+		cfg->maxPictureWidth = gf_bs_read_u16(bs);
+		cfg->maxPictureHeight = gf_bs_read_u16(bs);
+		cfg->avgFrameRate = gf_bs_read_u16(bs);
+	}
+
+	count = gf_bs_read_int(bs, 8);
+	for (i=0; i<count; i++) {
+		u32 nalucount, j;
+		GF_NALUFFParamArray *ar;
+		GF_SAFEALLOC(ar, GF_NALUFFParamArray);
+		if (!ar) {
+			gf_odf_vvc_cfg_del(cfg);
+			GF_LOG(GF_LOG_ERROR, GF_LOG_CODING, ("[VVC] alloc failed while parsing vvc config\n"));
+			return NULL;
+		}
+		ar->nalus = gf_list_new();
+		gf_list_add(cfg->param_array, ar);
+
+		ar->array_completeness = gf_bs_read_int(bs, 1);
+		gf_bs_read_int(bs, 2);
+		ar->type = gf_bs_read_int(bs, 5);
+
+		if ((ar->type != GF_VVC_NALU_DEC_PARAM) && (ar->type != GF_VVC_NALU_OPI))
+			nalucount = gf_bs_read_int(bs, 16);
+		else
+			nalucount = 1;
+			
+		for (j=0; j<nalucount; j++) {
+			GF_NALUFFParam *sl;
+			u32 size = gf_bs_read_int(bs, 16);
+			if ((size>gf_bs_available(bs)) || (size<2)) {
+				GF_LOG(GF_LOG_ERROR, GF_LOG_CODING, ("[VVC] Wrong param set size %d\n", size));
+				gf_odf_vvc_cfg_del(cfg);
+				return NULL;
+			}
+			GF_SAFEALLOC(sl, GF_NALUFFParam );
+			if (!sl) {
+				gf_odf_vvc_cfg_del(cfg);
+				GF_LOG(GF_LOG_ERROR, GF_LOG_CODING, ("[VVC] alloc failed while parsing vvc config\n"));
+				return NULL;
+			}
+
+			sl->size = size;
+			sl->data = (char *)gf_malloc(sizeof(char) * sl->size);
+			if (!sl->data) {
+				gf_free(sl);
+				gf_odf_vvc_cfg_del(cfg);
+				GF_LOG(GF_LOG_ERROR, GF_LOG_CODING, ("[VVC] alloc failed while parsing vvc config\n"));
+				return NULL;
+			}
+			gf_bs_read_data(bs, sl->data, sl->size);
+			gf_list_add(ar->nalus, sl);
+		}
+	}
+	return cfg;
+}
+
+GF_EXPORT
+GF_VVCConfig *gf_odf_vvc_cfg_read(u8 *dsi, u32 dsi_size)
+{
+	GF_BitStream *bs = gf_bs_new(dsi, dsi_size, GF_BITSTREAM_READ);
+	GF_VVCConfig *cfg = gf_odf_vvc_cfg_read_bs(bs);
+	gf_bs_del(bs);
+	return cfg;
+}
 
 GF_EXPORT
 GF_AV1Config *gf_odf_av1_cfg_new()
@@ -1184,7 +1515,7 @@ GF_Err gf_odf_vp_cfg_write_bs(GF_VPConfig *cfg, GF_BitStream *bs, Bool is_v0)
 
 	if (!is_v0) {
 		if (cfg->codec_initdata_size) {
-			GF_LOG(GF_LOG_WARNING, GF_LOG_CONTAINER, ("[iso file] VP Configuration Box: invalid data, codec_initdata_size must be 0, was %d - ignoring\n", cfg->codec_initdata_size));
+			GF_LOG(GF_LOG_WARNING, GF_LOG_CODING, ("[VPX] Invalid data in configuration: codec_initdata_size must be 0, was %d - ignoring\n", cfg->codec_initdata_size));
 		}
 
 		gf_bs_write_int(bs, (u16)0, 16);
@@ -1231,7 +1562,7 @@ GF_VPConfig *gf_odf_vp_cfg_read_bs(GF_BitStream *bs, Bool is_v0)
 
 	// must be 0 according to spec
 	if (cfg->codec_initdata_size) {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[iso file] VP Configuration Box: invalid data, codec_initdata_size must be 0, was %d\n", cfg->codec_initdata_size));
+		GF_LOG(GF_LOG_ERROR, GF_LOG_CODING, ("[VPX] Invalid data in configuration: codec_initdata_size must be 0, was %d\n", cfg->codec_initdata_size));
 		gf_odf_vp_cfg_del(cfg);
 		return NULL;
 	}
@@ -1277,7 +1608,7 @@ GF_AV1Config *gf_odf_av1_cfg_read_bs_size(GF_BitStream *bs, u32 size)
 
 	reserved = gf_bs_read_int(bs, 3);
 	if (reserved != 0 || cfg->marker != 1 || cfg->version != 1) {
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[AV1] wrong avcC reserved %d / marker %d / version %d expecting 0 1 1\n", reserved, cfg->marker, cfg->version));
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_CODING, ("[AV1] wrong avcC reserved %d / marker %d / version %d expecting 0 1 1\n", reserved, cfg->marker, cfg->version));
 		gf_odf_av1_cfg_del(cfg);
 		return NULL;
 	}
@@ -1297,15 +1628,15 @@ GF_AV1Config *gf_odf_av1_cfg_read_bs_size(GF_BitStream *bs, u32 size)
 
 		pos = gf_bs_get_position(bs);
 		obu_size = 0;
-		if (gf_media_aom_av1_parse_obu(bs, &obu_type, &obu_size, NULL, &state) != GF_OK) {
-			GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[AV1] could not parse AV1 OBU at position "LLU". Leaving parsing.\n", pos));
+		if (gf_av1_parse_obu(bs, &obu_type, &obu_size, NULL, &state) != GF_OK) {
+			GF_LOG(GF_LOG_ERROR, GF_LOG_CODING, ("[AV1] could not parse AV1 OBU at position "LLU". Leaving parsing.\n", pos));
 			break;
 		}
 		assert(obu_size == gf_bs_get_position(bs) - pos);
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[AV1] parsed AV1 OBU type=%u size="LLU" at position "LLU".\n", obu_type, obu_size, pos));
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_CODING, ("[AV1] parsed AV1 OBU type=%u size="LLU" at position "LLU".\n", obu_type, obu_size, pos));
 
 		if (!av1_is_obu_header(obu_type)) {
-			GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[AV1] AV1 unexpected OBU type=%u size="LLU" found at position "LLU". Forwarding.\n", pos));
+			GF_LOG(GF_LOG_DEBUG, GF_LOG_CODING, ("[AV1] AV1 unexpected OBU type=%u size="LLU" found at position "LLU". Forwarding.\n", pos));
 		}
 		GF_SAFEALLOC(a, GF_AV1_OBUArrayEntry);
 		if (!a) break;
@@ -1321,12 +1652,13 @@ GF_AV1Config *gf_odf_av1_cfg_read_bs_size(GF_BitStream *bs, u32 size)
 		gf_list_add(cfg->obu_array, a);
 
 		if (size<obu_size) {
-			GF_LOG(GF_LOG_WARNING, GF_LOG_CONTAINER, ("[AV1] AV1 config misses %d bytes to fit the entire OBU\n", obu_size - size));
+			GF_LOG(GF_LOG_WARNING, GF_LOG_CODING, ("[AV1] AV1 config misses %d bytes to fit the entire OBU\n", obu_size - size));
 			break;
 		}
 		size -= (u32) obu_size;
 	}
 	gf_av1_reset_state(& state, GF_TRUE);
+	gf_bs_align(bs);
 	return cfg;
 #else
 	return NULL;
@@ -1360,20 +1692,19 @@ GF_DOVIDecoderConfigurationRecord *gf_odf_dovi_cfg_read_bs(GF_BitStream *bs)
 	cfg->rpu_present_flag = gf_bs_read_int(bs, 1);
 	cfg->el_present_flag = gf_bs_read_int(bs, 1);
 	cfg->bl_present_flag = gf_bs_read_int(bs, 1);
-	{
-		int i = 0;
-		u32 data[5];
-		memset(data, 0, sizeof(data));
-		gf_bs_read_data(bs, (char*)data, 20);
-		for (i = 0; i < 5; ++i) {
-			if (data[i] != 0) {
-				GF_LOG(GF_LOG_WARNING, GF_LOG_CONTAINER, ("[odf_cfg] dovi config reserved bytes are not zero\n"));
-			}
+	cfg->dv_bl_signal_compatibility_id = gf_bs_read_int(bs, 4);
+	if (gf_bs_read_int(bs, 28)) {
+		GF_LOG(GF_LOG_WARNING, GF_LOG_CODING, ("[DOVI] Configuration reserved bits are not zero\n"));
+	}
+	for (u32 i=0; i<4; i++) {
+		if (gf_bs_read_u32(bs)) {
+			GF_LOG(GF_LOG_WARNING, GF_LOG_CODING, ("[DOVII] Configuration reserved bits are not zero\n"));
 		}
 	}
 	return cfg;
 }
 
+GF_EXPORT
 void gf_odf_dovi_cfg_del(GF_DOVIDecoderConfigurationRecord *cfg)
 {
 	gf_free(cfg);
@@ -1388,10 +1719,106 @@ GF_Err gf_odf_dovi_cfg_write_bs(GF_DOVIDecoderConfigurationRecord *cfg, GF_BitSt
 	gf_bs_write_int(bs, cfg->rpu_present_flag, 1);
 	gf_bs_write_int(bs, cfg->el_present_flag, 1);
 	gf_bs_write_int(bs, cfg->bl_present_flag, 1);
-    gf_bs_write_u32(bs, 0);
+	gf_bs_write_int(bs, cfg->dv_bl_signal_compatibility_id, 4);
+    gf_bs_write_int(bs, 0, 28);
     gf_bs_write_u32(bs, 0);
     gf_bs_write_u32(bs, 0);
     gf_bs_write_u32(bs, 0);
     gf_bs_write_u32(bs, 0);
 	return GF_OK;
 }
+
+
+GF_Err gf_odf_ac3_cfg_write_bs(GF_AC3Config *cfg, GF_BitStream *bs)
+{
+	if (!cfg || !bs) return GF_BAD_PARAM;
+
+	if (cfg->is_ec3) {
+		u32 i;
+		gf_bs_write_int(bs, cfg->brcode, 13);
+		gf_bs_write_int(bs, cfg->nb_streams - 1, 3);
+		for (i=0; i<cfg->nb_streams; i++) {
+			gf_bs_write_int(bs, cfg->streams[i].fscod, 2);
+			gf_bs_write_int(bs, cfg->streams[i].bsid, 5);
+			gf_bs_write_int(bs, cfg->streams[i].bsmod, 5);
+			gf_bs_write_int(bs, cfg->streams[i].acmod, 3);
+			gf_bs_write_int(bs, cfg->streams[i].lfon, 1);
+			gf_bs_write_int(bs, 0, 3);
+			gf_bs_write_int(bs, cfg->streams[i].nb_dep_sub, 4);
+			if (cfg->streams[i].nb_dep_sub) {
+				gf_bs_write_int(bs, cfg->streams[i].chan_loc, 9);
+			} else {
+				gf_bs_write_int(bs, 0, 1);
+			}
+		}
+	} else {
+		gf_bs_write_int(bs, cfg->streams[0].fscod, 2);
+		gf_bs_write_int(bs, cfg->streams[0].bsid, 5);
+		gf_bs_write_int(bs, cfg->streams[0].bsmod, 3);
+		gf_bs_write_int(bs, cfg->streams[0].acmod, 3);
+		gf_bs_write_int(bs, cfg->streams[0].lfon, 1);
+		gf_bs_write_int(bs, cfg->brcode, 5);
+		gf_bs_write_int(bs, 0, 5);
+	}
+	return GF_OK;
+}
+
+GF_Err gf_odf_ac3_cfg_write(GF_AC3Config *cfg, u8 **data, u32 *size)
+{
+	GF_BitStream *bs = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
+	GF_Err e = gf_odf_ac3_cfg_write_bs(cfg, bs);
+
+	gf_bs_get_content(bs, data, size);
+	gf_bs_del(bs);
+	return e;
+}
+
+GF_Err gf_odf_ac3_config_parse_bs(GF_BitStream *bs, Bool is_ec3, GF_AC3Config *cfg)
+{
+	if (!cfg || !bs) return GF_BAD_PARAM;
+	memset(cfg, 0, sizeof(GF_AC3Config));
+	cfg->is_ec3 = is_ec3;
+	if (is_ec3) {
+		u32 j;
+		cfg->is_ec3 = 1;
+		cfg->brcode = gf_bs_read_int(bs, 13);
+		cfg->nb_streams = 1 + gf_bs_read_int(bs, 3);
+		for (j=0; j<cfg->nb_streams; j++) {
+			cfg->streams[j].fscod = gf_bs_read_int(bs, 2);
+			cfg->streams[j].bsid = gf_bs_read_int(bs, 5);
+			gf_bs_read_int(bs, 1);
+			cfg->streams[j].asvc = gf_bs_read_int(bs, 1);
+			cfg->streams[j].bsmod = gf_bs_read_int(bs, 3);
+			cfg->streams[j].acmod = gf_bs_read_int(bs, 3);
+			cfg->streams[j].lfon = gf_bs_read_int(bs, 1);
+			gf_bs_read_int(bs, 3);
+			cfg->streams[j].nb_dep_sub = gf_bs_read_int(bs, 4);
+			if (cfg->streams[j].nb_dep_sub) {
+				cfg->streams[j].chan_loc = gf_bs_read_int(bs, 9);
+			} else {
+				gf_bs_read_int(bs, 1);
+			}
+		}
+	} else {
+		cfg->nb_streams = 1;
+		cfg->streams[0].fscod = gf_bs_read_int(bs, 2);
+		cfg->streams[0].bsid = gf_bs_read_int(bs, 5);
+		cfg->streams[0].bsmod = gf_bs_read_int(bs, 3);
+		cfg->streams[0].acmod = gf_bs_read_int(bs, 3);
+		cfg->streams[0].lfon = gf_bs_read_int(bs, 1);
+		cfg->brcode = gf_bs_read_int(bs, 5);
+		gf_bs_read_int(bs, 5);
+	}
+	return GF_OK;
+}
+
+GF_Err gf_odf_ac3_config_parse(u8 *dsi, u32 dsi_len, Bool is_ec3, GF_AC3Config *cfg)
+{
+	GF_BitStream *bs;
+	GF_Err e;
+	if (!cfg || !dsi) return GF_BAD_PARAM;
+	bs = gf_bs_new(dsi, dsi_len, GF_BITSTREAM_READ);
+	e = gf_odf_ac3_config_parse_bs(bs, is_ec3, cfg);
+	gf_bs_del(bs);
+	return e;
+}
