@@ -39,7 +39,7 @@
 typedef struct _st_entry
 {
 	struct _st_entry *next;
-	/*as refered to by xlink-href*/
+	/*as referred to by xlink-href*/
 	char *stream_name;
 	/*stream id*/
 	u32 id;
@@ -1426,6 +1426,10 @@ static void svg_node_start(void *sax_cbck, const char *name, const char *name_sp
 			Bool rap;
 			time = 0;
 			rap =  GF_FALSE;
+			if (!parser->laser_es) {
+				svg_report(parser, GF_BAD_PARAM, "No LASER stream specified");
+				return;
+			}
 			if (!gf_list_count(parser->laser_es->AUs)) rap = GF_TRUE;
 			for (i=0; i<nb_attributes; i++) {
 				GF_XMLAttribute *att = (GF_XMLAttribute *) &attributes[i];
