@@ -88,7 +88,8 @@ void gf_sg_command_del(GF_Command *com)
 			GF_CommandField *inf = (GF_CommandField *)gf_list_get(com->command_fields, 0);
 			gf_list_rem(com->command_fields, 0);
 
-			if (inf->new_node) gf_node_unregister(inf->new_node, NULL);
+			if (inf->new_node)
+				gf_node_try_destroy(com->in_scene, inf->new_node, NULL);
 			else if (inf->node_list) {
 				GF_ChildNodeItem *child;
 				child = inf->node_list;
