@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2000-2018
+ *			Copyright (c) Telecom ParisTech 2000-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / Media terminal sub-project
@@ -65,20 +65,6 @@ Bool gf_term_send_event(GF_Terminal *term, GF_Event *evt)
 }
 
 
-#ifdef FILTER_FIXME
-
-static Bool gf_term_get_user_pass(void *usr_cbk, const char *site_url, char *usr_name, char *password)
-{
-	GF_Event evt;
-	GF_Terminal *term = (GF_Terminal *)usr_cbk;
-	evt.type = GF_EVENT_AUTHORIZATION;
-	evt.auth.site_url = site_url;
-	evt.auth.user = usr_name;
-	evt.auth.password = password;
-	return gf_term_send_event(term, &evt);
-}
-#endif
-
 static GF_Err gf_sc_step_clocks_intern(GF_Compositor *compositor, u32 ms_diff, Bool force_resume_pause)
 {
 	/*only play/pause if connected*/
@@ -207,7 +193,7 @@ void gf_sc_connect_from_time_ex(GF_Compositor *compositor, const char *URL, u64
 		/*disconnect*/
 		gf_sc_disconnect(compositor);
 	}
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[Terminal] Connecting to %s\n", URL));
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPTIME, ("[Terminal] Connecting to %s\n", URL));
 
 	assert(!compositor->root_scene);
 
@@ -230,7 +216,7 @@ void gf_sc_connect_from_time_ex(GF_Compositor *compositor, const char *URL, u64
 		scene->first_frame_pause_type = pause_at_first_frame;
 	}
 
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[Terminal] root scene created\n", URL));
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPTIME, ("[Terminal] root scene created\n", URL));
 
 	if (!strnicmp(URL, "views://", 8)) {
 		gf_scene_generate_views(compositor->root_scene, (char *) URL+8, (char*)parent_path);
@@ -240,19 +226,15 @@ void gf_sc_connect_from_time_ex(GF_Compositor *compositor, const char *URL, u64
 		gf_scene_generate_mosaic(compositor->root_scene, (char *) URL+9, (char*)parent_path);
 		return;
 	}
-	else if (!strnicmp(URL, "mosaic://", 9)) {
-		gf_scene_generate_mosaic(compositor->root_scene, (char *) URL+9, (char*)parent_path);
-		return;
-	}
 
-	gf_scene_ns_connect_object(scene, odm, (char *) URL, (char*)parent_path);
+	gf_scene_ns_connect_object(scene, odm, (char *) URL, (char*)parent_path, NULL);
 }
 
 
 GF_EXPORT
 Bool gf_term_is_type_supported(GF_Terminal *term, const char* mime)
 {
-	return gf_fs_mime_supported(term->fsess, mime);
+	return gf_fs_is_supported_mime(term->fsess, mime);
 }
 
 //todo: move this to compositor ?
@@ -279,17 +261,15 @@ static void gf_term_refresh_cache()
 		}
 
 		force_delete = 0;
-		if (file) {
-			FILE *t = gf_fopen(file, "r");
-			if (!t) force_delete = 1;
-			else gf_fclose(t);
+		if (file && gf_file_exists(file)) {
+			force_delete = 1;
 		}
 		sscanf(opt, "%u", &exp);
 		gf_net_get_ntp(&sec, &frac);
 		if (exp && (exp<sec)) force_delete=1;
 
 		if (force_delete) {
-			if (file) gf_file_delete((char*) opt);
+			if (file) gf_file_delete(file);
 
 			gf_opts_del_section(name);
 			i--;
@@ -309,11 +289,11 @@ GF_Terminal *gf_term_new(GF_User *user)
 	const char *opt;
 	char szArgs[200];
 
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[Terminal] Creating terminal\n"));
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPTIME, ("[Terminal] Creating terminal\n"));
 
 	tmp = (GF_Terminal*)gf_malloc(sizeof(GF_Terminal));
 	if (!tmp) {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[Terminal] Failed to allocate GF_Terminal : OUT OF MEMORY ?\n"));
+		GF_LOG(GF_LOG_ERROR, GF_LOG_COMPTIME, ("[Terminal] Failed to allocate GF_Terminal : OUT OF MEMORY ?\n"));
 		return NULL;
 	}
 	memset(tmp, 0, sizeof(GF_Terminal));
@@ -328,9 +308,9 @@ GF_Terminal *gf_term_new(GF_User *user)
 	sprintf(szArgs, "%d", user->init_flags);
 	gf_opts_set_key("Temp", "InitFlags", szArgs);
 
-	tmp->fsess = gf_fs_new_defaults(GF_FS_FLAG_NO_MAIN_THREAD);
+	tmp->fsess = gf_fs_new_defaults(GF_FS_FLAG_NON_BLOCKING);
 	if (!tmp->fsess) {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[Terminal] Failed to create filter session.\n"));
+		GF_LOG(GF_LOG_ERROR, GF_LOG_COMPTIME, ("[Terminal] Failed to create filter session.\n"));
 		gf_free(tmp);
 		return NULL;
 	}
@@ -343,7 +323,7 @@ GF_Terminal *gf_term_new(GF_User *user)
 	def_h = opt ? atoi(opt) : 0;
 
 	if (def_w && def_h) {
-		sprintf(szArgs, "compositor:FID=compose:player=base:size=%dx%d", def_w, def_h);
+		sprintf(szArgs, "compositor:FID=compose:player=base:osize=%dx%d", def_w, def_h);
 	} else {
 		strcpy(szArgs, "compositor:FID=compose:player=base");
 	}
@@ -351,22 +331,13 @@ GF_Terminal *gf_term_new(GF_User *user)
 	comp_filter = gf_fs_load_filter(tmp->fsess, szArgs, &e);
 	tmp->compositor = comp_filter ? gf_sc_from_filter(comp_filter) : NULL;
 	if (!tmp->compositor) {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[Terminal] Failed to load compositor filter: %s\n", gf_error_to_string(e) ));
+		GF_LOG(GF_LOG_ERROR, GF_LOG_COMPTIME, ("[Terminal] Failed to load compositor filter: %s\n", gf_error_to_string(e) ));
 		gf_fs_del(tmp->fsess);
 		gf_free(tmp);
 		return NULL;
 	}
 
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[Terminal] compositor loaded\n"));
-
-#ifdef FILTER_FIXME
-	gf_dm_set_auth_callback(tmp->downloader, gf_term_get_user_pass, tmp);
-
-	tmp->uri_relocators = gf_list_new();
-	tmp->locales.relocate_uri = term_check_locales;
-	tmp->locales.term = tmp;
-	gf_list_add(tmp->uri_relocators, &tmp->locales);
-#endif
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPTIME, ("[Terminal] compositor loaded\n"));
 
 	gf_term_refresh_cache();
 	gf_fs_run(tmp->fsess);
@@ -379,10 +350,10 @@ GF_Err gf_term_del(GF_Terminal * term)
 {
 	if (!term) return GF_BAD_PARAM;
 
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[Terminal] Destroying terminal\n"));
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPTIME, ("[Terminal] Destroying terminal\n"));
 	/*close main service*/
 	gf_term_disconnect(term);
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[Terminal] main service disconnected\n"));
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPTIME, ("[Terminal] main service disconnected\n"));
 
 	term->in_destroy = GF_TRUE;
 	/*stop the media manager */
@@ -391,7 +362,7 @@ GF_Err gf_term_del(GF_Terminal * term)
 	gf_sys_close();
 	if (term->reload_url) gf_free(term->reload_url);
 	gf_free(term);
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[Terminal] Terminal destroyed\n"));
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPTIME, ("[Terminal] Terminal destroyed\n"));
 	return GF_OK;
 }
 
@@ -446,6 +417,8 @@ const char *gf_term_get_url(GF_Terminal *term)
 	return compositor->root_scene->root_od->scene_ns->url;
 }
 
+GF_DownloadManager *gf_filter_get_download_manager(GF_Filter *filter);
+
 /*set rendering option*/
 GF_EXPORT
 GF_Err gf_term_set_option(GF_Terminal * term, u32 type, u32 value)
@@ -455,11 +428,13 @@ GF_Err gf_term_set_option(GF_Terminal * term, u32 type, u32 value)
 	case GF_OPT_PLAY_STATE:
 		gf_term_set_play_state(term->compositor, value, 0, 1);
 		return GF_OK;
-#ifdef FILTER_FIXME
 	case GF_OPT_HTTP_MAX_RATE:
-		gf_dm_set_data_rate(term->downloader, value);
+	{
+		GF_DownloadManager *dm = gf_filter_get_download_manager(term->compositor->filter);
+		if (!dm) return GF_SERVICE_ERROR;
+		gf_dm_set_data_rate(dm, value);
 		return GF_OK;
-#endif
+	}
 	default:
 		return gf_sc_set_option(term->compositor, type, value);
 	}
@@ -506,11 +481,11 @@ u32 gf_sc_term_get_option(GF_Compositor *compositor, u32 type)
 	case GF_OPT_CAN_SELECT_STREAMS:
 		return (compositor->root_scene && compositor->root_scene->is_dynamic_scene) ? 1 : 0;
 	case GF_OPT_HTTP_MAX_RATE:
-#if FILTER_FIXME
-		return gf_dm_get_data_rate(compositor->downloader);
-#else
-		return 0;
-#endif
+	{
+		GF_DownloadManager *dm = gf_filter_get_download_manager(compositor->filter);
+		if (!dm) return 0;
+		return gf_dm_get_data_rate(dm);
+	}
 	case GF_OPT_VIDEO_BENCH:
 		return compositor->bench_mode ? GF_TRUE : GF_FALSE;
 	case GF_OPT_ORIENTATION_SENSORS_ACTIVE:
@@ -541,26 +516,6 @@ typedef struct
 } GF_TermConnectObject;
 
 
-#ifdef FILTER_FIXME
-
-
-/* Browses all registered relocators (ZIP-based, ISOFF-based or file-system-based to relocate a URI based on the locale */
-GF_EXPORT
-Bool gf_term_relocate_url(GF_Terminal *term, const char *service_url, const char *parent_url, char *out_relocated_url, char *out_localized_url)
-{
-	u32 i, count;
-
-	count = gf_list_count(term->uri_relocators);
-	for (i=0; i<count; i++) {
-		Bool result;
-		GF_URIRelocator *uri_relocator = gf_list_get(term->uri_relocators, i);
-		result = uri_relocator->relocate_uri(uri_relocator, parent_url, service_url, out_relocated_url, out_localized_url);
-		if (result) return 1;
-	}
-	return 0;
-}
-#endif
-
 GF_EXPORT
 u32 gf_sc_play_from_time(GF_Compositor *compositor, u64 from_time, u32 pause_at_first_frame)
 {
@@ -574,7 +529,7 @@ u32 gf_sc_play_from_time(GF_Compositor *compositor, u64 from_time, u32 pause_at_
 			pause_at_first_frame = 0;
 	}
 
-	/*for dynamic scene OD ressources are static and all object use the same clock, so don't restart the root
+	/*for dynamic scene OD resources are static and all object use the same clock, so don't restart the root
 	OD, just act as a mediaControl on all playing streams*/
 	if (compositor->root_scene->is_dynamic_scene) {
 
@@ -731,11 +686,25 @@ GF_Err gf_term_scene_update(GF_Terminal *term, char *type, char *com)
 	if (!term || !com) return GF_BAD_PARAM;
 
 	if (type && (!stricmp(type, "application/ecmascript") || !stricmp(type, "js")) )  {
-		return gf_scene_execute_script(compositor->root_scene->graph, com);
+#if defined(GPAC_HAS_QJS) && !defined(GPAC_DISABLE_SVG)
+		u32 tag;
+		GF_Node *root = gf_sg_get_root_node(compositor->root_scene->graph);
+		if (!root) return GF_BAD_PARAM;
+		tag = gf_node_get_tag(root);
+		if (tag >= GF_NODE_RANGE_FIRST_SVG) {
+			if (compositor->root_scene->graph->svg_js) {
+				GF_Err svg_exec_script(struct __tag_svg_script_ctx *svg_js, GF_SceneGraph *sg, const char *com);
+				return svg_exec_script(compositor->root_scene->graph->svg_js, compositor->root_scene->graph, (char *)com);
+			}
+			return GF_NOT_FOUND;
+		}
+		return GF_NOT_SUPPORTED;
+#else
+		return GF_NOT_SUPPORTED;
+#endif
 	}
 
 	if (!type && !strncmp(com, "gpac ", 5)) {
-#ifdef FILTER_FIXME
 		com += 5;
 		//new add-on
 		if (compositor->root_scene && !strncmp(com, "add ", 4)) {
@@ -747,7 +716,7 @@ GF_Err gf_term_scene_update(GF_Terminal *term, char *type, char *com)
 			return GF_OK;
 		}
 		//new splicing add-on
-		if (term->root_scene && !strncmp(com, "splice ", 7)) {
+		if (compositor->root_scene && !strncmp(com, "splice ", 7)) {
 			char *sep;
 			Double start, end;
 			Bool is_pts = GF_FALSE;
@@ -772,9 +741,9 @@ GF_Err gf_term_scene_update(GF_Terminal *term, char *type, char *com)
 			}
 			//splice end, locate first splice with no end set and set it
 			else if (sscanf(com, "%lf", &end)==1) {
-				u32 count = gf_list_count(term->root_scene->declared_addons);
+				u32 count = gf_list_count(compositor->root_scene->declared_addons);
 				for (i=0; i<count; i++) {
-					GF_AddonMedia *addon = gf_list_get(term->root_scene->declared_addons, i);
+					GF_AddonMedia *addon = gf_list_get(compositor->root_scene->declared_addons, i);
 					if (addon->is_splicing && (addon->splice_end<0) ) {
 						addon->splice_end = end;
 						break;
@@ -790,16 +759,15 @@ GF_Err gf_term_scene_update(GF_Terminal *term, char *type, char *com)
 			addon_info.splice_start_time = start;
 			addon_info.splice_end_time = end;
 			addon_info.splice_time_pts = is_pts;
-			gf_scene_register_associated_media(term->root_scene, &addon_info);
+			gf_scene_register_associated_media(compositor->root_scene, &addon_info);
 			return GF_OK;
 		}
 		//select object
-		if (term->root_scene && !strncmp(com, "select ", 7)) {
+		if (compositor->root_scene && !strncmp(com, "select ", 7)) {
 			u32 idx = atoi(com+7);
-			gf_term_select_object(term, gf_list_get(term->root_scene->resources, idx));
+			gf_term_select_object(term, gf_list_get(compositor->root_scene->resources, idx));
 			return GF_OK;
 		}
-#endif
 		return GF_OK;
 	}
 
@@ -964,50 +932,6 @@ GF_Err gf_term_paste_text(GF_Terminal *term, const char *txt, Bool probe_only)
 }
 
 
-enum
-{
-	GF_ACTION_PLAY,
-	GF_ACTION_STOP,
-	GF_ACTION_STEP,
-	GF_ACTION_EXIT,
-	GF_ACTION_MUTE,
-	GF_ACTION_VOLUP,
-	GF_ACTION_VOLDOWN,
-	GF_ACTION_JUMP_FORWARD,
-	GF_ACTION_JUMP_BACKWARD,
-	GF_ACTION_JUMP_START,
-	GF_ACTION_JUMP_END,
-	GF_ACTION_VERY_FAST_FORWARD,
-	GF_ACTION_FAST_FORWARD,
-	GF_ACTION_SLOW_FORWARD,
-	GF_ACTION_VERY_FAST_REWIND,
-	GF_ACTION_FAST_REWIND,
-	GF_ACTION_SLOW_REWIND,
-	GF_ACTION_NEXT,
-	GF_ACTION_PREVIOUS,
-	GF_ACTION_QUALITY_UP,
-	GF_ACTION_QUALITY_DOWN,
-};
-
-#ifdef FILTER_FIXME //unused for now, need to patch compositor shortcuts
-static void set_clocks_speed(GF_Compositor *compositor, Fixed ratio)
-{
-	u32 i, j;
-	GF_SceneNamespace *ns;
-
-	/*pause all clocks on all services*/
-	i=0;
-	while ( (ns = (GF_SceneNamespace*)gf_list_enum(compositor->root_scene->namespaces, &i)) ) {
-		GF_Clock *ck;
-		j=0;
-		while ( (ck = (GF_Clock *)gf_list_enum(ns->Clocks, &j)) ) {
-			Fixed s = gf_mulfix(ck->speed, ratio);
-			gf_clock_set_speed(ck, s);
-		}
-	}
-}
-#endif
-
 GF_EXPORT
 GF_Err gf_term_set_speed(GF_Terminal *term, Fixed speed)
 {
@@ -1022,16 +946,15 @@ GF_Err gf_term_set_speed(GF_Terminal *term, Fixed speed)
 	if (speed<0) {
 		i=0;
 		while ( (ns = (GF_SceneNamespace*)gf_list_enum(term->compositor->root_scene->namespaces, &i)) ) {
-#ifdef FILTER_FIXME
-			GF_NetworkCommand com;
-			GF_Err e;
-			memset(&com, 0, sizeof(GF_NetworkCommand));
-			com.base.command_type = GF_NET_SERVICE_CAN_REVERSE_PLAYBACK;
-			e = gf_term_service_command(ns, &com);
-			if (e != GF_OK) {
-				return e;
+			u32 k=0;
+			GF_ObjectManager *odm;
+			if (!ns->owner || !ns->owner->subscene) continue;
+
+			while ( (odm = gf_list_enum(ns->owner->subscene->resources, &k))) {
+				const GF_PropertyValue *p = gf_filter_pid_get_property(odm->pid, GF_PROP_PID_PLAYBACK_MODE);
+				if (!p || (p->value.uint!=GF_PLAYBACK_MODE_REWIND))
+					return GF_NOT_SUPPORTED;
 			}
-#endif
 		}
 	}
 
@@ -1071,15 +994,70 @@ GF_Err gf_term_set_speed(GF_Terminal *term, Fixed speed)
 		speed = -speed;
 
 	fps = term->compositor->fps;
-	fps.num = fps.num * (u32) (1000 * FIX2FLT(speed));
-	fps.den *= 1000;
+	if (fps.den<1000) {
+		fps.num = fps.num * (u32) (1000 * FIX2FLT(speed));
+		fps.den *= 1000;
+	} else {
+		fps.num = (u32) (fps.num * FIX2FLT(speed));
+	}
 	gf_media_get_reduced_frame_rate(&fps.num, &fps.den);
 	gf_sc_set_fps(term->compositor, fps);
 	return GF_OK;
 }
 
-#ifdef FILTER_FIXME
-GF_EXPORT
+#if 0 //no more support for shortcuts in MP4 client, this must be done through GUI
+
+enum
+{
+	GF_ACTION_PLAY,
+	GF_ACTION_STOP,
+	GF_ACTION_STEP,
+	GF_ACTION_EXIT,
+	GF_ACTION_MUTE,
+	GF_ACTION_VOLUP,
+	GF_ACTION_VOLDOWN,
+	GF_ACTION_JUMP_FORWARD,
+	GF_ACTION_JUMP_BACKWARD,
+	GF_ACTION_JUMP_START,
+	GF_ACTION_JUMP_END,
+	GF_ACTION_VERY_FAST_FORWARD,
+	GF_ACTION_FAST_FORWARD,
+	GF_ACTION_SLOW_FORWARD,
+	GF_ACTION_VERY_FAST_REWIND,
+	GF_ACTION_FAST_REWIND,
+	GF_ACTION_SLOW_REWIND,
+	GF_ACTION_NEXT,
+	GF_ACTION_PREVIOUS,
+	GF_ACTION_QUALITY_UP,
+	GF_ACTION_QUALITY_DOWN,
+};
+
+#define	MAX_SHORTCUTS	200
+
+typedef struct
+{
+	u8 code;
+	u8 mods;
+	u8 action;
+} GF_Shortcut;
+
+static void set_clocks_speed(GF_Compositor *compositor, Fixed ratio)
+{
+	u32 i, j;
+	GF_SceneNamespace *ns;
+
+	/*pause all clocks on all services*/
+	i=0;
+	while ( (ns = (GF_SceneNamespace*)gf_list_enum(compositor->root_scene->namespaces, &i)) ) {
+		GF_Clock *ck;
+		j=0;
+		while ( (ck = (GF_Clock *)gf_list_enum(ns->Clocks, &j)) ) {
+			Fixed s = gf_mulfix(ck->speed, ratio);
+			gf_clock_set_speed(ck, s);
+		}
+	}
+}
+
 void gf_term_process_shortcut(GF_Terminal *term, GF_Event *ev)
 {
 	GF_Event evt;
@@ -1323,7 +1301,7 @@ Bool gf_term_process_step(GF_Terminal *term)
 		}
 	}
 
-	gf_fs_run_step(term->fsess);
+	gf_fs_run(term->fsess);
 	return term->compositor->frame_was_produced;
 }
 
@@ -1749,3 +1727,17 @@ void gf_term_print_graph(GF_Terminal *term)
 	if (term->fsess)
 		gf_fs_print_connections(term->fsess);
 }
+
+
+
+GF_EXPORT
+Bool gf_term_is_supported_url(GF_Terminal *term, const char *fileName, Bool use_parent_url, Bool no_mime_check)
+{
+	char *parent_url = NULL;
+	if (!term || !term->compositor || !term->compositor->root_scene || !term->compositor->root_scene->root_od || !term->compositor->root_scene->root_od->scene_ns)
+		return GF_FALSE;
+		
+	if (use_parent_url && term->compositor->root_scene)
+		parent_url = term->compositor->root_scene->root_od->scene_ns->url;
+	return gf_filter_is_supported_source(term->compositor->filter, fileName, parent_url);
+}
