@@ -99,7 +99,7 @@ void gf_path_del(GF_Path *gp)
 
 #define GF_2D_REALLOC(_gp)	\
 	if (_gp->n_alloc_points < _gp->n_points+3) {	\
-		_gp->n_alloc_points = (_gp->n_alloc_points<5) ? 10 : (_gp->n_alloc_points*3/2);	\
+		_gp->n_alloc_points = (_gp->n_alloc_points<5) ? 10 : (_gp->n_alloc_points*2);	\
 		_gp->points = (GF_Point2D *)gf_realloc(_gp->points, sizeof(GF_Point2D)*(_gp->n_alloc_points));	\
 		_gp->tags = (u8 *) gf_realloc(_gp->tags, sizeof(u8)*(_gp->n_alloc_points));	\
 	}	\
@@ -304,14 +304,16 @@ GF_Err gf_path_add_subpath(GF_Path *gp, GF_Path *src, GF_Matrix2D *mx)
 	if (!gp->points) return GF_OUT_OF_MEM;
 	gp->tags = (u8*)gf_realloc(gp->tags, sizeof(u8)*gp->n_alloc_points);
 	if (!gp->tags) return GF_OUT_OF_MEM;
-	memcpy(gp->points + gp->n_points, src->points, sizeof(GF_Point2D)*src->n_points);
-	if (mx) {
-		for (i=0; i<src->n_points; i++) {
-			gf_mx2d_apply_coords(mx, &gp->points[i+gp->n_points].x, &gp->points[i+gp->n_points].y);
+	if (src->n_points) {
+		memcpy(gp->points + gp->n_points, src->points, sizeof(GF_Point2D)*src->n_points);
+		if (mx) {
+			for (i=0; i<src->n_points; i++) {
+				gf_mx2d_apply_coords(mx, &gp->points[i+gp->n_points].x, &gp->points[i+gp->n_points].y);
+			}
 		}
+		memcpy(gp->tags + gp->n_points, src->tags, sizeof(u8)*src->n_points);
+		gp->n_points += src->n_points;
 	}
-	memcpy(gp->tags + gp->n_points, src->tags, sizeof(u8)*src->n_points);
-	gp->n_points += src->n_points;
 	gf_rect_union(&gp->bbox, &src->bbox);
 	if (!(src->flags & GF_PATH_FLATTENED)) gp->flags &= ~GF_PATH_FLATTENED;
 	if (src->flags & GF_PATH_BBOX_DIRTY) gp->flags |= GF_PATH_BBOX_DIRTY;
@@ -998,6 +1000,7 @@ GF_EXPORT
 void gf_path_flatten(GF_Path *gp)
 {
 	GF_Path *res;
+	u32 flags = gp->flags;
 	if (gp->flags & GF_PATH_FLATTENED) return;
 	if (!gp->n_points) return;
 	res = gf_path_get_flatten(gp);
@@ -1005,6 +1008,7 @@ void gf_path_flatten(GF_Path *gp)
 	if (gp->points) gf_free(gp->points);
 	if (gp->tags) gf_free(gp->tags);
 	memcpy(gp, res, sizeof(GF_Path));
+	gp->flags |= (flags & (GF_PATH_FILL_ZERO_NONZERO|GF_PATH_FILL_EVEN));
 	gf_free(res);
 }
 
