@@ -690,37 +690,50 @@ void gf_sha1_finish(GF_SHA1Context *context, u8 output[GF_SHA1_DIGEST_SIZE] )
  * Output = SHA-1( file contents )
  */
 GF_EXPORT
-GF_Err gf_sha1_file( const char *path, u8 output[GF_SHA1_DIGEST_SIZE] )
+GF_Err gf_sha1_file_ptr(FILE *f, u8 output[GF_SHA1_DIGEST_SIZE] )
 {
-	FILE *f;
+	u64 pos = gf_ftell(f);
 	size_t n;
 	GF_SHA1Context *ctx;
 	u8 buf[1024];
 
-	if (!strncmp(path, "gmem://", 7)) {
-		u32 size;
-		u8 *mem_address;
-		GF_Err e = gf_blob_get_data(path, &mem_address, &size);
-		if (e) return e;
-
-		gf_sha1_csum(mem_address, size, output);
-		return GF_OK;
-	}
-
-	if( ( f = gf_fopen( path, "rb" ) ) == NULL )
-		return GF_URL_ERROR;
-
 	ctx  = gf_sha1_starts();
+	gf_fseek(f, 0, SEEK_SET);
 
 	while( ( n = gf_fread( buf, sizeof( buf ), f ) ) > 0 )
 		gf_sha1_update(ctx, buf, (s32) n );
 
 	gf_sha1_finish(ctx, output );
 
-	gf_fclose( f );
+	gf_fseek(f, pos, SEEK_SET);
 	return GF_OK;
 }
 
+GF_EXPORT
+GF_Err gf_sha1_file( const char *path, u8 output[GF_SHA1_DIGEST_SIZE] )
+{
+	FILE *f;
+	GF_Err e;
+
+	if (!strncmp(path, "gmem://", 7)) {
+		u32 size;
+		u8 *mem_address;
+		e = gf_blob_get(path, &mem_address, &size, NULL);
+		if (e) return e;
+
+		gf_sha1_csum(mem_address, size, output);
+        gf_blob_release(path);
+		return GF_OK;
+	}
+
+	if( ( f = gf_fopen( path, "rb" ) ) == NULL )
+		return GF_URL_ERROR;
+
+	e = gf_sha1_file_ptr(f, output);
+	gf_fclose( f );
+	return e;
+}
+
 /*
  * Output = SHA-1( input buffer )
  */
